# TEMPLATE CMAKE TOOLCHAIN FILE

# This file can be used for cross-compiling the Core C libraries. 
# The file is to be used as a template for different toolchain files. 
# The intention of this file is not to provide every property or variable that may or may not need to be set for various toolchains.
# Rather the file can be used as a starting point for creating the needed toolchain file for a given project that requires cross-compiling.   
# The template below is for a typical cross-compiling toolchain for Linux. Fill in the values labeled 'undefined' accordingly as directed. 
# Required and optional fields are labeled. Optional fields that are not used must be commented out.
#
# For detailed information about toolchains and cross-compiling with CMake see https://cmake.org/cmake/help/latest/manual/cmake-toolchains.7.html 
# and https://cmake.org/Wiki/CMake_Cross_Compiling.
#
# Referenced "Cross Compiling for Linux" section from https://cmake.org/cmake/help/latest/manual/cmake-toolchains.7.html 
# and CMake_Cross_Compiling from https://cmake.org/Wiki/CMake_Cross_Compiling for property descriptions and examples.

# CMAKE_SYSTEM_NAME, field is required. CMake-identifier of the target platform to build for. In this case 'Linux' is populated already where this is a typical toolchain for Linux.
set(CMAKE_SYSTEM_NAME Linux)

# CMAKE_SYSTEM_PROCESSOR, field is optional. CMake-identifier of the target architecture to build for. Generally only used when using a cross compiler where every target hardware needs special build settings.
set(CMAKE_SYSTEM_PROCESSOR undefined)

# CMAKE_<LANG>_COMPILER variables may be set to full paths, or to names of compilers to search for in standard locations. At least one CMAKE_<LANG>_COMPILER variable should be set, though multiple can be.
# In this template C and C++ compilers are set. An example path would be this 'usr/bin/arm-linux-gnueabihf-gcc'.
set(CMAKE_C_COMPILER undefined)
set(CMAKE_CXX_COMPILER undefined)

# CMAKE_SYSROOT, field is optional. May be specified if a sysroot is available on the given system. An example path would like '/home/devel/rasp-pi-rootfs'.
set(CMAKE_SYSROOT undefined)

# CMAKE_FIND_ROOT_PATH, field is optional. CMake-identifier for specifying where the target environment is installed for finding particular artifacts. The value is a list of directories.
# An example value for this field would be this '/opt/eldk-2007-01-19/ppc_74xx /home/user/eldk-ppc74xx-inst'
set (CMAKE_FIND_ROOT_PATH undefined)

# CMake 'find_*' commands will look in the sysroot, and the CMAKE_FIND_ROOT_PATH entries by default in all cases, as well as looking in the host system root prefix.
# Although this can be controlled on a case-by-case basis, when cross-compiling, it can be useful to exclude looking in either the host or the target for particular artifacts. 
# Generally, includes, libraries and packages should be found in the target system prefixes, whereas executables which must be run as part of the build should be found only on the host and not on the target. # This is the purpose of the CMAKE_FIND_ROOT_PATH_MODE_* variables. These fields are optional.
set(CMAKE_FIND_ROOT_PATH_MODE_PROGRAM NEVER)
set(CMAKE_FIND_ROOT_PATH_MODE_LIBRARY ONLY)
set(CMAKE_FIND_ROOT_PATH_MODE_INCLUDE ONLY)
set(CMAKE_FIND_ROOT_PATH_MODE_PACKAGE ONLY)