/*
**  Copyright (c) 2014-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef __avl_tree_H
#define __avl_tree_H

#include <dof/core.h>

/* made with the aid of Neil Brown's non-recursive AVL implementation http://neil.brown.name/blog/20041124101820
    and his article on non-recursive deletion at http://cgi.cse.unsw.edu.au/~neilb/me/01101186893

    but it's modified for specific requirements. */

typedef const void *OALAVLTreeKey_t;
typedef const void *OALAVLTreeItem_t;

typedef boolean (*OALAVLTree_CompareGreaterThan_fn_t)(OALAVLTreeKey_t key, OALAVLTreeItem_t item, void *context);
typedef boolean (*OALAVLTree_CompareEqual_fn_t)(OALAVLTreeKey_t key, OALAVLTreeItem_t item, void *context);

typedef struct OALAVLTree_t *OALAVLTree;

typedef boolean (*OALAVLTree_Traversal_fn_t)(OALAVLTreeItem_t item, void *arg);
typedef void (*OALAVLTree_RemoveAll_fn_t)(OALAVLTreeItem_t item, void *arg);

typedef enum OALAVLTree_InsertionStatus
{
    OALAVLTree_InsertFailure   = 0,     /* Force to be equal to FALSE */
    OALAVLTree_InsertSuccess   = 1,     /* Force to be equal to TRUE */
    OALAVLTree_AlreadyInserted          /* Value doesn't matter, except timer.c needs it not be DOFERROR_REJECTED (6).*/
} OALAVLTree_InsertionStatus;

/* compare_fn should return TRUE if d1>d2. you can pass NULL here for a default compare of the data pointer itself */
extern OALAVLTree OALAVLTree_Create(OALAVLTree_CompareGreaterThan_fn_t gt_fn, OALAVLTree_CompareEqual_fn_t eq_fn, void *context);
extern void OALAVLTree_Destroy(OALAVLTree tree, OALAVLTree_RemoveAll_fn_t remove_fn, void *arg);
extern void OALAVLTree_Empty(OALAVLTree tree, OALAVLTree_RemoveAll_fn_t remove_fn, void *arg);

extern OALAVLTree_InsertionStatus OALAVLTree_Insert(OALAVLTree tree, OALAVLTreeKey_t key, OALAVLTreeItem_t item, OALAVLTreeItem_t *ret);
extern boolean OALAVLTree_Delete(OALAVLTree tree, OALAVLTreeKey_t key);

extern OALAVLTreeItem_t OALAVLTree_Retrieve(OALAVLTree tree, OALAVLTreeKey_t key);

extern void OALAVLTree_Traverse(OALAVLTree tree, OALAVLTree_Traversal_fn_t traverse_fn, void *arg);

extern uint32 OALAVLTree_GetNumItems(OALAVLTree tree);

extern OALAVLTree OALAVLTree_CreateSubTree(OALAVLTree parent, OALAVLTree_CompareGreaterThan_fn_t gt_fn, OALAVLTree_CompareEqual_fn_t eq_fn, void *context);

#endif
