/*
**  Copyright (c) 2014-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DTLCONNECTION_H
#define DTLCONNECTION_H

#include <dof/pcr.h>
#include "socket.h"
#include <dof/oal.h>
#include <dof/inet.h>

typedef struct InetConnection_t
{
    DTLConnection_t     super;

    DOFAddress          addr;

    InetSocket          socket;

    DTLTransportHandler transportHandler;

    DTLServer           server;

    void              * context;

    boolean             isClosed;

    boolean             isBound;

    DOFConnectionType   type;

    DOFAddressType      aType;

    DTLConnectionConfig config;
} InetConnection_t;

typedef struct InetConnection_t * InetConnection;

typedef struct DOFInetConnectionConfig_t
{
    DOFAddress        addr;

    InetSocket        socket;

    DOFConnectionType type;
} DOFInetConnectionConfig_t;

typedef struct DOFInetConnectionConfig_t * DOFInetConnectionConfig;

InetConnection InetConnection_Create(DOFAddress            addr,
                                     InetSocket            socket,
                                     DTLTransportHandler   transportHandler,
                                     DTLServer             server,
                                     DOFConnectionType     type,
                                     boolean               isBound,
                                     DOFAddressType        aType,
                                     DTLConnectionConfig   config,
                                     void                * context);

void InetConnection_Destroy(InetConnection connection);

DOFInetConnectionConfig DOFInetConnectionConfig_Create(DOFAddress addr,
                                                       InetSocket socket);

void DOFInetConnectionConfig_Destroy(DOFInetConnectionConfig config);

#endif
