/*
**  Copyright (c) 2014-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include "config.h"
#include <dof/pcr.h>
#include "socket.h"
#include <dof/oal.h>
#include <dof/inet.h>

#include <errno.h>

#include "inet_send.h"

void Inet_ConnectedSend(InetSocketNode node)
{
    /* inetMutex must already be locked */
    PCR_Assert(node != NULL);
    if(node->socketData.sendQueue != NULL && !node->socketData.shouldDestroy)
    {
        PartialSendData sendData = OALQueue_GetFront(node->socketData.sendQueue);

        PCR_Assert(sendData != NULL);
        {
            uint32 toWrite = sendData->length - sendData->offset;

            uint32 bytesWritten = InetSocket_Send(node->socketData.sockFd,
                                                  &sendData->data[sendData->offset],
                                                  toWrite, MSG_DONTWAIT);

            if(bytesWritten < toWrite)
            {
                sendData->offset += bytesWritten;
            }
            else
            {
                if ((bytesWritten != 0xFFFFFFFF) || (errno != EAGAIN && errno != EWOULDBLOCK))
                {
                    OALQueue_Dequeue(node->socketData.sendQueue);
                    PCRMem_Free(sendData->data);
                    PCRMem_Free(sendData);
                    if(OALQueue_GetSize(node->socketData.sendQueue) == 0 || bytesWritten == 0xFFFFFFFF)
                    {
                        InetManager_PartialSendComplete(node->socketData.sockFd);
                    }
                    if(bytesWritten == 0xFFFFFFFF)
                    {
                        InetManager_RemoveNode(node, INETM_SEND_FAILED);
                    }
                }
            }
        }
    }
    else
    {
        InetManager_PartialSendComplete(node->socketData.sockFd);
    }
}

void Inet_ConnectionlessSend(InetSocketNode node)
{
    /* inetMutex must already be locked */
    PCR_Assert(node != NULL);
    if(node->socketData.sendQueue != NULL && !node->socketData.shouldDestroy)
    {
        PartialConnectionlessSendData sendData = OALQueue_GetFront(node->socketData.sendQueue);

        PCR_Assert(sendData != NULL);

        Inet_Connectionless_SendTo( node->socketData.sockFd, sendData->address, sendData->sendData.length, sendData->sendData.data );
        /* Ignore errors on directed datagram socket sends. */
        OALQueue_Dequeue(node->socketData.sendQueue);
        PCRMem_Free(sendData->sendData.data);
        DOFAddress_Destroy(sendData->address);
        PCRMem_Free(sendData);
        if ( OALQueue_GetSize(node->socketData.sendQueue) != 0 )
            return;
    }
    InetManager_PartialSendComplete(node->socketData.sockFd);
}
