/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_ITEM_H
#define DOF_ITEM_H

#include <dof/core.h>

#ifdef  __cplusplus
extern "C" {
#endif

/*
**  "Public" Type and Method Declarations
*/


/** @defgroup DOFInterfaceException DOFInterfaceException
    @ingroup DOFInterface

@{
**/

/* DOFInterfaceException Methods */

/**
    Get the Interface ID associated with an Exception.
    @param except The @ref DOFInterfaceException. This must not be NULL.
    @note A valid Exception is obtained from calling @ref DOFInterface_GetException().
    @return    The @ref DOFInterfaceID.
**/
extern DOF_OAL_DECLSPEC DOFInterfaceID DOFInterfaceException_GetInterfaceID( const DOFInterfaceException except );

/**
    Get the Interface associated with an Exception.
    @param except The @ref DOFInterfaceException. This must not be NULL.
    @note A valid Exception is obtained from calling @ref DOFInterface_GetException().
    @return    The @ref DOFInterface.
**/
extern DOF_OAL_DECLSPEC DOFInterface DOFInterfaceException_GetInterface( const DOFInterfaceException except );

/**
    Get the Item ID associated with an Exception.
    @param except The @ref DOFInterfaceException. This must not be NULL.
    @note A valid Exception is obtained from calling @ref DOFInterface_GetException().
    @return The @ref DOFItemID.
**/
extern DOF_OAL_DECLSPEC DOFItemID DOFInterfaceException_GetItemID( const DOFInterfaceException except );

/**@}**/



/** @defgroup DOFInterfaceEvent DOFInterfaceEvent
    @ingroup DOFInterface

@{
**/

/* DOFInterfaceEvent Methods */

/**
    Get the Interface ID associated with an Event.
    @param event The @ref DOFInterfaceEvent. This must not be NULL.
    @note A valid Event is obtained from calling @ref DOFInterface_GetEvent().
    @return    The @ref DOFInterfaceID.
**/
extern DOF_OAL_DECLSPEC DOFInterfaceID DOFInterfaceEvent_GetInterfaceID( const DOFInterfaceEvent event );

/**
    Get the Interface associated with an Event.
    @param event The @ref DOFInterfaceEvent. This must not be NULL.
    @note A valid Event is obtained from calling @ref DOFInterface_GetEvent().
    @return    The @ref DOFInterface.
**/
extern DOF_OAL_DECLSPEC DOFInterface DOFInterfaceEvent_GetInterface( const DOFInterfaceEvent event );

/**
    Get the Item ID associated with an Event.
    @param event The @ref DOFInterfaceEvent. This must not be NULL.
    @note A valid Event is obtained from calling @ref DOFInterface_GetEvent().
    @return The @ref DOFItemID.
**/
extern DOF_OAL_DECLSPEC DOFItemID DOFInterfaceEvent_GetItemID( const DOFInterfaceEvent event );

/**@}**/


/** @defgroup DOFInterfaceMethod DOFInterfaceMethod
    @ingroup DOFInterface

@{
**/

/* DOFInterfaceMethod Methods */

/**
    Get the Interface ID associated with a Method.
    @param method The @ref DOFInterfaceMethod. This must not be NULL.
    @note A valid Method is obtained from calling @ref DOFInterface_GetMethod().
    @return    The @ref DOFInterfaceID.
**/
extern DOF_OAL_DECLSPEC DOFInterfaceID DOFInterfaceMethod_GetInterfaceID( const DOFInterfaceMethod method );

/**
    Get the Interface associated with a Method.
    @param method The @ref DOFInterfaceMethod. This must not be NULL.
    @note A valid Method is obtained from calling @ref DOFInterface_GetMethod().
    @return    The @ref DOFInterface.
**/
extern DOF_OAL_DECLSPEC DOFInterface DOFInterfaceMethod_GetInterface( const DOFInterfaceMethod method );

/**
    Get the Item ID associated with a Method.
    @param method The @ref DOFInterfaceMethod. This must not be NULL.
    @note A valid Method is obtained from calling @ref DOFInterface_GetMethod().
    @return The @ref DOFItemID.
**/
extern DOF_OAL_DECLSPEC DOFItemID DOFInterfaceMethod_GetItemID( const DOFInterfaceMethod method );

/**@}**/


/** @defgroup DOFInterfaceProperty DOFInterfaceProperty
    @ingroup DOFInterface

@{
**/

/* DOFInterfaceProperty Methods */

/**
    Get the Interface ID associated with a Property.
    @param property The @ref DOFInterfaceProperty. This must not be NULL.
    @note A valid Property is obtained from calling @ref DOFInterface_GetProperty().
    @return    The @ref DOFInterfaceID.
**/
extern DOF_OAL_DECLSPEC DOFInterfaceID DOFInterfaceProperty_GetInterfaceID( const DOFInterfaceProperty property );

/**
    Get the Interface associated with a Property.
    @param property The @ref DOFInterfaceProperty. This must not be NULL.
    @note A valid Property is obtained from calling @ref DOFInterface_GetProperty().
    @return    The @ref DOFInterface.
**/
extern DOF_OAL_DECLSPEC DOFInterface DOFInterfaceProperty_GetInterface( const DOFInterfaceProperty property );

/**
    Get the Item ID associated with a Property.
    @param property The @ref DOFInterfaceProperty. This must not be NULL.
    @note A valid Property is obtained from calling @ref DOFInterface_GetProperty().
    @return The @ref DOFItemID.
**/
extern DOF_OAL_DECLSPEC DOFItemID DOFInterfaceProperty_GetItemID( const DOFInterfaceProperty property );

/**@}**/


#ifdef  __cplusplus
}
#endif

#endif /* DOF_ITEM_H */
