/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_LOG_H
#define DOF_LOG_H


#include <dof/core.h>


#ifdef  __cplusplus
extern "C" {
#endif

/** @defgroup DOFLog DOFLog
    @ingroup DOF

    DOFLog is a static logging class for use by the application to facilitate
    notification of runtime log information.

    The library itself must be initialized before utilizing the DOFLog API,
    using @ref DOF_Init.

@{
**/

/*
**  "Public" Type and Method Declarations
*/


/**
 * Adds a callback to be notified of log messages.
 *
 * @param priority The priority level(s) of messages to be delivered to the callback (multiple @ref DOFLogLevel enumerations can be ORed together to indicate the levels desired).
 * @param callback The callback to add.  This should not be NULL.
 * @retval Non-zero (@c TRUE) if the listener was added.
 * @exception Zero (@c FALSE) if the listener could not be added, illegal arguments were detected, or the listener has already been addded.
 */
extern DOF_OAL_DECLSPEC boolean DOFLog_AddListener( DOFLogLevel priority, DOFLogListener callback );

/**
 * Removes a previously added log callback.  If the callback cannot be found, no action is taken.
 *
 * @param callback The callback to remove.  This should not be NULL.
 */
extern DOF_OAL_DECLSPEC void DOFLog_RemoveListener( DOFLogListener callback );

/**
 * Compares a log priority level against all of a dof's activated log listener priority levels.
 *
 * @param priority The priority level to be compared. @see DOFLogLevel
 * @retval TRUE The priority level is activated.
 * @retval FALSE The priority level is not activated.
 */
extern DOF_OAL_DECLSPEC boolean DOFLog_IsLogLevel( DOFLogLevel priority );

/**
 * Logs a message to all Log Listeners on a dof with the Appropriate priority level.
 * Each log listener can receive logs for multiple priority levels, but all listeners
 * will only receive 1 log message per logged message.
 *
 * @param priority The priority level of the log. @see DOFLogLevel
 * @param message The String message to log
 */
extern DOF_OAL_DECLSPEC void DOFLog_Message( DOFLogLevel priority, const char * message );


#ifdef  __cplusplus
}
#endif

/**
@}
**/

#endif /* DOF_LOG_H */
