/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_OAL_H
#define DOF_OAL_H

/** @mainpage The C Object Access Library (OAL)
    @section intro Introduction

        The C Object Access Library (OAL) enables software developers
        to create applications for the Distributed Objects Framework (DOF).
        The C OAL implements the DOF Object Access Library
        Specification, allowing the full capabilities and power of DOF as a
        library to standard C applications.  Please refer to the DOF
        Overview document for a more comprehensive description of DOF and
        its powerful capabilities.

        The C OAL allows for easy discovery of and communication with other
        nodes within the DOF network.  A single node can choose to Provide
        information to the network and other nodes, Request information of the
        network or other nodes, or a combination of both.

        The C OAL is implemented as a portable, cross-platform library.  For
        any given target platform, there must be a platform-specific library
        providing the appropriate access to hardware and OS functionality used
        by the C OAL.

        An application need only use the portions of the library that are
        appropriate; unused features will be removed by the linker to maintain
        a minimal embedded application code size.

    @section introUsage Using This Reference

        This reference defines the APIs for use by a C-based OAL application.

    @section bugs How to Report a Bug

        See the README file associated with this project.

    @section addinfo Additional Information

        See the COPYING and README files associated with this project.
**/

/** @defgroup DOF_Framework Distributed Object Framework (DOF)

    The Distributed Object Framework (DOF) is the core collection of data
    types, strucutres, interfaces, and functions that are part of the C Object
    Access Library.

    Refer to the individual DOF modules for more information.
**/

/** @file
    Header file for the C OAL functions, defines and data types.
    @ingroup DOF_Framework
    @ingroup DTL
    @ingroup DOF
    @ingroup   DOFConfig
    @ingroup   DOFState
    @ingroup   DOFLog
    @ingroup   DOFMem
    @ingroup DOFAddress
    @ingroup DOFAuthenticator
    @ingroup   DOFAuthenticatorConfig
    @ingroup   DOFAuthenticatorState
    @ingroup DOFCipherRegistry
    @ingroup DOFConfig
    @ingroup DOFConnection
    @ingroup   DOFConnectionConfig
    @ingroup   DOFConnectionState
    @ingroup DOFCredentials
    @ingroup DOFDomain
    @ingroup   DOFDomainConfig
    @ingroup   DOFDomainState
    @ingroup DOFError
    @ingroup DOFException
    @ingroup DOFGroupAddress
    @ingroup DOFInterface
    @ingroup   DOFInterfaceEvent
    @ingroup   DOFInterfaceException
    @ingroup   DOFInterfaceMethod
    @ingroup   DOFInterfaceProperty
    @ingroup DOFInterfaceID
    @ingroup DOFObject
    @ingroup DOFObjectID
    @ingroup   DOFObjectIDAttribute
    @ingroup   DOFObjectIDAuthentication
    @ingroup   DOFObjectIDDomain
    @ingroup   DOFObjectIDSource
    @ingroup DOFOperation
    @ingroup   DOFOperationControl
    @ingroup   DOFOperationBridgeConfig
    @ingroup   DOFOperationFilter
    @ingroup DOFPermissionSet
    @ingroup DOFProtocolNegotiator
    @ingroup   DOFDefaultProtocolNegotiator
    @ingroup DOFProviderInfo
    @ingroup DOFQuery
    @ingroup DOFRequest
    @ingroup DOFResult
    @ingroup DOFRuntime
    @ingroup DOFSecurityMode
    @ingroup   DOFSecurityModeCCM
    @ingroup DOFSecurityScope
    @ingroup DOFServer
    @ingroup   DOFServerConfig
    @ingroup   DOFServerState
    @ingroup DOFSystem
    @ingroup   DOFSystemConfig
    @ingroup   DOFSystemState
    @ingroup DOFType
    @ingroup DOFValue
    @ingroup   DOFValueArray
    @ingroup   DOFValueStructure
    @ingroup DOFTypes
**/


/* Include lower-level and platform-specific information. */
#include <dof/core.h>

/* Include the DOF OAL API */
#include <dof/address.h>
#include <dof/authenticator.h>
#include <dof/bridge.h>
#include <dof/cfg.h>
#include <dof/cipherregistry.h>
#include <dof/connection.h>
#include <dof/control.h>
#include <dof/credentials.h>
#include <dof/dof.h>
#include <dof/domain.h>
#include <dof/error.h>
#include <dof/exception.h>
#include <dof/filter.h>
#include <dof/interface.h>
#include <dof/interfaceid.h>
#include <dof/item.h>
#include <dof/log.h>
#include <dof/object.h>
#include <dof/objectid.h>
#include <dof/operation.h>
#include <dof/permission.h>
#include <dof/permissionset.h>
#include <dof/providerinfo.h>
#include <dof/query.h>
#include <dof/request.h>
#include <dof/result.h>
#include <dof/routeinfo.h>
#include <dof/runtime.h>
#include <dof/securitymode.h>
#include <dof/securityscope.h>
#include <dof/server.h>
#include <dof/system.h>
#include <dof/transport.h>
#include <dof/type.h>
#include <dof/value.h>


#endif /* DOF_OAL_H */
