/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_OALTYPES_H
#define DOF_OALTYPES_H

#include <dof/pcr.h>
#include <dof/dofdefs.h>

#ifdef  __cplusplus
extern "C" {
#endif


/** @defgroup DOFTypes Standard DOF Types and Definitions
    @ingroup DOF_Framework

    Common types and definitions of the DOF C-Language Object Access Library.

@{
**/

/* Data types */
typedef struct DOF_t *                DOF;                        /**< DOF is the foundation class of the DOF Framework.  @ingroup DOF **/
typedef struct DOFQuery_t *           DOFQuery;                   /**< Describes interest for query operations. @ingroup DOFQuery  **/
typedef struct DOFQueryBuilder_t *    DOFQueryBuilder;            /**< Describes a builder for query operations. @ingroup DOFQUERY **/
typedef struct DOFInterface_t *       DOFInterface;               /**< DOF Interface representation. @ingroup DOFInterface **/
typedef struct DOFInterfaceID_t *     DOFInterfaceID;             /**< DOF Interface Identifier. @ingroup DOFInterfaceID **/
typedef uint16                        DOFItemID;                  /**< Item identifier, used in defining properties, events, methods, and exceptions of a DOF Interface. **/
typedef const struct DOFInterfaceProperty_t *  DOFInterfaceProperty;                /**< Definition of a Property within a DOF Interface. @ingroup DOFInterface **/
typedef const struct DOFInterfaceEvent_t *     DOFInterfaceEvent;                   /**< Definition of an Event within a DOF Interface. @ingroup DOFInterface **/
typedef const struct DOFInterfaceMethod_t *    DOFInterfaceMethod;                  /**< Definition of a Method within a DOF Interface. @ingroup DOFInterface **/
typedef const struct DOFInterfaceException_t * DOFInterfaceException;               /**< Definition of an Exception within a DOF Interface. @ingroup DOFInterface **/
typedef struct DOFException_t *       DOFException;               /**< DOF Exception return type.  This can contain either an error code, or a @ref DOFInterfaceException and a list of @ref DOFValue parameters, defined in an Interface.  Where appropriate, this is used as an output parameter to indicate an exception occurred (since the C language does not natively support exceptions). **/
typedef uint32                        DOFObjectIDSize;            /**< DOF Object Identifier storage size type. @ingroup DOFObjectID **/
typedef uint8                         DOFObjectIDDataSize;        /**< DOF Object Identifier data field size type. @ingroup DOFObjectID **/
typedef uint8                         DOFObjectIDAttributeDataSize;/**< DOF Object Identifier Attribute size type. @ingroup DOFObjectID **/
typedef struct DOFOperation_t *       DOFOperation;               /**< Represents an asynchronous operation. @ingroup DOFOperation **/
typedef struct DOFRequest_t *         DOFRequest;                 /**< Represents a request from another node in the DOF network, and contains a place to store error information for the Provider callbacks. @ingroup DOFRequest **/
typedef struct DOFOperationControl_t * DOFOperationControl;         /**< Configuration for a request operation. @ingroup DOFOperationControl **/
typedef struct DOFAddress_t *         DOFAddress;                 /**< Represents a network address **/
typedef struct DOFResult_t *          DOFResult;                  /**< A collection of information pertaining to the result of a remote method invocation or property accessor. @ingroup DOFResult **/
typedef struct DOFRuntime_t *         DOFRuntime;                 /**< The Runtime information and statistics for a DOF. @ingroup DOFRuntime **/
typedef struct DOFRouteInfo_t *       DOFRouteInfo;               /**< A collection of meta data about the route used by specified objects and interfaces. **/
typedef struct DOFServer_t *          DOFServer;                  /**< A representation of a standard DOF server. @ingroup DOFServer **/
typedef struct DOFServerState_t *     DOFServerState;             /**< A snapshot of the current state of a @ref DOFServer. @ingroup DOFServer **/
typedef uint32                        DOFError;                   /**< Standard OAL error codes type. @ingroup DOFException **/
typedef struct DOFSystem_t *          DOFSystem;                  /**< Represents the DOF System, from the application's perspecitive, including security identification. @ingroup DOFSystem **/
typedef struct DOFObject_t *          DOFObject;                  /**< DOF Object representation, used for both locally defined and remote objects. @ingroup DOFObject **/
typedef uint32                        DOFObjectIDClass;           /**< DOF Object ID Class representation.  Typically one of the enumerations found in @ref DOFObjectIDClass_t. @ingroup DOFObjectID. */
typedef struct DOFInterfaceIDList_t * DOFInterfaceIDList;         /**< DOF InterfaceID List. @ingroup DOFInterfaceID **/
typedef struct DOFObjectIDList_t *    DOFObjectIDList;            /**< DOF ObjectID List. @ingroup DOFObjectID **/
typedef struct DOFObjectID_t *        DOFObjectID;                /**< DOF Object Identifier. @ingroup DOFObjectID **/
typedef struct DOFObjectIDDomain_t *  DOFObjectIDDomain;          /**< DOF Domain Identifier. @ingroup DOFObjectID **/
typedef struct DOFObjectIDSource_t *  DOFObjectIDSource;          /**< DOF Source Identifier. @ingroup DOFObjectID **/
typedef struct DOFObjectIDAuthentication_t *DOFObjectIDAuthentication;  /**< DOF Authentication Identifier. @ingroup DOFObjectID **/
typedef const struct DOFType_t *      DOFType;                    /**< DOF Data Type Representation.  @ingroup DOFType **/
typedef struct DOFValue_t *           DOFValue;                   /**< A container for any value.  This is the base class for any constructed value, including DOFValueArray and DOFValueStructure.  Used as the value of a Property, or a parameter of a Method, Event, or Exception. @ingroup DOFValue **/
typedef uint16                        DOFTypeID;                  /**< Identifies the DOF Data Type Identifier of a DOFType. @ingroup DOFType **/
typedef struct DOFSecurityScope_t *   DOFSecurityScope;           /**< A complete, immutable security scope definition.  This respresents the scope in which a permission, operation, or connection is valid, and governs its security compatibility. @ingroup DOFSecurityScope **/
typedef struct DOFPermission_t*       DOFPermission;              /**< Immutable Permission.  @ingroup DOFPermission **/
typedef struct DOFPermissionBuilder_t * DOFPermissionBindingBuilder; /**< Binding Permission Builder.  @ingroup DOFPermission **/
typedef struct DOFPermissionSet_t*    DOFPermissionSet;           /**< Immutable Permission Set.  Always built from a @ref DOFPermissionSetBuilder.  @ingroup DOFPermissionSet **/
typedef struct DOFPermissionSetBuilder_t * DOFPermissionSetBuilder;    /**< Permission Set Builder.  @ingroup DOFPermissionSet **/
typedef struct DOFCredentials_t *     DOFCredentials;             /**< Authentication credentials (identity, secret, etc.).  @ingroup DOFCredentials **/
typedef struct DOFSecurityMode_t *    DOFSecurityMode;            /**< Security Mode Configuration.  @ingroup DOFSecurityMode **/
typedef struct DOFConnection_t*       DOFConnection;              /**< A connection definition. @ingroup DOFConnection **/
typedef struct DOFConnectionState_t*  DOFConnectionState;         /**< A snapshot of the current state of a @ref DOFConnection. @ingroup DOFConnection **/
typedef struct DOFConnectionConfig_t *DOFConnectionConfig;        /**< Immutable Connection configuration. @ingroup DOFConnectionConfig **/
typedef struct DOFConnectionConfigBuilder_t *DOFConnectionConfigBuilder;  /**< Connection configuration builder. @ingroup DOFConnectionConfig **/
typedef struct DOFServerConfig_t *    DOFServerConfig;            /**< Immutable Server configuration. @ingroup DOFServerConfig **/
typedef struct DOFServerConfigBuilder_t * DOFServerConfigBuilder;     /**< Server configuration builder. @ingroup DOFServerConfig **/
typedef struct DOFProtocolNegotiator_t *DOFProtocolNegotiator;    /**< An interface for negotiating the protocols for inbound and outbound connections. @ingroup DOFProtocolNegotiator **/
typedef struct DOFAuthenticator_t *          DOFAuthenticator;                  /**< A representation of a standard DOF Authenticator. @ingroup DOFAuthenticator **/
typedef struct DOFAuthenticatorList_t *      DOFAuthenticatorList;              /**< DOF Authenticator List. @ingroup DOFAuthenticator **/
typedef struct DOFAuthenticatorConfig_t *    DOFAuthenticatorConfig;            /**< Authenticator configuration. @ingroup DOFAuthenticator **/
typedef struct DOFAuthenticatorConfigBuilder_t * DOFAuthenticatorConfigBuilder; /**< Authenticator configuration Builder. @ingroup DOFAuthenticator **/
typedef struct DOFAuthenticatorState_t * DOFAuthenticatorState;   /**< Authenticator state. @ingroup DOFAuthenticator **/
typedef struct DOFDomain_t*           DOFDomain;                  /**< A representation of a standard DOF Domain. @ingroup DOFDomain **/
typedef struct DOFDomainList_t*       DOFDomainList;              /**< DOF Domain List. @ingroup DOFDomain **/
typedef struct DOFDomainConfig_t*     DOFDomainConfig;            /**< Domain configuration. @ingroup DOFDomain **/
typedef struct DOFDomainConfigBuilder_t* DOFDomainConfigBuilder;  /**< Domain configuration builder. @ingroup DOFDomain **/
typedef struct DOFDomainState_t*      DOFDomainState;             /**< Domain state. @ingroup DOFDomain **/
typedef struct DOFProviderInfo_t *    DOFProviderInfo;            /**< Information about the source (Provider) of a result or exception. @ingroup DOFProviderInfo **/
typedef struct DOFConfig_t *          DOFConfig;                  /**< The configuration for a DOF instance.  Defined as a pointer to the DOF Configuration structure. @ingroup DOFConfig **/
typedef struct DOFConfigBuilder_t*    DOFConfigBuilder;           /**< The configuration builder to build a DOFConfig from.  Defined as a pointer to the DOF Configuration builder structure. @ingroup DOFConfig **/
typedef struct DOFState_t*            DOFState;                   /**< The configuration state.  Defined as a pointer to the DOF Configuration state structure. @ingroup DOFConfig **/
typedef struct DOFOperationFilter_t * DOFOperationFilter;         /**< A type to filter unwanted DOFOperations. @ingroup DOFOperation **/
typedef struct DOFOperationFilterBuilder_t* DOFOperationFilterBuilder;  /**< A type to build and configure a DOFOperationFilter. @ingroup DOFOperation **/
typedef struct DOFOperationBridgeConfig_t* DOFOperationBridgeConfig;    /**< A type to control the DOFDomain a DOFOperation is presented in. @ingroup DOFOperation **/
typedef struct DOFOperationBridgeConfigBuilder_t* DOFOperationBridgeConfigBuilder;  /**< A type to build and configure a DOFOperationBridgeConfig. @ingroup DOFOperation **/
typedef struct DOFSystemState_t *     DOFSystemState;             /**< Provides current information about a DOFSystem. @ingroup DOFSystem **/
typedef struct DOFSystemConfig_t *    DOFSystemConfig;            /**< A type to control certain behavior in a DOFSystem. @ingroup DOFSystem **/
typedef struct DOFSystemConfigBuilder_t* DOFSystemConfigBuilder;  /**< A type to configure and build a DOFSystemConfig. @ingroup DOFSystem **/
typedef const struct BlockCipher *    DOFCipher;                  /**< An implementation of a block cipher. @see BlockCipher @ingroup DOFCipherRegistry **/
typedef struct DOFConnectionStreamRequest_t *DOFConnectionStreamRequest; /**< A representation of a request for a streaming connection. @ingroup DOFConnection **/

/* Enumerations */

/**
    Predefined DOF Query match styles, defining query match behavior.
    @ingroup DOFQuery
**/
typedef enum DOFQueryMatchStyle_t
{
    DOFQUERYMATCHSTYLE_AT_LEAST, /**< Match base OIDs and attribute sets, but allow the ID to contain additional attributes not specified by the other. **/
    DOFQUERYMATCHSTYLE_BASE,     /**< Match base OIDs only (no attributes). **/
    DOFQUERYMATCHSTYLE_EXACT     /**< Match base OIDs and attribute sets exactly. **/
    /* Note: do not reorder these, DOFFilterBlock_IsObjInBlock() requires their values to be 0, 1, 2, as shown. */
} DOFQueryMatchStyle_t;


/**
    Predefined DOF Object Identifier Attribute ID codes, defining the data
    with which an Object ID can be augmented. These values must fit in 7 bits.
    @ingroup DOFObjectIDAttribute
**/
typedef enum DOFObjectIDAttributeID_t
{
    /**
     * Provider attribute. This attribute identifies an object as being
     * provided by a specific service provider. The associated data must
     * be an object identifier.
     */
    DOFOBJECTIDATTRIBUTE_PROVIDER  =  0,

    /**
     * Session attribute. This attribute associates the object with the
     * specified session. The associated data must be exactly 16 bytes long.
     */
    DOFOBJECTIDATTRIBUTE_SESSION   =  1,

    /**
     * Group attribute. This attribute is normally used in association
     * with the BROADCAST object identifier. It defines a target that is
     * a multicast group in the DOF network (as opposed to the transport).
     * The associated data must be an object identifier.
     */
    DOFOBJECTIDATTRIBUTE_GROUP     =  2,

    /**
     * Binding Provider attribute.This Attribute identifies the provider
     * of a binding. It can be used in cases where the provider of a
     * specific binding wants to provide additional related bindings.
     */
    DOFOBJECTIDATTRIBUTE_BINDING_PROVIDER = 3,

    /**
     * Related Time attribute. For objects that are associated with a specific
     * date and time, this attribute can express that relationship directly
     * as part of the OID.
     */
    DOFOBJECTIDATTRIBUTE_RELATED_TIME = 4,

    /**
     * Created Time attribute. For objects that are created at a specific date and
     * time, this attribute can express that relationship directly as part of the OID.
     */
    DOFOBJECTIDATTRIBUTE_CREATED_TIME = 5,

    /**
     * Invalid, used to signal that an error has occurred.
     */
    DOFOBJECTIDATTRIBUTE_INVALID   =  128
} DOFObjectIDAttributeType;

/**
 * DOF Object Identifier Classification codes, used for defining the format
 * of an Object ID.  This enumeration contains the standard, registered Object
 * ID classes at the time of the library's release.  Additional classes may
 * have beed defined; refer to the DOF Protocol Specification for more
 * information.  These are 12-bit constants.
 *
 * @ingroup DOFObjectID
 */
typedef enum DOFObjectIDClass_t
{
    DOFOBJECTIDCLASS_BROADCAST   = 0, /**< The Broadcast Object ID Class.  This is also used as a wildcard, or with the Group Attribute for multicast.  The data field of the broadcast ID class must be empty. **/
    DOFOBJECTIDCLASS_MAC         = 2, /**< The MAC address Object ID Class.  The data field must contain at least 6 bytes, starting with a valid MAC address. **/
    DOFOBJECTIDCLASS_EMAIL       = 3, /**< The E-mail Object ID Class.  The data field must contain at least 5 bytes, starting with a valid email address string, excluding any terminating NULL. **/
    DOFOBJECTIDCLASS_IPV4        = 4, /**< The IPv4 address Object ID Class.  The data field must contain at least 4 bytes, starting with a valid IPv4 address. **/
    DOFOBJECTIDCLASS_IPV6        = 5, /**< The IPv6 address Object ID Class.  The data field must contain at least 16 bytes, starting with a valid IPv6 address. **/
    DOFOBJECTIDCLASS_DOMAIN      = 6, /**< The Domain (FQDN) Object ID Class.  The data field must contain at least 3 bytes, representing the domain as specified in RFC2822, "dot-atom". **/
    DOFOBJECTIDCLASS_LODLC       = 7, /**< The LoDLC address Object ID Class.  The data field must contain at least 4 bytes, starting with a valid LoDLC address. **/
    DOFOBJECTIDCLASS_AS          = 8, /**< The Authentication Server (AS) Assigned Object ID Class.  The data field must contain at least 7 bytes, as described by the DOF Common Types Specification. **/
    DOFOBJECTIDCLASS_EUI48       = 9, /**< The EUI-48&trade; Object ID Class.  The data field must contain a 48-bit Extended Unique Identifier (as defined by IEEE), MSB first. **/
    DOFOBJECTIDCLASS_EUI64       = 10, /**< The EUI-64&trade; Object ID Class.  The data field must contain a 64-bit Extended Unique Identifier (as defined by IEEE), MSB first. **/
    DOFOBJECTIDCLASS_HIDLC       = 11, /**< The HiDLC Object ID Class.  This is assigned to Panasonic Electric Works who defines the structure and guarantees uniqueness. **/
    DOFOBJECTIDCLASS_SMART_ENERGY_SYSTEM_SERIAL_NUMBER = 12, /**< The Smart Energy System Serial Number object identifier class. Contains a maximum of 32 characters. Non-printing characters are not allowed. */
    DOFOBJECTIDCLASS_AWS_RESOURCE_ID = 13, /**< The AWS Resource ID object identifier class. The data contains a case-sensitive ASCII string. Contains a maximum of 32 characters. Non-printing characters are not allowed.*/
    DOFOBJECTIDCLASS_IMEI        = 14, /**< The IMEI Object ID Class. */
    DOFOBJECTIDCLASS_ODP         = 127, /**< The The OpenDOF Project Object ID Class. OpenDOF Project Assigned Object ID Class.  The data field must contain at least 1 byte, as described by the DOF Common Types Specification. **/
    DOFOBJECTIDCLASS_GUID        = 128,  /**< The Globally Unique Identifier (GUID) Object ID Class.  The data field must contain exactly 16 bytes, representing a GUID. **/
    DOFOBJECTIDCLASS_ODP_STRING  = 129, /**< The OpenDOF Project String Object ID Class. */
    DOFOBJECTIDCLASS_PANASONIC_SEBD_FUEL_CELL = 16384, /**< The Panasonic SEBD Fuel-cell Object ID Class. */
    DOFOBJECTIDCLASS_ECHONET_LITE_NODE_ID = 16385 /**< The ECHONET Lite Node Identification Object ID Class. */
} DOFObjectIDClass_t;

/**
 * Predefined response levels for reducing unwanted response traffic.
 *
 * @ingroup DOFOperationControl
 */
typedef enum DOFOperationResponseLevel_t
{
    DOFOPERATIONRESPONSELEVEL_ALL       =    3,  /**< Specifies that all responses (successful, excptional, or intermediate) are desired. **/
    DOFOPERATIONRESPONSELEVEL_EXCEPTION =    2,  /**< Specifies that all final responses (successful or exceptional) are desired (excludes intermediate exceptions). **/
    DOFOPERATIONRESPONSELEVEL_SUCCESS   =    1,  /**< Specifies that only successful responses are desired (all exceptions on the provider are ignored).  If the request results in an exception, there will be no indication -- the operation will simply time out.  Locally-generated exceptions (such as timeout) are always reported. **/
    DOFOPERATIONRESPONSELEVEL_NONE      =    0   /**< Specifies that no response is desired (execute the request, but do not respond).  This level does not apply to operations where there is no purpose in the request without the response (e.g., a Get).  Locally-generated exceptions (such as timeout) are always reported. **/
} DOFOperationResponseLevel;

/**
 *  The different levels of Interest that can be requested.
 *
 *  @see DOFSystem_BeginInterest
 */
typedef enum DOFInterestLevel_t
{
    /* WARNING: router's interest stuff depends on these in the order of WATCH, ACTIVATE, CONNECT */
    DOFINTERESTLEVEL_WATCH      = 0,    /**< Watch interest level: notification only (there is no requirement to take any action). **/
    DOFINTERESTLEVEL_ACTIVATE   = 1,    /**< Activate interest level: make available if possible (e.g., by starting a BeginProvide), in addition to Watch. **/
    DOFINTERESTLEVEL_CONNECT    = 2     /**< Connect interest level: make available over a streaming connection if possible (e.g., by connecting to a provider), in addition to Activate. **/
} DOFInterestLevel;

/**
 *  The type of request used for a received operation.
 *  This corresponds to the OAP addressing type, not the transport addressing type.
 */
typedef enum DOFRequestType_t
{
    DOFREQUESTTYPE_UNICAST,     /**< Unicast request; non-wildcard ObjectID. **/
    DOFREQUESTTYPE_MULTICAST,   /**< Multicast request; wildcard ObjectID with a Group Attribute. **/
    DOFREQUESTTYPE_BROADCAST    /**< Broadcast request; wildcard ObjectID (no Attributes). **/
} DOFRequestType;

/**
 *  The transport type associated with an address.
 */
typedef enum DOFAddressType_t
{
    DOFADDRESSTYPE_UNICAST,     /**< Transport Unicast. **/
    DOFADDRESSTYPE_MULTICAST,   /**< Transport Multicast. **/
    DOFADDRESSTYPE_BROADCAST    /**< Transport Broadcast. **/
} DOFAddressType;

/**
 * The direction of a connection.  Each connection may be either inbound or outbound.
 */
typedef enum DOFConnectionDirection_t
{
    DOFCONNECTIONDIRECTION_INBOUND,  /**< Indicates that a connection is inbound, where the node is acting as the server. **/
    DOFCONNECTIONDIRECTION_OUTBOUND  /**< Indicates that a connection is outbound, where the node is acting as the client. **/
} DOFConnectionDirection;

/**
 * The type of connection represented by a @ref DOFConnectionConfig.
 */
typedef enum DOFConnectionType_t
{
    DOFCONNECTIONTYPE_STREAM,               /**< Connection to a host via a lossless, ordered (guaranteed) session. Does not support transport-level multicast or broadcast. **/
    DOFCONNECTIONTYPE_DATAGRAM,             /**< Pseudo-connection to a host via a lossy, orderless (non-guaranteed) session (the peer host knows about and can track the session). Does not support transport-level multicast or broadcast. **/
    DOFCONNECTIONTYPE_DATAGRAM_STATELESS,   /**< Stateless pseudo-connection to a host without a session (the peer host does not know about a session). Supports transport-level multicast or broadcast. **/
    DOFCONNECTIONTYPE_HUB,                  /**< Pseudo-connection, similar to @ref DOFCONNECTIONTYPE_GROUP, but as the concentrator or hub for @ref DOFCONNECTIONTYPE_POINT connections.  Requires security.  The concentrator/hub combines @ref DOFCONNECTIONTYPE_POINT connections (similar to a @ref DOFCONNECTIONTYPE_GROUP) to reduce the overhead of keeping a large number of individual connections.  Broadcasting data to the group is supported, but may take a long time as separate packets are needed to send to each point connection. **/
    DOFCONNECTIONTYPE_POINT,                /**< Pseudo-connection, similar to @ref DOFCONNECTIONTYPE_GROUP, to a concentrator or hub (a @ref DOFCONNECTIONTYPE_HUB connection).  Requires security.  The concentrator/hub combines this connection with others, more like a @ref DOFCONNECTIONTYPE_GROUP to reduce the overhead of keeping a large number of individual connections. **/
    DOFCONNECTIONTYPE_GROUP,                /**< Pseudo-connection to a group of nodes via a lossy, orderless (non-guaranteed) session. Requires security and utilizes a form of transport-level multicast or broadcast. **/
    DOFCONNECTIONTYPE_GROUP_MEMBER_ONLY     /**< Pseudo-connection, similar to @ref DOFCONNECTIONTYPE_GROUP, but the node cannot manage the group. Without another member to act as the group manager, this node cannot join the group or remain a member of the group. **/
} DOFConnectionType;


/**
 * The type of credentials (password, secret, etc.).
 * @ingroup DOFCredentials
 */
typedef enum DOFCredentialsType_t
{
    DOFCREDENTIALSTYPE_NONE = 0,      /**< Reserved. Used to represent no credentials. **/
    DOFCREDENTIALSTYPE_KEY = 1,       /**< Credentials based on a 256-bit key. **/
    DOFCREDENTIALSTYPE_PASSWORD = 2   /**< Credentials based on a password. **/
} DOFCredentialsType;


/**
 * The type of server represented by a @ref DOFServerConfig.
 */
typedef enum DOFServerType_t
{
    DOFSERVERTYPE_STREAM,       /**< Server accepting connections from a host via a guaranteed, streaming protocol. **/
    DOFSERVERTYPE_DATAGRAM      /**< Server accepting pseudo-connections from a host via a non-guaranteed, datagram protocol. **/
} DOFServerType;

/**
 * The route state change type.
 */
typedef enum DOFRouteInfoState_t {
    DOFROUTEINFOSTATE_ADDED,        /**< A new route has been added. **/
    DOFROUTEINFOSTATE_REMOVED,      /**< An existing route has been removed. **/
    DOFROUTEINFOSTATE_CHANGED       /**< An existing route has changed. **/
} DOFRouteInfoState;

/**
 * Logging priority levels.
 */
typedef enum DOFLogLevel_t {
    DOFLOGLEVEL_ALL      = 0xFF, /**< Logging level indicating all priority levels. **/
    DOFLOGLEVEL_ALERT    = 0x80, /**< Logging priority level indicating action is required immediately. **/
    DOFLOGLEVEL_CRITICAL = 0x40, /**< Logging priority level indicating a critical error condition. **/
    DOFLOGLEVEL_ERROR    = 0x20, /**< Logging priority level indicating an error condition. **/
    DOFLOGLEVEL_WARNING  = 0x10, /**< Logging priority level indicating a warning condition. **/
    DOFLOGLEVEL_NOTICE   = 0x08, /**< Logging priority level indicating a normal, but significant condition. **/
    DOFLOGLEVEL_INFO     = 0x04, /**< Logging priority level indicating information only. **/
    DOFLOGLEVEL_DEBUG    = 0x02, /**< Logging priority level indicating debug information only. **/
    DOFLOGLEVEL_TRACE    = 0x01, /**< Logging priority level indicating detailed trace debug information. **/
    DOFLOGLEVEL_NONE     = 0x00  /**< Logging priority level indicating no logging. */
} DOFLogLevel;

/**
 * The DOFSecurityDesire enumeration types represent the security levels that can be imposed on connections (@ref DOFConnection) 
 * and servers (@ref DOFServer). For a connection and server to be compatible, they must have compatible security desires; 
 * otherwise, the connection fails. 
 * 
 * @see DOFConnectionConfigBuilder_SetSecurityDesire
 * @see DOFServerConfigBuilder_SetSecurityDesire
 * @see DOFConnectionState_GetSecurityDesire
 * @see DOFServerState_GetSecurityDesire
 */
typedef enum DOFSecurityDesire
{
    DOFSECURITYDESIRE_NOT_SECURE                = 0x00, /**< Indicates that the connection or server must not be secure. Connections and servers with this security desire are compatible only with others that have the NOT_SECURE or ANY security desire. **/
    DOFSECURITYDESIRE_SECURE_ANY                = 0x01, /**< Indicates that authentication is required, and encryption is allowed but not required. Connections and servers with this security desire are compatible with all others, except those that have the NOT_SECURE security desire. **/
    DOFSECURITYDESIRE_SECURE_AUTHENTICATE_ONLY  = 0x03, /**< Indicates that authentication is required, but encryption is not allowed. Connections and servers with this security desire are compatible only with others that have the SECURE_AUTHENTICATE_ONLY, SECURE_ANY, or ANY security desire. **/
    DOFSECURITYDESIRE_SECURE                    = 0x05, /**< Indicates that both authentication and encryption are required. Connections and servers with this security desire are compatible with others that have the SECURE, SECURE_ANY, or ANY security desire. **/
    DOFSECURITYDESIRE_ANY                       = 0x08  /**< Indicates that the connection or server accepts any of the other security levels. Connections and servers with this security desire are compatible with all others, as long as they are capable of authentication and encryption when required by the other component. **/
}DOFSecurityDesire;

/**
 * Block Cipher Algorithms.
 */
typedef enum DOFCipherAlgorithm
{
    DOFCIPHERALGORITHM_AES = 0x00 /**< Advanced Encryption Standard (AES), as described by FIPS PUB 197 (Rijndael). **/
} DOFCipherAlgorithm;

/**
 * Block Cipher Key Strengths.
 *
 * These constants are controlled by the OpenDOF Project. See the OpenDof Security Strength Registry http://opendof.org/support/registries/security-strength-registry/.
 */
typedef enum DOFCipherStrength
{
    DOFCIPHERSTRENGTH_256 = 1, /**< 256-bit security strength. **/
    DOFCIPHERSTRENGTH_192 = 2, /**< 192-bit security strength. **/
    DOFCIPHERSTRENGTH_128 = 3  /**< 128-bit security strength. **/
} DOFCipherStrength;

/* Exposed Structures */
typedef struct DOFSystemList_t *  DOFSystemList;          /**< DOF DOFSystem List. @ingroup DOFSystem @see DOFSystemList_Destroy **/
/**
 * DOF DOFSystem List structure.
 *
 * This is structure containing both a count and a list.  This is only used
 * when a pointer to this structure is returned.  The pointer must be freed via
 * @ref DOFSystemList_Destroy.
 *
 * @ingroup DOFSystem
 */
struct DOFSystemList_t
{
    uint32          systemCount;   /**< Count of systems in the list. **/
    DOFSystem       *systemArray;  /**< Array of @ref DOFSystemList_t::systemCount "systemCount" systems. **/
};

typedef struct DOFConnectionList_t *  DOFConnectionList;          /**< DOF DOFConnection List. @ingroup DOFConnection @see DOFConnectionList_Destroy **/
/**
 * DOF DOFConnection List structure.
 *
 * This is structure containing both a count and a list.  This is only used
 * when a pointer to this structure is returned.  The pointer must be freed via
 * @ref DOFConnectionList_Destroy.
 *
 * @ingroup DOFConnection
 */
struct DOFConnectionList_t
{
    uint32          connectionCount;   /**< Count of connections in the list. **/
    DOFConnection * connectionArray;   /**< Array of @ref DOFConnectionList_t::connectionCount "connectionCount" connections. **/
};

typedef struct DOFServerList_t *  DOFServerList;          /**< DOF DOFServer List. @ingroup DOFServer @see DOFServerList_Destroy **/
/**
 * DOF DOFServer List structure.
 *
 * This is structure containing both a count and a list.  This is only used
 * when a pointer to this structure is returned.  The pointer must be freed via
 * @ref DOFServerList_Destroy.
 *
 * @ingroup DOFServer
 */
struct DOFServerList_t
{
    uint32          serverCount;   /**< Count of servers in the list. **/
    DOFServer *     serverArray;   /**< Array of @ref DOFServerList_t::serverCount "serverCount" servers. **/
};

/**
 * DOF ObjectID List structure.
 *
 * This is structure containing both a count and a list.  This is only used
 * when a pointer to this structure is returned.  The pointer must be freed via
 * @ref DOFObjectIDList_Destroy.
 *
 * @ingroup DOFObjectID
 */
struct DOFObjectIDList_t
{
    uint32        count;   /**< Count of Object IDs in the list. **/
    DOFObjectID * list;    /**< Array of @ref DOFObjectIDList_t::count "count" Object IDs. **/
};

/**
 * DOF InterfaceID List structure.
 *
 * This is structure containing both a count and a list.  This is only used
 * when a pointer to this structure is returned.  The pointer must be freed via
 * @ref DOFInterfaceIDList_Destroy.
 *
 * @ingroup DOFInterfaceID
 */
struct DOFInterfaceIDList_t
{
    uint32           count;    /**< Count of Interface IDs in the list. **/
    DOFInterfaceID * list;     /**< Array of @ref DOFInterfaceIDList_t::count "count" Interface IDs. **/
};

/**
 * DOF Authenticators List structure.
 *
 * This is structure containing both a count and a list.  This is only used
 * when a pointer to this structure is returned.  The pointer must be freed via
 * @ref DOFAuthenticatorList_Destroy.
 *
 * @ingroup DOFAuthenticator
 */
struct DOFAuthenticatorList_t
{
    uint32              count;    /**< Count of Authenticators in the list. **/
    DOFAuthenticator*   list;     /**< Array of @ref DOFAuthenticatorList_t::count "count" Authenticator. **/
};

/**
 * DOF Domain List structure.
 *
 * This is structure containing both a count and a list.  This is only used
 * when a pointer to this structure is returned.  The pointer must be freed via
 * @ref DOFDomainList_Destroy.
 *
 * @ingroup DOFDomain
 */
struct DOFDomainList_t
{
    uint32      count;  /**< Count of Domains in the list. **/
    DOFDomain*   list;  /**< Array of @ref DOFDomainList_t::count "count" Domain. **/
};

/**
 * DOF Object ID Attribute Descriptor.
 *
 * This is the structure containing the attribute id and data.
 *
 * @ingroup DOFObjectIDAttribute
 */
typedef struct DOFObjectIDAttribute_t
{
    uint8                          id;         /**< Attribute Identifier.  Intentionally defined as @ref uint8 for size, but holds all valid values for @ref DOFObjectIDAttributeType.  **/
    DOFObjectIDAttributeDataSize   dataSize;   /**< Size of the attribute data. **/
    const uint8 *                  data;       /**< Attribute data. **/
} DOFObjectIDAttribute;


/* Callback Interfaces */

/**
    The DOF Interest Listener interface.  Defined as a pointer to the @ref DOFInterestListener_t structure.  Refer to the structure for more information.
**/
typedef struct DOFInterestListener_t * DOFInterestListener;

/**
 * The DOFInterestListener interface enables you to create functionality for monitoring all interest requests. 
 * It is intended only for monitoring:
 * 
 * - To react by providing a binding in response to a @ref DOFINTERESTLEVEL_ACTIVATE interest, 
 *   use @ref DOFSystemActivateInterestListener instead.
 * - To react by connecting in response to a @ref DOFINTERESTLEVEL_CONNECT interest, use 
 *   @ref DOFConnectionStreamRequestListener instead.
 *
 * @ifacestruct{DOFInterestListener}
 * 
 * @threadsafety
 *
 * @see DOF_AddInterestListener
 * @see DOF_RemoveInterestListener
 */
typedef struct DOFInterestListener_t
{

    /**
     * The listener interface function structure, containing a list of
     * function pointers that implement the interface.  It has the same role as
     * a vtable in C++.  This must not be NULL.
     */
    const struct DOFInterestListenerFns_t
    {

        /**
         * Called when the DOFInterestListener is removed from the DOF. Use this function to free resources 
         * associated with the listener. The
         * @ref DOFInterestListener_t::DOFInterestListenerFns_t::InterestRemoved "InterestRemoved" callback will not be 
         * called after the listener is removed, so it may be necessary to clean up resources 
         * associated with 
         * @ref DOFInterestListener_t::DOFInterestListenerFns_t::InterestAdded "InterestAdded" calls for which 
         * no InterestRemoved call was received when the listener was removed.
         *         
         *  This callback may be @c NULL to ignore notification of removal.
         *
         *  @param self The DOFInterestListener object.  This will never be @c NULL.
         *  @param dof The DOF where this listener was added. This will never be @c NULL.
         *  @param except An exception that caused the listener to be removed prematurely. This will be @c NULL 
         *                if the application removed the listener normally.
         */
        void (*Removed) ( DOFInterestListener self, DOF dof, DOFException except );

        /**
         * Called when the node receives any level of interest for a binding. Program this function to monitor interest requests. 
         * The @ref DOFRequest is passed in this function for informational purposes, but this function should not be used to 
         * respond to the request.
         *
         *  This callback may be @c NULL to ignore notifications about interest being added.
         *
         *  @param self The listener object. This will never be @c NULL.
         *  @param dof The DOF where this listener was added. This will never be @c NULL.
         *  @param request An object that contains information about the request. This will never be @c NULL.
         *  @param objectID The DOFObjectID the requestor has expressed interest in. This will never be @c NULL. 
         *                  This may be @ref DOFOBJECTID_BROADCAST, indicating that the interest is for the specified 
         *                  interface on all objects. It will not be DOFOBJECTID_BROADCAST if the 
         *                  @a interfaceID is @ref DOFINTERFACEID_WILDCARD.
         *  @param interfaceID The DOFInterfaceID the requestor has expressed interest in. This will never be @c NULL. 
         *                     This may be DOFINTERFACEID_WILDCARD, indicating that the interest is for all interfaces 
         *                     on the specified object. It will not be DOFINTERFACEID_WILDCARD if the 
         *                     @a objectID is DOFOBJECTID_BROADCAST.
         *  @param level The level of interest being added. This will never be @c NULL.
         */
        void (*InterestAdded)( DOFInterestListener self, DOF dof, DOFRequest request, DOFObjectID objectID, DOFInterfaceID interfaceID, DOFInterestLevel level );

        /**
         * Called when an interest request is canceled or times out. When this callback is called, it does not always 
         * mean that no interest is requested for the given object and interface, because more than one interest request 
         * may have been received for the same binding. If needed, use @ref DOFSystem_IsInterestRequested to check if interest 
         * is still requested.
         *
         * @note This callback is primarily for monitoring; no action needs to be taken, and the @a request should not be 
         * completed in this callback.
         *
         * This callback may be @c NULL to ignore notification of interest removal.
         *
         *  @param self The listener object. This will never be @c NULL.
         *  @param dof The DOF where this listener was added. This will never be @c NULL.
         *  @param request The interest request that was canceled. This will never be @c NULL.
         *  @param objectID The DOFObjectID associated with the original interest request. This may be @ref DOFOBJECTID_BROADCAST. 
         *                  This will never be @c NULL.
         *  @param interfaceID The DOFInterfaceID associated with the original interest request. This may be 
         *                     @ref DOFINTERFACEID_WILDCARD. This will never be @c NULL.
         *  @param level The level of interest that was canceled. This will never be @c NULL.
         */
        void (*InterestRemoved)( DOFInterestListener self, DOF dof, DOFRequest request, DOFObjectID objectID, DOFInterfaceID interfaceID, DOFInterestLevel level );

    } * fns;

    /* Implementations may extend this structure. */

} DOFInterestListener_t;

/**
    The DOFSystem Activate Interest Listener interface.  Defined as a pointer to the @ref DOFSystemActivateInterestListener_t structure.  Refer to the structure for more information.
**/
typedef struct DOFSystemActivateInterestListener_t * DOFSystemActivateInterestListener;

/**
 * The DOFSystemActivateInterestListener interface enables you to create functionality in providers that begins providing 
 * a binding in response to a @ref DOFINTERESTLEVEL_ACTIVATE or higher level of interest. @ref DOFINTERESTLEVEL_WATCH interest
 * does not trigger the callbacks in this interface. 
 * 
 * - To instead monitor all levels of interest, use @ref DOFInterestListener instead.
 * - To react by connecting in response to a @ref DOFINTERESTLEVEL_CONNECT interest, use 
 *   @ref DOFConnectionStreamRequestListener instead. 
 *
 * @ifacestruct{DOFSystemActivateInterestListener}
 * 
 * @threadsafety
 *
 * @see DOFSystem_AddActivateInterestListener
 * @see DOFSystem_RemoveActivateInterestListener
 */
typedef struct DOFSystemActivateInterestListener_t
{

    /**
     * The listener interface function structure, containing a list of
     * function pointers that implement the interface.  It has the same role as
     * a vtable in C++.  This must not be NULL.
     */
    const struct DOFSystemActivateInterestListenerFns_t
    {
        /**
         *  Callback used to let the listener free resources associated with the listener.
         *
         *  This callback is to notify the listener that it is safe to remove any data allocated for
         *  this listener.  After this callback is called, the listener will no longer be called.
         *
         *  Because the listener is updated as the state of the dof changes,
         *  interest added by
         *  @ref DOFSystemActivateInterestListener_t::DOFSystemActivateInterestListenerFns_t::Activate "Activate"
         *  will @b not be removed by
         *  @ref DOFSystemActivateInterestListener_t::DOFSystemActivateInterestListenerFns_t::CancelActivate "CancelActivate"
         *  after this (Removed) is called.  It may therefore be necessary to cleanup data "orphaned" by
         *  the removal of the listener (since the InterestRemoved call will not be called again for this listener).
         *
         *  This callback may be NULL to ignore notification of removal.
         *
         *  @param self The listener object.  This will never be NULL.
         *  @param system The DOFSystem. This will never be NULL.
         *  @param except The exception causing the listener to be removed prematurely. This will be NULL if the listener is removed normally by the application.
         */
        void (*Removed)( DOFSystemActivateInterestListener self, DOFSystem system, DOFException except );

        /**
         *  Callback for a Provider when Activate-level interest is added (requested) for an Object and Interface.
         *
         *  This callback is called whenever an Activate-level interest request is received for the given Object ID and
         *  Interface ID. This call is responsible for starting the provider for the binding(s), if applicable.
         *
         *  If the @a interfaceID is the wildcard IID, then the request is to provide ALL applicable interfaces on the specified
         *  Object.
         *
         * @note The @a request is for information only and must not be completed via this callback.
         *
         *  This callback may be NULL to ignore notification of interest addition.
         *
         *  @param self The listener object.  This will never be NULL.
         *  @param system The DOFSystem. This will never be NULL.
         *  @param request A representation of the requestor.  This will never be NULL.
         *  @param objectID The Object ID for which the interest is being requested.  This will never be NULL.
         *  @param interfaceID The Interface for which interest is being requested (or the Wildcard IID).  This will match the original ID from the request.  This will never be NULL.
         */
        void (*Activate)( DOFSystemActivateInterestListener self, DOFSystem system, DOFRequest request, DOFObjectID objectID, DOFInterfaceID interfaceID );

        /**
         *  Callback for a Provider when Activate-level interest is removed for an Object and Interface.
         *
         *  This callback is called whenever an Activate-level interest request for the given Object ID and Interface ID is
         *  cancelled or times out.  This does not always mean the given Object and Interface no longer needs to be active; use
         *  @ref DOFSystem_IsInterestRequested to check if it is still needed.  This call is responsible for removing the
         *  binding(s) added by the previous call to @ref Activate.
         *
         *  If the @a interfaceID is the wildcard IID, then the application can
         *  check whether any specific Interface IDs are still active for the Object via
         *  @ref DOFSystem_IsInterestRequested.
         *
         * @note The @a request is for information only and must not be completed via this callback.
         *
         *  This callback may be NULL to ignore notification of interest removal.
         *
         *  @param self The listener object.  This will never be NULL.
         *  @param system The DOFSystem. This will never be NULL.
         *  @param request A representation of the requestor.  This will never be NULL.
         *  @param objectID The Object ID for which the interest is being requested.  This will never be NULL.
         *  @param interfaceID The Interface that is being removed from interest (or the Wildcard IID).  This will match the original ID from the request.  This will never be NULL.
         */
        void (*CancelActivate)( DOFSystemActivateInterestListener self, DOFSystem system, DOFRequest request, DOFObjectID objectID, DOFInterfaceID interfaceID );
    } * fns;

    /* Implementations may extend this structure. */

} DOFSystemActivateInterestListener_t;

/**
    The Query Callback interface.  Defined as a pointer to the @ref DOFSystemQueryCallback_t structure.  Refer to the structure for more information.
**/
typedef struct DOFSystemQueryCallback_t * DOFSystemQueryCallback;

/**
 * The callback interface structure used to obtain changes in the interest set
 * for a query.
 *
 * @ifacestruct{DOFSystemQueryCallback}
 * 
 * @threadsafety
 *
 * @see DOFSystem_BeginQuery
 */
typedef struct DOFSystemQueryCallback_t
{

    /**
     * The callback interface function structure, containing a list of
     * function pointers that implement the interface.  It has the same role as
     * a vtable in C++.  This must not be NULL.
     */
    const struct DOFSystemQueryCallbackFns_t
    {

        /**
         * Callback for a Requestor when the @ref DOFOperation, representing the
         * Query, has timed out or is cancelled.
         *
         *  Because the callbacks are notified as the state of the system changes,
         *  providers added by
         *  @ref DOFSystemQueryCallback_t::DOFSystemQueryCallbackFns_t::InterfaceAdded "InterfaceAdded"
         *  will @b not be removed by
         *  @ref DOFSystemQueryCallback_t::DOFSystemQueryCallbackFns_t::InterfaceRemoved "InterfaceRemoved", or
         *  @ref DOFSystemQueryCallback_t::DOFSystemQueryCallbackFns_t::ProviderRemoved "ProviderRemoved"
         *  after this (Complete) is called.  It may therefore be necessary to cleanup data "orphaned" by
         *  the completion of the operation (since the callbacks will not be called again for this operation).
         *
         *  This callback may be NULL to ignore completion of the query.
         *
         * @param self The callback object.  This will never be NULL.
         * @param operation The Query Operation that is complete.  This will never be NULL.
         * @param except An exception causing the completion of the Operation.
         *               If NULL, no exception occurred (the operation completed
         *               normally).
         */
        void (* Complete) ( DOFSystemQueryCallback self, DOFOperation operation, DOFException except );

        /**
         * Callback for Requestor when a Provider's Interface matching the query for
         * the @ref DOFOperation is discovered within the system.
         *
         *  This callback may be NULL to ignore notification of interface addition.
         *
         * @param self The callback object.  This will never be NULL.
         * @param operation The Query Operation.  This will never be NULL.
         * @param objectID The Object ID for which the Interface is being added.  This will never be NULL.
         * @param interfaceID The Interface ID being added on the Object.  This will never be NULL.
         */
        void (* InterfaceAdded) ( DOFSystemQueryCallback self,
                                  DOFOperation operation,
                                  DOFObjectID objectID,
                                  DOFInterfaceID interfaceID );

        /**
         * Callback for Requestor when a Provider's Interface matching the query for
         * the @ref DOFOperation is removed from the system.
         *
         *  This callback may be NULL to ignore notification of interface removal.
         *
         * @param self The callback object.  This will never be NULL.
         * @param operation The Query Operation.  This will never be NULL.
         * @param objectID The Object ID for which the Interface is being removed.  This will never be NULL.
         * @param interfaceID The Interface ID being removed on the Object.  This will never be NULL.
         */
        void (* InterfaceRemoved) ( DOFSystemQueryCallback self,
                                    DOFOperation operation, DOFObjectID objectID,
                                    DOFInterfaceID interfaceID );

        /**
         * Callback for Requestor when a Provider matching the query for the @ref
         * DOFOperation no longer provides any matching Interfaces.  This can be used
         * to identify when a @ref DOFObject can safely be cleaned up and destroyed.
         *
         *  This callback may be NULL to ignore notification of object removal.
         *
         * @param self The callback object.  This will never be NULL.
         * @param operation The Query Operation.  This will never be NULL.
         * @param objectID The Object ID being removed from the system.  This will never be NULL.
         */
        void (* ProviderRemoved) ( DOFSystemQueryCallback self,
                                   DOFOperation operation, DOFObjectID objectID );

    } * fns;

    /* Implementations may extend this structure. */

} DOFSystemQueryCallback_t;


/**
    The Requestor Session Callback interface.  Defined as a pointer to the @ref DOFObjectSessionCallback_t structure.  Refer to the structure for more information.
**/
typedef struct DOFObjectSessionCallback_t * DOFObjectSessionCallback;

/**
 * The callback interface structure used to be notified of session requests.
 *
 * @ifacestruct{DOFObjectSessionCallback}
 * 
 * @threadsafety
 *
 * @see DOFObject_BeginSession
 */
typedef struct DOFObjectSessionCallback_t
{

    /**
     * The callback interface function structure, containing a list of
     * function pointers that implement the interface.  It has the same role as
     * a vtable in C++.  This must not be NULL.
     */
    const struct DOFObjectSessionCallbackFns_t
    {

        /**
         * Callback for a Requestor when the @ref DOFOperation, representing the
         * Session, has timed out or is cancelled.
         *
         * The session object (returned by
         * @ref DOFObjectSessionCallback_t::DOFObjectSessionCallbackFns_t::SessionOpen "SessionOpen")
         * no longer can be used to source requests
         *
         * This callback may be NULL to ignore completion of the session.
         *
         * @param self The callback object.  This will never be NULL.
         * @param operation The Session Operation that is complete.  This will never be NULL.
         * @param except An exception causing the completion of the Operation.
         *               If NULL, no exception occurred (the operation completed
         *               normally).
         */
        void (* Complete) ( DOFObjectSessionCallback self, DOFOperation operation, DOFException except );

        /**
         * Callback for a Requestor when a requested session request is ready.
         *
         * When successful, the @a session parameter is the new @ref DOFObject
         * for the session.  This is only maintained for the duration of the
         * callback itself, so if needed after the callback is finished, it
         * must be copied (via @ref DOFObject_Copy) and later freed by the
         * application (via @ref DOFObject_Destroy).
         *
         * This callback may be NULL to ignore notification of the available session object.
         *
         * @param self The callback object.  This will never be NULL.
         * @param operation The Session Operation.  This will never be NULL.
         * @param providerInfo The @ref DOFProviderInfo associated with the @a session or @a exception.  This will never be NULL.
         * @param session The result of the Session request.  If @a exception is non-NULL, this must be ignored and may be NULL.  Otherwise, it will be a valid DOFObject that may be used by the application within the context of this callback.  If the application needs to keep this DOFObject beyond the scope of this callback, it must be copied via @ref DOFObject_Copy and therefore freed when no longer needed.
         * @param except An exception, if any.  If not NULL, this indicates the error that occurred.
         */
        void (* SessionOpen) ( DOFObjectSessionCallback self, DOFOperation operation, DOFProviderInfo providerInfo, DOFObject session, DOFException except );

    } * fns;

    /* Implementations may extend this structure. */

} DOFObjectSessionCallback_t;


/**
    The Provider Callback interface.  Defined as a pointer to the @ref DOFObjectProvider_t structure.  Refer to the structure for more information.
**/
typedef struct DOFObjectProvider_t * DOFObjectProvider;

/**
 * The callback interface structure used to Provide information to a Requestor.
 *
 * @ifacestruct{DOFObjectProvider}
 * 
 * @threadsafety
 *
 * @see DOFObject_BeginProvide
 */
typedef struct DOFObjectProvider_t
{

    /**
     * The callback interface function structure, containing a list of
     * function pointers that implement the interface.  It has the same role as
     * a vtable in C++.  This must not be NULL.
     */
    const struct DOFObjectProviderFns_t
    {

        /**
         * Callback for a Provider when the @ref DOFOperation, representing the act of
         * Providing, has timed out or is cancelled.
         *
         *  Because the callbacks are notified as requests are made and completed,
         *  subscriptions, registrations, and sessions created by
         *  @ref DOFObjectProvider_t::DOFObjectProviderFns_t::Subscribe "Subscribe",
         *  @ref DOFObjectProvider_t::DOFObjectProviderFns_t::Register "Register", and
         *  @ref DOFObjectProvider_t::DOFObjectProviderFns_t::Session "Session"
         *  will @b not be removed by
         *  @ref DOFObjectProvider_t::DOFObjectProviderFns_t::SubscribeComplete "SubscribeComplete",
         *  @ref DOFObjectProvider_t::DOFObjectProviderFns_t::RegisterComplete "RegisterComplete", and
         *  @ref DOFObjectProvider_t::DOFObjectProviderFns_t::SessionComplete "SessionComplete", respectively,
         *  after this (Complete) is called.  It may therefore be necessary to cleanup data "orphaned" by
         *  the completion of the provider (since the callbacks will not be called again for this operation).
         *
         * This callback may be NULL to ignore the completion of the Provide.
         *
         * @param self The callback object.  This will never be NULL.
         * @param operation The Provide Operation that is complete.  This will never be NULL.
         * @param except An exception causing the completion of the Operation.
         *               If NULL, no exception occurred (the operation completed
         *               normally).
         */
        void (* Complete) ( DOFObjectProvider self, DOFOperation operation, DOFException except );

        /**
         * Callback for a Provider when a @ref DOFInterfaceMethod is to be invoked.
         *
         * This callback may be NULL if the application does not implement the capability to invoke a Method
         * (if the interface supports methods, any attempt to invoke will cause a generic exception response).
         *
         * @param self    The callback object.  This will never be NULL.
         * @param operation  The Provide Operation.  This will never be NULL.
         * @param method  The Method being Invoked.  This will never be NULL.
         * @param parameterCount The number of parameters in the @a value array.
         * @param parameters  An array of @ref DOFValue elements, representing the input parameters to the Method.  This will never be NULL if @a parameterCount is not 0.
         * @param request A representation of the requestor.  This will never be NULL.
         *
         * @note The @a request must be completed by either returning the output parameters of the Method via @ref DOFRequestInvoke_Return, or returning an exception via @ref DOFRequest_Throw.  All parameters to this callback remain valid and may be referenced by the application until either the end of the callback or the request is completed as described, whichever is LATER.  It is the responsibility of the application to ensure the request is eventually completed.
         */
        void (* Invoke) ( DOFObjectProvider self, DOFOperation operation, DOFRequest request, DOFInterfaceMethod method, uint16 parameterCount, const DOFValue parameters[] );

        /**
         * Callback for a Provider when a @ref DOFInterfaceProperty is to be read.
         *
         * This callback may be NULL if the application does not implement the capability to get a Property
         * (if the interface supports readable properties, any attempt to get them will cause a generic exception response).
         *
         * @param self    The callback object.  This will never be NULL.
         * @param operation The Provide Operation.  This will never be NULL.
         * @param property  The Property being queried.  This will never be NULL.
         * @param request A representation of the requestor.  This will never be NULL.
         *
         * @note The @a request must be completed by either returning the current value of the Property via @ref DOFRequestGet_Return, or returning an exception via @ref DOFRequest_Throw.  All parameters to this callback remain valid and may be referenced by the application until either the end of the callback or the request is completed as described, whichever is LATER.  It is the responsibility of the application to ensure the request is eventually completed.
         */
        void (* Get) ( DOFObjectProvider self, DOFOperation operation, DOFRequest request, DOFInterfaceProperty property );

        /**
         * Callback for a Provider when a @ref DOFInterfaceProperty is to be set.
         *
         * This callback may be NULL if the application does not implement the capability to set a Property
         * (if the interface supports writable properties, any attempt to set them will cause a generic exception response).
         *
         * @param self    The callback object.  This will never be NULL.
         * @param operation The Provide Operation.  This will never be NULL.
         * @param property The Property being Set.  This will never be NULL.
         * @param value   The new Value for the Property.  This will never be NULL.
         * @param request A representation of the requestor.  This will never be NULL.
         *
         * @note The @a request must be completed by either indicating successful completion via @ref DOFRequestSet_Return, or returning an exception via @ref DOFRequest_Throw.  All parameters to this callback remain valid and may be referenced by the application until either the end of the callback or the request is completed as described, whichever is LATER.  It is the responsibility of the application to ensure the request is eventually completed.
         * @note Successful completion of the Set request will cause the library to assume the property changed, and will automatically notify any subscribers.  There is no need for the application to call @ref DOFObject_Changed as a result of the Set request (doing so may cause excess subscription notifications to be generated).
         */
        void (* Set) ( DOFObjectProvider self, DOFOperation operation, DOFRequest request, DOFInterfaceProperty property,
                       DOFValue value );

        /**
         * Callback for a Provider when a @ref DOFInterfaceProperty is to be subscribed.
         *
         * This callback must decide whether or not to accept the new subscription.
         * The Provider may choose to not allow the subscription by throwing an exception on the request (see @ref DOFRequest_Throw).
         * The @a request will remain valid until the SubscribeComplete callback is called.  If the application chooses not to supply a SubscribeComplete callback,
         * then the subscription request @b must be accepted or rejected synchronously inside this callback.
         * If the application initially accepts the subscription, the application can terminate it (as long as the @a request remains valid) by calling @ref DOFRequest_Throw.
         *
         * This callback may be NULL if the application does not monitor nor control subscriptions to any Property
         * (subscriptions to any readable properties will be automatically accepted).
         *
         * @param self      The callback object.  This will never be NULL.
         * @param operation The Provide Operation.  This will never be NULL.
         * @param property The Property being Subscribed.  This will never be NULL.
         * @param minPeriod The minimum period between updates.
         * @param request   A representation of the requestor.  This will never be NULL.
         *
         * @note The @a request must be accepted by calling @ref DOFRequestSubscribe_Return, or rejected via @ref DOFRequest_Throw.  All parameters to this callback remain valid and may be referenced by the application until either the end of the callback or the SubscribeComplete callback is called, whichever is LATER.
         */
        void (* Subscribe) ( DOFObjectProvider self, DOFOperation operation, DOFRequest request, DOFInterfaceProperty property, uint32 minPeriod);

        /**
         * Callback for a Provider when a subscription is removed for a @ref DOFInterfaceProperty.
         * This callback informs the application that the subscription is complete and the @a request can no longer be used.
         *
         * This callback may be NULL if the application does not care to receive notification when a Subscribe completes.
         *
         * @param self      The callback object.  This will never be NULL.
         * @param operation The Provide Operation.  This will never be NULL.
         * @param property  The Property no longer Subscribed.  This will never be NULL.
         * @param request   A representation of the requestor.  This will never be NULL.
         */
        void (* SubscribeComplete)( DOFObjectProvider self, DOFOperation operation, DOFRequest request, DOFInterfaceProperty property );

        /**
         * Callback for a Provider when a @ref DOFInterfaceEvent handler is registered.
         *
         * This callback must decide whether or not to accept the new registration.
         * The Provider may choose to not allow the registration by throwing an exception on the request (see @ref DOFRequest_Throw).
         * The @a request will remain valid until the RegisterComplete callback is called.  If the application chooses not to supply a RegisterComplete callback,
         * then the registration request @b must be accepted or rejected synchronously inside this callback.
         * If the application initially accepts the registration, the application can terminate it (as long as the @a request remains valid) by calling @ref DOFRequest_Throw.
         *
         * This callback may be NULL if the application does not monitor nor control registrations to any Event
         * (registrations to any event will be automatically accepted).
         *
         * @param self      The callback object.  This will never be NULL.
         * @param operation The Provide Operation.  This will never be NULL.
         * @param evt       The Event being Registered.  This will never be NULL.
         * @param request   A representation of the requestor.  This will never be NULL.
         *
         * @note The @a request must be accepted by calling @ref DOFRequestRegister_Return, or rejected via @ref DOFRequest_Throw.  All parameters to this callback remain valid and may be referenced by the application until either the end of the callback or the RegisterComplete callback is called, whichever is LATER.
         */
        void (* Register)( DOFObjectProvider self, DOFOperation operation, DOFRequest request, DOFInterfaceEvent evt );

        /**
         * Callback for a Provider when an event handler is removed for a @ref DOFInterfaceEvent.
         * This callback informs the application that the registration is complete and the @a request can no longer be used.
         *
         * This callback may be NULL if the application does not care to receive notification when a Register completes.
         *
         * @param self      The callback object.  This will never be NULL.
         * @param operation The Provide Operation.  This will never be NULL.
         * @param evt       The Event being Registered.  This will never be NULL.
         * @param request   A representation of the requestor.  This will never be NULL.
         */
        void (* RegisterComplete)( DOFObjectProvider self, DOFOperation operation, DOFRequest request, DOFInterfaceEvent evt );


        /**
         * Callback for a Provider when a session is requested.
         *
         * This callback must decide whether or not to accept the new session.
         * The Provider may choose to not allow the session by throwing an exception on the request (see @ref DOFRequest_Throw).
         * The @a request will remain valid until the SessionComplete callback is called.  If the application chooses not to supply a SessionComplete callback,
         * then the session request @b must be accepted or rejected synchronously inside this callback.
         * If the application initially accepts the session, the application can terminate it (as long as the @a request remains valid) by calling @ref DOFRequest_Throw.
         *
         * Typically, an application creates a new Session @ref DOFObject (based on the given @a sessionID) and starts one or more Provide Operations on it.
         *
         * This callback may be NULL if the application does not implement the capability to open a Session
         * (any attempt to open a session will cause a generic exception response).
         *
         * @param self        The callback object.  This will never be NULL.
         * @param operation   The Provide Operation.  This will never be NULL.
         * @param object      The provider Object on which a Session is requested.  This will never be NULL.
         * @param interfaceID The Interface ID of the specific provider on which a session is being requested.  This will never be NULL.
         * @param sessionID   The Session ID (a @ref DOFObjectID).  This uniquely identifies the session, and includes a Session Attribute chosen by the requestor.  A DOFObject can be created directly from this ID to provide interfaces on the session.  This will never be NULL.
         * @param sessionType The requested Session type (a @ref DOFInterfaceID), defining the purpose and contract for the session.  This interface is typically provided on the session (possibly in addition to others), but no specific behavior is required.  This will never be NULL.
         * @param request     A representation of the requestor.  This will never be NULL.
         *
         * @note The @a request must be accepted by calling @ref DOFRequestSession_Return, or rejected via @ref DOFRequest_Throw.  All parameters to this callback remain valid and may be referenced by the application until either the end of the callback or the SessionComplete callback is called, whichever is LATER.
         */
        void (* Session) ( DOFObjectProvider self, DOFOperation operation, DOFRequest request,
                           DOFObject object, DOFInterfaceID interfaceID,
                           DOFObjectID sessionID, DOFInterfaceID sessionType );

        /**
         * Callback for a Provider when a session request is completed and the session should be terminated.
         *
         * This callback informs the application that the session is complete and the @a request can no longer be used.
         * Typically, an application terminates a session by cleaning up the
         * @ref DOFObject created for the Session and any Provide Operations
         * started on it.
         *
         * This callback may be NULL if the application does not care to receive notification when a Session completes.
         *
         * @param self        The callback object.  This will never be NULL.
         * @param operation   The Provide Operation.  This will never be NULL.
         * @param object      The provider Object on which a Session is requested.  This will never be NULL.
         * @param interfaceID The Interface ID for which a session is requested on the provider Object.  This will never be NULL.
         * @param sessionID   The Session ID (a @ref DOFObjectID).  This will never be NULL.
         * @param sessionType The requested Session type (a @ref DOFInterfaceID), defining the purpose and contract for the session.  This will never be NULL.
         * @param request     A representation of the requestor.  This will never be NULL.
         *
         * @note The @a request is for information only and must not be completed via this callback.
         */
        void (* SessionComplete) ( DOFObjectProvider self, DOFOperation operation, DOFRequest request,
                                   DOFObject object, DOFInterfaceID interfaceID,
                                   DOFObjectID sessionID, DOFInterfaceID sessionType );

    } * fns;

    /* Implementations may extend this structure. */

} DOFObjectProvider_t;

/**
    The Requestor Callback interface.  Defined as a pointer to the @ref DOFObjectGetCallback_t structure.  Refer to the structure for more information.
**/
typedef struct DOFObjectGetCallback_t * DOFObjectGetCallback;

/**
 * The callback interface structure used to receive results from a Provider for Get
 * Operations.
 *
 * @ifacestruct{DOFObjectGetCallback}
 * 
 * @threadsafety
 *
 * @see DOFObject_BeginGet
 */
typedef struct DOFObjectGetCallback_t
{

    /**
     * The callback interface function structure, containing a list of
     * function pointers that implement the interface.  It has the same role as
     * a vtable in C++.  This must not be NULL.
     */
    const struct DOFObjectGetCallbackFns_t
    {

        /**
         * Callback for a Requestor for the completion of an asynchronous operation.
         *
         * This callback may be NULL to ignore notification of completion.
         *
         * @param self The callback object.  This will never be NULL.
         * @param operation The original request operation.  This will never be NULL.
         * @param except An exception causing the completion of the Operation.
         *               If NULL, no exception occurred (the operation completed
         *               normally). A DOFEXCEPTION_TIMEOUT will occur
         *               only if no results were received.
         */
        void (* Complete) ( DOFObjectGetCallback self, DOFOperation operation, DOFException except );

        /**
         * Callback for a Requestor for the results of an asynchronous operation.
         *
         * This callback may be NULL to ignore notification of results.
         *
         * @param self The callback object.  This will never be NULL.
         * @param operation The original request operation.  This will never be NULL.
         * @param providerInfo The @ref DOFProviderInfo associated with the
         *                 @a value or @a except.  This will never be NULL.
         * @param value  A @ref DOFValue result of the Get Operation.  Must be
         *               ignored if @a except is non-NULL.  This may be NULL if
         *               the returned value is "NULL" (from a nullable type).
         * @param except An exception received from a Provider.  If NULL, no
         *               exception occurred (the Get was successful) and
         *               @a value is valid.
         */
        void (* GetResult) ( DOFObjectGetCallback self, DOFOperation operation, DOFProviderInfo providerInfo, DOFValue value, DOFException except );

    } * fns;

    /* Implementations may extend this structure. */

} DOFObjectGetCallback_t;


/**
    The Requestor Callback interface.  Defined as a pointer to the @ref DOFObjectSetCallback_t structure.  Refer to the structure for more information.
**/
typedef struct DOFObjectSetCallback_t * DOFObjectSetCallback;

/**
 * The callback interface structure used to receive results from a Provider for Set
 * Operations.
 *
 * @ifacestruct{DOFObjectSetCallback}
 * 
 * @threadsafety
 *
 * @see DOFObject_BeginSet
 */
typedef struct DOFObjectSetCallback_t
{

    /**
     * The callback interface function structure, containing a list of
     * function pointers that implement the interface.  It has the same role as
     * a vtable in C++.  This must not be NULL.
     */
    const struct DOFObjectSetCallbackFns_t
    {

        /**
         * Callback for a Requestor for the completion of an asynchronous operation.
         *
         * This callback may be NULL to ignore notification of completion.
         *
         * @param self The callback object.  This will never be NULL.
         * @param operation The original request operation.  This will never be NULL.
         * @param except An exception causing the completion of the Operation.
         *               If NULL, no exception occurred (the operation completed
         *               normally). A DOFEXCEPTION_TIMEOUT will occur
         *               only if no results were received.
         */
        void (* Complete) ( DOFObjectSetCallback self, DOFOperation operation, DOFException except );

        /**
         * Callback for a Requestor for the results of an asynchronous operation.
         *
         * This callback may be NULL to ignore notification of results.
         *
         * @param self The callback object.  This will never be NULL.
         * @param operation The original request operation.  This will never be NULL.
         * @param providerInfo The @ref DOFProviderInfo associated with the
         *                 response or @a except.  This will never be NULL.
         * @param except An exception received from a Provider.  If NULL, no
         *               exception occurred (the Set was successful).
         */
        void (* SetResult) ( DOFObjectSetCallback self, DOFOperation operation, DOFProviderInfo providerInfo, DOFException except );

    } * fns;

    /* Implementations may extend this structure. */

} DOFObjectSetCallback_t;


/**
    The Requestor Callback interface.  Defined as a pointer to the @ref DOFObjectInvokeCallback_t structure.  Refer to the structure for more information.
**/
typedef struct DOFObjectInvokeCallback_t * DOFObjectInvokeCallback;

/**
 * The callback interface structure used to receive results from a Provider for Invoke
 * Operations.
 *
 * @ifacestruct{DOFObjectInvokeCallback}
 * 
 * @threadsafety
 *
 * @see DOFObject_BeginInvoke
 */
typedef struct DOFObjectInvokeCallback_t
{

    /**
     * The callback interface function structure, containing a list of
     * function pointers that implement the interface.  It has the same role as
     * a vtable in C++.  This must not be NULL.
     */
    const struct DOFObjectInvokeCallbackFns_t
    {

        /**
         * Callback for a Requestor for the completion of an asynchronous operation.
         *
         * This callback may be NULL to ignore notification of completion.
         *
         * @param self The callback object.  This will never be NULL.
         * @param operation The original request operation.  This will never be NULL.
         * @param except An exception causing the completion of the Operation.
         *               If NULL, no exception occurred (the operation completed
         *               normally). A DOFEXCEPTION_TIMEOUT will occur
         *               only if no results were received.
         */
        void (* Complete) ( DOFObjectInvokeCallback self, DOFOperation operation, DOFException except );

        /**
         * Callback for a Requestor for the results of an asynchronous operation.
         *
         * This callback may be NULL to ignore notification of results.
         *
         * @param self The callback object.  This will never be NULL.
         * @param operation The original request operation.  This will never be NULL.
         * @param providerInfo The @ref DOFProviderInfo associated with the
         *                 @a result or @a except.  This will never be NULL.
         * @param resultCount The number of values in the @a result array.
         * @param result An array of @ref DOFValue elements, representing the output parameters of the Method used in the Invoke Operation.
         *               Must be ignored if @a except is non-NULL.  This will never be NULL if @a resultCount is not 0.  This may contain NULL values for parameters that are "NULL" (from nullable types).
         * @param except An exception received from a Provider.  If NULL, no
         *               exception occurred (the Invoke was successful) and
         *               @a result is valid.
         */
        void (* InvokeResult) ( DOFObjectInvokeCallback self, DOFOperation operation, DOFProviderInfo providerInfo, uint16 resultCount, const DOFValue result[], DOFException except );

    } * fns;

    /* Implementations may extend this structure. */

} DOFObjectInvokeCallback_t;


/**
    The Requestor Callback interface.  Defined as a pointer to the @ref DOFObjectSubscribeCallback_t structure.  Refer to the structure for more information.
**/
typedef struct DOFObjectSubscribeCallback_t * DOFObjectSubscribeCallback;

/**
 * The callback interface structure used to receive results from a Provider for Subscribe
 * Operations.
 *
 * @ifacestruct{DOFObjectSubscribeCallback}
 * 
 * @threadsafety
 *
 * @see DOFObject_BeginSubscribe
 */
typedef struct DOFObjectSubscribeCallback_t
{

    /**
     * The callback interface function structure, containing a list of
     * function pointers that implement the interface.  It has the same role as
     * a vtable in C++.  This must not be NULL.
     */
    const struct DOFObjectSubscribeCallbackFns_t
    {

        /**
         * Callback for a Requestor for the completion of an asynchronous operation.
         *
         * This callback may be NULL to ignore notification of completion.
         *
         * @param self The callback object.  This will never be NULL.
         * @param operation The original request operation.  This will never be NULL.
         * @param except An exception causing the completion of the Operation.
         *               If NULL, no exception occurred (the operation completed
         *               normally). A DOFEXCEPTION_TIMEOUT will occur
         *               only if no results were received.
         */
        void (* Complete) ( DOFObjectSubscribeCallback self, DOFOperation operation, DOFException except );

        /**
         * Callback for a Requestor for the results of an asynchronous operation.
         *
         * This callback may be NULL to ignore notification of results.
         *
         * @param self The callback object.  This will never be NULL.
         * @param operation The original request operation.  This will never be NULL.
         * @param providerInfo The @ref DOFProviderInfo associated with the
         *                 @a value or @a except.  This will never be NULL.
         * @param value  The current @ref DOFValue of the property for the
         *               Subscribe Operation.  Must be ignored if @a
         *               except is non-NULL.  This may be NULL if
         *               the returned value is "NULL" (from a nullable type).
         * @param except An exception received from a Provider.  If NULL, no
         *               exception occurred (the Subscribe was successful)
         *               and @a value is valid.
         */
        void (* PropertyChanged) ( DOFObjectSubscribeCallback self, DOFOperation operation, DOFProviderInfo providerInfo, DOFValue value, DOFException except );

    } * fns;

    /* Implementations may extend this structure. */

} DOFObjectSubscribeCallback_t;


/**
    The Requestor Callback interface.  Defined as a pointer to the @ref DOFObjectEventCallback_t structure.  Refer to the structure for more information.
**/
typedef struct DOFObjectEventCallback_t * DOFObjectEventCallback;

/**
 * The callback interface structure used to receive results from a Provider for
 * RegisterHandler Operations.
 *
 * @ifacestruct{DOFObjectEventCallback}
 * 
 * @threadsafety
 *
 * @see DOFObject_BeginRegister
 */
typedef struct DOFObjectEventCallback_t
{

    /**
     * The callback interface function structure, containing a list of
     * function pointers that implement the interface.  It has the same role as
     * a vtable in C++.  This must not be NULL.
     */
    const struct DOFObjectEventCallbackFns_t
    {

        /**
         * Callback for a Requestor for the completion of an asynchronous operation.
         *
         * This callback may be NULL to ignore notification of completion.
         *
         * @param self The callback object.  This will never be NULL.
         * @param operation The original request operation.  This will never be NULL.
         * @param except An exception causing the completion of the Operation.
         *               If NULL, no exception occurred (the operation completed
         *               normally). A DOFEXCEPTION_TIMEOUT will occur
         *               only if no results were received.
         */
        void (* Complete) ( DOFObjectEventCallback self, DOFOperation operation, DOFException except );

        /**
         * Callback for a Requestor for the results of an asynchronous operation.
         *
         * This callback may be NULL to ignore notification of results.
         *
         * @param self The callback object.  This will never be NULL.
         * @param operation The original request operation.  This will never be NULL.
         * @param provider The @ref DOFProviderInfo associated with the
         *                 @a parameters or @a except.  This will never be NULL.
         * @param parameterCount The number of values in the @a parameters array.
         * @param parameters An array of @ref DOFValue elements, representing the parameters of the Event used in the Register Operation.
         *               Must be ignored if @a except is non-NULL.  Otherwise, NULL indicates
         *               that the Register is successfully established and
         *               non-NULL indicates that a Signal is received.
         *               This may contain NULL values for parameters that are "NULL" (from nullable types).
         * @param except An exception received from a Provider.  If NULL, no
         *               exception occurred (the Register was successful)
         *               and @a parameters is valid.
         */
        void (* EventSignaled) ( DOFObjectEventCallback self, DOFOperation operation, DOFProviderInfo providerInfo, uint16 parameterCount, const DOFValue parameters[], DOFException except );

    } * fns;

    /* Implementations may extend this structure. */

} DOFObjectEventCallback_t;


/**
    The Requestor Callback interface.  Defined as a pointer to the @ref DOFObjectDefineCallback_t structure.  Refer to the structure for more information.
**/
typedef struct DOFObjectDefineCallback_t * DOFObjectDefineCallback;

/**
 * The callback interface structure used to receive an Interface definition from the
 * network.
 *
 * @ifacestruct{DOFObjectDefineCallback}
 * 
 * @threadsafety
 *
 * @see DOFObject_BeginDefine
 */
typedef struct DOFObjectDefineCallback_t
{

    /**
     * The callback interface function structure, containing a list of
     * function pointers that implement the interface.  It has the same role as
     * a vtable in C++.  This must not be NULL.
     */
    const struct DOFObjectDefineCallbackFns_t
    {

        /**
         * Callback for a Requestor for the completion of an asynchronous operation.
         *
         * This callback may be NULL to ignore notification of completion.
         *
         * @param self The callback object.  This will never be NULL.
         * @param operation The original request operation.  This will never be NULL.
         * @param except An exception causing the completion of the Operation.
         *               If NULL, no exception occurred (the operation completed
         *               normally). A DOFEXCEPTION_TIMEOUT will occur
         *               only if no results were received.
         */
        void (* Complete) ( DOFObjectDefineCallback self, DOFOperation operation, DOFException except );

        /**
         * Callback for a Requestor for the results of an asynchronous operation.
         *
         * This callback may be NULL to ignore notification of results.
         *
         * @param self The callback object.  This will never be NULL.
         * @param operation The original request operation.  This will never be NULL.
         * @param definition The complete @ref DOFInterface for the
         *                   Define Operation.  This will never be NULL.
         */
        void (* Define) ( DOFObjectDefineCallback self, DOFOperation operation, DOFInterface definition );

    } * fns;

    /* Implementations may extend this structure. */

} DOFObjectDefineCallback_t;


/**
    The Requestor Interest Callback interface.  Defined as a pointer to the @ref DOFSystemInterestCallback_t structure.  Refer to the structure for more information.
**/
typedef struct DOFSystemInterestCallback_t * DOFSystemInterestCallback;

/**
 * The callback interface structure used to be notified of interest operation completion.
 *
 * @ifacestruct{DOFSystemInterestCallback}
 * 
 * @threadsafety
 *
 * @see DOFSystem_BeginInterest
 */
typedef struct DOFSystemInterestCallback_t
{

    /**
     * The callback interface function structure, containing a list of
     * function pointers that implement the interface.  It has the same role as
     * a vtable in C++.  This must not be NULL.
     */
    const struct DOFSystemInterestCallbackFns_t
    {

        /**
         * Callback for a Requestor for the completion of an asynchronous interest
         * operation.
         *
         * This callback must not be NULL.
         *
         * @param self The callback object.  This will never be NULL.
         * @param operation The original request operation.  This will never be NULL.
         * @param except An exception causing the completion of the Operation.
         *               If NULL, no exception occurred (the operation completed
         *               normally).
         */
        void (* Complete) ( DOFSystemInterestCallback self, DOFOperation operation, DOFException except );

    } * fns;

    /* Implementations may extend this structure. */

} DOFSystemInterestCallback_t;

/**
    The Session Data Transform Callback interface.  Defined as a pointer to the @ref DOFOperationSessionDataTransform_t
    structure. Refer to the structure for more information.
**/
typedef struct DOFOperationSessionDataTransform_t * DOFOperationSessionDataTransform;

/**
 * The callback interface structure used to transform data on an OAP session.
 *
 * @ifacestruct{DOFOperationSessionDataTransform}
 * 
 * @threadsafety
 *
 * @see DOFOperationSession_SetDataTransform
 * @see DOFRequestSession_SetDataTransform
 */
typedef struct DOFOperationSessionDataTransform_t
{

    /**
     * The callback interface function structure, containing a list of
     * function pointers that implement the interface.  It has the same role as
     * a vtable in C++.  This must not be NULL.
     */
    const struct DOFOperationSessionDataTransformFns_t
    {

        /**
         *  Transform data received from the other end of an OAP Session.
         *
         *  @param  self    The DOFOperationSessionDataTransform this function is being called on.  Will never be @c NULL.
         *  @param  sessionId   The DOFInterfaceID associated with the OAP session.  Will never be @c NULL.
         *  @param  pSize   A pointer to the length of @a buffer.  If new data is allocated to accomdate the transform, @a *pSize should be updated to the allocated size.
         *  @param  buffer  The untransformed data.  This will never be @c NULL.
         *
         *  @return The transformed data.  If the pointer differs from @a buffer, the returned pointer will be freed using @ref PCRMem_Free.
         *
         *  @retval NULL    An error occurred while performing the data transformation.
         */
        uint8 * (* TransformReceiveData)(DOFOperationSessionDataTransform self, DOFInterfaceID sessionId, uint32 * pSize, uint8 * buffer);

        /**
         *  Transform data that will be sent over an OAP Session.
         *
         *  @param  self    The DOFOperationSessionDataTransform this function is being called on.  Will never be @c NULL.
         *  @param  sessionId   The DOFInterfaceID associated with the OAP session.  Will never be @c NULL.
         *  @param  pSize   A pointer to the length of @a buffer.  If new data is allocated to accomdate the transform, @a *pSize should be updated to the allocated size.
         *  @param  buffer  The untransformed data.  This will never be @c NULL.
         *
         *  @return The transformed data.  If the pointer differs from @a buffer, the returned pointer will be freed using @ref PCRMem_Free.
         *
         *  @retval NULL    An error occurred while performing the data transformation.
         */
        uint8 * (* TransformSendData)(DOFOperationSessionDataTransform self, DOFInterfaceID sessionId, uint32 * pSize, uint8 * buffer);

    } * fns;

} DOFOperationSessionDataTransform_t;


/**
    The System Route Listener interface.  Defined as a pointer to the @ref DOFSystemRouteListener_t structure.  Refer to the structure for more information.
**/
typedef struct DOFSystemRouteListener_t * DOFSystemRouteListener;

/**
 * The listener interface structure used to be notified of routing changes within the DOF.
 *
 * @ifacestruct{DOFSystemRouteListener}
 * 
 * @threadsafety
 *
 * @see DOFSystem_AddRouteListener
 */
typedef struct DOFSystemRouteListener_t
{

    /**
     * The listener interface function structure, containing a list of
     * function pointers that implement the interface.  It has the same role as
     * a vtable in C++.  This must not be NULL.
     */
    const struct DOFSystemRouteListenerFns_t
    {

        /**
         *  Callback used to let the listener free resources associated with the listener.
         *
         *  This callback is to notify the listener that it is safe to remove any data allocated for
         *  this listener.  After this callback is called, the listener will no longer be called.
         *
         *  Because the listener is updated as the state of the system changes,
         *  routes added or updated by
         *  @ref DOFSystemRouteListener_t::DOFSystemRouteListenerFns_t::RouteStatus "RouteStatus"
         *  will @b not be removed after this (Removed) is called.  It may therefore be necessary to cleanup data "orphaned" by
         *  the removal of the listener (since the RouteStatus call will not be called again for this listener).
         *
         *  This callback may be NULL to ignore notification of removal.
         *
         *  @param self The listener object.  This will never be NULL.
         *  @param system The @ref DOFSystem on which the listener was added.  This will never be NULL.
         *  @param except The exception causing the listener to be removed prematurely. This will be NULL if the listener is removed normally by the application.
         */
        void (* Removed) ( DOFSystemRouteListener self, DOFSystem system, DOFException except );

        /**
         * Callback to notify the application that something has changed for a route.
         *
         * @note The @a request is for information only and must not be completed via this callback.
         *
         * This callback must not be NULL.
         *
         * @param self The listener object.  This will never be NULL.
         * @param system The @ref DOFSystem on which the listener was added.  This will never be NULL.
         * @param routeInfo The routing information that is being updated.  This will never be NULL.
         * @param reason The reason for the update (@ref DOFROUTEINFOSTATE_ADDED, @ref DOFROUTEINFOSTATE_REMOVED, or @ref DOFROUTEINFOSTATE_CHANGED).
         *
         * @note The @a request is for information only and must not be completed via this callback.
         */
        void (* RouteStatus ) ( DOFSystemRouteListener self, DOFSystem system, DOFRouteInfo routeInfo, DOFRouteInfoState reason );

    } * fns;

    /* Implementations may extend this structure. */

} DOFSystemRouteListener_t;


/**
    The Retry Callback interface.  Defined as a pointer to the @ref DOFOperationRetryCallback_t structure.  Refer to the structure for more information.
**/
typedef struct DOFOperationRetryCallback_t * DOFOperationRetryCallback;

/**
 * The callback interface structure used to determine when to send the next retry.
 *
 * @ifacestruct{DOFOperationRetryCallback}
 * 
 * @threadsafety
 *
 * @see DOFOperationControl_SetRetryCallback
 */
typedef struct DOFOperationRetryCallback_t
{

    /**
     * The callback interface function structure, containing a list of
     * function pointers that implement the interface.  It has the same role as
     * a vtable in C++.  This must not be NULL.
     */
    const struct DOFOperationRetryCallbackFns_t
    {

        /**
         * Callback for a Requestor when a retry is about to take place.
         *
         * This callback must not be NULL.
         *
         * @param self The callback object.  This will never be NULL.
         * @param  operation The original request operation.  This will never be NULL.
         */
        uint32 (* Retry) ( DOFOperationRetryCallback self, DOFOperation operation );

    } * fns;

    /* Implementations may extend this structure. */

} DOFOperationRetryCallback_t;


/**
    The Connection Peer Listener interface.  Defined as a pointer to the @ref DOFConnectionPeerListener_t structure.
    Refer to the structure for more information.
**/
typedef struct DOFConnectionPeerListener_t * DOFConnectionPeerListener;

/**
 * The listener interface structure used to monitor connection peers.
 *
 * @ifacestruct{DOFConnectionPeerListener}
 * 
 * @threadsafety
 *
 * @see DOFConnection_AddStateListener
 */
typedef struct DOFConnectionPeerListener_t
{

    /**
     * The listener interface function structure, containing a list of
     * function pointers that implement the interface.  It has the same role as
     * a vtable in C++.  This must not be NULL.
     */
    const struct DOFConnectionPeerListenerFns_t
    {

        /**
         *  Callback used to let listener free any allocated memory from a removed listener.
         *
         *  This callback is to notify the listener that it is safe to remove any data allocated for
         *  this listener.  After this callback is called, the listener will no longer be called.
         *
         *  This callback may be NULL to ignore notification of removal.
         *
         *  @param self The listener object.  This will never be NULL.
         *  @param connection The connection for which the state is being reported.  This will never be NULL.
         *  @param except The exception causing the listener to be removed prematurely. This will be NULL if the listener is removed normally by the application.
         */
        void (* Removed) ( DOFConnectionPeerListener self, DOFConnection connection, DOFException except );

        /**
         * Callback for when a peer has joined the session represented by the Connection (for example, a node joined the secure
         * group or hub).
         *
         * This callback will be called periodically when a peer is known to have joined the session.  Only updates after the
         * listener was added will be reported.
         *
         * This callback may be NULL to ignore notification of added peers.
         *
         * @param self The listener object.  This will never be NULL.
         * @param connection The connection for which the peer addition is being reported.  This will never be NULL.
         * @param address The address of the peer that joined.  This will never be NULL.
         * @param scope The security scope of the peer that joined.  This may be the same as the connection, or some subset of
         *              it. This will never be NULL.
         */
        void (* PeerAdded) ( DOFConnectionPeerListener self, DOFConnection connection, DOFAddress address, DOFSecurityScope scope );

        /**
         * Callback for when a peer has left the session represented by the Connection (for example, a node has disconnected
         * from a hub).
         *
         * This callback will be called periodically when a peer is known to have left the session.  Only updates after the
         * listener was added will be reported.
         *
         * This callback may be NULL to ignore notification of removed peers.
         *
         * @param self The listener object.  This will never be NULL.
         * @param connection The connection for which the peer removal is being reported.  This will never be NULL.
         * @param address The address of the peer that left.  This will never be NULL.
         */
        void (* PeerRemoved) ( DOFConnectionPeerListener self, DOFConnection connection, DOFAddress address );

    } * fns;

    /* Implementations may extend this structure. */

} DOFConnectionPeerListener_t;


/**
    The Connection State Listener interface.  Defined as a pointer to the @ref DOFConnectionStateListener_t structure.  Refer to the structure for more information.
**/
typedef struct DOFConnectionStateListener_t * DOFConnectionStateListener;

/**
 * The listener interface structure used to monitor connection state.
 *
 * @ifacestruct{DOFConnectionStateListener}
 * 
 * @threadsafety
 *
 * @see DOFConnection_AddStateListener
 */
typedef struct DOFConnectionStateListener_t
{

    /**
     * The listener interface function structure, containing a list of
     * function pointers that implement the interface.  It has the same role as
     * a vtable in C++.  This must not be NULL.
     */
    const struct DOFConnectionStateListenerFns_t
    {

        /**
         *  Callback used to let listener free any allocated memory from a removed listener.
         *
         *  This callback is to notify the listener that it is safe to remove any data allocated for
         *  this listener.  After this callback is called, the listener will no longer be called.
         *
         *  This callback may be NULL to ignore notification of removal.
         *
         *  @param self The listener object.  This will never be NULL.
         *  @param connection The connection for which the state is being reported.  This will never be NULL.
         *  @param except The exception causing the listener to be removed prematurely. This will be NULL if the listener is removed normally by the application.
         */
        void (* Removed) ( DOFConnectionStateListener self, DOFConnection connection, DOFException except );

        /**
         * Callback for when the state of a Connection changes (for example, when the
         * Connection is established or when it is closed or fails).
         *
         * This callback will be called immediately when the listener is added to the
         * connection with the current state of the connection (whether connected or
         * disconnected) and then will be called whenever the state changes.
         *
         * All calls to connection state listeners are synchronized in a single
         * thread, so application locking is not required if all access to required
         * data structures is contained solely inside the listener.
         *
         * This callback must not be NULL.
         *
         * @param self The listener object.  This will never be NULL.
         * @param connection The connection for which the state is being reported.  This will never be NULL.
         * @param state The current state of the Connection.  This will never be NULL.
         */
        void (* StateChanged) ( DOFConnectionStateListener self, DOFConnection connection, DOFConnectionState state );

    } * fns;

    /* Implementations may extend this structure. */

} DOFConnectionStateListener_t;


/**
    The Connection Complete Callback interface.  Defined as a pointer to the @ref DOFConnectionConnectCallback_t structure.  Refer to the structure for more information.
**/
typedef struct DOFConnectionConnectCallback_t * DOFConnectionConnectCallback;

/**
 * The callback interface structure used to monitor connection complete.
 *
 * @ifacestruct{DOFConnectionConnectCallback}
 * 
 * @threadsafety
 *
 * @see DOFConnection_BeginConnect
 */
typedef struct DOFConnectionConnectCallback_t
{

    /**
     * The callback interface function structure, containing a list of
     * function pointers that implement the interface.  It has the same role as
     * a vtable in C++.  This must not be NULL.
     */
    const struct DOFConnectionConnectCallbackFns_t
    {

        /**
         * Callback for when an asynchronous connection request completes.
         * The exception can be checked to determine whether the request completed successfully or not.
         *
         * This callback must not be NULL.
         *
         * @param self The callback object.  This will never be NULL.
         * @param connection The connection for which the state is being reported.  This will never be NULL.
         * @param context The context that was associated with the callback during the @ref DOFConnection_BeginConnect
         * call, or NULL if no context was associated.
         * @param except An exception, if one occurred.  This will be NULL if there was no exception (success).
         */
        void (* Complete) ( DOFConnectionConnectCallback self, DOFConnection connection, void * context, DOFException except );

    } * fns;

    /* Implementations may extend this structure. */

} DOFConnectionConnectCallback_t;

/**
    The DOFConnection Stream Request Listener interface.  Defined as a pointer to the @ref DOFConnectInterestListener_t structure. Refer to the structure for more information.
**/
typedef struct DOFConnectionStreamRequestListener_t * DOFConnectionStreamRequestListener;

/**
 * The listener interface structure used to obtain notification of streaming connection requests, based on the
 * connection, security, interest and available bindings.
 *
 * @ifacestruct{DOFConnectionStreamRequestListener}
 * 
 * @threadsafety
 *
 * @see DOFConnectionConfigBuilder_SetStreamRequestListener
 */
typedef struct DOFConnectionStreamRequestListener_t
{

    /**
     * The listener interface function structure, containing a list of
     * function pointers that implement the interface.  It has the same role as
     * a vtable in C++.  This must not be NULL.
     */
    const struct DOFConnectionStreamRequestListenerFns_t
    {
        /**
         *  Callback used to let the listener free resources associated with the listener.
         *
         *  This callback is to notify the listener that it is safe to remove any data allocated for
         *  this listener.  After this callback is called, the listener will no longer be called.
         *
         *  Because the listener is updated as the state of the dof changes, notifications by
         *  @ref DOFConnectionStreamRequestListener_t::DOFConnectionStreamRequestListenerFns_t::StreamRequest "StreamRequest"
         *  will @b not be removed by
         *  @ref DOFConnectionStreamRequestListener_t::DOFConnectionStreamRequestListenerFns_t::StreamRequestComplete
         *  "StreamRequestComplete" after this (Removed) is called.  It may therefore be necessary to cleanup data "orphaned" by
         *  the removal of the listener (since the StreamRequestComplete call will not be called again for this listener).
         *
         *  This callback may be NULL to ignore notification of removal.
         *
         *  @param self The listener object.  This will never be NULL.
         *  @param except The exception causing the listener to be removed prematurely.  This will be NULL if the listener is removed normally.
         */
        void (*Removed)( DOFConnectionStreamRequestListener self, DOFException except );

        /**
         *  Callback for a DOFConnection when connect-level interests and matching providers indicate that a streaming
         *  connection is desired.
         *
         *  This callback may indicate to a datagram (or any non-stream) connection that it should "upgrade" itself to a
         *  streaming connection (to the same host).  This callback may indicate to a stream connection that the connection is
         *  still desired. This call is responsible for establishing or maintaining the streaming connection, if possible.
         *
         *  This callback may be NULL to ignore notification of this condition.
         *
         *  @param self The listener object.  This will never be NULL.
         *  @param config  The DOFConnectionConfig on which the listener was set. In addition, this contains the valid security
         *                 information assocaited with the existing connection so that a compatible streaming connection can be
         *                 made. This will never be NULL.
         *  @param request An object representing the request for a sreaming connection. This will never be NULL. This request
         *                 includes the following information:
         *  # The DOFConnection on which the request is being made. This connection instance is unique to the request/listener
         *  (it is not the same DOFConnection the application has, but it represents the same underlying connection). This also
         *  gives access to the current connection state, useful for determining if the request is on a datagram connection (for
         *  which a stream connection should be opened) or on a stream connection (which needs to be kept open, and possibly
         *  monitored, to satisfy the request).
         *  # The DOFAddress representing the address to which to connect. In some cases, the address associated with the
         *  connection (and its configuration) may not be appropriate for opening a streaming connection (e.g., a multicast,
         *  broadcast, or group address); this parameter is always a unicast address, and is always the correct address to which
         *  to connect.
         */
        void (* StreamRequest) ( DOFConnectionStreamRequestListener self, DOFConnectionConfig config, DOFConnectionStreamRequest request );

        /**
         *  Callback for a DOFConnection when connect-level interests and a matching providers indicate that a streaming
         *  connection is no longer desired.
         *
         *  This callback may indicate to a stream connection that the connection is no longer needed. This call is responsible
         *  for cleaning up any state established by the corresponding StreamRequest callback.
         *
         *  This callback may be NULL to ignore notification of this condition.
         *
         *  @param self The listener object.  This will never be NULL.
         *  @param config  The DOFConnectionConfig associated with this listener. This will never be NULL.
         *  @param requet The DOFConnectionStreamRequest that is now complete. This will never be NULL.
         */
        void (* StreamRequestComplete) ( DOFConnectionStreamRequestListener self, DOFConnectionConfig config, DOFConnectionStreamRequest request );
    } * fns;

    /* Implementations may extend this structure. */

} DOFConnectionStreamRequestListener_t;


/**
    The Server State Listener interface.  Defined as a pointer to the @ref DOFServerStateListener_t structure.  Refer to the structure for more information.
**/
typedef struct DOFServerStateListener_t * DOFServerStateListener;

/**
 * The listener interface structure used to monitor server state.
 *
 * @ifacestruct{DOFServerStateListener}
 * 
 * @threadsafety
 *
 * @see DOFServer_AddStateListener
 */
typedef struct DOFServerStateListener_t
{

    /**
     * The listener interface function structure, containing a list of
     * function pointers that implement the interface.  It has the same role as
     * a vtable in C++.  This must not be NULL.
     */
    const struct DOFServerStateListenerFns_t
    {

        /**
         *  Callback used to let listener free any allocated memory from a removed listener.
         *
         *  This callback is to notify the listener that it is safe to remove any data allocated for
         *  this listener.  After this callback is called, the listener will no longer be called.
         *
         *  This callback may be NULL to ignore notification of removal.
         *
         *  @param self The listener object.  This will never be NULL.
         *  @param server The server for which the state is being reported.
         *  @param except The exception causing the listener to be removed prematurely. This will be NULL if the listener is removed normally by the application.
         */
        void (* Removed) ( DOFServerStateListener self, DOFServer server, DOFException except );

        /**
         * Callback for when the state of a Server changes (for example, when the
         * Server is started or when it is stopped or fails).
         *
         * This callback will be called immediately when the listener is added to the
         * server with the current state of the server (whether started or stopped)
         * and then will be called whenever the state changes.
         *
         * All calls to server state listeners are synchronized in a single
         * thread, so application locking is not required if all access to required
         * data structures is contained solely inside the listener.
         *
         * This callback must not be NULL.
         *
         * @param self The listener object.  This will never be NULL.
         * @param server The server for which the state is being reported.  This will never be NULL.
         * @param state The current state of the Server.  This will never be NULL.
         */
        void (* StateChanged) ( DOFServerStateListener self, DOFServer server, DOFServerState state );

    } * fns;

    /* Implementations may extend this structure. */

} DOFServerStateListener_t;


/**
    The Server Complete Callback interface.  Defined as a pointer to the @ref DOFServerStartCallback_t structure.  Refer to the structure for more information.
**/
typedef struct DOFServerStartCallback_t * DOFServerStartCallback;

/**
 * The callback interface structure used to monitor connection complete.
 *
 * @ifacestruct{DOFServerStartCallback}
 * 
 * @threadsafety
 *
 * @see DOFServer_BeginStart
 */
typedef struct DOFServerStartCallback_t
{
    /**
     * The callback interface function structure, containing a list of
     * function pointers that implement the interface.  It has the same role as
     * a vtable in C++.  This must not be NULL.
     */
    const struct DOFServerStartCallbackFns_t
    {

        /**
         * Callback for when an asynchronous server startup completes.
         * The exception can be checked to determine whether the request completed successfully or not.
         *
         * This callback must not be NULL.
         *
         * @param self The callback object.  This will never be NULL.
         * @param server The server for which the state is being reported.  This will never be NULL.
         * @param context The context that was associated with the callback during the @ref DOFServer_BeginStart
         * call, or NULL if no context was associated.
         * @param except An exception, if one occurred.  This will be NULL if there was no exception (success).
         */
        void (* Complete) ( DOFServerStartCallback self, DOFServer server, void * context, DOFException except );

    } * fns;

    /* Implementations may extend this structure. */

} DOFServerStartCallback_t;


/**
    The Runtime Connection State Listener interface. Defined as a pointer to the @ref DOFRuntimeConnectionStateListener_t structure. Refer to the structure for more information.
**/
typedef struct DOFRuntimeConnectionStateListener_t * DOFRuntimeConnectionStateListener;

/**
 * The DOFRuntimeConnectionStateListener interface structure, when implemented, enables you to receive 
 * notifications when any of this DOF's connections, either inbound or outbound, are established or go down.
 *
 * @ifacestruct{DOFRuntimeConnectionStateListener}
 * 
 * @threadsafety
 *
 * @see DOFRuntime_AddConnectionListener
 * @see DOFConnectionStateListener
 */
typedef struct DOFRuntimeConnectionStateListener_t
{

    /**
     * The listener interface function structure, containing a list of
     * function pointers that implement the interface. It has the same role as
     * a vtable in C++. This must not be @c NULL.
     */
    const struct DOFRuntimeConnectionStateListenerFns_t
    {

        /**
         * Called when this listener is removed from the DOFRuntime, either through use of @ref DOFRuntime_RemoveConnectionListener 
         * or because an exception caused the listener to be removed. Use this callback to free any memory, 
         * resources, or data allocated in association with this listener. After this callback is called, the listener will 
         * no longer be called.
         *
         * This callback may be @c NULL to ignore notification of removal.
         *
         * @param self The listener object. This will never be @c NULL.
         * @param runtime The @ref DOFRuntime on which the listener was added. This will never be @c NULL.
         * @param except The exception that caused this listener to be removed. This will be @c NULL if the 
         *               listener was removed normally.
         */
        void (* Removed) ( DOFRuntimeConnectionStateListener self, DOFRuntime runtime, DOFException except );

        /**
         * Called whenever the @ref DOFConnectionState_IsConnected value changes for any of the DOF's connections. 
         * This function is called immediately for all connections when the listener is added, and then will be 
         * called whenever a connection's connected state changes. For each connection, the library guarantees 
         * that calls to this function will occur in the same order as the associated state change, so that, for 
         * example, if a connection rapidly connects and then disconnects, you won't mistakenly receive the 
         * disconnect notification before the connect notification.
         *
         * This callback must not be @c NULL.
         *
         * @param self The listener object. This will never be @c NULL.
         * @param runtime The @ref DOFRuntime on which the listener was added. This will never be @c NULL.
         * @param connection The @ref DOFConnection that changed state. This will never be @c NULL.
         * @param state The updated state of the connection. This will never be @c NULL.
         */
        void (* StateChanged) ( DOFRuntimeConnectionStateListener self, DOFRuntime runtime, DOFConnection connection, DOFConnectionState state );

    } * fns;

    /* Implementations may extend this structure. */

} DOFRuntimeConnectionStateListener_t;


/**
    The Runtime Server State Listener interface.  Defined as a pointer to the @ref DOFRuntimeServerStateListener_t structure.  Refer to the structure for more information.
**/
typedef struct DOFRuntimeServerStateListener_t * DOFRuntimeServerStateListener;

/**
 * The DOFRuntimeServerStateListener interface, when implemented, enables you to receive notifications when any of the 
 * DOF's servers start or stop.
 *
 * @ifacestruct{DOFRuntimeServerStateListener}
 * 
 * @threadsafety
 *
 * @see DOFRuntime_AddServerListener
 * @see DOFServerStateListener
 */
typedef struct DOFRuntimeServerStateListener_t
{

    /**
     * The listener interface function structure, containing a list of
     * function pointers that implement the interface. It has the same role as
     * a vtable in C++. This must not be @c NULL.
     */
    const struct DOFRuntimeServerStateListenerFns_t
    {

        /**
         * Called when this listener is removed from the DOFRuntime, either through use of @ref DOFRuntime_RemoveServerListener 
         * or because an exception caused the listener to be removed. Use this callback to free any memory, 
         * resources, or data allocated in association with this listener. After this callback is called, the listener will 
         * no longer be called.
         * 
         * This callback may be @c NULL to ignore notification of removal.
         *
         * @param self The listener object. This will never be @c NULL.
         * @param runtime The @ref DOFRuntime on which the listener was added. This will never be @c NULL.
         * @param except The exception that caused this listener to be removed. This will be @c NULL if the 
         *               listener was removed normally.
         */
        void (* Removed) ( DOFRuntimeServerStateListener self, DOFRuntime runtime, DOFException except );

        /**
         * Called whenever the @ref DOFServerState_IsStarted value changes for any of the DOF's servers. 
         * This function is called immediately for all servers when the listener is added, and then will be 
         * called whenever a server's started state changes. For each server, the library guarantees that 
         * calls to this function will occur in the same order as the associated state change, so that, 
         * for example, if a server rapidly starts and then stops, you won't mistakenly receive the stop 
         * notification before the start notification.
         *
         * This callback must not be @c NULL.
         *
         * @param self The listener object. This will never be @c NULL.
         * @param runtime The @ref DOFRuntime on which the listener was added. This will never be @c NULL.
         * @param server The @ref DOFServer that changed state. This will never be @c NULL.
         * @param state The updated state of the server. This will never be @c NULL.
         */
        void (* StateChanged) ( DOFRuntimeServerStateListener self, DOFRuntime runtime, DOFServer server, DOFServerState state );

    } * fns;

    /* Implementations may extend this structure. */

} DOFRuntimeServerStateListener_t;


/**
    The Log Listener interface.  Defined as a pointer to the @ref DOFLogListener_t structure.  Refer to the structure for more information.
**/
typedef struct DOFLogListener_t * DOFLogListener;

/**
 * The listener interface structure used to monitor log messages.
 *
 * @ifacestruct{DOFLogListener}
 * 
 * @threadsafety
 *
 * @see DOFLog_AddListener
 */
typedef struct DOFLogListener_t
{

    /**
     * The listener interface function structure, containing a list of
     * function pointers that implement the interface.  It has the same role as
     * a vtable in C++.  This must not be NULL.
     */
    const struct DOFLogListenerFns_t
    {

        /**
         *  Callback used to let listener free any allocated memory from a removed listener.
         *
         *  This callback is to notify the listener that it is safe to remove any data allocated for
         *  this listener.  After this callback is called, the listener will no longer be called.
         *
         *  This callback may be NULL to ignore notification of removal.
         *
         *  @param self The listener object.  This will never be NULL.
         *  @param except The exception causing the listener to be removed prematurely. This will be NULL if the listener is removed normally by the application.
         */
        void (* Removed) ( DOFLogListener self, DOFException except );

        /**
         * Callback for when a new message is to be logged.
         * Messages are logged and notified in-order (per listener), but may be delayed due to the asynchronous handling of the listener.
         *
         * This callback must not be NULL.
         *
         * @param self The listener object.  This will never be NULL.
         * @param timestamp The timestamp when the message was generated, based on @ref PCRTime_GetMS.
         * @param level The priority level of the message.
         * @param message The log message.  This will never be NULL.
         */
        void (* LogMessage) ( DOFLogListener self, uint32 timestamp, DOFLogLevel level, const char * message );

    } * fns;

    /* Implementations may extend this structure. */

} DOFLogListener_t;


/**
    The DOFDomainListener interface.  Defined as a pointer to the @ref DOFDomainListener_t structure.  Refer to the structure for more information.
**/
typedef struct DOFDomainListener_t*   DOFDomainListener;

/**
 * The DomainListener interface enables you to be notified when a route to the Authentication Server 
 * for a remote domain is discovered. The DomainListener is primarily useful on DOF Center Servers or
 * other centralized nodes that route traffic between multiple remote security domains.
 *
 * @ifacestruct{DOFDomainListener}
 * 
 * @threadsafety
 *
 * @see DOF_AddDomainListener
 */
typedef struct DOFDomainListener_t
{

    /**
     * The listener interface function structure, containing a list of
     * function pointers that implement the interface.  It has the same role as
     * a vtable in C++.  This must not be NULL.
     */
    const struct DOFDomainListenerFns_t
    {
        /**
         * Called when the Domain Listener is removed from the DOF. Use this callback to free resources associated with 
         * the listener or free the listener itself if it was allocated.
         *
         * @note This callback may be @c NULL to ignore notification of removal.
         *
         * @param self This listener. This will never be @c NULL.
         * @param dof The DOF where this listener was added. This will never be @c NULL.
         * @param exception An exception that caused the listener to be removed prematurely. This will be null 
         *                  if the application removed the listener normally.
         * @see DOF_RemoveDomainListener
         */
        void (*Removed)( DOFDomainListener self, DOF dof, DOFException exception );

        /**
         * Called when you have created a DOFServer or DOFConnection and configured it using one of the following functions 
         * and a route to the remote Authentication Server is discovered:
         * 
         * - @ref DOFServerConfigBuilder_SetDomainDiscoveryCredentials
         * - @ref DOFConnectionConfigBuilder_SetDomainDiscoveryCredentials
         *
         * @param self This listener. This will never be @c NULL.
         * @param dof The DOF where this listener and the discovered domain were added. This will never be @c NULL.
         * @param domain The DOFDomainConfig for the discovered domain. This will never be @c NULL.
         * @param creds The credentials that were used to connect in the domain. These are the credentials that were used in 
         *              the SetDomainDiscoveryCredentials function. This will never be @c NULL.
         * @param state The state of the connection that the domain was discovered over. This will never be @c NULL.
         */
        void (*DomainAdded)( DOFDomainListener self, DOF dof, DOFDomainConfig domain, DOFCredentials creds, DOFConnectionState state );
    } * fns;

    /* Implementations may extend this structure. */

}DOFDomainListener_t;

/**
@}
**/


#ifdef  __cplusplus
}
#endif

#endif /* DOF_OALTYPES_H */
