/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef _DOF_ADDRESSINTERNAL_H
#define _DOF_ADDRESSINTERNAL_H


#include <dof/core.h>
#include <dof/transport.h>
#include <internal/observable.h>
#include <dof/address.h>


#ifdef  __cplusplus
extern "C" {
#endif

/** @addtogroup DOFAddress
@{
**/

typedef struct DOFAddress_t
{
    uint32          len;    /* actual length of the sockaddr structure. If a DOFGroupAddress, is 4 (length of pointer)*/
    DTLTransport    transport;
    OALObservable   addrDownObservable;
    uint32          refcnt; /* Reference count. */
    uint32          enpPort;
    uint32          addr;   /* 4-byte aligned location for sockaddr structure (not really a uint32).
                               If a DOFGroupAddress, is pointer to the OID that is tied to this DOFAddress. */
                            /* addr MUST be last structure member! */
}DOFAddress_t;


extern void          DOFAddress_ClearObservable    ( DOFAddress self );
extern OALObserver   DOFAddress_AddObserver        ( DOFAddress self, OALObserver_fn_t fn, void * arg );
extern void          DOFAddress_RemoveObserver     ( DOFAddress self, OALObserver observer );
extern DOFAddress    DOFAddress_Create_AddENPPort  ( DOFAddress self, uint32 enpPort );
extern uint32        DOFAddress_GetENPPort         ( DOFAddress self );
extern boolean       DOFAddress_IsAddressInfoEqual ( const DOFAddress self, const DOFAddress addr );
extern int           DOFAddress_CompareAddressInfo ( const DOFAddress self, const DOFAddress addr );


/**
@}
**/

#ifdef  __cplusplus
}
#endif

#endif /* _DOF_ADDRESSINTERNAL_H */
