/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef __avl_multitree_H
#define __avl_multitree_H

#include "avl_tree.h"
#include <dof/core.h>

/* made with the aid of Neil Brown's non-recursive AVL implementation http://neil.brown.name/blog/20041124101820
    and his article on non-recursive deletion at http://cgi.cse.unsw.edu.au/~neilb/me/01101186893

    but it's heavily modified for my evil purposes */

typedef const void *OALAVLMultiTreeKey_t;
typedef const void *OALAVLMultiTreeItem_t;

typedef OALAVLTree_CompareGreaterThan_fn_t OALAVLMultiTree_CompareGreaterThan_fn_t;
typedef OALAVLTree_CompareEqual_fn_t OALAVLMultiTree_CompareEqual_fn_t;

typedef struct OALAVLMultiTree_t *OALAVLMultiTree;

typedef enum OALAVLMultiTree_Traverse_All_Return_Type_fn_t {
    OALAVLMULTITREE_NORMAL,
    OALAVLMULTITREE_TERMINATE_SET,
    OALAVLMULTITREE_TERMINATE_TRAVERSE,
    OALAVLMULTITREE_INVALID_RETURN
} OALAVLMultiTree_Traverse_All_Return_Type_fn_t;

typedef boolean (*OALAVLMultiTree_Traversal_fn_t)(OALAVLMultiTreeItem_t item, void *arg);
typedef OALAVLMultiTree_Traverse_All_Return_Type_fn_t (*OALAVLMultiTree_Traverse_All_fn_t)(OALAVLMultiTreeItem_t item, void *arg, boolean new_set);
typedef void (*OALAVLMultiTree_RemoveAll_fn_t)(OALAVLMultiTreeItem_t item, void *arg);

/* compare_fn should return TRUE if d1>d2. you can pass NULL here for a default compare of the data pointer itself */
extern OALAVLMultiTree OALAVLMultiTree_Create(OALAVLMultiTree_CompareGreaterThan_fn_t key_gt_fn, OALAVLMultiTree_CompareEqual_fn_t key_eq_fn, OALAVLMultiTree_CompareGreaterThan_fn_t item_gt_fn, OALAVLMultiTree_CompareEqual_fn_t item_eq_fn, void *context);
extern void OALAVLMultiTree_Destroy(OALAVLMultiTree tree, OALAVLMultiTree_RemoveAll_fn_t remove_fn, void *arg);

extern OALAVLTree_InsertionStatus OALAVLMultiTree_Insert(OALAVLMultiTree tree, OALAVLMultiTreeKey_t key, OALAVLMultiTreeItem_t item);
extern boolean OALAVLMultiTree_Delete(OALAVLMultiTree tree, OALAVLMultiTreeKey_t key, OALAVLMultiTreeItem_t item);

/* this just returns one item! */
extern OALAVLMultiTreeItem_t OALAVLMultiTree_Retrieve(OALAVLMultiTree tree, OALAVLMultiTreeKey_t key);

/* returns your item if it's in the tree; otherwise NULL */
extern OALAVLMultiTreeItem_t OALAVLMultiTree_Retrieve_Item(OALAVLMultiTree tree, OALAVLMultiTreeKey_t key, OALAVLMultiTreeItem_t item);

/* traverse over all items that are in the tree with key data */
extern void OALAVLMultiTree_Traverse(OALAVLMultiTree tree, OALAVLMultiTreeItem_t key, OALAVLMultiTree_Traversal_fn_t traverse_fn, void *arg);

/* traverse over all items in the table one key at a time */
extern void OALAVLMultiTree_Traverse_All(OALAVLMultiTree tree, OALAVLMultiTree_Traverse_All_fn_t traverse_fn, void *arg);

extern uint32 OALAVLMultiTree_GetNumItems(OALAVLMultiTree tree);

#endif
