/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef AVL_TREELIST_H
#define AVL_TREELIST_H

#include <dof/core.h>

#ifdef  __cplusplus
extern "C" {
#endif

typedef const void * OALAVLTreeListKey_t;
typedef const void * OALAVLTreeListItem_t;

typedef enum OALAVLTreeList_Traverse_All_Return_Type_fn_t {
    OALAVLTREELIST_NORMAL,
    OALAVLTREELIST_TERMINATE_LIST,
    OALAVLTREELIST_TERMINATE_TRAVERSE
} OALAVLTreeList_Traverse_All_Return_Type_fn_t;

typedef boolean (*OALAVLTreeList_CompareGreaterThan_fn_t)(OALAVLTreeListKey_t key, OALAVLTreeListItem_t item, void *context);
typedef boolean (*OALAVLTreeList_CompareEqual_fn_t)(OALAVLTreeListKey_t key, OALAVLTreeListItem_t item, void *context);

typedef boolean (*OALAVLTreeList_Traversal_fn_t)(OALAVLTreeListItem_t item, void *arg);
typedef OALAVLTreeList_Traverse_All_Return_Type_fn_t (*OALAVLTreeList_TraverseAll_fn_t)(OALAVLTreeListItem_t item, void *context, boolean newKey);

typedef void (*OALAVLTreeListRemoveList_fn)(OALAVLTreeListItem_t item, void *arg);

typedef struct OALAVLTreeList_t* OALAVLTreeList;

extern OALAVLTreeList          OALAVLTreeList_Create(OALAVLTreeList_CompareGreaterThan_fn_t gt_fn, OALAVLTreeList_CompareEqual_fn_t eq_fn, void *context);
extern OALAVLTreeList          OALAVLTreeList_Copy(OALAVLTreeList self);
extern void                    OALAVLTreeList_Destroy(OALAVLTreeList self, OALAVLTreeListRemoveList_fn remove_fn, void *arg);
extern boolean                 OALAVLTreeList_Insert(OALAVLTreeList self, OALAVLTreeListKey_t key, OALAVLTreeListItem_t item);
extern void                    OALAVLTreeList_DeleteList(OALAVLTreeList tree, OALAVLTreeListKey_t key, OALAVLTreeListRemoveList_fn remove_fn, void *arg);
extern void                    OALAVLTreeList_TraverseList(OALAVLTreeList tree, OALAVLTreeListKey_t key, OALAVLTreeList_Traversal_fn_t traverse_fn, void *arg);                
extern void                    OALAVLTreeList_TraverseAll(OALAVLTreeList tree, OALAVLTreeList_TraverseAll_fn_t traverse_fn, void *arg);
extern OALAVLTreeListItem_t    OALAVLTreeList_FindAndRemove(OALAVLTreeList tree, OALAVLTreeListKey_t key, OALAVLTreeList_Traversal_fn_t fn, void *arg);
extern boolean                 OALAVLTreeList_IsKeyInTree(OALAVLTreeList tree, OALAVLTreeListKey_t key);
extern uint32                  OALAVLTreeList_GetNumLists(OALAVLTreeList tree);
extern uint32                  OALAVLTreeList_GetNumItems(OALAVLTreeList self);
extern uint32                  OALAVLTreeList_GetNumItemsInList(OALAVLTreeList self, OALAVLTreeListKey_t key );

#ifdef  __cplusplus
}
#endif

#endif
