/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_BLACKBOARDDATA_H
#define DOF_BLACKBOARDDATA_H

#include <dof/core.h>
#include <dof/address.h>
#include <internal/protocollayerresult.h>
#include <internal/stream.h>
#include <internal/packet.h>
#include <internal/channel.h>
#include <internal/cipher.h>

#ifdef  __cplusplus
extern "C" {
#endif

/* @addtogroup blackboarddata
@{
**/

/*
**  "Public" Type and Method Declarations
*/

typedef struct BlackboardData_t
{
    DOFOperationID      opID; /* this is the RID + opCount */
    DOFAddress          transportAddr;

    OALStream           receiveStream;
    OALPacket           sendPacket;
    OALChannel          associatedChannel;
    DOFOperation        providerOp;
    DOFSecurityScope    securityScope;
    DOFSecurityScope    remoteOpSecurityScope;
    OALCipherKeyState   keyState;
    uint32              delay; /* in milliseconds */
    uint32              delayWithoutGracePeriod;
    uint32              securityStateID;
    uint32              secureMarkingID;
    uint32              remoteDomainID;
    struct
    {
        unsigned int isMulticast                                    : 1;
        unsigned int isFlooded                                      : 1;
        unsigned int isCommandNotResponse                           : 1;
        unsigned int denyPacketReplayProtectionIfSecureUnicastSend  : 1;
        unsigned int getChannelBySSIDFromAsInProtocolStack          : 1;
        unsigned int securityRequiredSecureHeaderUnsecure           : 1;
        unsigned int securityRequiredUnsecure                       : 1;
        unsigned int securityRequiredAuthenticate                   : 1;
        unsigned int securityRequiredEncrypt                        : 1;
        unsigned int securityRequiredEncryptOrAuthenticate          : 1;
        unsigned int receivedUnsecure                               : 1;
        unsigned int receivedAuthenticated                          : 1;
        unsigned int receivedEncrypted                              : 1;
        unsigned int receivedUnsecureOnSecureChannel                : 1;
        unsigned int streamingProtocolNotAllowed                    : 1;
        unsigned int datagramProtocolP2PNotAllowed                  : 1;    /* Datagram Point to Point */
        unsigned int multicastProtocolNotAllowed                    : 1;    /* Datagram Multicast */
        unsigned int ifProperPlaceNextBaseNIDOnWire                 : 1;
        unsigned int responseIsIntermediate                         : 1;
        unsigned int commandCompletesOnAnyFinal                     : 1;
        unsigned int forceUnsecure                                  : 1;
    } flags;
    uint8               seq;
    uint8               retry;
} BlackboardData_t;


typedef struct BlackboardData_t * BlackboardData;

extern void BlackboardData_Init(BlackboardData blackboardData);
extern void BlackboardData_Clear(BlackboardData blackboardData);
/* Note: BlackboardData_Save and BlackboardData_Restore only copy the pointers and values and do not retain or release any objects. */
extern BlackboardData BlackboardData_Save(BlackboardData blackboardData);
extern void BlackboardData_Restore(BlackboardData dest, BlackboardData src);

/**
@}
**/

#ifdef  __cplusplus
}
#endif

#endif /* DOF_BLACKBOARDDATA_H */
