/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_BRIDGE_INTERNAL_H
#define DOF_BRIDGE_INTERNAL_H

#ifdef  __cplusplus
extern "C" {
#endif

#include <dof/oaltypes.h>
#include <internal/channel.h>
#include <internal/cipher.h>
#include <internal/security_membrane.h>

extern boolean DOFOperationBridgeConfig_Init(void);

extern void DOFOperationBridgeConfig_Shutdown(void);

extern OALOperationBridge OALOperationBridge_CreateNonSecureBridge(DOF                dof,
                                                                   DOFOperationBridgeConfig config);

extern OALOperationBridge OALOperationBridge_CreatePassThroughBridge(OALChannel channel);

extern OALOperationBridge OALOperationBridge_CreateSecureBridge(DOF                       dof,
                                                                DOFCredentials            credentials,
                                                                DOFObjectIDAuthentication id,
                                                                DOFPermissionSet          inboundPermissions,
                                                                DOFSecurityScope          inboundScope,
                                                                uint8                     inboundDuration,
                                                                DOFPermissionSet          outboundPermissions,
                                                                DOFSecurityScope          outboundScope,
                                                                uint8                     outboundDuration,
                                                                boolean                   extendablePermissions);

extern DOFSecurityMembrane DOFOperationBridgeConfig_GetMembrane(OALOperationBridge self);

extern DOFObjectIDAuthentication DOFOperationBridgeConfig_GetAlternateIdentity(DOFOperationBridgeConfig self);

extern OALOperationBridge OALOperationBridge_Copy(OALOperationBridge self);

extern void OALOperationBridge_Destroy(OALOperationBridge self);

#ifdef  __cplusplus
}
#endif

#endif
