/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_CHANNEL_STACK_MESSAGES_H
#define DOF_CHANNEL_STACK_MESSAGES_H

#include <dof/core.h>
#include <internal/blackboarddata.h>
#include <internal/messaging.h>
#include <internal/channel.h>
#include <dof/address.h>

#ifdef  __cplusplus
extern "C" {
#endif

typedef struct OLAMessage_PeerEventMessage_t * OLAMessage_PeerEventMessage;
typedef struct OLAMessage_PeerEventMessage_t
{
    /* The message must be the first member of this structure. */
    OALMessage_t        message;

    DOFAddress          address;
    DOFSecurityScope    scope;
    boolean             added;
}OLAMessage_PeerEventMessage_t;

typedef struct OALMessage_StreamRequestMessage_t * OALMessage_StreamRequestMessage;
typedef struct OALMessage_StreamRequestMessage_t
{
    /* The message must be the first member of this structure. */
    OALMessage_t        message;

    DOFAddress          addr;
    boolean             complete;
}OALMessage_StreamRequestMessage_t;

typedef struct OALMessage_CreateBridgeMessage_t * OALMessage_CreateBridgeMessage;
typedef struct OALMessage_CreateBridgeMessage_t
{
    /* The message must be the first member of this structure. */
    OALMessage_t                message;

    DOF                         dof;
    DOFOperationBridgeConfig    bridgeConfig;
}OALMessage_CreateBridgeMessage_t;

typedef struct OALMessage_CreateBridgeCompleteMessage_t * OALMessage_CreateBridgeCompleteMessage;
typedef struct OALMessage_CreateBridgeCompleteMessage_t
{
    /* The message must be the first member of this structure. */
    OALMessage_t                message;

    OALOperationBridge          bridge;
}OALMessage_CreateBridgeCompleteMessage_t;

typedef enum ChannelMsgType_t
{
    /* the order here is coupled with an array of function pointers in channel message handler stuff. So dont change it. */
    CMT_ROUTE_TO_PROVIDER = 0,
    CMT_ROUTE_TO_REQUESTOR,
    CMT_RECEIVE_FROM_TRANSPORT,
    CMT_INITIALIZE,
    CMT_INITIALIZE_COMPLETE,
    CMT_SHUTDOWN,
    CMT_TRANSPORTDOWN,
    CMT_PROCESS_BLOCKED_MESSAGE,
    CMT_CONNECTION_INITIATED_SHUTDOWN,
    CMT_REGISTER_SECURE_CHANNEL,
    CMT_APPID_QUERY_TIMEOUT,
    CMT_REGISTER_ENPPORT_CHANNEL,
    CMT_REMOTE_DOMAIN_CALLBACK,
    CMT_RESOLVE_CREDS_CALLBACK,
    CMT_GET_LIBRARY_PERMS_CALLBACK,
    CMT_SERVER_INITIATED_SHUTDOWN,
    CMT_PROCESS_RECEIVE_SILENCE,
    CMT_SET_DOMAIN_CREDENTIALS_CALLBACK,
    CMT_PEER_LISTENER_EVENT,
    CMT_CREATEBRIDGE,
    CMT_CREATEBRIDGE_COMPLETE,
    CMT_CONNECTION_STREAM_REQUEST,
    /* Add new message types here, and dont forget to update the message handlers */

    CMT_LAST /* keep this at the end */
} ChannelMsgType_t;


typedef enum StackMsgType_t
{
    /* the order here is coupled with an array of function pointers in stack message handler stuff. So dont change it. */
    SMT_ROUTE_TO_PROVIDER = 0,
    SMT_ROUTE_TO_REQUESTOR,
    SMT_RECEIVE_FROM_TRANSPORT,
    SMT_BEGIN_NEGOTIATION_AUTHORIZATION,
    SMT_SHUTDOWN,
    SMT_CONNECTED_NEGOTIATION_TIMEOUT,
    SMT_NEGOTIATION_SERVER_SIDE,
    SMT_REMOTE_DOMAIN_CALLBACK,
    SMT_RESOLVE_CREDS_CALLBACK,
    SMT_GET_LIBRARY_PERMS_CALLBACK,
    SMT_ROUTE_SAVED_EPP,
    /* Add new message types here, and dont forget to update the message handlers */

    SMT_LAST /* keep this at the end */
} StackMsgType_t;


extern OALMessage CMT_RouteToProvider_Create( void* recipient, DOFOperation req_op, DOFOperation prov_op, DOFAddress addr );
extern OALMessage CMT_RouteToRequestor_Create( void* recipient, DOFOperation resp_op, DOFResult result );
extern OALMessage CMT_ReceiveFromTransport_Create( void* recipient, OALStream data, DOFAddress rcvdFrom, boolean isMulticast );
extern OALMessage CMT_Initialize_Create(void* recipient);
extern OALMessage CMT_InitializeComplete_Create(void* recipient);
extern OALMessage CMT_Shutdown_Create(void* recipient, DOFException exception);
extern void       CMT_Shutdown_SetException(OALMessage msg, DOFException exception);
extern OALMessage CMT_TransportDown_Create(void* recipient, DTL transport, DOFException exception);
extern OALMessage CMT_ProcessBlockedMessage(void * recipient, boolean clearSendBlock);
extern OALMessage CMT_ConnectionInitiatedShutdown_Create(void * recipient, OALConnection OALConn);
extern OALMessage CMT_RegisterSecureChannel_Create(void * recipient, uint32 SSID, boolean AS);
extern OALMessage CMT_APPIDQueryTimeout_Create(void * recipient);
extern OALMessage CMT_RegisterChannel_ENPPort_Create(void * recipient);
extern OALMessage CMT_RemoteDomainCallback_Create(void *recipient, DOFCredentials creds);
extern OALMessage CMT_ResolveCredsCallback_Create(void *recipient, DOFCredentials creds);
extern OALMessage CMT_GetLibraryPermsCallback_Create(void *recipient, DOFPermissionSet perms, sid_t * secureSid);
extern OALMessage CMT_ServerInitiatedShutdown_Create(void *recipient, OALServer oalSrv);
extern OALMessage CMT_ProcessReceiveSilence_Create(void *recipient);
extern OALMessage CMT_PeerListenerEvent_Create( void *recipient, DOFAddress addr, DOFSecurityScope scope, boolean added);
extern OALMessage CMT_ConnectionStreamRequest_Create( void *recipient, DOFAddress addr, boolean complete );
extern OALMessage CMT_SetDomainCredentials_Create(void *recipient, DOFCredentials creds, void *context);
extern OALMessage CMT_CreateBridge_Create(void *recipient, DOFOperationBridgeConfig bridge, DOF dof);
extern OALMessage CMT_CreateBridgeComplete_Create(void *recipient, OALOperationBridge bridge);

void SMT_BeginNegotiationAuthorization_Dtor( OALMessage msg );
extern OALMessage SMT_RouteToProvider_Convert( OALMessage message, void * recipient, OALChannel associatedChannel, DOFAddress address );
extern OALMessage SMT_RouteToRequestor_Convert( OALMessage message, void * recipient, DOFAddress address, OALChannel associatedChannel );
extern OALMessage SMT_Shutdown_Create( void* recipient );
extern void SMT_Shutdown_SetException(OALMessage msg, DOFException exception);

extern OALMessage SMT_Connected_Negotiation_Timeout_Create(void *recipient);
extern OALMessage SMT_Connected_Negotiation_Server_Side_Create(void *recipient);
extern OALMessage SMT_RemoteDomainCallback_Create(void *recipient, DOFCredentials creds);
extern OALMessage SMT_ResolveCredsCallback_Create(void *recipient, DOFCredentials creds);
extern OALMessage SMT_GetLibraryPermsCallback_Create(void *recipient, DOFPermissionSet perms, sid_t * secureSid);
extern OALMessage SMT_RouteSavedEPP_Create(void *recipient, BlackboardData data);


#ifdef  __cplusplus
}
#endif

#endif /* DOF_CHANNEL_STACK_MESSAGES_H */
