/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_CONNECTIONMANAGER_H
#define DOF_CONNECTIONMANAGER_H

#include <dof/core.h>
#include <dof/address.h>
#include <dof/connection.h>

#include <internal/channel.h>
#include <internal/connection_internal.h>

#ifdef  __cplusplus
extern "C" {
#endif

typedef struct connLstnNode_t * connLstnNode;
typedef struct connPeerLstnNode_t * connPeerLstnNode;

extern boolean OALConnectionMgr_Init(void);
extern void    OALConnectionMgr_Shutdown(void);

/* The connection mutex must be locked when this is called. */
/* OALConn must not be NULL. */
void OALConnection_DestroyStreamRequestTree(OALConnection OALConn);

/* The connection mutex must be locked when this is called. */
/* OALConn must not be NULL. */
void OALConnection_EmptyStreamRequestTree(OALConnection OALConn);

extern void    OALConnectionMgr_CloseInboundConnections(void);

extern void OALConnectionMgr_FindOALConnection(OALConnection_findContext_t * findCtx);
extern boolean OALConnectionMgr_AddOALConnection(OALConnection OALConn);
extern void OALConnectionMgr_RemoveOALConnection(OALConnection OALConn);
extern uint32 OALConnectionMgr_GetConnectionCount(void);

/* Note: this function will take ownership of the cbNode member of the pendingConnection. */
extern void OALConnectionMgr_enqueueConnectComplete(pendingConnection_t *pendingConnection, DOFException except);

extern DOFError OALConnectionMgr_NotifySystem_ListenerRemoved(DOFConnection connection, connLstnNode_t *listenNode);
extern DOFError OALConnectionMgr_NotifySystem_PeerListenerRemoved(DOFConnection connection, connPeerLstnNode_t *listenNode);
extern DOFError OALConnectionMgr_NotifySystem_Destroyed(DOFConnection connection);
extern void OALConnectionMgr_NotifySystem_ConnectionEvent(DOFConnection connection, DOFConnectionState state,
                                                          DOFConnectionStateListener newListener);
extern void OALConnectionMgr_NotifySystem_PeerListenerEvent(DOFConnection connection, DOFAddress address, DOFSecurityScope scope, boolean added);
extern boolean OALConnectionMgr_NotifySystem_StreamListenerEvent(DOFConnectionStreamRequest request, boolean complete);
extern boolean OALConnectionMgr_NotifySystem_StreamListenerRemoved(DOFConnectionSRL srl, DOFException except);

typedef boolean (*connection_manager_traverse_fn_t)(OALConnection connection, void *context);
extern void OALConnectionMgr_TraverseAllConnections(connection_manager_traverse_fn_t fn, void *context);

#ifdef  __cplusplus
}
#endif

#endif /* DOF_CONNECTIONMANAGER_H */


