/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_DOMAIN_INTERNAL_H
#define DOF_DOMAIN_INTERNAL_H

#include <dof/core.h>
#include <dof/domain.h>
#include <dof/authenticator.h>
#include <internal/avl_tree.h>
#include <internal/timer.h>

#ifdef  __cplusplus
extern "C" {
#endif

/* Default Max Silence Time */
#define DOFDOMAIN_HOURSILENCE 3600000

typedef struct DOFDomain_t
{
    OALEnqueueItem_t    base;       /* DOFDomains are enqueueable so we can always call removed */
    DOFCredentials      creds;      /* Creds basically for the domain */
    OALAVLTree          listeners;
    OALTimerItem        timerItem;
    DOF                 dof;
    uint32              maxSilence; 
    uint32              refCount;
    struct
    {
        unsigned int isConnected    : 1;
        unsigned int validating1    : 1;
        unsigned int validating2    : 1;
        unsigned int validating3    : 1;
    } flags;
}DOFDomain_t;

typedef struct DOFDomainConfig_t
{
    DOFCredentials  creds;
    uint32          maxSilence;
    uint32          refCount;
}DOFDomainConfig_t;

typedef struct DOFDomainConfigBuilder_t
{
    DOFDomainConfig_t base;
    boolean           error;
}DOFDomainConfigBuilder_t;

typedef struct DOFDomainState_t
{
    DOFCredentials creds; /* this must be neutered credentials */
    uint32         refCount;
    uint32         maxSilence;
    boolean        isConnected;
}DOFDomainState_t;

extern boolean DOFDomain_Init1( void );
extern void DOFDomain_Shutdown1( void );
extern boolean DOFDomain_Init2( void );
extern void DOFDomain_Shutdown2( void );

extern void DOFDomain_Lock(void);
extern void DOFDomain_Unlock(void);

extern boolean DOFDomain_IsEqual( const DOFDomain self, const DOFDomain other );

extern DOFDomain DOFDomain_Copy( DOFDomain self );

extern DOFCredentials DOFDomain_GetCredentials( DOFDomain self );
extern DOFDomain DOFDomain_CreateInternal( DOFDomainConfig config );
extern DOFDomainState DOFDomainState_Create( DOFDomain domain );
extern void DOFDomain_SetUpSilenceTimer( DOFDomain self );
extern void DOFDomain_UpdateSilenceTimer( DOFDomain self, uint32 silence );
extern void DOFDomain_NotifyRemovedDOF( DOFDomain self );
extern void DOFDomain_RemoveDOF( DOFDomain self );
extern DOFDomainConfig DOFDomainConfig_Copy( DOFDomainConfig self );
extern DOFDomainState DOFDomainState_Copy( DOFDomainState self );

/* DOFAuthenticator */
extern DOFAuthenticator DOFAuthenticator_Copy( const DOFAuthenticator self );
extern DOFAuthenticator DOFAuthenticator_CreateInternal( DOF dof, DOFAuthenticatorConfig config );
/* this is dofs destroy of the Authenticator this will clear the storage */
extern void DOFAuthenticator_DOFTriggeredDestroy( DOFAuthenticator self );
extern DOFAuthenticatorStorage DOFAuthenticator_GetStorage(const DOFAuthenticator self);
extern uint8 DOFAuthenticator_GetID(const DOFAuthenticator self);
extern uint8 DOFAuthenticator_GetMaxLifetime(const DOFAuthenticator self);
extern boolean DOFAuthenticator_IsDebug( DOFAuthenticator self );
extern uint8* DOFAuthenticator_GetDebugKey( DOFAuthenticator self );
extern DOFObjectIDDomain DOFAuthenticator_GetDomainID( const DOFAuthenticator self );

#ifdef  __cplusplus
}
#endif

#endif /* DOF_DOMAIN_INTERNAL_H */
