/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef _DOF_ENP_V1_H
#define _DOF_ENP_V1_H

#include <dof/core.h>
#include <internal/stream.h>
#include <internal/packet.h>
#include <internal/protocollayerresult.h>
#include <internal/blackboarddata.h>

#ifdef  __cplusplus
extern "C" {
#endif

/** @internal @addtogroup Protocols
@{
**/

/* @internal @addtogroup grp_enp_v1 ENP
@{
**/

#define ENP_V1_VERSION  0x01

#define ENP_INVALID_PORT    0xFFFFFFFF
#define ENP_MAX_PORT        0x003FFFFF

/*
**  "Public" Type and Method Declarations
*/

typedef struct ENP_V1_t * ENP_V1;

/* "Static" methods */

extern ENP_V1  ENP_V1_Create(BlackboardData blackboardData, boolean lengthInferencePermitted, boolean copyStreamOnReceive);
extern void    ENP_V1_Destroy(ENP_V1 enp);

/* "Object" methods */

/**
 *
 * @param self  Must not be NULL.
 *
 * @return ProtocolLayerResult
 */
extern ProtocolLayerResult ENP_V1_Receive(ENP_V1 self);

extern ProtocolLayerResult ENP_V1_Send(ENP_V1 self);


/**
@}
**/

#ifdef  __cplusplus
}
#endif

#endif /* _DOF_ENP_V1_H */


