/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef _DOF_EPP_NEGATIVE_ONE_H
#define _DOF_EPP_NEGATIVE_ONE_H

#include <dof/core.h>
#include <internal/protocollayerresult.h>
#include <internal/stream.h>
#include <internal/packet.h>
#include <internal/blackboarddata.h>
#include <internal/queue.h>

#ifdef  __cplusplus
extern "C" {
#endif

#define EPP_NEGONE_ID 0x7fff

#define EPP_NEGONE_MAX_PING_DELAY 10000

#define EPP_NEGONE_NO_SECURE_MARKING_ID     0xffffffff

typedef struct EPP_NegOne_t * EPP_NegOne;

extern void        EPP_NegOne_Mutex_Init(void);
extern boolean     EPP_NegOne_Init(void);
extern void        EPP_NegOne_Mutex_Shutdown(void);
extern void        EPP_NegOne_Shutdown(void);

extern boolean     EPP_NegOne_UpdateOperationSid(DOFOperation op, sid_t * newSid);
extern EPP_NegOne  EPP_NegOne_Create(BlackboardData blackboardData, boolean isConnectionless);
extern void        EPP_NegOne_Destroy(EPP_NegOne epp);

extern ProtocolLayerResult EPP_NegOne_Receive(EPP_NegOne self);
extern ProtocolLayerResult EPP_NegOne_Send(EPP_NegOne self, DOFOperation op);

extern boolean EPP_NegOne_ProcessNodeDown( DOFObjectID oid, OALChannel channel, DOFAddress addr, boolean notifyChannelDown );
extern int32 EPP_NegOne_AddCommandOperation(DOFOperation op, OALChannel channel); /* returns 0 on success */
extern void EPP_NegOne_NotifyChannelDown(OALChannel channel, DOFAddress address);
extern ProtocolLayerResult EPP_NegOne_ReparentOperation(DOFOperation op, OALChannel new_channel, DOFAddress new_address, boolean isConnectionless);

extern void EPP_NegOne_GoingDown(void);
extern void EPP_NegOne_NodeDown(OALChannel channel);
extern void EPP_NegOne_CancelAll(void);
extern void EPP_NegOne_Heartbeat(OALChannel channel, DOFAddress address, boolean denyPacketReplayProtectionIfSecureUnicastSend);
extern void EPP_NegOne_Ping(OALChannel channel, DOFAddress address, uint32 secureMarkingID);
extern boolean EPP_NegOne_Query(OALChannel channel, DOFAddress address, uint32 timeout, uint32 retry, void * context, EPPNegOneQueryCallback callbackFn);
extern boolean EPP_NegOne_SourceLost(DOFOperation op);

#ifdef  __cplusplus
}
#endif

#endif /* _DOF_EPP_NEGATIVE_ONE_H */
