/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef _INTERNAL_ERROR_H
#define _INTERNAL_ERROR_H

#include <dof/core.h>

#ifdef  __cplusplus
extern "C" {
#endif

/** @addtogroup DOFError DOFError
@{
**/


/* For internal use only, but can properly pass to the application because DOFERROR_FAILED or DOFERROR_NOT_FOUND or
   DOFERROR_SERVICE_UNAVAILABLE are in the public API.*/
#define DOFERROR_PARSE_ERROR                         DOFERROR_FAILED
#define DOFERROR_REQUEST_FAILED                      DOFERROR_FAILED
#define DOFERROR_RECEIVED_NODE_IDENTIFIER_NOT_US     DOFERROR_FAILED
#define DOFERROR_NON_SGM_SENT_NEXTNODEID             DOFERROR_FAILED
#define DOFERROR_REJECT_SLOT                         DOFERROR_FAILED
#define DOFERROR_REJECT_INCOMING_PACKET_NUMBER       DOFERROR_FAILED
#define DOFERROR_SPECIFIED_DECRYPT_KEY_NOT_IN_HAND   DOFERROR_FAILED
#define DOFERROR_TARGETNID_FOUND_BUT_NOT_SECUREGROUP DOFERROR_FAILED
#define DOFERROR_KEY_NOT_SET                         DOFERROR_FAILED
#define DOFERROR_REJECT_OUTGOING_PACKET_NUMBER       DOFERROR_FAILED
#define DOFERROR_NODEID_NOT_VALIDATED                DOFERROR_FAILED
#define DOFERROR_REQUIRED_TARGETNID_NOT_KNOWN        DOFERROR_FAILED
#define DOFERROR_ENCIPHER_UNMARSHAL_ERROR            DOFERROR_FAILED
#define DOFERROR_INTERNAL_ERROR                      DOFERROR_FAILED
#define DOFERROR_MISSING_SENDER_ADDRESS              DOFERROR_FAILED
#define DOFERROR_MISSING_TICKET                      DOFERROR_FAILED
#define DOFERROR_NO_AUTHENTICATION_SERVER            DOFERROR_NOT_FOUND
#define DOFERROR_SGM_CANNOT_GET_GROUP_KEY            DOFERROR_NOT_FOUND
#define DOFERROR_GROUP_NOT_AVAILABLE                 DOFERROR_SERVICE_UNAVAILABLE


/* NOTE: Any assignments below must be in the range of 0xFFFF0000 through 0xFFFFFFFF. These are reserved in the public
   API error codes of include/dof/error.h. */


/* For internal use only, but not for passing to any application- these are not in the public API. */
#define DOFERROR_DECIPHER_UNMARSHAL_ERROR 0xFFFF0000/*For coding convenience, gets overwritten to DOFERROR_PARSE_ERROR*/
#define DOFERROR_OK_NODEID_COLLISION      0xFFFF0001 /* Allow EPP to detect a condition that it must react to. */
#define DOFERROR_DECRYPTION_FAILED        0xFFFF0002 /* We would like to distinguish this from DOFERROR_FAILED. */
/* These next two are for internal TRP use, need be distinguished from each other, and are not convenient to replace. */
#define DOFERROR_OPERATION_IGNORED        0xFFFF0003/*Callback processed rq but DOFSystem needn't respond to requestor*/
#define DOFERROR_INVALID_REQUEST_STRUCT   0xFFFF0004 /* The Request structure was found to be NULL. */


/**
@}
**/

#ifdef  __cplusplus
}
#endif

#endif /* _INTERNAL_ERROR_H */
