/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef _DOF_ESP_V2_H
#define _DOF_ESP_V2_H

#include <dof/core.h>
#include <internal/protocollayerresult.h>
#include <internal/esp_v2_options.h>
#include <internal/packet.h>
#include <internal/stream.h>
#include <internal/blackboarddata.h>

#ifdef  __cplusplus
extern "C" {
#endif

/* @addtogroup ccp_v1
@{
**/

/*
**  "Public" Type and Method Declarations
*/

/* CPP packet opcodes */
/* In header because test needs it */
#define LCP_CONFIGURE_REQUEST   1
#define LCP_CONFIGURE_ACK       2
#define LCP_CONFIGURE_NAK       3
#define LCP_CONFIGURE_REJECT    4
#define LCP_TERMINATE_REQUEST   5


/* Type of payload in ESP packets */
/* In header because test needs it */
#define ESP_ATTR_TYPE_TEP         0
#define ESP_ATTR_TYPE_OAP         1
#define ESP_ATTR_TYPE_CCM         2
#define ESP_ATTR_TYPE_TRP         3
#define ESP_ATTR_TYPE_Gen       255


#define ESP_PROTOCOLID_CONT     ((uint8)0x80)
#define ESP_PROTOCOLID          0

/* ***** Config Handler Stuff ***** */
#define ESP_V2_CFGHANDLER_RESPONSE_ACK      0x00
#define ESP_V2_CFGHANDLER_RESPONSE_NEGACK   0x01
#define ESP_V2_CFGHANDLER_RESPONSE_REJECT   0x02
#define ESP_V2_CFGHANDLER_RESPONSE_REQUEST  0x03
#define ESP_V2_CFGHANDLER_RESPONSE_CLOSE    0x04

#define ESP_V2_NEGOTIATION_TIMEOUT      30000   /* 30 second timeout for negotiation */

typedef uint8   ESP_V2_CfgHandlerResponseCode;

typedef struct ESP_V2_CfgHandlerResponse_t
{
    ESP_V2_CfgHandlerResponseCode       responseCode;
    ConfigOptionList                    optList;
} ESP_V2_CfgHandlerResponse_t;


typedef ESP_V2_CfgHandlerResponse_t   (*ConfigRequestHandler_t)   (void* protocolStack, ConfigOptionList cfgRequestOpts);
typedef ESP_V2_CfgHandlerResponse_t   (*ConfigNegAckHandler_t)    (void* protocolStack, ConfigOptionList cfgNegAckOpts);
typedef ESP_V2_CfgHandlerResponse_t   (*ConfigRejectHandler_t)    (void* protocolStack, ConfigOptionList cfgRejectOpts);

/* ******************************************************************************************************************/


typedef struct ESP_V2_t * ESP_V2;

extern ESP_V2 ESP_V2_Create(BlackboardData blackboardData, void* protocolStack, ConfigRequestHandler_t reqHandler, ConfigNegAckHandler_t negAckHandler, ConfigRejectHandler_t rejectHandler);

extern void ESP_V2_Destroy(ESP_V2 ccp);

extern ProtocolLayerResult ESP_V2_SendInitialNegotiation(ESP_V2 self, ConfigOptionList desiredOpts);
extern ProtocolLayerResult ESP_V2_TCP_Receive(ESP_V2 self);


extern ProtocolLayerResult ESP_V2_Receive(BlackboardData blackboardData);

extern ProtocolLayerResult ESP_V2_Send(BlackboardData blackboardData, DOFOperation op);

extern boolean ESP_V2_SendQuery(OALChannel channel, uint32 timeout, uint32 retry);
extern boolean ESP_V2_SendTerminate(OALChannel channel);
extern boolean ESP_V2_SendOpen(OALChannel channel);

/**
@}
**/

#ifdef  __cplusplus
}
#endif

#endif /* _DOF_ESP_V2_H */


