/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef _DOF_ESP_V2_OPTIONS_H
#define _DOF_ESP_V2_OPTIONS_H

#include <dof/core.h>
#include <internal/stream.h>
#include <internal/pointerlist.h>

#ifdef  __cplusplus
extern "C" {
#endif

/* @addtogroup ccp_v1_options
@{
**/

/*
**  "Public" Type and Method Declarations
*/

/* This struct is not intended to be hidden from it's users. */
typedef struct ESPV2_ConfigOption_t
{
    uint8   refCount;
    uint8   attributeCode;
    uint16  attributeData;     /* if you change this struct make sure to update the offsetof call in the create function. */
    uint8   valueLength;
    uint8   valueData[1];
} ESPV2_ConfigOption_t;

typedef struct ESPV2_Reject_t
{
    uint8   refCount;
    uint8   attributeCode;
    uint16  attributeDataCnt;
    uint16  attributeData[1];
} ESPV2_Reject_t;

typedef struct ESPV2_ConfigOption_t* ESPV2_ConfigOption;
typedef struct ESPV2_Reject_t* ESPV2_Reject;

typedef OALPtrList ConfigOptionList;
typedef OALPtrList RejectList;

extern ESPV2_ConfigOption ESPV2_ConfigOption_Create(uint8 attributeCode, uint16 attributeData, uint8 valueLength);
extern ESPV2_ConfigOption ESPV2_ConfigOption_CreateSec(DOFSecurityDesire desire);
extern ESPV2_ConfigOption ESPV2_ConfigOption_CreateFromMode(DOFSecurityMode mode);

extern DOFSecurityMode ESPV2_ConfigOption_SecModeFromConfigOption(ESPV2_ConfigOption self, DOFSecurityDesire desire, boolean* nak);

extern ESPV2_ConfigOption ESPV2_ConfigOption_Unmarshall(OALStream stream);
extern ConfigOptionList ESPV2_ConfigOption_Unmarshall_List(OALStream stream);

extern boolean ESPV2_ConfigOption_Marshall(ESPV2_ConfigOption cfgOpt, OALStream stream);
extern OALStream ESPV2_ConfigOption_Marshall_List(ConfigOptionList cfgOptList);

extern void ESPV2_ConfigOption_Destroy(ESPV2_ConfigOption configOpt);
extern ESPV2_ConfigOption ESPV2_ConfigOption_Retain(ESPV2_ConfigOption self);

extern void ESPV2_ConfigOptionList_Destroy(ConfigOptionList);
extern ConfigOptionList ESPV2_ConfigOptionList_Copy(ConfigOptionList optList);


extern ESPV2_Reject ESPV2_Reject_Create( uint8 attributeCode );
extern ESPV2_Reject ESPV2_TEPReject_Create( void );
extern ESPV2_Reject ESPV2_OAPReject_Create( void );
extern ESPV2_Reject ESPV2_CCMReject_Create( void );
extern ESPV2_Reject ESPV2_TRPReject_Create( void );
extern ESPV2_Reject ESPV2_GenReject_Create( void );

extern ESPV2_Reject ESPV2_Reject_Unmarshall(OALStream stream);
extern RejectList ESPV2_Reject_Unmarshall_List(OALStream stream);

extern boolean ESPV2_Reject_Marshall(ESPV2_Reject reject, OALStream stream);
extern OALStream ESPV2_Reject_Marshall_List(RejectList rejectList);

extern void ESPV2_Reject_Destroy( ESPV2_Reject self );
extern void ESPV2_RejectList_Destroy(RejectList optList);

extern boolean ESPV2_CheckRejectOrNakIfNakReturnASecurityMode(OALStream stream, DOFSecurityDesire desire, DOFSecurityMode* secMode, boolean * streamError);
extern boolean ESPV2_MarshalCCMNak(OALStream stream, DOFSecurityDesire desire);        /* Size is 8 */
extern boolean ESPV2_MarshalCCMReject(OALStream stream);                               /* Size is 1 */

/**
@}
**/

#ifdef  __cplusplus
}
#endif

#endif /* _DOF_ESP_V2_OPTIONS_H */
