/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef _INTERNAL_EXCEPTION_H
#define _INTERNAL_EXCEPTION_H


#include <dof/core.h>
#include <internal/stream.h>


#ifdef  __cplusplus
extern "C" {
#endif

/** @defgroup DOFException DOFException
    @ingroup DOF_Framework

    DOFException encapsulates DOFError codes and provider item exceptions.

@{
**/

extern DOFException DOFEXCEPTION_NOT_SUPPORTED;
extern DOFException DOFEXCEPTION_TYPE_MISMATCH;
extern DOFException DOFEXCEPTION_INSUFFICIENT_RESOURCES;
extern DOFException DOFEXCEPTION_TERMINATED;
extern DOFException DOFEXCEPTION_REJECTED;
extern DOFException DOFEXCEPTION_NOT_FOUND;
extern DOFException DOFEXCEPTION_DUPLICATE;
extern DOFException DOFEXCEPTION_TOO_MANY;
extern DOFException DOFEXCEPTION_APPLICATION_ERROR;

extern DOFException DOFEXCEPTION_FAILED;
extern DOFException DOFEXCEPTION_TIMEOUT;
extern DOFException DOFEXCEPTION_NOT_READABLE;
extern DOFException DOFEXCEPTION_NOT_WRITABLE;
extern DOFException DOFEXCEPTION_RESULT_LOST;
extern DOFException DOFEXCEPTION_PROTOCOL_NEGOTIATION_FAILED;
extern DOFException DOFEXCEPTION_TRANSPORT_FAILED;
extern DOFException DOFEXCEPTION_IO_FAILED;
extern DOFException DOFEXCEPTION_AUTHENTICATION_FAILED;
extern DOFException DOFEXCEPTION_ACK_TIMEOUT;
extern DOFException DOFEXCEPTION_STALE_CREDENTIAL;

extern DOFException DOFEXCEPTION_ACCESS_DENIED;
extern DOFException DOFEXCEPTION_UNKNOWN_INITIATOR;
extern DOFException DOFEXCEPTION_UNKNOWN_RESPONDER;
extern DOFException DOFEXCEPTION_UNKNOWN_DOMAIN;
extern DOFException DOFEXCEPTION_SERVICE_UNAVAILABLE;
extern DOFException DOFEXCEPTION_BAD_MODE;
extern DOFException DOFEXCEPTION_NOT_COMPATIBLE;
extern DOFException DOFEXCEPTION_NOT_IN_GROUP;
extern DOFException DOFEXCEPTION_UNSUPPORTED_INITIATOR;
extern DOFException DOFEXCEPTION_UNSUPPORTED_RESPONDER;
extern DOFException DOFEXCEPTION_INVALID_INITIATOR;
extern DOFException DOFEXCEPTION_INVALID_RESPONDER;
extern DOFException DOFEXCEPTION_RESOLUTION_FAILED;
extern DOFException DOFEXCEPTION_VALIDATION_FAILED;

extern DOFException DOFEXCEPTION_ILLEGAL_ARGUMENT;


/*
**  Internal Type and Method Declarations
*/

extern void DOFException_Init( void );
extern void DOFException_Shutdown( void );
extern DOFException DOFErrorException_GetFromError(DOFError error);
extern DOFException DOFErrorException_Create_ProviderInfo( DOFError error, const char * errorString, DOFProviderInfo info );
extern DOFException DOFErrorException_Create_Data( DOFError error, const char * errorString, const void * data );
extern DOFException DOFProviderException_Create_ProviderInfo( DOFInterfaceException except, uint8 parameterCount, const DOFValue * parameters, DOFProviderInfo info );
extern DOFException DOFProviderException_CreateStream_ProviderInfo( OALStream valueStream, DOFProviderInfo info, DOFItemID itemId );
extern DOFException DOFException_Copy_ProviderInfo( DOFException self, DOFProviderInfo info );

boolean DOFException_IsStream(DOFException e);
OALStream DOFException_GetStream(DOFException e);
DOFItemID DOFStreamException_GetItemId(DOFException e);

/**
@}
**/


#ifdef  __cplusplus
}
#endif


#endif /* _INTERNAL_TIME_H */
