/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef _DOF_INTERESTINTERNAL_H
#define _DOF_INTERESTINTERNAL_H

#include <dof/core.h>
#include <internal/oaltypesinternal.h>

#ifdef  __cplusplus
extern "C" {
#endif

typedef struct bindingList
{
    uint32              nBindings;
    struct binding_t*   pBindings;
} bindingList;

typedef struct binding_t
{
    DOFObjectID    oid;
    DOFInterfaceID iid;
} binding_t;


/* Note that the flags byte of DOFQuery_t uses the lower couple of bits for other purposes, so we use upper two bits. */
#define FILTER_FLAGS_NO_CATCH_ALL           0
#define FILTER_FLAGS_CATCH_ALL_OBJECTS      0x40
#define FILTER_FLAGS_CATCH_ALL_INTERFACES   0x80
#define FILTER_FLAGS_CATCH_ALL              (FILTER_FLAGS_CATCH_ALL_OBJECTS | FILTER_FLAGS_CATCH_ALL_INTERFACES)


typedef struct interestBlock_t
{
    struct interestBlock_t  *next; /* points to next interestBlock_t or NULL if its the end */
    DOFFilterID             blockID; /* for easy removal */
    uint32                  oidCnt; /* number of oids [0...*] */
    uint32                  iidCnt; /* number of iids [0...*] */
    DOFObjectID*            pOIDs;
    DOFInterfaceID*         pIIDs;
    uint8                   flags;  /* upper 2 bits for _CATCH_ALL info, lower 2 bits for match style. */
}interestBlock_t;

typedef struct DOFQuery_t
{
    DOFFilterID          blockCnt; /* number of blocks */
    interestBlock_t      *pInterestBlocks; /* linked list of blocks */
    uint32               totalBinds; /* keeps the number of bindings that this interest holds see GetBindings*/
    struct DOFInterfaceIDList_t restrictionIIDs;
    uint8                flags;
}DOFQuery_t;

typedef struct DOFQueryBuilder_t
{
    DOFFilterID nextBlock;
    struct DOFQuery_t q;
}DOFQueryBuilder_t;

boolean DOFQuery_IsCatchAll(DOFQuery  interest);
uint32 DOFQuery_GetBlockCount(DOFQuery  interest);

DOFInterfaceIDList DOFQuery_GetRestrictIIDs(DOFQuery  filter);
boolean DOFQuery_IsRestricted(DOFQuery  filter, DOFInterfaceID iid);
boolean DOFQuery_QueryMatch(DOFQuery  filter, DOFObjectID obj, DOFSecurityScope securityScope);
DOFQuery  DOFQuery_DeepCopy(DOFQuery  filter);

bindingList* DOFQuery_GetBindings(DOFQuery  filter);

void bindingList_Destroy(bindingList* bindings);

#ifdef  __cplusplus
}
#endif

#endif

