/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_KEY_REQUEST_H_
#define DOF_KEY_REQUEST_H_

#include <dof/oaltypes.h>
#include <internal/stream.h>

#define DEFAULT_NONCE_SIZE          8
#define MAX_KEYREQUEST_NONCE_SIZE   16

/* DOFKeyRequest is in the body of structures. do not free or allocate memory for it in its ctor/dtor */
typedef struct DOFKeyRequest_t
{
    uint32 refCount;
    DOFCredentials credentials;
    DOFPermissionSet permission_set;

    uint8 nonce[MAX_KEYREQUEST_NONCE_SIZE];

    uint8 nonceLength;
    boolean newLogin;
    boolean fixedPermissions;
} DOFKeyRequest_t;

typedef struct DOFKeyRequest_t* DOFKeyRequest;


DOFKeyRequest DOFKeyRequest_Create( DOFCredentials credentials, uint8 nonce_size, boolean new_login, boolean fixedPermissions, const uint8 *nonce, DOFPermissionSet permission_set);

DOFKeyRequest DOFKeyRequest_Retain(DOFKeyRequest self);

void DOFKeyRequest_Destroy(DOFKeyRequest self);

/*  This function ONLY cleans up the internals of a DOFKeyRequest (does not free the memory).
 */
void DOFKeyRequest_Cleanup(DOFKeyRequest self);

boolean DOFKeyRequest_Marshal(OALStream     stream,
                              DOFKeyRequest self,
                              boolean       isRequest);
boolean DOFKeyRequest_Unmarshal(OALStream           stream,
                                DOFKeyRequest       self,
                                DOFObjectIDDomain   domain,
                                boolean             isRequest,
                                boolean             unknownAllowed);
uint32 DOFKeyRequest_MarshallByteLength(DOFKeyRequest self,
                                        boolean       isRequest);
boolean DOFKeyRequest_ArePermissionsFixed(DOFKeyRequest self);

#endif
