/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef _DOF_MESSAGING_H
#define _DOF_MESSAGING_H

#include <dof/core.h>

#include <internal/enqueue.h>

#ifdef  __cplusplus
extern "C" {
#endif

typedef struct OALMessage_t * OALMessage;

typedef void (*oalmsg_dtor_func_t)(OALMessage msg);

typedef struct OALMessage_t
{
    OALEnqueueItem_t    super;          /* OALMessage extends the Enqueue Item, This must be first. */

    void*               recipient;
    uint32              msg_type;
    oalmsg_dtor_func_t  dtor;
    uint32              refCount;
    uint16              argc;            /* Number of arguments for this item.  Must not be modified. */
    uint8               already;         /* already enqueued? */
    uint8               shutdown;        /* Channel resources are shutting down. */
    void                *argv[1];        /* Array of arguments for this item (space is allocated for the number of pointers specified by argc). */
} OALMessage_t;

extern void OALMessage_Init(OALMessage message, void *recipient, int32 msg_type, uint32 argc, oalmsg_dtor_func_t dtor);

extern OALMessage OALMessage_Create(uint32               argc,
                                    void               * recipient,
                                    int32                msg_type,
                                    oalmsg_dtor_func_t   dtor);

extern void OALMessage_Destroy(OALMessage message);
extern OALMessage OALMessage_Retain(OALMessage message);


#ifdef  __cplusplus
}
#endif

#endif /* _DOF_MESSAGING_H */
