/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef _DOF_OALTYPESINTERNAL_H
#define _DOF_OALTYPESINTERNAL_H

#include <dof/oaltypes.h>
#include <dof/pcrtypes.h>

#ifdef  __cplusplus
extern "C" {
#endif


/** @internal
    @addtogroup grp_OALTypes  OAL Types and Definitions
    Common internal types and definitions of the DOF C-Language Object Access Library.
@{
**/

#define DOF_SECURITY_KEY_LENGTH 32

#define DOFINTERESTLEVEL_CONNECT_FULL 3

typedef struct OALCipher_t *        OALCipher;
typedef struct OALChannel_t *       OALChannel;
typedef struct OALConnection_t *    OALConnection;

typedef uint32                        OALAlias;                   /**< @internal An alias to a full binding. **/
typedef uint32                        DOFOperationCount;          /**< @internal The operation count as used by the presentation layer. @ingroup DOFOperation **/
typedef uint32                        DOFFilterID;                /**< Refers to a single block of interest within a @ref DOFQuery . @ingroup DOFQuery  **/
typedef struct DOFSecureGroup_t *     DOFSecureGroup;
typedef struct DOFOperationID_t *     DOFOperationID;             /**< Uniquely identifies a DOFOperation. @ingroup DOFOperation **/
typedef const uint8 *                 DOFInterfaceDef;            /**< Original definition of a DOF Interface.  Typically generated by an external tool.  @ingroup DOFInterfaceDef **/
typedef struct DOFPartition_t *       DOFPartition;               /**< @internal An old-style Partition, representing a Security Domain and list of Security IDs.  Should be removed. @ingroup DOFPartition **/

typedef struct dtl_t* DTL;

typedef struct OALServer_t* OALServer;
typedef struct DOFServerManager_t* DOFServerManager;

typedef struct EPPNegOneQueryCallback_t * EPPNegOneQueryCallback;

typedef struct OALOperationBridge_t * OALOperationBridge;
typedef struct DOFSecurityMembrane_t * DOFSecurityMembrane;
typedef struct OALPermissionManager_t * OALPermissionManager;

typedef struct EPPNegOneQueryCallback_t
{
    const struct EPPNegOneQueryCallbackFns_t
    {
        void (* Complete) ( void * context, boolean success );

    } * fns;

    /* Implementations may extend this structure. */

} EPPNegOneQueryCallback_t;

/** @internal
    Callback for when the state of a Join changes (for example, when the Join succeeds or fails).

    @param sys    The DOFSystem for which the join state has changed.
    @param group  The group whose state has changed (an Object ID).
    @param status The new status of the Join activity.
**/
typedef void (* DOFJoinCallback) ( const DOFSystem sys, const DOFObjectID group, DOFError status );

/**
@}
**/

/* for request block and security */
typedef struct OALIdentity_t* OALIdentity;

#ifdef  __cplusplus
}
#endif

#endif /* _DOF_OALTYPESINTERNAL_H */

