/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_OBJECTIDINTERNAL_H
#define DOF_OBJECTIDINTERNAL_H


#include <dof/core.h>
#include <dof/objectid.h>
#include <internal/stream.h>


#ifdef  __cplusplus
extern "C" {
#endif


/* @addtogroup objectidinternal
@{
**/


/*
**  "Public" Type and Method Declarations
*/


#define DOFOBJECTID_MAX_CLASS_SIZE (4)


extern boolean                   DOFObjectID_Marshal                ( const DOFObjectID self, OALStream stream );
extern boolean                   DOFObjectID_IsWildcard             ( const DOFObjectID oid );
extern uint32                    DOFObjectID_GetClassSize           ( const DOFObjectID self );
extern boolean                   DOFObjectID_IsSourceID             ( const DOFObjectID id );
extern boolean                   DOFObjectID_IsASAssignedEqual      ( const DOFObjectID self, DOFObjectID other);


extern uint32                    DOFObjectID_Unmarshal_GetSidLength ( const uint8 * pBytes, uint32 max_length );
extern uint32                    DOFObjectID_GetClassSize_Bytes     ( const uint8 * pBytes );
extern DOFObjectIDSize           DOFObjectID_GetSize_Bytes          ( const uint8 * pBytes );
extern boolean                   DOFObjectID_MatchesAtLeast_Mcast   ( const DOFObjectID oid2, const DOFObjectID oid1 );
extern boolean                   DOFObjectIDAuthentication_IsEqual  ( const DOFObjectIDAuthentication oid1,
                                                                      const DOFObjectIDAuthentication oid2 );
extern DOFObjectID               DOFObjectID_Unmarshal              (       OALStream stream );
extern DOFObjectIDDomain         DOFObjectIDDomain_Unmarshal        (       OALStream stream );
extern DOFObjectIDAuthentication DOFObjectIDAuthentication_Unmarshal(       OALStream stream );
extern DOFObjectID               DOFObjectID_Create_Unmarshal       (       uint32 * length, const uint8 * buffer );
extern boolean                   DOFObjectID_Init                   ( void );
extern void                      DOFObjectID_Shutdown               ( void );


/**
@}
**/


#ifdef  __cplusplus
}
#endif


#endif /* DOF_OBJECTIDINTERNAL_H */
