/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef INTERNAL_PERMISSIONSCOPE_H
#define INTERNAL_PERMISSIONSCOPE_H

#include <internal/stream.h>

#ifdef  __cplusplus
extern "C" {
#endif

typedef struct OALPermissionScope_t * OALPermissionScope;

typedef struct OALPermissionScopeBuilder_t
{
    uint32 alc;          /* space Allocated */
    uint32 idCount;      /* cnt added */
    uint32 *ids;       /* must be last */
} *OALPermissionScopeBuilder;

typedef struct OALPermissionScopeList_t {
    uint32 permissionScopeCount;
    OALPermissionScope *permissionScopes;
} *OALPermissionScopeList;

extern OALPermissionScope OALPermissionScope_Create(uint32 idCount);
extern void OALPermissionScope_Destroy(OALPermissionScope scope);

extern OALPermissionScope OALPermissionScope_Copy(OALPermissionScope);

extern void OALPermissionScope_SetIDs(OALPermissionScope self, uint32 i_cnt, const uint32* ids, uint32 f_cnt, const uint32* flags, uint32 * realFlagsCount);
extern void OALPermissionScope_SetID(OALPermissionScope self, uint32* offset, uint32 id);

extern uint32 OALPermissionScope_CalculateMarshalSize(OALPermissionScope scope);

extern uint32 OALPermissionScope_GetIdCount(OALPermissionScope scope);
extern uint32 const * OALPermissionScope_GetIds(OALPermissionScope scope);
extern void OALPermissionScope_SetIdCount(OALPermissionScope scope,
                                          uint32             count);

extern boolean OALPermissionScope_IsMasking(OALPermissionScope self);

extern boolean OALPermissionScope_IsCompatible(OALPermissionScope self, OALPermissionScope scope);

extern boolean OALPermissionScope_Marshal(OALPermissionScope scope,
                                          OALStream          stream);

extern DOFError OALPermissionScope_Unmarshal(OALStream            stream,
                                             uint16               count,
                                             OALPermissionScope * permissionScopes,
                                             OALPermissionScope   connectionScope);

extern OALPermissionScope OALPermissionScope_GetCompatibleScope(OALPermissionScope self,
                                                                OALPermissionScope scope);

extern int32 OALPermissionScope_Compare(OALPermissionScope self, OALPermissionScope scope);

extern OALPermissionScopeBuilder OALPermissionScopeBuilder_Create( void );
extern OALPermissionScopeBuilder OALPermissionScopeBuilder_CreateCopy( OALPermissionScopeBuilder self );
extern boolean OALPermissionScopeBuilder_AddID( OALPermissionScopeBuilder self, uint32 ID );
extern boolean OALPermissionScopeBuilder_DoIntersection( OALPermissionScopeBuilder self, OALPermissionScopeBuilder other );
extern OALPermissionScope OALPermissionScopeBuilder_Build( OALPermissionScopeBuilder self );
extern OALPermissionScope OALPermissionScopeBuilder_BuildAndDestroy( OALPermissionScopeBuilder self );
extern void OALPermissionScopeBuilder_Destroy( OALPermissionScopeBuilder self );

extern boolean OALPermissionScope_Init(void);
extern void OALPermissionScope_Shutdown(void);

extern void OALPermissionScope_Lock(void);
extern void OALPermissionScope_Unlock(void);

extern DOFSecurityScope DOFSecurityScope_CreateCompatibleScope( DOFSecurityScope self, DOFSecurityScope other );
extern boolean DOFSecurityScope_Marshal(DOFSecurityScope self, OALStream stream);

#ifdef  __cplusplus
}
#endif

#endif
