/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_PERMISSIONSETINTERNAL_H
#define DOF_PERMISSIONSETINTERNAL_H

#include <dof/core.h>
#include <dof/authenticator.h>
#include <dof/permission.h>
#include <dof/permissionset.h>
#include <internal/avl_treelist.h>
#include <internal/stream.h>
#include <internal/permissionscope.h>

#ifdef  __cplusplus
extern "C" {
#endif

void DOFPermissionSet_Init(void);

void DOFPermissionSet_Shutdown(void);

DOFPermissionSet DOFPermissionSet_Create( void );

DOFPermissionSet DOFPermissionSet_CreateInternal( uint16 cnt, boolean builder );

DOFPermissionSet DOFPermissionSet_DeepCopy( DOFPermissionSet self );

boolean DOFPermissionSet_AddDeniedAccessPermission( DOFPermissionSet self, DOFPermission perm );

/**
 * Adds the permissions from another permission set
 *
 * @param self DOFPermissionSet to add permissions too
 * @param addFrom DOFPermissionSet that contains permissions to
 *                copy
 *
 * @return boolean TRUE means the permissions were copied
 *         successfully, FALSE means failure to copy at some
 *         point
 *
 * @note On failure the original the self DOFPermissionSet may
 *       be mutated from original.
 */
boolean DOFPermissionSet_AddPermissionsFromSet( DOFPermissionSet self, const DOFPermissionSet addFrom );

boolean DOFPermissionSet_AddTrustedPermission( DOFPermissionSet self, const DOFPermission perm );

OALPermissionScope DOFPermissionSet_Validate( DOFPermissionSet self, DOFPermission perm );

extern boolean DOFPermissionSet_Contains(DOFPermissionSet self,
                                         DOFPermission    perm);

OALPermissionScope DOFPermissionSet_ValidateDefine( DOFPermissionSet self, DOFInterfaceID iid );

OALPermissionScope DOFPermissionSet_ValidateRequestorProvider( DOFPermissionSet self, boolean provider );

OALPermissionScope DOFPermissionSet_ValidateBinding( DOFPermissionSet self, DOFPermissionBindingAction access, const DOFObjectID oid, DOFInterfaceID iid );

uint32 DOFPermissionSet_GetPermissionCount( DOFPermissionSet self );

/**
 * Creates a compliment DOFPermissionSet to the one passed in.
 *
 * @param self The DOFPermissionSet to create the complement
 *             DOFPermissionSet too.
 *
 * @return DOFPermissionSet The Complement DOFPermissionSet or
 *         NULL for failure.
 */
DOFPermissionSet DOFPermissionSet_CreateComplement(DOFPermissionSet self);

/* Start: These are only used in the AS validation  */
boolean DOFPermissionSet_StartBuildingValidation( DOFPermissionSet request );
void DOFPermissionSet_CleanupFailedBuildingValidation( DOFPermissionSet request );
boolean DOFPermissionSet_AddIDToValidation( DOFPermissionSet request, DOFAuthenticatorNodePermissionSetMap perm_map );
boolean DOFPermissionSet_AddALLScopesToValidation(DOFPermissionSet permission_set, DOFPermissionSet sids);
boolean DOFPermissionSet_FinishBuildingValidation( DOFPermissionSet request );
/* End: These are only used in the AS validation  */

uint32 DOFPermissionSet_MarshalValidationSize( DOFPermissionSet self );
boolean DOFPermissionSet_MarshalValidation( DOFPermissionSet self, OALStream stream );
boolean DOFPermissionSet_UnmarshalValidation( DOFPermissionSet self, OALStream stream, OALPermissionScope connectionScope );

uint32 DOFPermissionSet_MarshalRequestSize( DOFPermissionSet self );
boolean DOFPermissionSet_MarshalRequest( DOFPermissionSet self, OALStream stream );
DOFPermissionSet DOFPermissionSet_UnmarshalRequest( OALStream stream, boolean unknownAllowed);
boolean DOFPermissionSetBuilder_AddExactPermission( DOFPermissionSetBuilder self, DOFPermission perm );

/* IMPORTANT: do not add anything to the empty set. */
DOFPermissionSet DOFPermissionSet_GetEmptyPermissionSet(void);

/* IMPORTANT: Set that the permission set passed in is a grant permissions set.
   This was created to be used by TRP for group permissions please verify before using in any other case.
   New Case, the non-extendable permission case should also be treated as a permission grant. */
void DOFPermissionSet_SetGrantPerms( DOFPermissionSet self );

void DOFPermissionSet_ClearGrantPerms( DOFPermissionSet self );

#ifdef  __cplusplus
}
#endif

#endif
