/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_PROTOCOLSTACK_NEW_H
#define DOF_PROTOCOLSTACK_NEW_H

#include <dof/core.h>
#include <internal/messaging.h>
#include <internal/packet.h>
#include <internal/pointerlist.h>
#include <internal/channel.h>
#include <internal/blackboarddata.h>
#include <internal/stream.h>
#include <internal/esp_v2.h>
#include <internal/enqueue.h>
#include <internal/dofinternal.h>
#include <internal/operationinternal.h>

#ifdef  __cplusplus
extern "C" {
#endif

typedef struct OALProtocolStack_t * OALProtocolStack;

typedef void (*OALProtocolStack_Destroy_fn_t)( OALProtocolStack );
typedef void (*OALProtocolStack_MessageHandler_fn_t)( OALProtocolStack self, OALMessage message );
typedef ProtocolLayerResult (*OALProtocolStack_ENP_EPP_Send_fn_t)(OALProtocolStack self, BlackboardData);

typedef boolean (*KeyChangeCallback_fn_t)(void * context);

typedef struct OALProtocolStack_t
{
    OALProtocolStack_Destroy_fn_t           destroy_func;
    OALProtocolStack_MessageHandler_fn_t    msg_handler_func;
    OALProtocolStack_ENP_EPP_Send_fn_t      ENP_EPP_send_func;

    uint32 refCount;

    BlackboardData_t        blackboardData;     /*  This is a data structure shared between the stack and its layers. It contains information
                                                    which needs to be communicated but is not contained in the stream or packet. */
    OALMessage              shutdownMessage;
    OALTimerItem            negotiationTimerItem;

    boolean                 shutdownMessageSent;

    KeyChangeCallback_fn_t  keyChangeCallback;
    void *                  keyChangeCallbackContext;
} OALProtocolStack_t;

typedef enum ProtocolLayerIndicator_t
{
    PLI_OAP,
    PLI_TEP,
    PLI_TRP,
    PLI_SGMP,
    PLI_CCMMODE,
    PLI_ESP,
    PLI_EPP_NEGONE
} ProtocolLayerIndicator_t;

#define PROTOCOLID_ESP          0x0000
#define PROTOCOLID_ESP_V2       0x0000      /* Note: The version for ESP is tied to EPP, so EPP_V2 uses ESP_V2. */
#define PROTOCOLID_OAP_V1       0x0001
#define PROTOCOLID_EPP_NEGONE   0x7fff
#define PROTOCOLID_TEP_V1       0x0080
#define PROTOCOLID_TRP_V1       0x0081
#define PROTOCOLID_SGMP_V1      0x0082
#define PROTOCOLID_CCMMODE_V1   0x6001

#define PROTOCOLID_TRANSPORT    0x6000 /* used for sec neg*/

#define MAX_QUERY_DELAY         180000
#define QUERY_TYPE_ENP_V0       0
#define QUERY_TYPE_EPP_V0       1
#define QUERY_TYPE_ESP_V2       2


extern void OALProtocolStack_Init(void);
extern void OALProtocolStack_Cleanup(void);

extern OALProtocolStack OALProtocolStack_Retain( OALProtocolStack self );
extern void OALProtocolStack_Release( OALProtocolStack self );

extern boolean OALProtocolStack_CancelNegotiationTimer(OALProtocolStack self);
extern boolean OALProtocolStack_StartNetotiationTimeout(OALProtocolStack self, uint32 timeout);

extern void OALProtocolStack_HandleMessage( OALProtocolStack self, OALMessage message );

extern boolean OALProtocolStack_StartNegotiationTimeout(OALProtocolStack self, uint32 timeout);

extern void OALProtocolStack_Ctor( OALProtocolStack self,
                                   OALProtocolStack_Destroy_fn_t        destroy_func,
                                   OALProtocolStack_MessageHandler_fn_t msg_handler_func,
                                   OALProtocolStack_ENP_EPP_Send_fn_t enp_epp_send_func );

extern void OALProtocolStack_Dtor( OALProtocolStack self );

extern void OALProtocolStack_Shutdown( OALProtocolStack self, DOFException except );

extern ProtocolLayerIndicator_t OALProtocolStack_DetermineProtocol( DOFOperationType opType );

extern void OALProtocolStack_SetKeyChangeCallback( OALProtocolStack self, KeyChangeCallback_fn_t cb,void * context);

/* function used for multicast queries. */
extern boolean connectionlessV0Query_AddDelayedResponse(BlackboardData blackboardData);

#ifdef  __cplusplus
}
#endif

#endif /* DOF_PROTOCOLSTACK_NEW_H */
