/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_REMOTECHANNEL_H
#define DOF_REMOTECHANNEL_H

#include <dof/core.h>
#include <internal/channel.h>
#include <internal/cipher.h>
#include <internal/connectionmanager.h>
#include <internal/permissionsetinternal.h>

#ifdef  __cplusplus
extern "C" {
#endif

typedef struct KeyChangeInfo_t
{
    OALChannel  channel;
    OALCipher   cipher;
    DOFPermissionSet localPerms;
    DOFPermissionSet remotePerms;
    DOFSecurityMode  securityMode;
    DOFSecurityScope securityScope;
    uint32      stateId;
    boolean     replaceCipher;
    boolean     disableOld;
    boolean     promoteNew;
} KeyChangeInfo_t;

extern boolean OALRemoteChannel_MutexInit(void);
extern void OALRemoteChannel_MutexShutdown(void);

extern void OALRemoteChannel_Init(OALChannel self);
extern void OALRemoteChannel_NotifyInitComplete(OALChannel self);
extern void OALRemoteChannel_HandleMessage_InitializeComplete( OALChannel self, OALMessage message );

extern void OALRemoteChannel_ShutdownTransport(OALChannel self);

/* The channel must be valid when OALRemoteChannel_Shutdown is called, however it does not keep a refernece to the channel, since
   remoteChannelManager will be used to see if the channel is valid before the message is processed. */
extern void OALRemoteChannel_Shutdown(OALChannel self, DOFException exception);
extern OALMessage OALRemoteChannel_CreateShutdownMessage(OALChannel self);
extern void OALRemoteChannel_ConnectionInitiatedShutdown(OALChannel self, OALConnection OALConn);
extern void OALRemoteChannel_ServerInitiatedShutdown(OALChannel self, OALServer OALSrv);
extern void OALRemoteChannel_ProcessRecieveSilence(OALChannel self);
extern void OALRemoteChannel_APPIDQueryTimeout(OALChannel self);
extern void OALRemoteChannel_HandleMessage_RegisterSecureChannel(OALChannel channel, OALMessage message);
extern void OALRemoteChannel_HandleMessage_ProcessReceiveSilence(OALChannel channel, OALMessage message);
extern void OALRemoteChannel_RegisterSecureChannel(OALChannel self, uint32 SSID, boolean assignedFromAs);
extern void OALRemoteChannel_DeregisterSecureChannel(OALChannel self);
extern OALChannel OALRemoteChannel_GetSecureChannelByStateID_NonAs(OALChannel baseChannel, uint32 SSID);

typedef boolean (*remotechannel_ssid_traverse_fn_t)(OALChannel channel, void *context);

/* Note: this must only be called from the message enqueue worker thread. */
extern uint32 OALRemoteChannel_GetRegisteredChannelCountBySSID_As(OALChannel baseChannel, uint32 SSID);
extern void OALRemoteChannel_TraverseSecureChannelsBySSID_As(remotechannel_ssid_traverse_fn_t fn, OALChannel baseChannel,
                                                             uint32 SSID, void *context);

extern void OALRemoteChannel_RouteToProviderLossy(OALChannel self, DOFOperation req_op, DOFOperation prov_op, DOFAddress addr);
extern void OALRemoteChannel_RouteToProviderLossless(OALChannel self, DOFOperation req_op, DOFOperation prov_op, DOFAddress addr);
extern void OALRemoteChannel_RouteToRequestorLossy(OALChannel self, DOFOperation respOp, DOFResult result);
extern void OALRemoteChannel_RouteToRequestorLossless(OALChannel self, DOFOperation respOp, DOFResult result);

extern void OALRemoteChannel_ReceiveFromTransport(OALChannel self, OALStream data, DOFAddress rcvdFrom, boolean isMulticast, boolean isLossless );
extern void OALRemoteChannel_NotifyTransportDown(OALChannel self, DTL transport, DOFException exception);
extern void OALRemoteChannel_DestroyBlockedMessageQueue(OALChannel channel);
extern void OALRemoteChannel_ProcessBlockedMessages(OALChannel channel, boolean clearSendBlock);
extern void OALRemoteChannel_CreateBridge(OALChannel self, DOFAddress address, DOF dof, DOFOperationBridgeConfig bridgeConfig);

/* Note: this function must only be called from within a channel message_enqueue worker thread. */
extern OALChannel OALRemoteChannel_GetRegisteredChannel_ENPPort(OALChannel baseChannel, uint32 ENPPort);
extern boolean OALRemoteChannel_RegisterChannel_ENPPort(OALChannel channel);

typedef void (*createLibPermsCallback)(DOFError           status,
                                       sid_t            * secureSid,
                                       DOFPermissionSet   outPerms,
                                       void             * context);

extern void OALRemoteChannel_CreateLibPermissions(createLibPermsCallback   callback,
                                                  void                   * context,
                                                  DOFCredentials           credentials,
                                                  OALChannel               channel,
                                                  DOFAddress               address);

extern void OALRemoteChannel_HandleMessage_RegisterChannel_ENPPort(OALChannel channel, OALMessage message);

typedef void (*OALRemoteChannel_SetCredentials_Callback)(DOFError status, void* context);

extern void OALRemoteChannel_SetCredentialsCallback_ContextDtor(void *context);
extern void OALRemoteChannel_HandleMessage_SetDomainCredentialsCallback(OALChannel channel, OALMessage message);
extern void OALRemoteChannel_HandleMessage_PeerListenerEvent(OALChannel channel, OALMessage message);
extern void OALRemoteChannel_HandleMessage_ConnectionStreamListener(OALChannel channel, OALMessage message);
extern void OALRemoteChannel_SetCredentials(OALChannel channel, DOFObjectIDDomain domainID, uint32 timeout, OALRemoteChannel_SetCredentials_Callback fn, void* context);
extern DOFCredentials OALRemoteChannel_GetCredentials(RemoteChannelData self);
extern void OALRemoteChannel_PeerListenerEvent(OALChannel channel, DOFAddress address, DOFSecurityScope scope, boolean added);
extern void OALRemoteChannel_ConnectionStreamRequest(OALChannel channel, DOFAddress address, boolean complete);
extern KeyChangeInfo_t* OALRemoteChannel_CreateKeyChangeInfo( OALChannel channel, OALCipher cipher, uint32 stateId, DOFPermissionSet localPerms, DOFPermissionSet remotePerms, DOFSecurityMode securityMode, DOFSecurityScope securityScope, boolean disableOld, boolean promoteNew );
extern boolean OALRemoteChannel_KeyChangeCallback( void* ctx );

extern void OALRemoteChannel_RemoveOALConnection( OALChannel channel, DOFException exception );
extern void OALRemoteChannel_RemoveOALServer( OALChannel channel, DOFException exception );

extern DOFAddress              OALRemoteChannelConfig_GetAddress(OALChannel self);
extern DOFCredentials          OALRemoteChannelConfig_GetCredentials(OALChannel self);
extern DOFOperationBridgeConfig OALRemoteChannelConfig_GetBridgeConfig(OALChannel self);
extern DOFSecurityMode         OALRemoteChannelConfig_GetMode(OALChannel self);
extern DOFProtocolNegotiator   OALRemoteChannelConfig_GetNegotiator(OALChannel self);
extern DOFSecurityDesire       OALRemoteChannelConfig_GetSecurityDesire(OALChannel self);
extern DOFPermissionSet        OALRemoteChannelConfig_GetPermissions(OALChannel self);
extern uint32                  OALRemoteChannelConfig_GetMaxReceiveSilence(OALChannel self);
extern uint32                  OALRemoteChannelConfig_GetMaxSendSilence(OALChannel self);
extern DOFConnectionType       OALRemoteChannelConfig_GetConnectionType(OALChannel self);
extern DTLConnectionConfig     OALRemoteChannelConfig_GetDTLConnectionConfig(OALChannel self);
extern DOFServerType           OALRemoteChannelConfig_GetServerType(OALChannel self);
extern DTLServerConfig         OALRemoteChannelConfig_GetDTLServerConfig(OALChannel self);

extern void                    OALRemoteChannel_IncByteCount( OALChannel self, uint32 byteCount );

#ifdef  __cplusplus
}
#endif

#endif /* DOF_REMOTECHANNEL_H */
