/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_REMOTECHANNELDATA_H
#define DOF_REMOTECHANNELDATA_H

#include <dof/core.h>

#include <internal/avl_tree.h>
#include <internal/avl_treelist.h>
#include <internal/hashtable.h>
#include <internal/oaltypesinternal.h>
#include <internal/queue.h>
#include <internal/source.h>
#include <internal/timer.h>

#ifdef  __cplusplus
extern "C" {
#endif

#define V0_QUERY_ENP_BITMASK        0x00000001
#define V0_QUERY_BIT_ENP_V1         0x00000001
/* Modify the bitmask and enable the following as versions are added
#define V0_QUERY_BIT_ENP_V2         0x00000002
#define V0_QUERY_BIT_ENP_V3         0x00000004
#define V0_QUERY_BIT_ENP_V4         0x00000008
#define V0_QUERY_BIT_ENP_V5         0x00000010 */

#define V0_QUERY_EPP_BITMASK        0x00000001
/* Note: EPP V1 is not supported on the C OAL. */
#define V0_QUERY_BIT_EPP_V2         0x00000001
/* Modify the bitmask and enable the following as versions are added
#define V0_QUERY_BIT_EPP_V3         0x00000002
#define V0_QUERY_BIT_EPP_V4         0x00000004
#define V0_QUERY_BIT_EPP_V5         0x00000008
#define V0_Query_BIT_EPP_V6         0x00000010 */


#define ESP_V0_APPID_COUNT          5
#define ESP_V0_QUERY_BITMASK        0x0000001F
#define V0_QUERY_BIT_OAP_V1         0x00000001
#define V0_QUERY_BIT_TEP_V1         0x00000002
#define V0_QUERY_BIT_TRP_V1         0x00000004
#define V0_QUERY_BIT_SGMP_V1        0x00000008
#define V0_QUERY_BIT_CCMMODE_V1     0x00000010
/*  Modify the bitmask and add new AppIDs here.
    If more than 32 are required then a new bit field must be added to RemoteChanelData*/

#define V0_QUERY_SECURITYDESIRE_ANY_APPIDS  0x0000000E
#define V0_QUERY_UNSECURE_APPIDS    0x00000005

#define BLOCKED_MESSAGE_QUEUE_TIMEOUT 5000   /* 5 seconds. */

typedef struct ChannelConnectionKeyNode_t* ChannelConnectionKeyNode;
typedef struct ChannelConnectionKeyNode_t
{
    OALSListNode    next;
    void *          key; /* do not use as a connection */
}ChannelConnectionKeyNode_t;

typedef struct RemoteChannelData_t * RemoteChannelData;
typedef struct RemoteChannelData_t
{
    OALMessage              shutdownMessage;
    OALConnection           failedCIShutdown;    /* If a connection initiated shutdown fails, retain the OALConn here. */
    OALServer               failedSIShutdown;    /* If a server initiated shutdown fail, retain the OALServer here. */
    DOFSecurityMode         mode;
    DOFProtocolNegotiator   negotiator;
    uint32                  securityStateID;
    DOFAddress              address;
    union
    {
        DOFConnectionConfig connection;
        DOFServerConfig     server;
    }config;
    OALAVLTreeList          registeredSSIDs_AS;
    OALHashtable            registeredSSIDs_NonAs;
    OALAVLTree              registeredENPPorts;
    OALChannel              baseChannel;
    OALChannel              multicastChannel;
    OALServer               server;
    OALConnection           OALConn;
    DTL                     dtl; /* the server or connection */
    void *                  protocolStack;
    OALCipher               cipher;
    OALQueue                blockedMessageQueue;
    boolean                 blockedMessageQueueTimeoutEnabled;
    uint32                  blockedMessageQueueTimeoutExpirationTime;
    OALTimerItem            maxSendSilenceTimerItem;
    OALTimerItem            maxReceiveSilenceTimerItem;
    OALTimerItem            negotiationTimerItem;
    DOFCredentials          secureCredentials;
    sid_t                 * secureSid;
    DOFObjectIDSource       remoteSid;
    uint32                  ENP_V0_QueryVersions;       /* Used when a v0 query is in progress*/
    uint32                  EPP_V0_QueryVersions;       /* Used when a v0 query is in progress*/
    uint32                  ESP_V0_QueryAppids;         /* Used when a v0 query is in progress*/
    uint32                  ENP_Port;                   /* Used to identify a session, sent as sender port. */
    boolean                 waitingForData;             /* Used to determine whether we have timed out while waiting for an EPP Ping */
    struct
    {
        unsigned int    isSSIDAssignedFromAS        : 1;
        unsigned int    isInShutdownState           : 1;
        unsigned int    isUp                        : 1;
        unsigned int    ENP_V0_QueryInProgress      : 1;
        unsigned int    EPP_V0_QueryInProgress      : 1;
        unsigned int    ESP_V0_QueryInProgress      : 1;
        unsigned int    SecureSendBlocked           : 1;
        unsigned int    isConnection                : 1;
        unsigned int    isConnected                 : 1; /* Set only for isConnection, in _InitializeComplete_Notify()*/
        unsigned int    defaultAddressIsMulticast   : 1;
        unsigned int    tepRekeying                 : 1;
        unsigned int    secureDesire                : 1;
        unsigned int    sessionIsSecure             : 1;
        unsigned int    validateDomainCreds         : 1;
        unsigned int    TRP_AcceptsCommands         : 1;
        unsigned int    TRP_SendCommands            : 1;
        unsigned int    waitingForInitialDatagram   : 1;
        unsigned int    waitingForSecurityIndicator : 1;
        unsigned int    noSessionSecurity           : 1;
        unsigned int    mustTerminate               : 1;
        unsigned int    mustFinalizeTep             : 1;
        unsigned int    isOutbound                  : 1;
        unsigned int    lossySession                : 1; /* For secure, lossy sessions. Not always set for lossy channels in general. */
        unsigned int    setDtlInChannelInit         : 1;
        unsigned int    shutdownMessageSent         : 1;
        unsigned int    isLossy                     : 1; /* Always set if not streaming. */
        unsigned int    waitingForQueryResponse     : 1;
        unsigned int    createBridge                : 1;
    } flags;
} RemoteChannelData_t;

#ifdef  __cplusplus
}
#endif

#endif /* DOF_REMOTECHANNELDATA_H */
