/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_REMOTECHANNELMANAGER_H
#define DOF_REMOTECHANNELMANAGER_H

#include <dof/core.h>
#include <internal/channel.h>
#include <internal/protocolstack.h>
#include <internal/messaging.h>
#include <internal/connectionless_protocolstack.h>
#include <internal/permissionsetinternal.h>

#ifdef  __cplusplus
extern "C" {
#endif

#define OALRemoteChannelMgr_SendMessageToChannel(message)   OALRemoteChannelMgr_SendMessageToChannel_Internal(message, FALSE)

extern void    OALRemoteChannelMgr_SendToAll( DOFOperation operation );
extern void    OALRemoteChannelMgr_SendToAllStateless( DOFOperation operation );
extern void    OALRemoteChannelMgr_SendToAllProvider( DOFOperation operation );
extern void    OALRemoteChannelMgr_NotifyAllChannels_TransportDown(void * transport);

extern boolean OALRemoteChannelMgr_Init(void);
extern boolean OALRemoteChannelMgr_MutexInit(void);
extern void    OALRemoteChannelMgr_Shutdown(void);
extern void OALRemoteChannelMgr_MutexShutdown(void);

extern boolean OALRemoteChannelMgr_AddChannel(OALChannel channel);

/*  Note: This function can only be called from within a channel message_enqueue worker thread.
    Call OALRemoteChannel_Shutdown when not in the message_enqueue worker thread.  */
extern void    OALRemoteChannelMgr_RemoveChannel_Immediate(OALChannel channel);

extern boolean OALRemoteChannelMgr_AddProtocolStack(OALProtocolStack protocolStack);

/*  Note: This function can only be called from within a channel message_enqueue worker thread.
    call OALProtocolStack_Shutdown when not in the message_enqueue worker thread. */
extern void OALRemoteChannelMgr_RemoveProtocolStack_Immediate( OALProtocolStack protocolStack );

extern void    OALRemoteChannelMgr_SendMessageToChannel_Internal(OALMessage message, boolean forceEnqueue);
extern void    OALRemoteChannelMgr_SendMessageToStack(OALMessage message);

extern void    OALRemoteChannelMgr_SendShutdownToStack(OALProtocolStack protocolStack, DOFException except);

extern boolean OALRemoteChannelMgr_SetRecvSilenceTimerItem( OALChannel channel );
extern boolean OALRemoteChannelMgr_SetSendSilenceTimerItem( OALChannel channel );
extern void    OALRemoteChannelMgr_RemoveSilenceTimerItems( OALChannel channel );

extern boolean OALRemoteChannelMgr_CancelNegotiationTimer(OALChannel channel);
extern boolean OALRemoteChannelMgr_StartNegotiationTimeout(OALChannel channel, uint32 timeout);


extern OALConnectionlessProtocolStack gDefaultConnectionlessProtocolStack;

#ifdef  __cplusplus
}
#endif

#endif /* DOF_REMOTECHANNELMANAGER_H */
