/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_REQUESTINTERNAL_H
#define DOF_REQUESTINTERNAL_H


#include <dof/core.h>
#include <dof/request.h>
#include <dof/result.h>
#include <internal/channel.h>
#include <internal/iteminternal.h>
#include <internal/operationinternal.h>


#ifdef  __cplusplus
extern "C" {
#endif


/* @internal @addtogroup requestinternal
@{
**/


extern void DOFRequest_CleanupRequestTree( DOFRequest request, void * arg );


extern DOFRequest   DOFRequest_Create         (       DOFSystem system, DOFOperation op, DOFConnectionState connState );
extern void         DOFRequest_Retain         (       DOFRequest self );
extern void         DOFRequest_Release        (       DOFRequest request );
extern DOFException DOFRequest_GetException   ( const DOFRequest request );
extern DOFObjectID  DOFRequest_GetResultOID   ( const DOFRequest request );
extern DOFOperation DOFRequest_GetProviderOp  ( const DOFRequest request );
extern DOFOperation DOFRequest_GetReqOp       ( const DOFRequest request );
extern DOFItem      DOFRequest_GetReqItem     ( const DOFRequest request );
extern void         DOFRequest_Pend           (       DOFRequest request, DOFOperation reqOp, DOFOperation provOp );
extern void         DOFRequest_Unpend         (       DOFRequest request );
extern void         DOFRequestSubscribe_Accept(       DOFRequest request );


/** @internal
    Retrieve the associated @ref DOFSystem for a given DOFRequest.

    @param  request The Request.  This must not be NULL.

    @return The associated @ref DOFSystem.
**/
extern DOFSystem    DOFRequest_GetSystem      ( const DOFRequest request );


/** @internal
    Report whether the Request has been fully processed by the provider application code or if it is pending completion.

    @param  request The Request.  This must not be NULL.

    @retval Non-zero (@c TRUE) if the Request is pending.
    @retval Zero (@c FALSE) if the Request is not pending.

    @see DOFRequest_Pend
**/
extern boolean      DOFRequest_IsPending      ( const DOFRequest request );


/** @internal
    Disable the request for further processing by the provider application.

    @param  request The Request.  This must not be NULL.

    @see DOFRequest_Pend
**/
extern void         DOFRequest_DisablePend    ( DOFRequest request );


/**
@}
**/

#ifdef  __cplusplus
}
#endif

#endif /* DOF_REQUESTINTERNAL_H */
