/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_INTERNAL_RESOURCE_POOL_H
#define DOF_INTERNAL_RESOURCE_POOL_H

#include <dof/core.h>

#ifdef  __cplusplus
extern "C" {
#endif

typedef struct OALResourcePool_t *OALResourcePool;

typedef void *OALResourcePoolKey_t;

/** @internal
 *  Create a resource.
 *  @return A new resource.
 *  @exception NULL is returned if the resource could not be allocated.
 */
typedef void * (*OALResourcePool_Create_Resource_fn_t)(void);
/** @internal
 *  Destroy a resource.
 *  @param The resource. This may be NULL.
 */
typedef void  (*OALResourcePool_Destroy_Resource_fn_t)(void *resource);

extern OALResourcePool OALResourcePool_Create(uint16 num_resources, OALResourcePool_Create_Resource_fn_t create_fn, OALResourcePool_Destroy_Resource_fn_t destroy_fn);
extern void OALResourcePool_Destroy(OALResourcePool pool);

extern void * OALResourcePool_GetResource(OALResourcePool pool, OALResourcePoolKey_t key);
extern void * OALResourcePool_GetResourceNoRef(OALResourcePool pool, OALResourcePoolKey_t key);

#ifdef OALRESOURCEPOOL_KEEP_AFTER_USE
extern void OALRESOURCEPOOL_KEEP_AFTER_USE_OALResourcePool_ReleaseResource(OALResourcePool pool, OALResourcePoolKey_t key, boolean destroy);
#define OALResourcePool_ReleaseResource( pool, key )  OALRESOURCEPOOL_KEEP_AFTER_USE_OALResourcePool_ReleaseResource( pool, key, FALSE )
#define OALResourcePool_DestroyResource( pool, key )  OALRESOURCEPOOL_KEEP_AFTER_USE_OALResourcePool_ReleaseResource( pool, key, TRUE )
#else
extern void OALResourcePool_ReleaseResource(OALResourcePool pool, OALResourcePoolKey_t key);
#define OALResourcePool_DestroyResource( pool, key )
#endif

#ifdef  __cplusplus
}
#endif

#endif
