/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_RUNTIMEINTERNAL_H
#define DOF_RUNTIMEINTERNAL_H

#include <dof/core.h>
#include <internal/router.h>
#include <internal/enqueue.h>

#ifdef  __cplusplus
extern "C" {
#endif

typedef struct DOFRouteInfo_t
{
    DOFConnectionState  connectionState;
    DOFSecurityScope    securityScope;
    DOFObjectID         oid;
    DOFInterfaceID      iid;
    DOFAddress          address;
    uint16              distLossless;
    uint16              distLossy;
}DOFRouteInfo_t;

extern boolean DOFRuntime_Init( DOF dof );
extern void DOFRuntime_Shutdown( void );

extern void DOFRuntime_RoutingInfoChanged( provider_binding_table_entry_t *pbte, DOFRouteInfoState change );
extern void DOFRuntime_HandleRouteInfo( DOFOperation op );
extern void DOFRuntime_RemoveAllRoutingCallback( void );

extern void DOFRuntime_ConnectionStateChanged(DOFConnection conn, DOFConnectionState state);
extern void DOFRuntime_ServerStateChanged(DOFServer serv, DOFServerState state);

/**
 * Retrieve RouteInfo for a specific Binding (object ID and interface ID).  The
 * information returned represents the current best compatible match for the
 * binding, within the constraints of the specified security scope.  The actual
 * security scope in which the binding exists is returned as part of the result (it
 * may only be a subset of the specified security scope).
 *
 * @mustfree{DOFRouteInfo_Destroy}
 *
 * @param self          The DOF instance.  This must not be NULL.
 * @param oid           The ObjectID portion of the desired Binding.  Should not be NULL.
 * @param iid           The InterfaceID portion of the desired Binding.  Should not be NULL.
 * @param scope         The security scope.  If NULL, the default non-secure DOF scope is used.
 *
 * @return              The @ref DOFRouteInfo structure if the information is found; or
 *                      NULL Otherwise.
 *
 * @exception           NULL is returned if an invalid argument is detected or the information could not be created.
 */
extern DOFRouteInfo DOFRuntime_CreateProviderRoute( const DOFRuntime self, const DOFObjectID oid, const DOFInterfaceID iid, const DOFSecurityScope scope );

#ifdef  __cplusplus
}
#endif

#endif /* DOF_RUNTIMEINTERNAL_H */
