/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_SECUREGROUPINTERNAL_H
#define DOF_SECUREGROUPINTERNAL_H


#include <dof/core.h>
#include <internal/channel_stack_common.h>
#include <internal/key_request.h>
#include <internal/oaltypesinternal.h>
#include <internal/staticassertinternal.h>
#include <internal/security_membrane.h>


#ifdef  __cplusplus
extern "C" {
#endif


/** @addtogroup DOFSecureGroup
@{
**/


extern OALChannel     DOFSecureGroup_GetChannel                  ( DOFSecureGroup secureGroup );
extern void           DOFSecureGroup_ProcessRequestGroup         ( DOFSecureGroup secureGroup,
                                                                   DOFOperation requestGroupOp,
                                                                   DOFKeyRequest keyRequest, DOFObjectIDDomain domain,
                                                                   OALStream I_Stream, uint32 I_offset, uint32 Scope_offset, 
                                                                   uint16 epoch, uint16 hisTransmittedAPPID );
extern void           DOFSecureGroup_HeartbeatReceiveHandler     ( DOFSecureGroup secureGroup, DOFAddress hisAddress,
                                                                   uint16 hisHighestAPPID, uint8 hisRank,
                                                                   uint32 tieBreaker );
extern void           DOFSecureGroup_RekeyReceiveHandler         ( DOFSecureGroup secureGroup, uint16 secsPromoteDelay,
                                                                   OALStream streamCcmModeInitialState,
                                                                   const uint8 * pGroupKey, boolean receivedMulticast );
extern void           DOFSecureGroup_RekeyEpochReceiveHandler    ( DOFSecureGroup secureGroup, DOFAddress hisAddress,
                                                                   uint16 secsPromoteDelay,
                                                                   OALStream streamCcmModeInitialState,
                                                                   uint8 * pKey, uint16 epoch,
                                                                   boolean receivedMulticast );
extern void           DOFSecureGroup_RekeyMergeReceiveHandler    ( DOFSecureGroup secureGroup,
                                                                   OALStream streamCcmModeInitialState,
                                                                   const uint8 * pGroupKey, boolean receivedMulticast );
extern void           DOFSecureGroup_EpochChangedReceiveHandler  ( DOFSecureGroup secureGroup, DOFAddress address,
                                                                   uint16 epoch, uint16 getKEKWindowMinutes );
extern void           DOFSecureGroup_KEKKeyPromoted              ( DOFSecureGroup secureGroup, uint16 epoch );
extern void           DOFSecureGroup_SetHubConfiguredPointTimeout( DOFSecureGroup self, uint32 maxReceiveSilence );
extern boolean        DOFHub_UpdateAddressTable                  ( DOFSecureGroup self, const DOFAddress receivedAddr,
                                                                   boolean addNewEntry, uint16 hisTransmittedAPPID );
extern boolean        DOFSecureGroup_LogMessageNoTrailer         ( DOFLogLevel logLevel, const char * message );
extern boolean        DOFSecureGroup_LogMessage                  ( DOFLogLevel logLevel, const char * message,
                                                                   uint32 trailingValue );
extern boolean        DOFSecureGroups_Init                       ( void );
extern DOFSecureGroup DOFSecureGroup_Create                      ( const DOFObjectIDAuthentication groupOID,
                                                                   OALConnectionlessProtocolStack associatedStack,
                                                                   DTL associatedTransport,
                                                                   DOFSecurityMembrane securityConnector,
                                                                   OALServer listenServer,
                                                                   OALConnection relatedConnection,
                                                                   DOFConnectionConfig connConfig,
                                                                   DOFPermissionSet myPerms );
extern DOFSecurityScope DOFSecureGroup_GetSecurityScope          ( DOFSecureGroup self );
extern DOFObjectIDAuthentication DOFSecureGroup_GetGroupID       ( DOFSecureGroup self );
extern boolean   DOFSecureGroup_LastTwoKEKRekeysFromDifferentSGMs( DOFSecureGroup self );


/**
@}
**/


#ifdef  __cplusplus
}

#endif

#endif /* DOF_SECUREGROUPINTERNAL_H */
