/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef SECURITY_COMMON_H
#define SECURITY_COMMON_H

#include <dof/oal.h>
#include <internal/key_request.h>
#include <internal/stream.h>
#include <internal/securityscope.h>
#include <internal/securitymode.h>
#include <internal/permissionscope.h>

#ifdef  __cplusplus
extern "C" {
#endif

typedef struct OALModeOfOperation_t
{
    uint32  refCount;

    uint16  appId;

    uint8   attributeCode;
    uint8   valueLength;

    uint8 * options;        /* Must be last */
} OALModeOfOperation_t;

typedef OALModeOfOperation_t * OALModeOfOperation;

extern OALModeOfOperation OALModeOfOperation_Create(uint16 appId,
                                                    uint8  length);

extern  OALModeOfOperation OALModeOfOperation_Retain(OALModeOfOperation modeOfOperation);

extern void OALModeOfOperation_Release(OALModeOfOperation modeOfOperation);

extern uint32 OALModeOfOperation_MarshalSize(OALModeOfOperation modeOfOperation);

extern boolean OALModeOfOperation_Marshal(OALModeOfOperation self,
                                          OALStream          stream);

extern OALModeOfOperation OALModeOfOperation_Unmarshal(OALStream stream);

typedef struct OALSessionInitiatorBlock_t
{
    DOFKeyRequest     keyRequest;
    
    DOFSecurityMode     modeOfOperation;
    
    uint32              refCount;
    
    uint8               duration;
} OALSessionInitiatorBlock_t;

typedef OALSessionInitiatorBlock_t * OALSessionInitiatorBlock;

extern OALSessionInitiatorBlock OALSessionInitiatorBlock_Create(uint8               duration,
                                                                DOFSecurityMode     modeOfOperation,
                                                                DOFKeyRequest       keyRequest);

extern OALSessionInitiatorBlock OALSessionInitiatorBlock_Retain(OALSessionInitiatorBlock request);

extern void OALSessionInitiatorBlock_Release(OALSessionInitiatorBlock request);

extern OALSessionInitiatorBlock OALSessionInitiatorBlock_Unmarshal(OALStream   stream,
                                                                   DOFObjectIDDomain domain,
                                                                   boolean     isRequest,
                                                                   boolean     unknownAllowed);

extern uint32 OALSessionInitiatorBlock_MarshalSize(OALSessionInitiatorBlock self,
                                                   boolean                  isRequest);

extern boolean OALSessionInitiatorBlock_Marshal(OALSessionInitiatorBlock self,
                                                OALStream                stream,
                                                boolean                  isRequest);

typedef struct OALKeyGrant_t
{
    uint32      refCount;
    uint8       mac[32];
    uint8       key[32];
}OALKeyGrant_t;

typedef struct OALKeyGrant_t* OALKeyGrant;

extern OALKeyGrant OALKeyGrant_Create( uint8* mac, uint8* key );
extern OALKeyGrant OALKeyGrant_Retain( OALKeyGrant request );
extern void OALKeyGrant_Release( OALKeyGrant request );
extern OALKeyGrant OALKeyGrant_Unmarshal( OALStream stream );
extern boolean OALKeyGrant_Marshal( OALKeyGrant, OALStream stream );
extern uint32 OALKeyGrant_MarshalSize(OALKeyGrant self);
extern void OALKeyGrant_ExtractK(OALKeyGrant grant, const uint8 *shared_secret, uint8 *place_to_put_it);

typedef struct OALSessionAuthenticationResponseBlock_t
{
    uint8                   grantedDuration;
    DOFSecurityScope        connSecurityScope;
    OALPermissionScopeList  initPermValidation;
    OALPermissionScopeList  respPermValidation;  
}OALSessionAuthenticationResponseBlock_t;
typedef struct OALSessionAuthenticationResponseBlock_t* OALSessionAuthenticationResponseBlock;

extern boolean OALSessionAuthRespBlock_Unmarshal( uint8* grantedDuration, DOFSecurityScope * connSecurityScope, DOFPermissionSet initPermValidation, DOFPermissionSet respPermValidation, DOFObjectIDDomain domain, OALStream stream );
extern boolean OALSessionAuthRespBlock_Marshal( uint8 grantedDuration, DOFSecurityScope connSecurityScope, DOFPermissionSet initPermValidation, DOFPermissionSet respPermValidation, OALStream stream);
extern uint32 OALSessionAuthRespBlock_MarshalByteLength( DOFSecurityScope connSecurityScope, DOFPermissionSet initPermValidation, DOFPermissionSet respPermValidation );

#ifdef  __cplusplus
}
#endif

#endif
