/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef BRIDGE_STRUCTS_H
#define BRIDGE_STRUCTS_H

#include <dof/oaltypes.h>
#include <internal/cipher.h>

#ifdef  __cplusplus
extern "C" {
#endif

typedef enum DOFSecurityMembraneExtendReturn_t
{
    CANNOT_EXTEND,
    EXTEND_ELSEWHERE,
    EXTEND_PENDING
} DOFSecurityMembraneExtendReturn_t;

typedef void (*ExtendMembraneCompleteCB) (DOFError   status,
                                          void     * context);

typedef struct DOFSecurityMembrane_VTable_t
{
    void                               (*dtor)                   (DOFSecurityMembrane self);
    DOFObjectIDDomain                  (*getDomainId)            (DOFSecurityMembrane self);
    DOFSecurityScope                   (*validateInboundPerms)   (DOFSecurityMembrane self,
                                                                  DOFPermission       explicitPermission,
                                                                  DOFPermission       logicalPermission,
                                                                  OALCipherKeyState   keyState);
    DOFSecurityScope                   (*validateOutboundPerms)  (DOFSecurityMembrane self,
                                                                  DOFPermission       explicitPermission,
                                                                  DOFPermission       logicalPermission);
    DOFSecurityScope                   (*getInboundScope)        (DOFSecurityMembrane self);
    DOFSecurityScope                   (*getOutboundScope)       (DOFSecurityMembrane self);
    DOFSecurityMembraneExtendReturn_t  (*extendInboundPerms)     (DOFSecurityMembrane        self,
                                                                  DOFPermissionSet           extendPerms,
                                                                  ExtendMembraneCompleteCB   callback,
                                                                  void                     * context);
    DOFSecurityMembraneExtendReturn_t  (*extendOutboundPerms)    (DOFSecurityMembrane        self,
                                                                  DOFPermissionSet           extendPerms,
                                                                  ExtendMembraneCompleteCB   callback,
                                                                  void                     * context);

    DOFPermissionSet                   (*getInboundPerms)        (DOFSecurityMembrane self);

    DOFPermissionSet                   (*getOutboundPerms)       (DOFSecurityMembrane self);
} * DOFSecurityMembrane_VTable;

typedef struct DOFSecurityMembrane_t
{
    DOFSecurityMembrane_VTable vtable;

    uint32 refCount;
} DOFSecurityMembrane_t;

extern void DOFSecurityMembrane_Ctor(DOFSecurityMembrane        self,
                                     DOFSecurityMembrane_VTable vtable);

extern DOFSecurityScope DOFSecurityMembrane_ValidatePermission(DOFObjectIDDomain domainId,
                                                               DOFPermissionSet  permSet,
                                                               DOFPermission     perm);

extern DOFSecurityScope DOFSecurityMembrane_GetInboundScope(DOFSecurityMembrane self);

extern DOFSecurityScope DOFSecurityMembrane_GetOutboundScope(DOFSecurityMembrane self);

extern DOFSecurityScope DOFSecurityMembrane_ValidateInboundPermissions(DOFSecurityMembrane self,
                                                                       DOFPermission       explicitPermission,
                                                                       DOFPermission       logicalPermission,
                                                                       OALCipherKeyState   keyState);

extern DOFSecurityScope DOFSecurityMembrane_ValidateOutboundPermissions(DOFSecurityMembrane self,
                                                                        DOFPermission       explicitPermission,
                                                                        DOFPermission       logicalPermission);

extern DOFObjectIDDomain DOFSecurityMembrane_GetDomainID(DOFSecurityMembrane self);

extern DOFSecurityMembraneExtendReturn_t DOFSecurityMembrane_ExtendInboundPermissions(DOFSecurityMembrane        self,
                                                                                      DOFPermissionSet           extendPerms,
                                                                                      ExtendMembraneCompleteCB   callback,
                                                                                      void                     * context);

extern DOFSecurityMembraneExtendReturn_t DOFSecurityMembrane_ExtendOutboundPermissions(DOFSecurityMembrane        self,
                                                                                       DOFPermissionSet           extendPerms,
                                                                                       ExtendMembraneCompleteCB   callback,
                                                                                       void                     * context);

extern DOFPermissionSet DOFSecurityMembrane_GetInboundPermissions(DOFSecurityMembrane self);

extern DOFPermissionSet DOFSecurityMembrane_GetOutboundPermissions(DOFSecurityMembrane self);

extern DOFSecurityMembrane DOFSecurityMembrane_Copy(DOFSecurityMembrane self);

extern void DOFSecurityMembrane_Destroy(DOFSecurityMembrane self);

#ifdef  __cplusplus
}
#endif

#endif
