/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_SGMP_V1_H
#define DOF_SGMP_V1_H

#ifdef  __cplusplus
extern "C" {
#endif

#include <dof/permissionset.h>
#include <internal/blackboarddata.h>
#include <internal/channel.h>
#include <internal/cipher.h>
#include <internal/key_request.h>
#include <internal/packet.h>
#include <internal/security_common.h>
#include <internal/securitymode.h>
#include <internal/stream.h>


#define SGMP_V1_SEND_OP_TIMEOUT 3000

/* Order of these opcode must match order found in operationinternal.h. */
#define SGMP_V1_HEARTBEAT_OPCODE        0
#define SGMP_V1_EPOCHCHANGED_OPCODE     1
#define SGMP_V1_REKEY_OPCODE            2
#define SGMP_V1_REQUESTGROUP_OPCODE     3
#define SGMP_V1_REKEYEPOCH_OPCODE       5
#define SGMP_V1_REKEYMERGE_OPCODE       7

#define SGMP_V1_HIGHEST_OPCODE          7

#define SGMP_V1_APPID       130

#define SGMP_V1_KEY_SIZE    32
#define SGMP_V1_SIG_SIZE    32


typedef struct SGMP_V1_RequestGroupContext_t
{
    uint32          nodeID;
    DOFError        status;
    DOFKeyRequest   keyRequest;
    uint8 *         groupKey;
    uint8 *         MAC;
    uint8 *         KEY;
}SGMP_V1_RequestGroupContext_t;

typedef struct SGMP_V1_RequestGroupContext_t * SGMP_V1_RequestGroupContext;


/* Must match RequestGroupCallback() in securegroupGetGroupKey.c. */
typedef void (*SGMP_V1_RequestGroupCallback_fn)( DOFAddress address, OALKeyGrant ticket, uint16 hisHighestAPPID,
                                                 uint8 hisRank, DOFError status, void * ctx,
                                                 OALStream streamCcmModeInitialState, boolean receivedMulticast,
                                                 DOFOperation respOp, PCRMutex reqGRespMutex );


typedef struct SGMP_V1_t
{
    BlackboardData                blackboardData;
    /* boolean                       isConnectionless; */
} SGMP_V1_t;

typedef struct SGMP_V1_t * SGMP_V1;

extern SGMP_V1 SGMP_V1_Create( BlackboardData blackboardData );
extern void SGMP_V1_Destroy( SGMP_V1 self );


extern ProtocolLayerResult SGMP_V1_Receive( SGMP_V1 self );
extern ProtocolLayerResult SGMP_V1_Send( SGMP_V1 self, DOFOperation operation, boolean isProviderOperation,
                                         DOFResult result, PCREvent * keyChangedEvent );

extern boolean  SGMP_V1_SM_Init(void);
extern void     SGMP_V1_SM_Shutdown(void);

extern DOFError SGMP_V1_SendHeartbeat( OALChannel channel, DOFAddress address, DOFSecurityScope securityScope, uint16 appid,
                                       uint8 rank, uint32 tieBreaker );
extern DOFError SGMP_V1_SendEpochChanged( OALChannel channel, DOFAddress address, DOFSecurityScope securityScope, uint16 epoch,
                                          uint16 getKEKWindowMinutes, uint8 retryCount, uint32 retryPeriod );
extern DOFError SGMP_V1_SendRekey( OALChannel channel, DOFAddress address, DOFSecurityScope securityScope,
                                   uint8 secsPromoteDelay, OALStream streamCcmModeInitialState, uint8* key );
extern DOFError SGMP_V1_SendRekeyEpoch( OALChannel channel, DOFAddress address, DOFSecurityScope securityScope,
                                        uint8 secsPromoteDelay, uint16 epoch, OALStream streamCcmModeInitialState,
                                        uint8* key );
extern DOFError SGMP_V1_SendRekeyMerge( OALChannel channel, DOFAddress address, DOFSecurityScope securityScope, PCREvent event,
                                        OALStream streamCcmModeInitialState,  uint8* key );
extern DOFOperation SGMP_V1_SendRequestGroup( OALChannel channel, DOFAddress address, OALStream streamI,
                                              DOFObjectIDDomain domain, DOFSecurityScope securityScope,
                                              SGMP_V1_RequestGroupCallback_fn callbackFn, void * rgSendCtx,
                                              uint32 timeout, boolean secure, PCRMutex reqGRespMutex );
extern DOFError SGMP_V1_SendRequestGroupResponse( OALChannel channel, OALKeyGrant ticket, DOFOperation requestGroupOp,
                                                  uint16 appid, uint8 myRank, OALStream streamCcmModeInitialState );


#ifdef  __cplusplus
}
#endif

#endif /* DOF_SGMP_V1_H */

