/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef __source_H
#define __source_H

#include <internal/objectidinternal.h>
#include <internal/timer.h>

boolean source_mutex_init(void);
boolean source_init(void);
void source_mutex_shutdown(void);
void source_shutdown(void);

typedef struct sid_t {
    uint32 ref_count;
    DOFObjectIDSource oid;
    DOFObjectIDDomain domain;
    struct sid_t * explicitSid;
    OALChannel channel;
    DOFAddress address;
    OALTimerItem state_timeout;
    DOFPermission iamPermission;
    DOFPermission actAsPermission;
    /* uint8 flags; */
} sid_t;

#ifdef OALSOURCE_REF_COUNT_DEBUG

#define source_retain(sid) source_retain_Debug(sid, __FILE__, __FUNCTION__, __LINE__)
void source_retain_Debug(sid_t *sid, const char * file, const char * func, uint32 line);
#define source_release(sid) source_release_Debug(sid, __FILE__, __FUNCTION__, __LINE__)
void source_release_Debug(sid_t *sid, const char * file, const char * func, uint32 line);
#define source_set_channel_address(sid, channel, address) source_set_channel_address_Debug(sid, channel, address, __FILE__, __FUNCTION__, __LINE__)
int source_set_channel_address_Debug(sid_t *sid, OALChannel channel, DOFAddress address, const char * file, const char * func, uint32 line);
#define lookup_sid(channel, address) lookup_sid_Debug(channel, address, __FILE__, __FUNCTION__, __LINE__)
sid_t *lookup_sid_Debug(OALChannel channel, DOFAddress address, const char * file, const char * func, uint32 line);
#define lookup_explicit_sid(oid, domain) lookup_explicit_sid_Debug(oid, domain, __FILE__, __FUNCTION__, __LINE__)
sid_t *lookup_explicit_sid_Debug(DOFObjectIDSource oid, DOFObjectIDDomain domain, const char * file, const char * func, uint32 line);

#else

void source_retain(sid_t *sid);
void source_release(sid_t *sid);
int source_set_channel_address(sid_t *sid, OALChannel channel, DOFAddress address);
sid_t *lookup_sid(OALChannel channel, DOFAddress address);
sid_t *lookup_explicit_sid(DOFObjectIDSource oid, DOFObjectIDDomain domain);

#endif

sid_t *create_sid_from_oid(DOFObjectIDSource oid, DOFObjectIDDomain domain);
sid_t *create_sid_from_channel_address(OALChannel channel, DOFAddress address, DOFObjectIDDomain domain);
sid_t* create_sid_from_channel_address_explicit(OALChannel channel, DOFAddress address, DOFObjectIDDomain domain, sid_t* sid_explicit);




boolean source_isOIDSet(sid_t *sid);

/* returns non-zero if fails */
int source_set_oid(sid_t *sid, DOFObjectIDSource oid);

#ifdef OALSOURCE_REF_COUNT_DEBUG
#define OALSource_Clear(channel, address) OALSource_Clear_Debug(channel, address, __FILE__, __FUNCTION__, __LINE__)
void OALSource_Clear_Debug(OALChannel channel, DOFAddress address, const char * file, const char * func, uint32 line);
#else
void OALSource_Clear(OALChannel channel, DOFAddress address);
#endif

#endif
