/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_INTERNAL_TIMER_H
#define DOF_INTERNAL_TIMER_H

#include <dof/core.h>
#include <internal/enqueue.h>
#include <internal/dlist.h>

#ifdef  __cplusplus
extern "C" {
#endif

typedef struct OALTimer_t *OALTimer;
typedef struct OALTimerItem_t *OALTimerItem;

/**
 Timer function.

 This is the standard prototype for a worker function to be used with a timer.

 @param item The timer item that is being fired. Will be NULL if internally created. You can use this item with OALTimer_Insert to re-use a timer item (more optimal) in your callback.
 @param  argc  The number of arguments passed in @a argv. Must be less than 0x40000000
 @param  argv  An array of @a argc arguments for the function.
 @param  shutdown True when the timer is shutting down. If this is set then your timer item must be freed in the callback.
 @return Return non-zero if the item is periodic and you don't wnat it to be anymore.
 **/
typedef int (*OALTimer_fn_t)(OALTimerItem item, uint32 argc, void *argv[], boolean shutdown);

extern void         OALTimer_Init(void);
extern void         OALTimer_Shutdown(void);

extern OALTimer     OALTimer_Create( OALThreadPool pool, boolean synchronous );
extern void         OALTimer_Destroy( OALTimer timer );

extern boolean      OALTimer_Insert( OALTimer timer, uint32 relative_time, OALTimer_fn_t fn, uint32 argc, void *argv[],
                                     boolean periodic );
extern OALTimerItem OALTimer_CreateItem( OALTimer timer, OALTimer_fn_t, uint32 argc, void *argv[] ); /* use this if you need to save the item */
/* you can call this again to change the timeout */
extern DOFError     OALTimerItem_Insert( OALTimerItem item, uint32 relative_time, boolean periodic,
                                         boolean notOnTimerItemFiring );
extern boolean      OALTimerItem_Remove( OALTimerItem item ); /* only call this if you need to remove the item. returns TRUE if item was actually removed */
extern void         OALTimerItem_Destroy( OALTimerItem item, boolean wait ); /* call this on any timer item made with OALTimer_CreateItem */

extern uint32       OALTimerItem_TimeRemaining( OALTimerItem item );

extern void         OALTimerItem_SetPeriodic( OALTimerItem item, boolean periodic );
extern boolean      OALTimerItem_IsPeriodic( OALTimerItem item );
extern boolean      OALTimerItem_IsInserted( OALTimerItem item );

#ifdef  __cplusplus
}
#endif

#endif
