/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef _INTERNAL_TYPE_H
#define _INTERNAL_TYPE_H

#include <dof/core.h>
#include <dof/type.h>
#include <dof/value.h>
#include <internal/stream.h>

#ifdef  __cplusplus
extern "C" {
#endif


/*
**  "Public" Type and Method Declarations
*/

/** @addtogroup DOFType
@{
**/

typedef struct DOFTypeContext_t * DOFTypeContext;

boolean DOFType_Init( void );
void DOFType_Shutdown( void );

DOFType DOFValueStructureType_Create_Values( uint16 count, const DOFValue fields[] );

DOFType       DOFType_Create_Unmarshal  ( uint32 * length, const uint8 * buffer, DOFTypeContext context, boolean must_define);
uint32        DOFType_GetUnmarshalSize  ( uint32 length, const uint8 *buffer );
uint32        DOFType_GetMarshalSize    ( const DOFType self, DOFTypeContext context );
uint32        DOFType_Marshal           ( const DOFType self, uint32 length, uint8 *buffer, DOFTypeContext context );

DOFValue      DOFType_CreateValue_Unmarshal          ( const DOFType self, uint32 * length, const uint8 *buffer );
DOFValue      DOFType_CreateValue_UnmarshalFromStream( const DOFType self, OALStream stream, boolean * failed );

/**
    Report whether or not the type represents a Nullable type.

    @param  type  The DOFType.  This must not be NULL.

    @return TRUE if the type is Nullable.
    @return FALSE if the type is not Nullable.
**/
#define DOFType_IsNullable( type )    ( DOFType_GetTypeID( type ) == DOFTYPE_NULLABLE )

DOFTypeContext DOFTypeContext_Create_Unmarshal( uint32 * length, const uint8 * buffer );
void DOFTypeContext_Destroy( DOFTypeContext context );
DOFType DOFTypeContext_UnmarshalTypeRef( DOFTypeContext context, uint32 * length, const uint8 * buffer );

void DOFTypeList_Destroy( uint16 valueCount, DOFType typeList[] );
DOFType * DOFTypeListAlloc_Create( uint16 typeCount );
DOFType * DOFTypeListAlloc_Copy( uint16 typeCount, const DOFType typeList[] );
void DOFTypeListAlloc_Destroy( uint16 typeCount, DOFType * typeList );

boolean DOFTypeList_IsValueCompatibleWith( uint16 count, const DOFType types[], const DOFValue values[] );
uint32 DOFTypeList_GetMarshalValueSize( uint16 count, const DOFType types[], const DOFValue values[] );
boolean DOFTypeList_MarshalValueToStream( uint16 count, const DOFType types[], const DOFValue values[], OALStream stream );

/**
@}
**/


#ifdef  __cplusplus
}
#endif

#endif /* _DOF_TYPE_H */
