/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include <dof/core.h>
#include <dof/address.h>
#include <dof/objectid.h>
#include <internal/addressinternal.h>


/** @addtogroup DOFGroupAddress
@{
**/


/*
**  "Private" Type and Method Declarations
*/


static int GAddrCompare( DTLTransport transport, uint32 size, const void* addr1, const void* addr2 )
{
    return DOFObjectID_Compare( DOFObjectIDAuthentication_GetObjectID( *(DOFObjectIDAuthentication*)addr1 ), DOFObjectIDAuthentication_GetObjectID( *(DOFObjectIDAuthentication*)addr2 ) );
}

static void destroyer( DTLTransport transport, uint32 size, const void * addr )
{
    DOFObjectIDAuthentication_Destroy( *(DOFObjectIDAuthentication*)addr );
}

static DOFAddressType GAddrType( DTLTransport transport, DOFAddress address )
{
    return DOFADDRESSTYPE_MULTICAST;
}

static boolean copyAddress(DTLTransport transport, uint32 size, void * addr )
{
    return (DOFObjectIDAuthentication_Copy(*(DOFObjectIDAuthentication *)addr) != NULL);
}

static const struct DTLTransportFns_t tportFns = {
    NULL, /* Open */
    NULL, /* Start */
    NULL, /* GetMaxSize */
    NULL, /* GetMinSize */
    GAddrType,      /* GetAddressType */
    NULL, /* IsConnectionConfigEqual */
    NULL, /* IsServerConfigEqual */
    GAddrCompare,   /* AddrCompare */
    copyAddress,    /* AddressCopy */
    destroyer       /* DestroyAddress */
};

static DTLTransport_t tport = { &tportFns };


/*
**  "Public" Type and Method Declarations
*/


/*
**  "Public" Methods
*/


DOFAddress DOFGroupAddress_Create( DOFObjectIDAuthentication groupOID )
{
    return DOFAddress_Create( sizeof( groupOID ), &groupOID, &tport );;
}


DOFObjectIDAuthentication DOFGroupAddress_GetGroupID( const DOFAddress self )
{
    if ( DOFAddress_GetTransport( self ) == &tport )
    {
        DOFObjectIDAuthentication *pOID = (DOFObjectIDAuthentication *)DOFAddress_GetAddress( self );

        if (pOID)
            return *pOID;
    }
    return NULL;
}

void DOFGroupAddress_Destroy( DOFAddress self )
{
    DOFAddress_Destroy( self );
}


/**
@}
**/
