/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include <internal/bignum.h>
#ifdef BN_MP_INIT_MULTI_C

#ifdef OALVARARGS_ALLOWED
#include <stdarg.h>

int32 mp_init_multi(mp_int *mp, ...) 
{ 
    mp_err res = MP_OKAY;      /* Assume ok until proven otherwise */
    int32 n = 0;                 /* Number of ok inits */
    mp_int *cur_arg = mp; 
    va_list args; 
    
    va_start(args, mp);        /* init args to next argument from caller */
    while(cur_arg != NULL) 
    {
        if(mp_init(cur_arg) != MP_OKAY)
        {
            /* Oops - error! Back-track and mp_clear what we already
               succeeded in init-ing, then return error.
            */
            va_list clean_args; 
            
            /* end the current list */
            va_end(args); 
            
            /* now start cleaning up */            
            cur_arg = mp; 
            va_start(clean_args, mp); 
            while(n--) 
            {
                mp_clear(cur_arg); 
                cur_arg = va_arg(clean_args, mp_int *);
            }
            va_end(clean_args); 
            res = MP_MEM; 
            break;
        }
        n++; 
        cur_arg = va_arg(args, mp_int*);
    }
    va_end(args); 
    return res;                /* Assumed ok, if error flagged above. */
}
#endif

#endif
