/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_CIPHERCCMMODE_V1_H
#define DOF_CIPHERCCMMODE_V1_H


#ifdef  __cplusplus
extern "C" {
#endif


#include <internal/blackboarddata.h>
#include <internal/ccmmode_v1.h>


/* Operation variables */


/* send variables */
typedef struct CCMMODE_V1_SendData_t * CCMMODE_V1_SendData;
typedef void (*CCMMODE_V1_SendFunction) (CCMMODE_V1_SendData data, BlackboardData blackboardData);
typedef boolean (*CCMMODE_V1_SendUtils) (CCMMODE_V1_SendData data);


typedef enum CCMMODE_V1_SendStatus_e
{
    CCMMODE_V1_SendStatus_OK,
    CCMMODE_V1_SendStatus_OKNOOPID,
    CCMMODE_V1_SendStatus_MEMORYERROR,
    CCMMODE_V1_SendStatus_STREAMERROR,
    CCMMODE_V1_SendStatus_TIMEOUTERROR,
    CCMMODE_V1_SendStatus_INVALID_OPCODE
} CCMMODE_V1_SendStatus_e;


typedef struct CCMMODE_V1_SendData_t
{
    CCMMODE_V1_SendUtils const  * functions;
    CCMMODE_V1_SendStatus_e     status;
    DOFOperation                operation;
    OALPacket                   packet;
    CCMMODE_V1_OperationContext context;
    CCMMODE_V1                  ccmMode;
    uint8                       opCode;
} CCMMODE_V1_SendData_t;


/* receive variables */
typedef struct CCMMODE_V1_ReceiveData_t * CCMMODE_V1_ReceiveData;
typedef void (*CCMMODE_V1_ReceiveFunction) (CCMMODE_V1_ReceiveData data);
typedef boolean (*CCMMODE_V1_ReceiveUtils) (CCMMODE_V1_ReceiveData data);


typedef enum CCMMODE_V1_ReceiveStatus_e
{
    CCMMODE_V1_ReceiveStatus_OK,
    CCMMODE_V1_ReceiveStatus_OK_DONOTHING,
    CCMMODE_V1_ReceiveStatus_ENDING,
    CCMMODE_V1_ReceiveStatus_STREAMERROR,
    CCMMODE_V1_ReceiveStatus_MEMORYERROR
} CCMMODE_V1_ReceiveStatus_e;


typedef struct CCMMODE_V1_ReceiveData_t
{
    CCMMODE_V1                      ccmMode;
    DOFOperation                    operation;
    OALStream                       stream;
    OALStream                       streamCcmModeInitialState;
    CCMMODE_V1_ReceiveUtils const * functions;
    CCMMODE_V1_ReceiveStatus_e      status;
    uint32                          nodeID;
    uint8                           errorCode;
} CCMMODE_V1_ReceiveData_t;

/* Operation functions */
extern DOFOperation DOFOperation_Create_CCMMODE_Query( OALChannel channel, DOFAddress address, uint32 timeout,
                                                       boolean providerOperation );
extern DOFOperation DOFOperation_Create_CCMMODE_Request( OALChannel channel, DOFAddress address, uint32 timeout,
                                                         boolean providerOperation );


#ifdef  __cplusplus
}
#endif


#endif /* DOF_CIPHERCCMMODE_V1_H */
