/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include <dof/core.h>
#include <dof/cipherregistry.h>
#include <dof/blockcipher.h>
#include <internal/cipher.h>

/** @internal @addtogroup DOFCipherRegistry
@{
**/


/*
**  "Private" Type and Method Declarations
*/

extern PCRMutex ref_count_mutex;

/* External pre-registered block ciphers. */
extern const struct BlockCipher BlockCipher_AES_256;

DOFCipher DOFCipherRegistry_AES256Cipher =
#ifdef HAVE_BlockCipher_AES_256
    &BlockCipher_AES_256
#else
    NULL
#endif
;


/*
**  Public Methods.
*/


boolean DOFCipherRegistry_Register( DOFCipherAlgorithm algorithm, DOFCipherStrength strength, DOFCipher cipher )
{
    PCRMutex_Lock( ref_count_mutex );
    if ( DOFCipherRegistry_AES256Cipher == NULL && algorithm == DOFCIPHERALGORITHM_AES && strength == DOFCIPHERSTRENGTH_256 && cipher && cipher->keyStateSize <= sizeof(OALCryptExpandedKey) && cipher->blockSize == 16 && cipher->GenerateKeyState && cipher->Encrypt )
    {
        DOFCipherRegistry_AES256Cipher = cipher;
        PCRMutex_Unlock( ref_count_mutex );
        return TRUE;
    }
    PCRMutex_Unlock( ref_count_mutex );
    return FALSE;
}


DOFCipher DOFCipherRegistry_Get( DOFCipherAlgorithm algorithm, DOFCipherStrength strength )
{
    if ( algorithm == DOFCIPHERALGORITHM_AES && strength == DOFCIPHERSTRENGTH_256 )
        return DOFCipherRegistry_AES256Cipher;
    return NULL;
}


/**
@}
**/
