/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_CONNECTION_LOCAL_H
#define DOF_CONNECTION_LOCAL_H

#include <internal/connection_internal.h>

#ifdef  __cplusplus
extern "C" {
#endif

typedef struct DOFConnectionSRLremoveNode_t * DOFConnectionSRLremoveNode;

typedef struct DOFConnectionSRLremoveNode_t
{
    OALEnqueueItem_t                    super;

    DOFConnectionStreamRequestListener  streamListener;
    DOFException                        except;
} DOFConnectionSRLremoveNode_t;

typedef struct DOFConnectionSRL_t
{
    DOFConnectionStreamRequestListener  streamListener;
    DOFConnectionSRLremoveNode          removeNode;
    uint32                              refCount;
    boolean                             removed;
} DOFConnectionSRL_t;

typedef struct DOFConnectionStreamRequest_t
{
    OALEnqueueItem_t    super;          /* The stream request is enqueued for the callbacks. */

    uint32              requestCount;   /* Number of calls from router. */
    DOFConnection       connection;     /* This must not change. */
    DOFAddress          address;        /* This must not change. */
    struct
    {
        unsigned int enqueued               : 1;
        unsigned int doNothing              : 1;
        unsigned int complete               : 1;
    } flags;
} DOFConnectionStreamRequest_t;


extern DOFConnectionSRL DOFConnectionSRL_Create(DOFConnectionStreamRequestListener streamListener);
extern DOFConnectionSRL DOFConnectionSRL_Copy(DOFConnectionSRL srl);
extern void DOFConnectionSRL_Destroy(DOFConnectionSRL srl);
extern DOFConnectionStreamRequestListener DOFConnectionSRL_GetStreamRequestListener(DOFConnectionSRL srl);

extern DOFConnectionStreamRequest DOFConnectionStreamRequest_Create(DOFConnection conn, DOFAddress addr);
extern void DOFConnectionStreamRequest_Destroy(DOFConnectionStreamRequest request);


#ifdef  __cplusplus
}
#endif

#endif /*DOF_CONNECTION_LOCAL_H */


