/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_CREDENTIALS_COMMON_INTERNAL_H
#define DOF_CREDENTIALS_COMMON_INTERNAL_H

#ifdef __cplusplus
extern "C" {
#endif

#define DOFCREDENTIALSTYPE_REMOTEKEY (DOFCredentialsType)0x8000

typedef struct OALCredentials_t* OALCredentials;

typedef struct OALCredentials_fns_t
{
    void (*OALCredentials_Dtor)( OALCredentials self );
    const uint8* (*OALCredentials_GetPrivateStorage)( OALCredentials self );
    uint32 (*OALCredentials_GetPrivateStorageSize)( OALCredentials self );
    boolean (*OALCredentials_SetPrivateStorage)( OALCredentials self, uint32 len, const uint8* data );
    boolean (*OALCredentials_Marshal)( OALCredentials self, OALStream stream, boolean isRequest );
    uint32 (*OALCredentials_MarshalSize)( OALCredentials self, boolean isRequest );
    CredentialsUnmarshalReturn (*OALCredentials_Unmarshal)( OALCredentials self, OALStream stream, boolean isRequest );
    boolean (*OALCredentials_IsEqual)( OALCredentials self, OALCredentials other);
    const uint8* (*OALCredentials_GetKeyFromStorage)( OALCredentials self, const uint8* privateStorage );
    const uint8* (*OALCredentials_GetKey)( OALCredentials self );
    const uint8* (*OALCredentials_GetStorageKey)( OALCredentials self );
    uint32 (*OALCredentials_GetStorageKeySize)( OALCredentials self );
    boolean (*OALCredentials_IsCached)( OALCredentials self );
    void (*OALCredentials_ResetCache)( OALCredentials self );
    boolean (*OALCredentials_Update)( OALCredentials self, OALCredentials from );
    boolean (*OALCredentials_HasAnotherResolutionStage)( OALCredentials self );
    uint32 (*OALCredentials_MarshalStore)( OALCredentials self, uint32 size, uint8 * buffer );
}OALCredentials_fns_t;

typedef struct OALCredentials_t
{
    const OALCredentials_fns_t* fns;
    OALObservable               changeObserver;
    DOFObjectIDAuthentication   identity;       /**< Authentication Identity. **/
    DOFObjectIDDomain           domain;         /**< Authentication Domain. **/
    DOFObjectIDSource           sid;            /**< Authenticated Source ID. **/
    uint32                      refCount;       /**< Reference Count. **/
    DOFCredentialsType          type;           /**< Type of authentication data: key or password. **/
    uint8                       stage;
    boolean                     isDomRes;
} OALCredentials_t;

typedef struct DOFCredentials_t
{
    OALCredentials_t    base;
}DOFCredentials_t;

extern void Credentials_Lock(void);
extern void Credentials_Unlock(void);
extern void CredentialsTree_Lock(void);
extern void CredentialsTree_Unlock(void);

extern boolean DOFCredentials_Update( DOFCredentials self, DOFCredentials from );
extern boolean DOFCredentials_NotifyObservers( DOFCredentials self );

#ifdef __cplusplus
}
#endif

#endif
