/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_CREDENTIALS_PASSWORD_INTERNAL_H
#define DOF_CREDENTIALS_PASSWORD_INTERNAL_H

#ifdef __cplusplus
extern "C" {
#endif

typedef struct SRPCredentialsData_t /* Expanded Credentials for SRP */
{
    uint8               pubA[128];
    uint8               pubB[128];
    uint8               validB[32];
    uint8               KEY[32];
    uint8               Ksrp[32];
    uint8               a[32];
    uint8               srpS[128];
    uint8               srps[32];
    uint8               v[128];
    uint8*              storage;    /* holds s, v and Kshared for private storage *//* copy key to end if GetStorage is called needed*/
}SRPCredentialsData_t;

typedef struct SRPCredentials_t /* Expanded Credentials for SRP */
{
    OALCredentials_t        base;
    SRPCredentialsData_t*   data;
    uint8                   validA[40];
    uint8                   key[32];
    uint8                   partial_x[32]; /* calculated at create SHA( ID || PASS ) */
} SRPCredentials_t;

extern DOFCredentials SRPCredentials_InternalCreate( DOFObjectIDDomain domain );
extern DOFCredentials DOFCredentialsPassword_Internal_Create( const DOFObjectIDDomain domain, const DOFObjectIDAuthentication identity, const uint8* partial_x );

#ifdef __cplusplus
}
#endif

#endif
