/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include <internal/dlist.h>
#include <internal/slist.h>

/*
 **  "Private" type and method declarations
 */

/* "Object" methods */

void OALDList_Insert(OALDListNode *head, OALDListNode item)
{
    item->next = *head;
    item->prev = NULL;
    if(item->next) item->next->prev = item;
    *head = item;
}

void OALDList_Remove(OALDListNode *head, OALDListNode item)
{
    PCR_Assert( head );
    PCR_Assert( item );

    if(*head==item) {
        *head = item->next;
        item->prev = item->next = NULL;
        if(*head) (*head)->prev = NULL;
    }
    else {
        /* item is not the head, so its prev must be valid */
        PCR_Assert(item->prev);
        item->prev->next = item->next;
        if(item->next) item->next->prev = item->prev;

        item->next = item->prev = NULL;
    }
}

void OALDList_RemoveAll(OALDListNode *head, OALDListRemoveAll_fn_t fn, void *arg)
{
    OALDListNode item;

    PCR_Assert( head );
    item = *head;
    while ( item )
    {
        *head = item->next;
        item->prev = item->next = NULL;
        if(*head) (*head)->prev = NULL;
        if ( fn )
            fn( item, arg );
        item = *head;
    }
}

boolean OALDList_Traverse(OALDListNode head, OALDListTraversal_fn_t fn, void *arg)
{
    return OALSList_Traverse((OALSListNode)head, (OALSListTraversal_fn_t)fn, arg);
}

OALDListNode OALDList_Retrieve(OALDListNode head, OALDListTraversal_fn_t fn, void *arg)
{
    return (OALDListNode)OALSList_Retrieve((OALSListNode)head, (OALSListTraversal_fn_t)fn, arg);
}

OALDListNode OALDList_FindAndRemove(OALDListNode * head, OALDListTraversal_fn_t fn, void *arg)
{
    PCR_Assert(fn);
    PCR_Assert(head != NULL);

    if (*head != NULL)
    {
        OALDListNode ptr;

        if (fn(*head, arg))
        {
            ptr = *head;
            *head = ptr->next;
            ptr->prev = ptr->next = NULL;
            if(*head) (*head)->prev = NULL;
            return ptr;
        }
        else
        {
            ptr = (*head)->next;
            while (ptr)
            {
                if (fn(ptr, arg))
                {
                    ptr->prev->next = ptr->next;
                    if(ptr->next) ptr->next->prev = ptr->prev;
                    ptr->next = ptr->prev = NULL;
                    return ptr;
                }
                ptr = ptr->next;
            }
        }
    }
    return NULL;
}

/**
@}
**/
