/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include <dof/interfaceid.h>
#include <internal/dofinternal.h>
#include <internal/interfaceinternal.h>
#include "interfaceGetItem.h"


/** @internal @addtogroup DOFInterface
    DOF Interface Definition
@{
**/


/*
**  "Public" Methods
*/


/** @internal
    Get reference to the item descriptor that describes a specific item with a specific itemID from an interfaceDef.

    @param iface    Reference to the interface descriptor that describes the interface definition.  This must not be NULL.
    @param itemID   itemID value of the item to be found.
    @param itemType Type (property, event, method, exception, typedef) of the item to be found.

    @return Reference to the item descriptor; or NULL if
        - the item number is not found in the interface descriptor within its matching section of item descriptors.
          For example, if submitted itemType is DOFITEM_PROPERTY and the submitted itemNumber is not found within
          the stored set of property item descriptors, then NULL is returned (even if that itemNumber is found
          in the DOFITEM_EVENT or DOFITEM_METHOD sections of the interface descriptor.
**/
DOFItem DOFInterface_GetItem( const DOFInterface iface, DOFItemID itemID, uint8 itemType )
{
    if (iface)
    {
        DOFItem item = (DOFItem)OALAVLTree_Retrieve( iface->itemTree, (uint8 *)0 + itemID );

        if (item && item->itemType == itemType)
            return item;
    }

    return NULL;
}


/**
@}
**/
