/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include <dof/core.h>
#include <dof/interfaceid.h>
#include <internal/interfaceidinternal.h>
#include "interfaceidprivate.h"


/** @addtogroup DOFInterfaceID
@{
**/

static const uint8 IWild_ID_Bytes[] = {0x01,0x00};

DOFInterfaceID DOFINTERFACEID_WILDCARD;


/*
**  "Public" Methods
*/


int32 DOFInterfaceID_Compare( const DOFInterfaceID iid1, const DOFInterfaceID iid2 )
{
    if (iid1 == iid2)
        return 0;                       /* Even if both are NULL. */
    if (!iid1)
        return -1;
    if (!iid2)
        return 1;
    /*
     * Perform straight memcmp().
     * Since the length is first, any discrepancies will be discovered before accessing outside either OID.
     */
    return PCRMem_Compare( DOFInterfaceID_GetBytes( iid1 ), DOFInterfaceID_GetBytes( iid2 ),
                           DOFInterfaceID_GetBytesSize( iid1 ) );
    /* Note: Our return value must be int32- not int8! For cygwin implementation, an int8 loses info. */
}

/** @internal
    Check if the given interface ID is wildcard.

    @param iid The interface ID to check.

    @return TRUE if the interface ID is wildcard; otherwise FALSE.
**/
boolean DOFInterfaceID_IsWildcard( const DOFInterfaceID iid )
{
    PCR_Assert( iid );
    if (iid->len == sizeof( IWild_ID_Bytes ) && !PCRMem_Compare( IWild_ID_Bytes, iid->iid, sizeof( IWild_ID_Bytes ) ))
        return TRUE;

    return FALSE;
}


/*
**  "Private" Methods
*/


/* @internal */
boolean DOFInterfaceID_Init( void )
{
    return ((DOFINTERFACEID_WILDCARD = DOFInterfaceID_Create_Bytes( sizeof(IWild_ID_Bytes), IWild_ID_Bytes )) != NULL);
}


/* @internal */
void DOFInterfaceID_Shutdown( void )
{
    DOFInterfaceID_Destroy( DOFINTERFACEID_WILDCARD );
    DOFINTERFACEID_WILDCARD = NULL;
}

/**
@}
**/

