/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include <dof/core.h>
#include <dof/interfaceid.h>
#include <internal/oalstring.h>
#include <internal/dofinternal.h>

#include "interfaceidprivate.h"


/** @addtogroup DOFInterfaceID
@{
**/

/* These might save over calling the equivalent DOFInterfaceID_GetBytes() and DOFInterfaceID_GetRegistry() calls. */
#define GetIIDData( iid )                 ( (const uint8 *)(iid) + 1 )
#define GetIIDRegistryValue( iid )        ( (const uint8)(*(const uint8 *)(iid) >> 2) )

/* Kept for pure cool, cheaper code and time to call DOFInterfaceID_GetBytesSize() and subtract 1. */
#define GetIIDDataSize( iid )             ( ( ( ( ~0 << ( ((*(const uint8 *)(iid)) & 0x03) - 1 ) ) ^ ~0 ) & 0x0f ) + 1 )


/*
**  "Public" Methods
*/


char * DOFInterfaceID_CreateStandardString( const DOFInterfaceID iid )
{
    char * pBuf = NULL;

    if (iid)
    {
        uint8 iidSize = DOFInterfaceID_GetBytesSize( iid ) - 1;

        /* Use PCRMem_Alloc() and not DOFMem_Alloc() because app caller will be freeing memory with PCRMem_Destroy(). */
        pBuf = (char *)PCRMem_Alloc( (iidSize * 2) + 10 );                   /* 10: [{xx}:{...}] and terminating NULL */
        if (pBuf)
        {
            uint32        len      = 0;
            const uint8 * pIIDData = GetIIDData( iid->iid );

            pBuf[len++] = '[';
            pBuf[len++] = '{';
            pBuf[len++] = OALString_HexDigitToChar( (GetIIDRegistryValue( iid->iid ) >> 4) & 0x0F );
            pBuf[len++] = OALString_HexDigitToChar( (GetIIDRegistryValue( iid->iid )     ) & 0x0F );
            pBuf[len++] = '}';
            pBuf[len++] = ':';
            pBuf[len++] = '{';
            while (iidSize--)
            {
                pBuf[len++] = OALString_HexDigitToChar( (*pIIDData >> 4) & 0x0F );
                pBuf[len++] = OALString_HexDigitToChar( (*pIIDData     ) & 0x0F );
                pIIDData++;
            }
            pBuf[len++] = '}';
            pBuf[len++] = ']';
            pBuf[len] = 0;
        }
    }

    return pBuf;
}


/**
@}
**/
