/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include <dof/interfaceid.h>
#include <dof/item.h>
#include <dof/value.h>
#include <internal/dofinternal.h>
#include <internal/interfaceinternal.h>
#include <internal/iteminternal.h>
#include <internal/type.h>
#include <internal/value.h>


/** @addtogroup DOFInterfaceEvent
    Represents an Event in a DOF Interface.
@{
**/


/*
**  "Private" Type and Method Declarations
*/


/*
**  "Public" Methods, internal
*/


uint8 DOFInterfaceEvent_GetParameterCount( const DOFInterfaceEvent evt )
{
    PCR_Assert( evt != NULL );
    PCR_Assert( ((DOFItem)evt)->itemType == DOFITEM_EVENT );
    return ((DOFItem)evt)->typeCount[0];
}


DOFType * DOFInterfaceEvent_GetParameterTypeList( const DOFInterfaceEvent evt )
{
    PCR_Assert( evt != NULL );
    PCR_Assert( ((DOFItem)evt)->itemType == DOFITEM_EVENT );
    return ((DOFItem)evt)->types[0];
}


DOFValue * DOFInterfaceEvent_CreateValueListFromStream( const DOFInterfaceEvent evt, OALStream stream )
{
    /* Called by OAP on the requestor side to process an incoming event signal report. */
    return DOFInterfaceItem_CreateValueListFromStream( (DOFItem)evt, stream, TRUE );
}


/*
**  "Public" Methods
*/


DOFInterfaceID DOFInterfaceEvent_GetInterfaceID( const DOFInterfaceEvent evt )
{
    PCR_Assert( evt );
    PCR_Assert( ((DOFItem)evt)->itemType == DOFITEM_EVENT );

    return DOFItem_GetInterfaceID( (const DOFItem)evt );
    /* Computes to valid DOFInterfaceID only if evt is valid. DOFInterface_GetEvent() delivers valid evt. */
}


DOFInterface DOFInterfaceEvent_GetInterface( const DOFInterfaceEvent evt )
{
    PCR_Assert( evt );
    PCR_Assert( ((DOFItem)evt)->itemType == DOFITEM_EVENT );

    return DOFItem_GetInterface( (const DOFItem)evt );
    /* Computes to valid DOFInterfaceID only if evt is valid. DOFInterface_GetEvent() delivers valid evt. */
}


DOFItemID DOFInterfaceEvent_GetItemID( const DOFInterfaceEvent evt )
{
    PCR_Assert( evt );
    PCR_Assert( ((DOFItem)evt)->itemType == DOFITEM_EVENT );

    return ((DOFItem)evt)->itemID;
}


/**
@}
**/
