/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include <dof/core.h>
#include <dof/interfaceid.h>
#include <dof/item.h>
#include <dof/value.h>
#include <internal/dofinternal.h>
#include <internal/interfaceinternal.h>
#include <internal/iteminternal.h>
#include <internal/type.h>
#include <internal/value.h>


/** @addtogroup DOFInterfaceException
    Represents an exception in a DOF Interface.
@{
**/


/*
**  "Private" Type and Method Declarations
*/


/*
**  "Public" Methods, internal
*/

DOFValue * DOFInterfaceException_CreateValueListFromStream( const DOFInterfaceException xcp, OALStream stream )
{
    /* Called by OAP on the requestor side to process an incoming exception report. */
    return DOFInterfaceItem_CreateValueListFromStream( (DOFItem)xcp, stream, FALSE );
}

uint8 DOFInterfaceException_GetParameterCount( const DOFInterfaceException xcp )
{
    PCR_Assert( xcp != NULL );
    PCR_Assert( ((DOFItem)xcp)->itemType == DOFITEM_EXCEPTION );

    return ((DOFItem)xcp)->typeCount[0];
}


DOFType * DOFInterfaceException_GetParameterTypeList( const DOFInterfaceException xcp )
{
    PCR_Assert( xcp != NULL );
    PCR_Assert( ((DOFItem)xcp)->itemType == DOFITEM_EXCEPTION );
    return ((DOFItem)xcp)->types[0];
}


/*
**  "Public" Methods
*/


DOFInterfaceID DOFInterfaceException_GetInterfaceID( const DOFInterfaceException xcp )
{
    PCR_Assert( xcp );
    PCR_Assert( ((DOFItem)xcp)->itemType == DOFITEM_EXCEPTION );

    return DOFItem_GetInterfaceID( (const DOFItem)xcp );
}


DOFInterface DOFInterfaceException_GetInterface( const DOFInterfaceException xcp )
{
    PCR_Assert( xcp );
    PCR_Assert( ((DOFItem)xcp)->itemType == DOFITEM_EXCEPTION );

    return DOFItem_GetInterface( (const DOFItem)xcp );
}


DOFItemID DOFInterfaceException_GetItemID( const DOFInterfaceException xcp )
{
    PCR_Assert( xcp );
    PCR_Assert( ((DOFItem)xcp)->itemType == DOFITEM_EXCEPTION );

    return ((DOFItem)xcp)->itemID;
}


/**
@}
**/
