/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include <dof/core.h>
#include <dof/interfaceid.h>
#include <dof/item.h>
#include <internal/stream.h>
#include <dof/value.h>
#include <internal/dofinternal.h>
#include <internal/interfaceinternal.h>
#include <internal/iteminternal.h>
#include <internal/type.h>
#include <internal/value.h>


/** @addtogroup DOFInterfaceMethod
    Represents a Method in a DOF Interface.
@{
**/


/*
**  "Private" Type and Method Declarations
*/



/*
**  "Public" Methods, internal
*/

uint8 DOFInterfaceMethod_GetInputParameterCount( const DOFInterfaceMethod mtd )
{
    PCR_Assert( mtd != NULL );
    PCR_Assert( ((DOFItem)mtd)->itemType == DOFITEM_METHOD );
    return ((DOFItem)mtd)->typeCount[0];
}


DOFType * DOFInterfaceMethod_GetInputParameterTypeList( const DOFInterfaceMethod mtd )
{
    PCR_Assert( mtd != NULL );
    PCR_Assert( ((DOFItem)mtd)->itemType == DOFITEM_METHOD );
    return ((DOFItem)mtd)->types[0];
}

uint8 DOFInterfaceMethod_GetOutputParameterCount( const DOFInterfaceMethod mtd )
{
    PCR_Assert( mtd != NULL );
    PCR_Assert( ((DOFItem)mtd)->itemType == DOFITEM_METHOD );
    return ((DOFItem)mtd)->typeCount[1];
}


DOFType * DOFInterfaceMethod_GetOutputParameterTypeList( const DOFInterfaceMethod mtd )
{
    PCR_Assert( mtd != NULL );
    PCR_Assert( ((DOFItem)mtd)->itemType == DOFITEM_METHOD );
    return ((DOFItem)mtd)->types[1];
}

DOFValue * DOFInterfaceMethod_CreateInputParamValueListFromStream( const DOFInterfaceMethod mtd, OALStream stream )
{
    /* Called by OAP on the provider side to process an incoming Invoke request. */
    uint8         i;
    uint8         elementCount = DOFInterfaceMethod_GetInputParameterCount( mtd );
    DOFType *     types = DOFInterfaceMethod_GetInputParameterTypeList( mtd );
    DOFValue *    retValues = DOFValueListAlloc_Create( elementCount );

    if (retValues)
    {
        for (i = 0; i < elementCount; i++)
        {
            boolean failed;

            retValues[ i ] = DOFType_CreateValue_UnmarshalFromStream( *(types++), stream, &failed );
            if ( failed )
            {
                DOFValueListAlloc_Destroy( elementCount, retValues );
                return NULL;
            }
        }
    }

    return retValues;
}

DOFValue * DOFInterfaceMethod_CreateOutputParamValueListFromStream( const DOFInterfaceMethod mtd, OALStream stream )
{
    /* Called by OAP on the requestor side to process an incoming Invoke response. */
    uint8         i;
    uint8         elementCount = DOFInterfaceMethod_GetOutputParameterCount( mtd );
    DOFType *     types = DOFInterfaceMethod_GetOutputParameterTypeList( mtd );
    DOFValue *    retValues = DOFValueListAlloc_Create( elementCount );

    if (retValues)
    {
        for (i = 0; i < elementCount; i++)
        {
            boolean failed;

            retValues[ i ] = DOFType_CreateValue_UnmarshalFromStream( *(types++), stream, &failed );
            if (failed)
            {
                DOFValueListAlloc_Destroy( elementCount, retValues );
                return NULL;
            }
        }
    }

    return retValues;
}


/*
**  "Public" Methods
*/


DOFInterfaceID DOFInterfaceMethod_GetInterfaceID( const DOFInterfaceMethod mtd )
{
    PCR_Assert( mtd );
    PCR_Assert( ((DOFItem)mtd)->itemType == DOFITEM_METHOD );

    return DOFItem_GetInterfaceID( (const DOFItem)mtd );
}


DOFInterface DOFInterfaceMethod_GetInterface( const DOFInterfaceMethod mtd )
{
    PCR_Assert( mtd );
    PCR_Assert( ((DOFItem)mtd)->itemType == DOFITEM_METHOD );

    return DOFItem_GetInterface( (const DOFItem)mtd );
}


DOFItemID DOFInterfaceMethod_GetItemID( const DOFInterfaceMethod mtd )
{
    PCR_Assert( mtd );
    PCR_Assert( ((DOFItem)mtd)->itemType == DOFITEM_METHOD );

    return ((DOFItem)mtd)->itemID;
}


/**
@}
**/
