/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include <dof/interfaceid.h>
#include <dof/item.h>
#include <internal/stream.h>
#include <dof/value.h>
#include <internal/dofinternal.h>
#include <internal/interfaceinternal.h>
#include <internal/iteminternal.h>
#include <internal/type.h>
#include <internal/value.h>


/** @addtogroup DOFInterfaceProperty
    Represents a Property in a DOF Interface.
@{
**/


/*
**  "Private" Type and Method Declarations
*/




/*
**  "Public" Methods, internal
*/


DOFType DOFInterfaceProperty_GetType( const DOFInterfaceProperty prp )
{
    PCR_Assert(prp != NULL);
    PCR_Assert( ((DOFItem)prp)->itemType == DOFITEM_PROPERTY );
    return (DOFType)(((DOFItem)prp)->types[0]);
}

DOFValue DOFInterfaceProperty_CreateValueFromStream(const DOFInterfaceProperty prp, OALStream stream, boolean * failed)
{
    /* Called by DOFSystem on the provider side to process an incoming Set request.
       Called by OAP on the requestor side to process an incoming Get or Subscribe response, or a subscription report.*/
    DOFType type;

    PCR_Assert( prp );
    type = DOFInterfaceProperty_GetType( prp );

    return DOFType_CreateValue_UnmarshalFromStream( type, stream, failed );
}

boolean DOFInterfaceProperty_IsReadable( const DOFInterfaceProperty prp )
{
    PCR_Assert( prp != NULL );
    PCR_Assert( ((DOFItem)prp)->itemType == DOFITEM_PROPERTY );
    return ((DOFItem)prp)->access & DOFITEM_ACCESS_READ;
}

boolean DOFInterfaceProperty_IsWritable( const DOFInterfaceProperty prp )
{
    PCR_Assert( prp != NULL );
    PCR_Assert( ((DOFItem)prp)->itemType == DOFITEM_PROPERTY );
    return ((DOFItem)prp)->access & DOFITEM_ACCESS_WRITE;
}


/*
**  "Public" Methods
*/


DOFInterfaceID DOFInterfaceProperty_GetInterfaceID( const DOFInterfaceProperty prp )
{
    PCR_Assert( prp );
    PCR_Assert( ((DOFItem)prp)->itemType == DOFITEM_PROPERTY );

    return DOFItem_GetInterfaceID( (const DOFItem)prp );
}


DOFInterface DOFInterfaceProperty_GetInterface( const DOFInterfaceProperty prp )
{
    PCR_Assert( prp );
    PCR_Assert( ((DOFItem)prp)->itemType == DOFITEM_PROPERTY );

    return DOFItem_GetInterface( (const DOFItem)prp );
}


DOFItemID DOFInterfaceProperty_GetItemID( const DOFInterfaceProperty prp )
{
    PCR_Assert( prp );
    PCR_Assert( ((DOFItem)prp)->itemType == DOFITEM_PROPERTY );

    return ((DOFItem)prp)->itemID;
}


/**
@}
**/
