/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include <dof/core.h>
#include <internal/dofinternal.h>
#include <internal/messaging.h>

extern PCRMutex ref_count_mutex;

void OALMessage_Init(OALMessage message, void *recipient, int32 msg_type, uint32 argc, oalmsg_dtor_func_t dtor)
{
    if(message)
    {
        message->refCount = 1;
        message->recipient = recipient;
        message->msg_type = msg_type;
        message->dtor = dtor;
        message->argc = argc;
        message->already = 0;
        message->shutdown = FALSE;
    }
}

OALMessage OALMessage_Create(uint32               argc,
                             void               * recipient,
                             int32                msg_type,
                             oalmsg_dtor_func_t   dtor)
{
    uint32 size = offsetof( struct OALMessage_t, argv ) + ( sizeof(void*) * ( argc ) );

    OALMessage message = (OALMessage)DOFMem_Alloc( size );

    if(message)
    {
        message->refCount = 1;
        message->recipient = recipient;
        message->msg_type = msg_type;
        message->dtor = dtor;
        message->argc = argc;
        message->already = 0;
        message->shutdown = FALSE;
    }

    return message;
}

void OALMessage_Destroy(OALMessage message)
{
    if(message)
    {
        boolean destroy = FALSE;

        PCRMutex_Lock(ref_count_mutex);
        message->refCount--;
        if(!message->refCount)
            destroy = TRUE;
        PCRMutex_Unlock(ref_count_mutex);

        if(destroy)
        {
            if(message->dtor)
                message->dtor(message);
            DOFMem_Free( message );
        }
    }
}

OALMessage OALMessage_Retain(OALMessage message)
{
    OALMessage ret = message;

    if(message)
    {
        PCRMutex_Lock(ref_count_mutex);
        if (message->refCount < 0xFFFFFFFF)
        {
            message->refCount++;
        }
        else
        {
            /* refCount is already at max so return NULL. */
            ret = NULL;
        }
        PCRMutex_Unlock(ref_count_mutex);
    }
    return ret;
}




