/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include <internal/oap_v1.h>

static OALTimer delayTimer;

static DOFOperation exceptionOperation;

OAP_V1 OAP_V1_Create(BlackboardData blackboardData,
                     boolean        isConnectionless)
{
    OAP_V1 ret;

    PCR_Assert(blackboardData);

    ret = (OAP_V1) DOFMem_Alloc(sizeof(OAP_V1_t));

    if(ret != NULL)
    {
        ret->blackboardData = blackboardData;
        ret->isConnectionless = isConnectionless;
    }

    return ret;
}

void OAP_V1_Destroy(OAP_V1 self)
{
    DOFMem_Free(self);
}

boolean OAP_V1_Init(void)
{
    delayTimer = OALTimer_Create(DOF_GetDof()->threadPool, FALSE);
    if(delayTimer != NULL)
    {
        exceptionOperation = DOFMem_Alloc(sizeof(DOFOperation_t));

        if(exceptionOperation != NULL)
        {
            PCRMem_Set(exceptionOperation, 0, sizeof(DOFOperation_t));
            exceptionOperation->ref_count = 1;
            return TRUE;
        }
        OALTimer_Destroy(delayTimer);
    }
    return FALSE;
}

void OAP_V1_Shutdown(void)
{
    DOFMem_Free(exceptionOperation);
    OALTimer_Destroy(delayTimer);
}

OALTimer OAP_V1_GetTimer(void)
{
    return delayTimer;
}

DOFOperation OAP_V1_GetExceptionOperation(void)
{
    return exceptionOperation;
}

OAP_V1_RequestType OAP_V1_DetermineRequestType(DOFObjectID objectId)
{
    PCR_Assert(objectId != NULL);

    if(DOFObjectID_IsMulticast(objectId))
    {
        return OAP_V1_MULTICAST;
    }
    else if(DOFObjectID_IsBroadcast(objectId))
    {
        return OAP_V1_BROADCAST;
    }
    else
    {
        return OAP_V1_UNICAST;
    }
}
