/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_OAP_V1_RECEIVEUTIL_H
#define DOF_OAP_V1_RECEIVEUTIL_H

#include <dof/interfaceid.h>
#include <dof/permission.h>
#include <internal/avl_tree.h>
#include <internal/oap_v1.h>
#include <internal/router.h>

#ifdef  __cplusplus
extern "C" {
#endif

enum OAP_V1_ReceiveStatus
{
    /* Valid status codes */
    OAP_V1_ReceiveStatus_OK,
    OAP_V1_ReceiveStatus_NOOPERATION,
    OAP_V1_ReceiveStatus_DONOTHING,

    /* Invalid/error status codes */
    OAP_V1_ReceiveStatus_INVALID_OPCODE,
    OAP_V1_ReceiveStatus_MEMORY_ERROR,
    OAP_V1_ReceiveStatus_NOT_SUPPORTED,
    OAP_V1_ReceiveStatus_STREAM_ERROR,

    OAP_V1_ReceiveStatus_INVALID_BINDING
};

typedef struct OAP_V1_ReceiveData_t * OAP_V1_ReceiveData;

typedef boolean (*OAP_V1_ReceiveHelper)(OAP_V1_ReceiveData data);
typedef void  (*OAP_V1_ReceiveFunction)(OAP_V1_ReceiveData data);

typedef struct OAP_V1_ReceiveData_t
{
    OALStream                 stream;       /* all */

    OAP_V1                    oap;          /* all */

    enum OAP_V1_ReceiveStatus status;       /* all */

    DOFAddress                aliasAddress;

    DOFObjectID               objectID;     /* interest
                                               advertise
                                               change
                                               get
                                               invoke */
    GUID                      sessionGUID;
    DOFInterfaceID            sessionIID;
    DOFObjectID               providerID;   /* getR
                                               invokeR */
    DOFInterfaceID            interfaceID;  /* interest
                                               advertise
                                               change
                                               get
                                               invoke
                                               define */

    OALAVLTree                ackTree;      /* interest
                                               get
                                               invoke */

    DOFOperation              operation;

    DOFResult                 result;       /* defineR
                                               getR
                                               invokeR */

    DOFItem                   item;

    OALAlias                  alias;        /* interest
                                               change
                                               get
                                               invoke */
    DOFInterface             interface;

    DOFOperationControl      opConfig;

    DOFSecurityScope         secScope;

    OAP_V1_RequestType        requestType;

    const OAP_V1_ReceiveHelper    * functions;

    uint16                    itemID;       /* change
                                               get
                                               invoke */
    uint16                    minimumDelta;
    uint16                    sequenceNumber; /* change */
    uint16                    losslessCount; /* provide */
    uint16                    lossyCount;    /* provide */

    boolean                   opDelay;      /* interest
                                               get
                                               invoke */
    boolean                   unmarshalLater;

    uint8                     firstByte;    /* all */
} OAP_V1_ReceiveData_t;

extern void OAP_V1_ReceiveData_Init(OAP_V1_ReceiveData self,
                                    OAP_V1             oap,
                                    DOFOperation       op);

extern void OAP_V1_ReceiveData_Clear(OAP_V1_ReceiveData self);

extern boolean OAP_V1_Receive_SetOperationConfig(OAP_V1_ReceiveData data);

extern boolean OAP_V1_Receive_VerifyPermissions(OAP_V1_ReceiveData data);

extern boolean OAP_V1_Receive_IterateFunctions(OAP_V1_ReceiveData data);

extern boolean OAP_V1_Receive_VerifyOpID(OAP_V1_ReceiveData data);

extern boolean OAP_V1_Receive_VerifyOperation(OAP_V1_ReceiveData data);

extern boolean OAP_V1_Receive_ReadBinding(OAP_V1_ReceiveData data);

extern boolean OAP_V1_Receive_ReadAlias(OAP_V1_ReceiveData data);

extern boolean OAP_V1_Receive_ReadFullBinding(OAP_V1_ReceiveData data);

extern boolean OAP_V1_Receive_ReadProviderID(OAP_V1_ReceiveData data);

extern boolean OAP_V1_Receive_ReadInterfaceID(OAP_V1_ReceiveData data);

extern boolean OAP_V1_Receive_ReadResponseControl(OAP_V1_ReceiveData data);

extern boolean OAP_V1_Receive_ReadItemID(OAP_V1_ReceiveData data);

extern boolean OAP_V1_Receive_ReadMinimumDelta(OAP_V1_ReceiveData data);

extern boolean OAP_V1_Receive_ReadSequenceNumber(OAP_V1_ReceiveData data);

extern boolean OAP_V1_Receive_ReadLosslessCount(OAP_V1_ReceiveData data);

extern boolean OAP_V1_Receive_ReadLossyCount(OAP_V1_ReceiveData data);

extern boolean OAP_V1_Receive_ReadSessionGUID(OAP_V1_ReceiveData data);

extern boolean OAP_V1_Receive_ReadSessionIID(OAP_V1_ReceiveData data);

extern boolean OAP_V1_Receive_CreateEmptyResult(OAP_V1_ReceiveData data);

extern boolean OAP_V1_Receive_CreateValueResult(OAP_V1_ReceiveData data);

extern boolean OAP_V1_Receive_CreateValueListResult(OAP_V1_ReceiveData data);

#ifdef  __cplusplus
}
#endif

#endif
