/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_OAP_V1_SENDUTIL_H
#define DOF_OAP_V1_SENDUTIL_H

#include <dof/permission.h>
#include <internal/oap_v1.h>
#include <internal/interfaceinternal.h>
#include <internal/operationinternal.h>

#ifdef  __cplusplus
extern "C" {
#endif

enum OAP_V1_SendStatus
{
    /* Valid status codes */
    OAP_V1_SendStatus_OK = 0,
    OAP_V1_SendStatus_OKNOOPID,

    OAP_V1_SendStatus_DONOTSEND,

    /* Invalid/error status codes */
    OAP_V1_SendStatus_INVALID_OPCODE,
    OAP_V1_SendStatus_BADALLOC,
    OAP_V1_SendStatus_STREAMERROR,
    OAP_V1_SendStatus_NOTENOUGHVALUES
};

typedef struct OAP_V1_SendData_t * OAP_V1_SendData;

typedef boolean (*OAP_V1_SendHelper)(OAP_V1_SendData data,
                                     boolean         marshalling);
typedef void  (*OAP_V1_SendFunction)(OAP_V1_SendData data);

#define OAP_V1_SENDDATA_BLANK       0x00
#define OAP_V1_SENDDATA_ALIAS       0x01
#define OAP_V1_SENDDATA_FLAGS       0x02
#define OAP_V1_SENDDATA_INTERNAL    0x04
#define OAP_V1_SENDDATA_FINAL       0x08
#define OAP_V1_SENDDATA_PID         0x10

#define OAP_V1_COMMANDCONTROL_NOEXECUTE   0x01
#define OAP_V1_COMMANDCONTROL_VERBOSITY   0x02
#define OAP_V1_COMMANDCONTROL_HEURISTIC   0x04
#define OAP_V1_COMMANDCONTROL_DELAY       0x08
#define OAP_V1_COMMANDCONTROL_CACHE       0x10
#define OAP_V1_COMMANDCONTROL_ACKLIST     0x20

#define MAX_ACK 10

typedef struct OAP_V1_SendData_t
{
    OALPacket              packet;

    OALStream              stream;

    OAP_V1                 oap;

    DOFOperation           operation;

    DOFInterfaceID         interfaceID;
    DOFObjectID            objectID;
    GUID                   sessionGUID;
    DOFInterfaceID         sessionIID;
    DOFObjectID            acks[MAX_ACK];
    DOFObjectID            providerID;

    DOFResult              result;

    DOFSecurityScope       secScope;

    enum OAP_V1_SendStatus status;

    const OAP_V1_SendHelper    * functions;

    OALStream              valueStream;

    uint32                 alias;

    uint16                 itemID;
    uint16                 exclusionListSize;
    uint16                 iIDSize;
    uint16                 oIDSize;
    uint16                 pIDSize;
    uint16                 sessionGUIDSize;
    uint16                 sessionIIDSize;
    uint16                 cacheDelay;
    uint16                 heuristic;

    boolean                noExecute;
    boolean                isDelayed;
    boolean                exitEarly;

    uint8                  firstByteFlags;
    uint8                  commandControlFlags;
    uint8                  firstByte;
    uint8                  opCode;
    uint8                  aliasSize;
    uint8                  flagsSize;
    uint8                  itemIDSize;
    uint8                  minimumDeltaSize;
    uint8                  sequenceNumberSize;
    uint8                  ackListCount;
    uint8                  verbosity;
    uint8                  hopSize;
} OAP_V1_SendData_t;

extern void OAP_V1_SendData_Init(OAP_V1_SendData    self,
                                 OAP_V1             oap,
                                 DOFOperation       operation,
                                 DOFResult          result,
                                 OAP_V1_RequestType requestType);

extern boolean OAP_V1_Send_VerifyPermissions(OAP_V1_SendData data,
                                             boolean         marshalling);

extern boolean OAP_V1_Send_IterateFunctions(OAP_V1_SendData data);

extern boolean OAP_V1_Send_WriteFirstByte(OAP_V1_SendData data,
                                          boolean         marshalling);

extern boolean OAP_V1_Send_WritePID(OAP_V1_SendData data,
                                    boolean         marshalling);

extern boolean OAP_V1_Send_WriteAlias(OAP_V1_SendData data,
                                      boolean         marshalling);

extern boolean OAP_V1_Send_WriteBinding(OAP_V1_SendData data,
                                        boolean         marshalling);

extern boolean OAP_V1_Send_WriteFullBinding(OAP_V1_SendData data,
                                            boolean         marshalling);

extern boolean OAP_V1_Send_WriteResponseControl(OAP_V1_SendData data,
                                                boolean         marshalling);

extern boolean OAP_V1_Send_WriteItemID(OAP_V1_SendData data,
                                       boolean         marshalling);

extern boolean OAP_V1_Send_WriteMinimumDelta(OAP_V1_SendData data,
                                             boolean         marshalling);

extern boolean OAP_V1_Send_WriteSequenceNumber(OAP_V1_SendData data,
                                               boolean         marshalling);

extern boolean OAP_V1_Send_WriteHopCount(OAP_V1_SendData data,
                                         boolean         marshalling);

extern boolean OAP_V1_Send_SessionGUID(OAP_V1_SendData data,
                                       boolean         marshalling);

extern boolean OAP_V1_Send_SessionIID(OAP_V1_SendData data,
                                      boolean         marshalling);

extern boolean OAP_V1_Send_WriteExceptionError(OAP_V1_SendData data,
                                               boolean         marshalling);

#ifdef  __cplusplus
}
#endif

#endif
