/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include <dof/core.h>
#include <dof/objectid.h>
#include <internal/stream.h>
#include <internal/objectidinternal.h>


/** @addtogroup DOFObjectIDAuthentication
@{
**/


/*
**  "Public" Methods
*/


DOFObjectIDAuthentication DOFObjectIDAuthentication_Create( const DOFObjectID id )
{
    /* Protocol spec Security.8 mandates that an authentication object id not be broadcast and not have attributes. */
    if ( id && !DOFObjectID_IsBroadcast( id ) && !DOFObjectID_HasAttributes( id ) )
        return (DOFObjectIDAuthentication)DOFObjectID_Copy( id );

    return NULL;
}


DOFObjectIDAuthentication DOFObjectIDAuthentication_Copy( const DOFObjectIDAuthentication self )
{
    return (DOFObjectIDAuthentication)DOFObjectID_Copy( (DOFObjectID)self );
}


DOFObjectID DOFObjectIDAuthentication_GetObjectID( const DOFObjectIDAuthentication self )
{
    return (DOFObjectID)self;
}


void DOFObjectIDAuthentication_Destroy( DOFObjectIDAuthentication self )
{
    DOFObjectID_Destroy( (DOFObjectID)self );
}


/*
**  "Private" Methods
*/


DOFObjectIDAuthentication DOFObjectIDAuthentication_Unmarshal( OALStream stream )
{
    DOFObjectID id = DOFObjectID_Unmarshal( stream );

    if (id)
    {
        /* Protocol spec Security.8 mandates that an authentication object id not be broadcast and not have attributes*/
        if(!DOFObjectID_IsBroadcast( id ) && !DOFObjectID_HasAttributes( id ) )
            return (DOFObjectIDAuthentication)id;
        DOFObjectID_Destroy( id );
    }
    return NULL;
}


boolean DOFObjectIDAuthentication_IsEqual( const DOFObjectIDAuthentication oid1, const DOFObjectIDAuthentication oid2 )
{
    return (boolean)(DOFObjectID_Compare( (DOFObjectID)oid1, (DOFObjectID)oid2 ) == 0);
}


/**
@}
**/


