/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include <dof/core.h>
#include <dof/objectid.h>
#include <internal/dofinternal.h>
#include "objectidprivate.h"


/** @addtogroup DOFObjectID
@{
**/


/*
**  "Public" Methods
*/


int32 DOFObjectID_CompareIgnoreBase( const DOFObjectID oid1, const DOFObjectID oid2 )
{
    uint32 oid1Len;
    uint32 oid2Len;

    if (oid1 == oid2)
        return 0;
    if (!oid1)
        return -1;
    if (!oid2)
        return 1;
    oid1Len = oid1->len - DOFObjectID_GetBaseSize( oid1 );
    oid2Len = oid2->len - DOFObjectID_GetBaseSize( oid2 );
    /*
     * Perform straight memcmp().
     * Since the attribute length is first, any discrepancies will be discovered before accessing outside either OID.
     */
    return (oid1Len == oid2Len) ?
            (PCRMem_Compare( oid1->oid + DOFObjectID_GetBaseSize( oid1 ),
                             oid2->oid + DOFObjectID_GetBaseSize( oid2 ), oid1Len )) :
            ((oid1Len > oid2Len) ? 1 : -1);
}


/**
@}
**/
