/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include <dof/core.h>
#include <dof/objectid.h>
#include "objectidprivate.h"


extern PCRMutex ref_count_mutex;


/** @addtogroup DOFObjectID
    DOF Object Identifier
@{
**/


/*
**  "Public" Methods
*/


DOFObjectID DOFObjectID_Copy( DOFObjectID oid )
{
    if (oid)
    {
        boolean overflow = FALSE;

        PCRMutex_Lock( ref_count_mutex );
        oid->refCount++;
        if (!oid->refCount)
        {
            overflow = TRUE;
            --oid->refCount;   /* Fail: restore back to the max refcount value. */
        }
        PCRMutex_Unlock( ref_count_mutex );
        if (!overflow)
            return( oid ); /* Success. */
    }

    return NULL;
}


/**
@}
**/

