/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include <dof/core.h>
#include <dof/objectid.h>
#include "objectidprivate.h"


/** @addtogroup DOFObjectID
    DOF Object Identifier
@{
**/


/*
**  "Public" Methods
*/


DOFObjectIDAttribute DOFObjectID_GetAttribute( const DOFObjectID self, const DOFObjectIDAttributeType id )
{
    DOFObjectIDAttribute retAttributeDescriptor = { DOFOBJECTIDATTRIBUTE_INVALID, 0, NULL };

    if (self && id < DOFOBJECTIDATTRIBUTE_INVALID && DOFObjectID_HasAttributes( self ))
    {
        const uint8 * pNextAttribute = (const uint8 *)self->oid + DOFObjectID_GetBaseSize( self );

        while (1)                            /* Parse through the Attributes and look for Attribute with matching id. */
        {
            /* NOTE: bit 7 is "next attribute present" flag; bits 0-6 are marshalled attribute id. */
            if ((*pNextAttribute & 0x7F) == (uint8)id)
            {
                retAttributeDescriptor.id = (uint8)id;
                retAttributeDescriptor.dataSize = (DOFObjectIDAttributeDataSize)*((const uint8 *)pNextAttribute + 1);
                retAttributeDescriptor.data = (const uint8 *)pNextAttribute + 2;
                break;                       /* Success. */
            }
            if (!(*pNextAttribute & 0x80))
                break;                      /* Fail: no more Attributes */
            pNextAttribute += (2 + *((const uint8 *)pNextAttribute + 1));
        }
    }

    return retAttributeDescriptor;
}


/**
@}
**/

