/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include <dof/core.h>
#include <dof/objectid.h>
#include "objectidprivate.h"


/** @addtogroup DOFObjectID
    DOF Object Identifier
@{
**/


/*
**  "Public" Methods
*/


boolean DOFObjectID_HasAttribute( const DOFObjectID self, const DOFObjectIDAttributeType id )
{
    if (self && id < DOFOBJECTIDATTRIBUTE_INVALID)
    {
        if (DOFObjectID_HasAttributes( self ))
        {
            const uint8 * pNextAttribute = (const uint8 *)self->oid + DOFObjectID_GetBaseSize( self );

            while (1)                              /* Parse through found Attributes- look for matching Attribute id. */
            {
                if ((*pNextAttribute & 0x7F) == (uint8)id)                          /* bits 0-6: stored attribute id. */
                    return TRUE;                   /* Success. */
                if (!(*pNextAttribute & 0x80))                                      /* bit 7: next attribute flag. */
                    break;                         /* No more AVPs. */
                pNextAttribute += (2 + *((const uint8 *)pNextAttribute + 1));
            }
        }
    }

    return FALSE;                                  /* Fail. */
}


/**
@}
**/

