/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include <dof/core.h>
#include <internal/objectidinternal.h>
#include "objectidprivate.h"


/** @addtogroup DOFObjectID
@{
**/


/*
**  "Public" Method, internal
*/


/** @internal
    Determine if the @ref DOFObjectID is of @ref DOFObjectIDClass_t type @ref DOFOBJECTIDCLASS_BROADCAST.

    @note This test is regardless of attributes held. A wildcard OID with no attributes is also a broadcast OID. A
          wildcard OID with attribute(s) is also a multicast OID.
    @param self The @ref DOFObjectID.  This must not be NULL.
    @retval TRUE if the ID is of @ref DOFObjectIDClass_t type @ref DOFOBJECTIDCLASS_BROADCAST.
    @retval FALSE if the ID is not of @ref DOFObjectIDClass_t type @ref DOFOBJECTIDCLASS_BROADCAST.
**/
boolean DOFObjectID_IsWildcard( const DOFObjectID self )
{
    if (self && DOFObjectID_GetIDClass( self ) == DOFOBJECTIDCLASS_BROADCAST)
    {
        PCR_Assert( DOFObjectID_GetDataSize( self ) == 0 ); /*COAL cannot create a broadcast OID with base data length*/
        return TRUE;
    }

    return FALSE;
}


boolean DOFObjectID_IsBroadcast( const DOFObjectID self )
{
    PCR_Assert( self );

    return DOFObjectID_IsWildcard( self ) && !DOFObjectID_HasAttributes( self );
}


boolean DOFObjectID_IsMulticast( const DOFObjectID self )
{
    PCR_Assert( self );

    return DOFObjectID_IsWildcard( self ) && DOFObjectID_HasAttributes( self );
}


boolean DOFObjectID_IsUnicast( const DOFObjectID self )
{
    PCR_Assert( self );

    return (!DOFObjectID_IsWildcard( self ));
}


/**
@}
**/
