/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_OBJECTIDPRIVATE_H
#define DOF_OBJECTIDPRIVATE_H


#ifdef  __cplusplus
extern "C" {
#endif

#include <dof/objectid.h>
#include <internal/dofinternal.h>
#include <internal/objectidinternal.h>


/** @addtogroup DOFObjectID
    DOF Object Identifier
@{
**/


#define MAX_OID_DATA_SIZE                 (63)
#define OID_DATA_LEN_MASK                 (MAX_OID_DATA_SIZE) /*Note: This works as a mask only because 63 is all ones*/
#define MAX_BASE_OID_SIZE                 (DOFOBJECTID_MAX_CLASS_SIZE + 1 + MAX_OID_DATA_SIZE)
#define MIN_AVAIL_BUF_COUNT               (100)/*So ObjectID_ToString need not check overflow til Attribute processing*/
#define MAX_OID_DESCRIPTION_STRING_LENGTH (10000)
#define MIN_OBJECTID_SIZE                 (2)  /* Min OID is Broadcast OID { 0x00, 0x00 }. */
#define DOFOBJECTID_MAX_SIDBYTES_SIZE     (20)              /* Note: this does not include the class or length bytes. */


typedef struct DOFObjectID_t
{
    uint32 refCount;
    uint16 len;                /* Actual length of oid's wire representation. Max is 32707: 4 + 1 + 63 + (127 * 257). */
    uint8  oid[1];             /* Extends beyond end of this defined structure, so oid MUST be last structure member! */
} DOFObjectID_t;


#define DOFOBJECTID_HEADER_SIZE     (offsetof( DOFObjectID_t, oid ))


extern DOFObjectIDSize DOFObjectID_GetBaseSize( const DOFObjectID oid );
extern void            DOFObjectID_InitStruct (       DOFObjectID newObjID, uint32 dataLen );

extern DOFError DOFObjectID_Create_ParseInternal( const char* oid, uint32* size, uint8* buffer );

/**
@}
**/


#ifdef  __cplusplus
}
#endif


#endif /* DOF_OBJECTIDPRIVATE_H */
