/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_PERMISSIONSET_STRUCTS_H
#define DOF_PERMISSIONSET_STRUCTS_H

#include <internal/permissionscope.h>
#include <internal/slist.h>

#ifdef  __cplusplus
extern "C" {
#endif

typedef struct DOFPermission_t
{
    uint32              refCount;
    DOFPermissionType   type;       /* type of permission as defined in spec. */
    uint16              dataLength; /* used to ease calculation of marshal length */
}DOFPermission_t;

typedef struct UnknownTypePermission_t
{
    DOFPermission_t base;
    uint8*          data;
}UnknownTypePermission_t;

typedef struct DOFPermissionBuilder_t
{
    DOFPermission_t base;
    uint8           flags;     /* lower 5 bits actions top 3 bits flags */
    OALAVLTree      objectIDs;
    OALAVLTree      interfaces;
    OALAVLTree      avps;
}DOFPermissionBuilder_t;

typedef struct DefinePermission_t
{
    DOFPermission_t base;
    DOFInterfaceID  iid;
}DefinePermission_t;

typedef struct EPPPermission_t
{
    DOFPermission_t     base;
    DOFObjectID         sid;
}EPPPermission_t;

typedef struct TunPermission_t
{
    DOFPermission_t     base;
    uint32              rdi;
}TunPermission_t;

typedef struct permission_node_t
{
    DOFPermission       perm;
    OALPermissionScope  scope;
}permission_node_t;

typedef struct DOFPermissionSet_t
{
    uint32              refCount;
    uint8               flags;
    uint16              free;               /* the number of permission_nodes_t alloced but not used pass the cnt */
    uint16              permsCnt;           /* number of permissions in set */
    uint32              rbyteLen;           /* wire byte len req */
    uint32              vbyteLen;           /* wire byte len val */
    permission_node_t*  permission_list;    /* < permission_node_t > */
}DOFPermissionSet_t;

#ifdef  __cplusplus
}
#endif

#endif
