/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include <internal/channel.h>
#include <internal/dofinternal.h>
#include <internal/blackboarddata.h>
#include <internal/epp_negative_one.h>
#include <internal/operationinternal.h>

#include <dof/operation.h>
#include <internal/securityscope.h>

void BlackboardData_Init(BlackboardData blackboardData)
{
    /* clear everything. */
    PCRMem_Set( blackboardData, 0, sizeof( BlackboardData_t ) );

    /* Set anything that should not be 0 to start with. */
    blackboardData->secureMarkingID = EPP_NEGONE_NO_SECURE_MARKING_ID;
    /* Only overwritten by epp_negative_one.c's send_ping(), and then with EPP_NEGONE_NO_SECURE_MARKING_ID. */
    blackboardData->securityStateID = INVALID_SECURITYSTATEID;
    blackboardData->remoteDomainID = INVALID_REMOTE_DOMAIN_ID;
}

void BlackboardData_Clear(BlackboardData blackboardData)
{
    /* Destroy anything that was retained or copied. */
    DOFOperationID_Destroy( blackboardData->opID );
    DOFAddress_Destroy(blackboardData->transportAddr);
    OALStream_Release( blackboardData->receiveStream, TRUE );
    OALPacket_Release( blackboardData->sendPacket );
    OALChannel_Release( blackboardData->associatedChannel );
    DOFOperation_Internal_Destroy(blackboardData->providerOp);
    DOFSecurityScope_Destroy(blackboardData->securityScope);
    DOFSecurityScope_Destroy(blackboardData->remoteOpSecurityScope);

    /* Reset to initial state. */
    BlackboardData_Init(blackboardData);
}

BlackboardData BlackboardData_Save(BlackboardData blackboardData)
{
    BlackboardData ret = (BlackboardData)DOFMem_Alloc(sizeof(BlackboardData_t));

    if (ret)
    {
        PCRMem_Copy(ret, blackboardData, sizeof(BlackboardData_t));
    }
    return ret;
}

void BlackboardData_Restore(BlackboardData dest, BlackboardData src)
{
    PCRMem_Copy(dest, src, sizeof(BlackboardData_t));
}

