/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include <dof/core.h>
#include <internal/queue.h>

struct QueueNode_t
{
    void *data;

    struct QueueNode_t *next;
};

struct OALQueue_t
{
    struct QueueNode_t *head;
    struct QueueNode_t *tail;

    uint32 count;
};

OALQueue OALQueue_Create(void)
{
    OALQueue ret = (OALQueue) DOFMem_Alloc(sizeof(struct OALQueue_t));

    if(ret) {
        ret->head = NULL;
        ret->tail = NULL;
        ret->count = 0;
    }

    return ret;
}

void OALQueue_Destroy(OALQueue self)
{
    struct QueueNode_t *temp;

    if(self == NULL)
    {
        return;
    }

    while(self->head != NULL)
    {
        temp = self->head;
        self->head = self->head->next;
        DOFMem_Free(temp);
    }
    DOFMem_Free(self);
}

boolean OALQueue_Enqueue(OALQueue self, void *item)
{
    struct QueueNode_t *temp = (struct QueueNode_t *) DOFMem_Alloc(sizeof(struct QueueNode_t));

    PCR_Assert(self);
    if (temp)
    {
        temp->data = item;
        temp->next = NULL;
        if(self->head == NULL)
        {
            self->head = temp;
            self->tail = temp;
        }
        else
        {
            self->tail->next = temp;
            self->tail = temp;
        }
        ++(self->count);
        return TRUE;
    }

    return FALSE;
}

void* OALQueue_Dequeue(OALQueue self)
{
    PCR_Assert(self && self->head);

    {
        struct QueueNode_t *temp = self->head;

        void *ret = temp->data;

        self->head = self->head->next;
        if(self->head == NULL)
        {
            self->tail = NULL;
        }
        DOFMem_Free(temp);
        --(self->count);

        return ret;
    }

    return NULL; /* no warnings */
}

void* OALQueue_GetFront(OALQueue self)
{
    PCR_Assert(self && self->head);

    return self->head->data;
}

uint32 OALQueue_GetSize(OALQueue self)
{
    PCR_Assert(self);
    return self->count;
}
