/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include "r.h"
#include "advertise.h"
#include <internal/avl_tree.h>
#include <internal/operationinternal.h>
#include <internal/interfaceidinternal.h>
#include <internal/securityscope.h>

/** @internal
    Handle advertise stuff for router.
@{
**/

static OALAVLTree advertise_list = NULL;

boolean advertise_init(void)
{
    advertise_list = OALAVLTree_Create(NULL, NULL, NULL);

    return ( advertise_list != NULL );
}

void advertise_cleanup(void)
{
    OALAVLTree_Destroy(advertise_list, NULL, NULL);
}

static void internalAdvertiseOpClosingRemove(OALObserver self, OALObservable observable, void* ctx)
{
    DOFOperation op = (DOFOperation)ctx;
    PCRMutex_Lock(router_mutex);
    OALAVLTree_Delete(advertise_list, op);
    PCRMutex_Unlock(router_mutex);
    DOFOperation_Internal_Destroy(op);
}

void DOFRouter_AddAdvertise(DOFOperation op)
{
    PCR_Assert(op);
    if( !op->flags.bits.cancelled )
    {
        if(OALAVLTree_Insert(advertise_list, op, op, NULL) == OALAVLTree_InsertSuccess)
        {
            DOFOperation_AddOpWillCloseObserver(op, internalAdvertiseOpClosingRemove);
        }
    }
}

static boolean internalRouteInterestToAdvertise(OALAVLTreeItem_t item, void *arg)
{
    DOFOperation advertise = (DOFOperation)item;
    DOFOperation interest = (DOFOperation)arg;
    if(DOFSecurityScope_IsCompatible(advertise->securityScope, interest->securityScope))
    {
        if ( DOFObjectID_IsBroadcast( interest->object_id.object_id ) ||
             DOFObjectID_MatchesAtLeast_Mcast( advertise->object_id.object_id, interest->object_id.object_id  ) )
        {
            if (DOFInterfaceID_IsWildcard(interest->interface_id) ||
                DOFInterfaceID_IsWildcard(advertise->interface_id) ||
                DOFInterfaceID_IsEqual( advertise->interface_id, interest->interface_id ))
            {
                advertise->sources->channel->routeToProvider(advertise->sources->channel, interest, NULL, advertise->sources->address);
            }
        }
    }
    return FALSE;
}

void DOFRouter_TraverseAdvertiseToRoutingInterests(DOFOperation op)
{
    OALAVLTree_Traverse(advertise_list, internalRouteInterestToAdvertise, op);
}

/**
@}
**/
