/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef _router_really_internal_H
#define _router_really_internal_H

#include <dof/core.h>
#include <internal/slist.h>
#include <internal/router.h>
#include <internal/enqueue.h>
#include <internal/channel.h>
#include <internal/hashtable.h>

#ifdef  __cplusplus
extern "C" {
#endif

extern PCRMutex router_mutex;
extern OALHashtable local_binding_tables[3];
extern OALHashtable remote_binding_tables[3];
extern OALHashtable local_alias_table;


extern void __make_keys(DOFSecurityScope securityScope, DOFInterfaceID iid, DOFObjectID oid, const void *keys[3], uint16 key_sizes[3]);
extern void _make_keys(DOFSecurityScope securityScope, DOFOperation op, OALChannel channel, const void *keys[3], uint16 key_sizes[3]);

#define BINDING_TABLE 0
#define BINDING_TABLE_INTERFACE 1
#define BINDING_TABLE_OBJECT 2
#define BINDING_TABLE_STARSTAR 3

extern boolean tables_add_binding(OALHashtable *tables, provider_binding_table_entry_t *pbte);
extern void tables_remove_binding(OALHashtable *tables, provider_binding_table_entry_t *pbte);

extern provider_binding_table_entry_t *binding_exists(OALHashtable table, DOFOperation op);

extern boolean subscription_add(DOFOperation op, provider_binding_table_entry_t *pbte);
extern boolean subscription_accept(DOFOperation op, DOFResult result);
extern DOFRouterStatusCode_t subscription_requestor_changed(DOFOperation op, DOFResult result);
extern void alias_remove(OALHashtable ht, OALAlias alias);
extern provider_binding_table_entry_t *alias_retrieve(OALHashtable ht, OALAlias alias);
extern boolean alias_insert(OALHashtable ht, OALAlias alias, provider_binding_table_entry_t *pbte);

extern void thread_create(void);
extern boolean subscriptions_init(void);
extern void subscriptions_cleanup(void);

extern void router_subscription_reparent(DOFOperation op, OALChannel oldChannel, DOFAddress address);

extern void interest_notify_new_channel(OALChannel channel);

extern boolean router_IsProvideCompatibleWithInterest( DOFOperation provOp, DOFOperation reqOp );

extern boolean router_forwards(void);

#ifdef  __cplusplus
}
#endif

#endif
