/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include "securegroupprivate.h"

/** @internal
    This routine is used to compute the key used in an OALKeyGrant ticket given the mac and using the KEK as the key.

    @param KEK      The KEK as provided by the auth server.
    @param key      [Output] The computed key.
    @param mac      The mac that will be used to generate the ticket.
    @param groupKey The SGM generated group key that will be used to generate the ticket.
**/
void DOFSecureGroup_GenerateKeyFromMacKekAndGroupKey( OALCryptKey KEK, OALCryptKey key, OALSecureHMACDigest mac,
                                                      OALCryptKey groupKey )
{
    OALCryptExpandedKey ekey;
    uint8 i;

    PCRMem_Copy(key, mac, 32);
    PCR_Validate( DOFCipherRegistry_AES256Cipher );
    DOFCipherRegistry_AES256Cipher->GenerateKeyState( ekey, KEK );
    DOFCipherRegistry_AES256Cipher->Encrypt( ekey, &key[0] );
    DOFCipherRegistry_AES256Cipher->Encrypt( ekey, &key[16] );

    for (i = 0; i < 32; i++)
        key[i] ^= groupKey[i];
}
