/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef _DOF_SERVERCOMMONINTERNAL_H
#define _DOF_SERVERCOMMONINTERNAL_H

#ifdef  __cplusplus
extern "C" {
#endif

typedef struct DOFServerManager_t
{
    DOF             dof;
    PCRMutex        server_mutex;
    OALTimer        server_timer;
    OALEnqueue      server_enqueue;
    uint32          count;
    OALSListNode    oalServers;
}DOFServerManager_t;

typedef struct serverContext_t
{
    OALSListNode                    next;
    OALTimerItem                    timer_item;
    DOFServer                       server;
    DOFServerStartCallback          callback;
    void*                           context;
    DOFException                    except;
    struct ServerContext_Wrapper_t* enqNode;
    struct StartCallbackCall_t*     enqComplete;
}serverContext_t;

typedef struct ServerContext_Wrapper_t
{
    OALEnqueueItem_t super;
    serverContext_t* item;
} ServerContext_Wrapper_t;

typedef struct DOFServerState_t
{
    DOFException exception;
    DOFAddress address;
    DTLServerConfig transportConfig;
    DOFProtocolNegotiator protocolNeg;
    DOFSecurityDesire securityDesire;
    DOFServerStatus status;
    DOFServerType type;
    uint64 byteInputCount;
    uint64 packetInputCount;
    uint64 byteOutputCount;
    uint64 packetOutputCount;
    uint32 sendSilence;
    uint32 receiveSilence;
    uint32 refCount;
    uint32 maxConnections;
}DOFServerState_t;

#if OALSERVER_PRINT_REF_COUNT_DEBUG
#   define OALServer_Create( dof, config, securityConnector, relatedServer ) OALServer_Create_old( (dof), (config), (securityConnector), (relatedServer), __func__, __FILE__, __LINE__ )
    extern OALServer OALServer_Create_old( DOF dof, DOFServerConfig config, DOFSecurityMembrane securityMembrane, DOFServer relatedServer, const char* function, const char* filename, int line);
#else
    extern OALServer OALServer_Create( DOF dof, DOFServerConfig config, DOFSecurityMembrane securityMembrane, DOFServer relatedServer );
#endif

extern void removeAllStateCallback(OALSListNode itemx, void* argx);
extern void removeAllContexts(OALSListNode item, void *user_arg);

extern void DOFServerContext_Destroy( serverContext_t* srvCtx );
extern void DOFServer_StopInternal( DOFServer srv, DOFException exception );
extern void DOFServer_RemoveStateListenerInternal( DOFServer self, DOFServerStateListener callback);
extern void DOFServer_EnqueueListenerCall(DOFServer self, DOFServerStateListener callback, DOFServerState serverState );

extern void OALServer_RealStop( OALServer srv, boolean fromRelated, DOFException exception );

extern void OALServer_RemoveManager( OALServer srv );

#ifdef  __cplusplus
}
#endif

#endif
