/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_SGMP_V1_OPERATION_H
#define DOF_SGMP_V1_OPERATION_H

#include <dof/address.h>
#include <dof/securitymode.h>
#include <internal/key_request.h>
#include <internal/sgmp_v1.h>


#ifdef  __cplusplus
extern "C" {
#endif


typedef struct SGMP_V1_OperationContext_t
{
    DOFObjectID                     group;
    DOFKeyRequest                   keyRequest;
    DOFObjectIDDomain               sgmpdomain;
    OALStream                       I_Rec_Stream;
    OALKeyGrant                     ticket;
    OALStream                       initialState;
    OALChannel                      secureGroupChannel;
    uint8 *                         key;
    OALStream                       sgmpStreamI;
    PCREvent                        keyChangedEvent;

    void *                          requestGroupContext;
    PCRMutex                        requestGroupResponseMutex;
    SGMP_V1_RequestGroupCallback_fn callbackFn;
    uint32                          I_Offset;
    uint32                          Scope_Offset;
    uint32                          tieBreaker;
    uint16                          Tmin;
    uint16                          epoch;
    uint16                          appid;
    uint8                           delay;
    uint8                           desire;
    boolean                         receivedMulticast;
    boolean                         secure;
} SGMP_V1_OperationContext_t;


typedef SGMP_V1_OperationContext_t * SGMP_V1_OperationContext;


extern SGMP_V1_OperationContext SGMP_V1_OperationContext_Create(void);
extern void SGMP_V1_OperationContext_Destroy(void* context);
extern DOFOperation DOFOperation_Create_Heartbeat( OALChannel channel, DOFAddress address, DOFSecurityScope securityScope,
                                                   uint32 tieBreaker, uint16 appid, uint8 desire, uint32 timeout,
                                                   boolean providerOperation );
extern DOFOperation DOFOperation_Create_EpochChanged( OALChannel channel, DOFAddress address, DOFSecurityScope securityScope,
                                                      uint16 epoch, uint16 Tmin, uint32 timeout, boolean providerOperation,
                                                      boolean doSendRetry, uint32 retryPeriod );
extern DOFOperation DOFOperation_Create_Rekey( OALChannel channel, DOFAddress address, DOFSecurityScope securityScope,
                                               OALStream initialState, uint8* key, uint8 delay, uint32 timeout,
                                               boolean providerOperation );
extern DOFOperation DOFOperation_Create_RekeyEpoch( OALChannel channel, DOFAddress address, DOFSecurityScope securityScope,
                                                    uint8* key, OALStream initialState, uint8 delay, uint16 epoch,
                                                    uint32 timeout, boolean providerOperation );
extern DOFOperation DOFOperation_Create_RekeyMerge( OALChannel channel, DOFAddress address, DOFSecurityScope securityScope,
                                                    PCREvent event,  uint8* key, OALStream initialState, uint32 timeout,
                                                    boolean providerOperation );
extern DOFOperation DOFOperation_Create_RequestGroup( OALChannel channel, DOFAddress address,
                                                      DOFSecurityScope securityScope, OALStream streamI,
                                                      DOFKeyRequest keyRequest, DOFObjectIDDomain domain,
                                                      OALStream I_Rec_Stream, uint32 IOffset, uint32 scopeOffset, 
                                                      SGMP_V1_RequestGroupCallback_fn callbackFn, void * rgSendCtx, 
                                                      uint16 epoch, uint32 timeout, boolean providerOperation, 
                                                      PCRMutex reqGRespMutex );


#ifdef  __cplusplus
}
#endif


#endif /* DOF_SGMP_V1_OPERATION_H */
