/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_SGMP_V1_RECEIVEUTIL_H
#define DOF_SGMP_V1_RECEIVEUTIL_H


#include <dof/securitymode.h>
#include <internal/key_request.h>
#include <internal/sgmp_v1.h>
#include <internal/stream.h>


#ifdef  __cplusplus
extern "C" {
#endif


typedef struct SGMP_V1_ReceiveData_t * SGMP_V1_ReceiveData;
typedef void (*SGMP_V1_ReceiveFunction) (SGMP_V1_ReceiveData data);
typedef boolean (*SGMP_V1_ReceiveUtils) (SGMP_V1_ReceiveData data);


typedef enum SGMP_V1_ReceiveStatus_e
{
    SGMP_V1_ReceiveStatus_OK,
    SGMP_V1_ReceiveStatus_OK_DONOTHING,
    SGMP_V1_ReceiveStatus_STREAMERROR,
    SGMP_V1_ReceiveStatus_MEMORYERROR
} SGMP_V1_ReceiveStatus_e;


typedef struct SGMP_V1_ReceiveData_t
{
    SGMP_V1                         sgmp;
    DOFOperation                    operation;
    OALStream                       stream;
    SGMP_V1_ReceiveUtils const *    functions;
    SGMP_V1_ReceiveStatus_e         status;
    DOFKeyRequest                   keyRequest;
    OALKeyGrant                     ticket;
    DOFObjectIDDomain               domain;
    OALStream                       initialState;
    OALChannel                      secureGroupChannel;
    uint8 *                         key;
    uint32                          securityStateID;
    uint32                          tieBreaker;
    uint32                          I_BlockOffset;
    uint16                          Tmin;
    uint16                          appid;
    uint16                          epoch;
    uint8                           delay;
    uint8                           desire;
    uint8                           errorCode;
} SGMP_V1_ReceiveData_t;


extern void SGMP_V1_Receive_InitData(SGMP_V1_ReceiveData data, OALStream stream, SGMP_V1 sgmp);
extern void SGMP_V1_Receive_CleanupData(SGMP_V1_ReceiveData data);
extern boolean SGMP_V1_Receive_IterateFunctions(SGMP_V1_ReceiveData data);
extern boolean SGMP_V1_Receive_ReadDelay(SGMP_V1_ReceiveData data);
extern boolean SGMP_V1_Receive_ReadAppID(SGMP_V1_ReceiveData data);
extern boolean SGMP_V1_Receive_ReadEpoch(SGMP_V1_ReceiveData data);
extern boolean SGMP_V1_Receive_ReadTieBreaker(SGMP_V1_ReceiveData data);
extern boolean SGMP_V1_Receive_ReadDesire(SGMP_V1_ReceiveData data);
extern boolean SGMP_V1_Receive_ReadTmin(SGMP_V1_ReceiveData data);
extern boolean SGMP_V1_Receive_ReadKey(SGMP_V1_ReceiveData data);
extern boolean SGMP_V1_Receive_ReadDomain(SGMP_V1_ReceiveData data);
extern boolean SGMP_V1_Receive_ReadKeyRequestBlock(SGMP_V1_ReceiveData data);
extern boolean SGMP_V1_Receive_ReadTicket(SGMP_V1_ReceiveData data);
extern boolean SGMP_V1_Receive_ReadInitialState(SGMP_V1_ReceiveData data);
extern boolean SGMP_V1_Receive_VerifyOpID(SGMP_V1_ReceiveData data);
extern boolean SGMP_V1_Receive_VerifyNewOpID(SGMP_V1_ReceiveData data);
extern boolean SGMP_V1_Receive_RetrieveOperationFromRouter(SGMP_V1_ReceiveData data);


#ifdef  __cplusplus
}
#endif


#endif /* DOF_SGMP_V1_RECEIVEUTIL_H */

