/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_SGMP_V1_SENDUTIL_H
#define DOF_SGMP_V1_SENDUTIL_H


#include <internal/blackboarddata.h>
#include <internal/key_request.h>
#include <internal/operationinternal.h>
#include <internal/sgmp_v1.h>
#include "sgmp_v1_operation.h"


#ifdef  __cplusplus
extern "C" {
#endif


typedef struct SGMP_V1_SendData_t * SGMP_V1_SendData;
typedef void (*SGMP_V1_SendFunction) (SGMP_V1_SendData data, BlackboardData blackboardData);
typedef boolean (*SGMP_V1_SendUtils) (SGMP_V1_SendData data);


typedef enum SGMP_V1_SendStatus_e
{
    SGMP_V1_SendStatus_OK,
    SGMP_V1_SendStatus_OKNOOPID,
    SGMP_V1_SendStatus_MEMORYERROR,
    SGMP_V1_SendStatus_STREAMERROR,
    SGMP_V1_SendStatus_TIMEOUTERROR,
    SGMP_V1_SendStatus_INVALID_OPCODE
} SGMP_V1_SendStatus_e;


typedef struct SGMP_V1_SendData_t
{
    SGMP_V1_SendUtils const  * functions;
    SGMP_V1_SendStatus_e       status;
    DOFOperation               operation;
    OALPacket                  packet;
    DOFKeyRequest              keyRequest;
    SGMP_V1_OperationContext   context;
    SGMP_V1                    sgmp;
    uint8                      opCode;
} SGMP_V1_SendData_t;


extern void SGMP_V1_Send_Init(SGMP_V1_SendData data, SGMP_V1 sgmp, DOFOperation operation);
extern boolean SGMP_V1_Send_IterateFunctions(SGMP_V1_SendData data);
extern boolean SGMP_V1_Send_WriteFirstByte(SGMP_V1_SendData data);
extern boolean SGMP_V1_Send_I( SGMP_V1_SendData data );
extern boolean SGMP_V1_Send_WriteDelay(SGMP_V1_SendData data);
extern boolean SGMP_V1_Send_WriteAppID(SGMP_V1_SendData data);
extern boolean SGMP_V1_Send_WriteTieBreaker(SGMP_V1_SendData data);
extern boolean SGMP_V1_Send_WriteDesire(SGMP_V1_SendData data);
extern boolean SGMP_V1_Send_WriteTmin(SGMP_V1_SendData data);
extern boolean SGMP_V1_Send_WriteKey(SGMP_V1_SendData data);
extern boolean SGMP_V1_Send_WriteEpoch(SGMP_V1_SendData data);
extern boolean SGMP_V1_Send_WriteTicket(SGMP_V1_SendData data);
extern boolean SGMP_V1_Send_WriteInitialState(SGMP_V1_SendData data);


#ifdef  __cplusplus
}
#endif


#endif /* DOF_SGMP_V1_SENDUTIL_H */
