/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include <internal/slist.h>

/*
 **  "Private" type and method declarations
 */

/* "Object" methods */

void OALSList_Insert(OALSListNode *head, OALSListNode item)
{
    item->next = *head;
    *head = item;
}

boolean OALSList_Remove(OALSListNode *head, OALSListNode item)
{
    if( head && *head && item )
    {
        if(*head==item) 
        {
            *head=(*head)->next;
            return TRUE;
        }
        else
        {
            OALSListNode ptr = (*head)->next;
            OALSListNode old = *head;
    
            for(; ptr/*&&ptr!=item*/; old=ptr, ptr=ptr->next) 
            {
                if(ptr==item) 
                {
                    old->next = ptr->next;
                    ptr->next = 0;
                    return TRUE;
                }
            }
        }
    }
    return FALSE;
}

void OALSList_RemoveAll(OALSListNode *head, OALSListRemoveAll_fn_t fn, void *arg)
{
    OALSListNode ptr = *head;
    OALSListNode next;
    
    while(ptr) {
        next = ptr->next;
        ptr->next = 0;
        if(fn) fn(ptr, arg);
        ptr = next;
    }
    
    *head = 0;
}

boolean OALSList_Traverse(OALSListNode head, OALSListTraversal_fn_t fn, void *arg)
{  
    while(head) {
        OALSListNode next = head->next;
        if(fn(head, arg)) return TRUE; /* early termination */
        head = next;
    }
    
    return FALSE;
}

OALSListNode OALSList_Retrieve(OALSListNode head, OALSListTraversal_fn_t fn, void *arg)
{
    while(head) {
        OALSListNode next = head->next;
        if(fn(head, arg)) return head; /* early termination */
        head = next;
    }
    return NULL;
}

OALSListNode OALSList_FindAndRemove(OALSListNode* head, OALSListNode* start, OALSListTraversal_fn_t fn, void* arg)
{
    PCR_Assert( fn );
    PCR_Assert( head );

    if( *head != NULL )
    {
        OALSListNode ptr;
        if((start == NULL || *start == NULL || *head == *start) && fn(*head, arg))
        {
            ptr = *head;
            *head = (*head)->next;
            ptr->next = NULL;
            if( start )
                (*start) = (*head);
            return ptr;
        }
        else
        {
            OALSListNode old;
            if( start && *start )
                old = *start;
            else
                old = *head;

            ptr = old->next;
            while( ptr )
            {
                if( fn( ptr, arg ) )
                {
                    old->next = ptr->next;
                    ptr->next = NULL;
                    if( start )
                        (*start) = old;
                    return ptr;
                }
                old = ptr;
                ptr = ptr->next;
            }
        }
    }

    if ( start )
        (*start) = NULL;
    return NULL;    
}
