/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include <dof/core.h>
#include <dof/address.h>
#include <internal/addressinternal.h>
#include <internal/channel.h>
#include <internal/systeminternal.h>


/*
**  "Private" Type and Method Declarations
*/


static const struct DTLTransportFns_t tportFns = {
    NULL, /* Open */
    NULL, /* Start */
    NULL, /* GetMaxSize */
    NULL, /* GetMinSize */
    NULL, /* GetAddressType */
    NULL, /* IsConnectionConfigEqual */
    NULL, /* IsServerConfigEqual */
    NULL, /* AddressCompare */
    NULL, /* AddressCopy */
    NULL  /* AddressDestroy */
};


static DTLTransport_t tport = { &tportFns };


/*
**  "Public" Type and Method Declarations
*/


DOFAddress DOFSystemAddress_Create( DOFSystem sys )
{
    DOFAddress retAddr = NULL;

    retAddr = DOFAddress_Create( sizeof( sys ), &sys, &tport );

    return retAddr;
}


DOFSystem DOFSystemAddress_GetSystem( const DOFAddress self )
{
    if ( DOFAddress_GetTransport( self ) == &tport )
    {
        DOFSystem * pSys = (DOFSystem *)DOFAddress_GetAddress( self );

        if (pSys)
            return *pSys;
    }
    return NULL;
}
