/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include <internal/marshal.h>
#include <internal/stream.h>

#include "security_mode.h"

OALSecurityMode OALSecurityMode_Create(uint16 length)
{
    OALSecurityMode mode = (OALSecurityMode) DOFMem_Alloc(offsetof(OALSecurityMode_t, initialState) +
                                                          (sizeof(uint8) * length));
    
    if(mode != NULL)
    {
        mode->length = length;
    }
    return mode;
}

void OALSecurityMode_Destroy(OALSecurityMode self)
{
    if(self != NULL)
    {
        DOFMem_Free(self);
    }
}

OALSecurityMode OALSecurityMode_Unmarshal(OALStream stream)
{
    uint16 length;
    
    if(OALMarshal_StreamUncompress2(stream, &length) == DOFERROR_OK)
    {
        OALSecurityMode ret = OALSecurityMode_Create(length);
        
        if(ret != NULL)
        {
            if(OALStream_Read(stream, length, ret->initialState))
            {
                return ret;
            }
            OALSecurityMode_Destroy(ret);
        }
    }
    return NULL;
}

boolean OALSecurityMode_Marshal(OALSecurityMode self,
                                OALStream       stream)
{
    if(OALMarshal_StreamCompress2(stream, self->length) == DOFERROR_OK)
    {
        return OALStream_Write(stream, self->length, self->initialState);
    }
    return FALSE;
}

uint32 OALSecurityMode_MarshalSize(OALSecurityMode self)
{
    return OALMarshal_GetCompressedValueSize(2, self->length) + self->length;
}
