/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef TEP_V1_OPERATION_H
#define TEP_V1_OPERATION_H

#include <internal/channel.h>
#include <internal/operationinternal.h>
#include <internal/security_common.h>
#include <internal/tep_v1.h>

#include "security_mode.h"

#ifdef  __cplusplus
extern "C" {
#endif

extern DOFOperation DOFOperation_Create_BeginSession(OALChannel               channel,
                                                     DOFAddress               address,
                                                     uint32                   timeout,
                                                     OALSessionInitiatorBlock initiatorBlock,
                                                     OALStream                iBlock,
                                                     TEP_V1                   tep,
                                                     DOFOperationID           operationId,
                                                     boolean                  isConnectionless);

extern DOFOperation DOFOperation_Create_ExtendPermission(OALChannel                 channel,
                                                         DOFAddress                 address,
                                                         uint32                     timeout,
                                                         OALSessionInitiatorBlock   initiatorBlock,
                                                         DOFOperationID             operationId,
                                                         ExtendCompleteCB           callback,
                                                         void                     * context,
                                                         OALStream                  iBlock,
                                                         OALCipherKeyState          keyState,
                                                         boolean                    isConnectionless);

extern DOFOperation DOFOperation_Create_TepRekey(OALChannel       channel,
                                                 DOFAddress       address,
                                                 DOFPermissionSet permissionSet,
                                                 uint8            duration,
                                                 TEP_V1           tep,
                                                 uint32           timeout);

extern DOFOperation DOFOperation_Create_TepRekey_Internal(OALChannel               channel,
                                                          DOFAddress               address,
                                                          OALStream                iBlock,
                                                          OALSessionInitiatorBlock initiatorBlock,
                                                          DOFOperationID           operationId,
                                                          TEP_V1                   tep,
                                                          uint32                   timeout);

extern DOFOperation DOFOperation_Create_Reject(OALChannel     channel,
                                               DOFAddress     address,
                                               DOFOperationID operationId,
                                               boolean        isConnectionless,
                                               uint8          rejectCode);

extern DOFOperation DOFOperation_Create_End(OALChannel channel,
                                            DOFAddress address);

extern DOFOperation DOFOperation_Create_SessionEnding(OALChannel channel,
                                                      DOFAddress address);

typedef struct TEP_V1_OperationContext_t
{
    DOFKeyRequest keyRequest;
    OALStream rBlock;

    TEP_V1 tep;

    sid_t * secureSid;

    OALSessionInitiatorBlock initiatorBlock;
    OALStream iBlock;
    OALKeyGrant initGrant;
    uint8   rejectCode;

    OALStream aBlock;
    OALStream rejectData;
    DOFObjectIDDomain domain;

    uint32          stateIdentifier;
    OALSecurityMode initialState;

    uint8            grantedDuration;
    OALCipherKeyState keyState;
    DOFSecurityScope connectionSecurityScope;

    ExtendCompleteCB extendCompleteCallback;
    void * extendContext;
    boolean calledCallback;

    uint8   sessionkey[32];
    uint8   responderConfirmation[32];
    uint8   initiatorConfirmation[32];

    struct
    {
        unsigned int rekeying            : 1;
        unsigned int extendingPermission : 1;
        unsigned int keyRequested        : 1;
    } flags;
} TEP_V1_OperationContext_t;

typedef TEP_V1_OperationContext_t * TEP_V1_OperationContext;

#define TEPOperation_GetInitiatorPermissions(op)        ((TEP_V1_OperationContext)op->tepContext)->initiatorBlock->keyRequest->permission_set
#define TEPOperation_GetResponderPermissions(op)        ((TEP_V1_OperationContext)op->tepContext)->keyRequest->permission_set

#ifdef  __cplusplus
}
#endif

#endif
