/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#if    !DOF_TEP_V1_SESSION_H
#define DOF_TEP_V1_SESSION_H 1

#include <internal/channel.h>
#include <internal/tep_v1.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef struct TEPV1SessionState_t * TEPV1SessionState;

extern boolean TEP_V1_Session_Init(void);

extern void TEP_V1_Session_Shutdown(void);

extern TEPV1SessionState TEP_V1_SessionLookup(OALChannel channel);

extern void TEP_V1_Session_Remove(TEPV1SessionState state,
                                  OALChannel        channel);

extern TEPV1SessionState TEP_V1_Session_CreateNewState(boolean            isResponder,
                                                       DOFPermissionSet   initialPermissions,
                                                       OALChannel         channel,
                                                       DOFCredentials     otherCredentials,
                                                       DOFAddress         address,
                                                       TEP_V1             tep,
                                                       void             * context);

extern OALTimerItem TEP_V1_Session_CreateTimerItem(OALTimer_fn_t   fn,
                                                   uint32          argc,
                                                   void          * argv[]);

extern TEPV1SessionState TEP_V1_Session_Retain(TEPV1SessionState state);

extern void TEP_V1_Session_Destroy(TEPV1SessionState state);

extern boolean TEP_V1_Session_SetupRekeyTimer(TEPV1SessionState state);

extern void TEP_V1_Session_BeginRekey(TEPV1SessionState state);

extern void TEP_V1_Session_SetDuration(TEPV1SessionState state,
                                       uint8             grantedDuration);

extern void TEP_V1_Session_IncrementKeyCount(TEPV1SessionState state);

extern uint32 TEP_V1_Session_GetKeyCount(TEPV1SessionState state);

extern TEP_V1 TEP_V1_Session_GetTEP(TEPV1SessionState state);

extern void TEP_V1_Session_SetOtherCredentials(TEPV1SessionState state,
                                               DOFCredentials    credentials);

extern DOFCredentials TEP_V1_Session_GetOtherCredentials(TEPV1SessionState state);

extern DOFAddress TEP_V1_Session_GetOtherAddress(TEPV1SessionState state);

extern DOFPermissionSet TEP_V1_Session_GetInitialPermissions(TEPV1SessionState state);

extern void TEP_V1_Session_SetInitialPermissions(TEPV1SessionState state,
                                                 DOFPermissionSet  initialPermissions);

extern void * TEP_V1_Session_GetContext(TEPV1SessionState state);

extern void TEP_V1_Session_SetContext(TEPV1SessionState   state,
                                      void              * newContext);

#ifdef __cplusplus
}
#endif

#endif
