/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include <internal/enqueue.h>

#include "internal_queue.h"

InternalQueue InternalQueue_Create(void)
{
    InternalQueue retQueue = DOFMem_Alloc(sizeof(struct InternalQueue_t));

    if(retQueue != NULL)
    {
        retQueue->head = NULL;
        retQueue->tail = NULL;
        retQueue->size = 0;
    }
    return retQueue;
}

void InternalQueue_Destroy(InternalQueue queue)
{
    PCR_Assert(queue != NULL);
    PCR_Assert(queue->head == NULL);
    PCR_Assert(queue->tail == NULL);
    PCR_Assert(queue->size == 0);

    DOFMem_Free(queue);
}

void InternalQueue_Enqueue(InternalQueue     queue,
                           OALThreadPoolItem node)
{
    PCR_Assert(queue != NULL);
    PCR_Assert(node != NULL);
    PCR_Assert(node->next == NULL);

    if(queue->tail != NULL)
    {
        queue->tail->next = node;
    }
    else
    {
        queue->head = node;
    }
    queue->tail = node;

    ++(queue->size);
}

OALThreadPoolItem InternalQueue_Dequeue(InternalQueue queue)
{
    OALThreadPoolItem temp;

    PCR_Assert(queue != NULL);
    PCR_Assert(queue->head != NULL);
    temp = queue->head;
    queue->head = queue->head->next;
    if(queue->head == NULL)
    {
        queue->tail = NULL;
    }
    --(queue->size);
    temp->next = NULL;      /* Not in the queue, but Enqueue often requeues */
    return temp;
}

uint32 InternalQueue_GetSize(InternalQueue queue)
{
    PCR_Assert(queue != NULL);

    return queue->size;
}
