/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_THREADPOOL_INTERNAL_H
#define DOF_THREADPOOL_INTERNAL_H

#include <internal/queue.h>

#include "internal_queue.h"

#ifdef  __cplusplus
extern "C" {
#endif

/**
 Internal thread pool instance data.
 **/
struct OALThreadPool_t {
    PCRCondition    cond;       /**< Condition variable to wakeup worker threads when new work arrives (threads efficiently sleep when there is no work). **/
    InternalQueue   queue;      /**< Queue of work to be handled by the threadpool. **/
    uint32          use_count;  /**< How many enqueues/timers have spawned off this threadpool? */
    uint16          count;      /**< Count of threads in the pool. **/
    struct {
        unsigned int running    : 1;   /**< Boolean flag to indicate the pool is running. **/
        unsigned int accepting  : 1; /**< Boolean flag to indicate the pool is accepting work items. **/
    }flags;
    PCRThread       thread[1];  /**< Thread handle array (when the structure is allocated, space is reserved for the exact number of threads). **/
};

/**
    Add work to be processed by the thread pool.
    Work for a thread pool consists of a function and a list of arguments.

    Called with threadpool mutex already locked, and it must stay locked.

    If a worker thread is available, processing will start almost immediately.
    If all of the worker threads are currently busy, the work is queued for
    later processing when a worker thread becomes available.

    @param  tpool   The thread pool to which work is to be added.  Must not be NULL.
    @param  item    An initialized threadpool item.  Must not be NULL.

    @return TRUE if the work items was enqueued, or FALSE if the work item
    could not be enqueued.
**/
extern void          OALThreadPool_RunInternal(OALThreadPool tpool, OALThreadPoolItem item);

#ifdef  __cplusplus
}
#endif


#endif
