/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include "trp_v1_operation.h"
#include "trp_v1_utils.h"
#include <dof/cfg.h>

DOFOperation TRP_V1_CreateOperation(DOFOperationType type, TRP_V1_Data context, DOFOperationID opID, uint32 timeout )
{
    if(TRP_V1_IsRunning())
    {
        if(opID) 
        {
            DOFOperation ret = (DOFOperation)DOFMem_Alloc(sizeof(DOFOperation_t)); 

            CancelOperationContext_t * cancelContext;

            if(!ret)
                return NULL;

            cancelContext = DOFOperationManager_CreateCancelContext(ret);
            if(cancelContext != NULL)
            {
                PCRMem_Set(ret, 0, sizeof(DOFOperation_t));
                ret->sources = (DOFOperationSource)DOFMem_Alloc(sizeof(DOFOperationSource_t));
                if( ret->sources )
                {
                    PCRMem_Set(ret->sources, 0, sizeof(DOFOperationSource_t));
                    ret->ref_count = 1;

                    ret->object_id.object_id = DOFObjectID_Copy( DOFOperationID_GetLocalOID() );
                    if( ret->object_id.object_id )
                    {
                        ret->securityScope = DOFSecurityScope_Copy(DOF_GetDefaultSecurityScope(DOF_GetDof()));
                        if( ret->securityScope )
                        {
                            ret->opID = opID;
                            ret->operation_type = type;
                            ret->sources->oal_timeout = timeout;
                            ret->context = (void*)context;
                            ret->cancelContext = cancelContext;
                
                            return ret;
                        }
                        DOFObjectID_Destroy( ret->object_id.object_id );
                    }
                    DOFMem_Free( ret->sources );
                }
                DOFMem_Free(cancelContext);
            }
            DOFMem_Free(ret);
        }
    }
    return NULL;
}

DOFOperation DOFOperation_TRP_V1_New(DOFOperationType type, TRP_V1_Data context, uint32 timeout )
{
    if(TRP_V1_IsRunning())
    {
        DOFOperationID opID = DOFOperationID_CreateLocal(); 
        return TRP_V1_CreateOperation(type, context, opID, timeout);
    }
    return NULL;
}

