/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_TRP_V1_RECEIVE_UTIL_H
#define DOF_TRP_V1_RECEIVE_UTIL_H

#include <internal/marshal.h>

#include "trp_v1_receive.h"
#include "trp_v1_utils.h"

#define TRP_V1_Read1ByteValue( stream, valueHolder ) OALStream_Read8(stream, &valueHolder)
#define TRP_V1_Read2ByteValue( stream, valueHolder ) OALStream_Read16(stream, &valueHolder)
#define TRP_V1_ReadSignature( stream, signature ) OALStream_Read(stream, 32, signature)
#define OALStateIdentifier_Read(stream, stateID)  OALMarshal_StreamUncompress4(stream, &stateID)
#define OALStateIdentifier_Write(stateID, stream) OALMarshal_StreamCompress4(stream, stateID)
#define OALStateIdentifier_GetStreamSize(stateID) OALMarshal_GetCompressedValueSize(4, stateID)

#ifdef  __cplusplus
extern "C" {
#endif

typedef uint32 OALStateIdentifier;

extern boolean TRP_V1_PermissionSet_Read(OALStream stream, DOFPermissionSet* permSet);
extern boolean TRP_V1_ReadPartition(OALStream stream, DOFPartition partition);
extern boolean DOFSecurityMode_Read(OALStream stream, DOFSecurityMode* config);
extern uint32 TRP_V1_GetObjectIDListCount( OALStream stream );
extern boolean TRP_V1_ReadObjectIDList( OALStream stream, DOFObjectIDList objIDList );
extern boolean TRP_V1_ReadStateID( OALStream stream, OALStateIdentifier* stateID );
extern void TRP_V1_StartHash( TRP_V1_Data data, DOFCredentials creds );
extern void TRP_V1_HASH( TRP_V1_ReceiveData_t opData, uint8* start);
extern boolean TRP_V1_Unmarshal_KeyRequest( OALStream stream, DOFKeyRequest* keyRequest, DOFObjectIDDomain domain );
extern boolean TRP_V1_Unmarshal_KeyGrant( OALStream stream, OALKeyGrant* keyGrant );
extern boolean TRP_V1_Unmarshal_KeyGrant_ToStream( OALStream from,  OALStream to );
extern boolean TRP_V1_Unmarshal_SourceNodeID( OALStream stream, DOFObjectIDSource* secNode );
extern boolean TRP_V1_Unmarshal_SecurityNodeID( OALStream stream, DOFObjectIDAuthentication* secNode );
extern boolean TRP_V1_Unmarshal_SecurityDomainID( OALStream stream, DOFObjectIDDomain* secDomain );
extern boolean TRP_V1_Unmarshal_KEKReqBlock( OALStream stream, TRP_V1_KEKData_t* kekData, DOFObjectIDDomain domain );
extern boolean TRP_V1_Unmarshal_KEKInitBlock( OALStream stream, TRP_V1_KEKData_t* kekData );
extern boolean TRP_V1_Unmarshal_SessInitBlock( OALStream stream, TRP_V1_SessionData_t* sessData, DOFObjectIDDomain domain );
extern boolean TRP_V1_Unmarshal_SessRespBlock( OALStream stream, TRP_V1_SessionData_t* sessData, DOFObjectIDDomain domain );
extern boolean TRP_V1_Unmarshal_SessAuthBlock( OALStream stream, TRP_V1_SessionData_t* sessData );
extern boolean TRP_V1_Unmarshal_SecurityScopeInit( OALStream stream, TRP_V1_SecurityScopeData_t* ssData );
extern boolean TRP_V1_Unmarshal_CredentialResolution( OALStream stream, TRP_V1_CredentialData_t* credData );

#ifdef  __cplusplus
}
#endif

#endif
