/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include "trp_v1_utils.h"
#include "trp_v1_receive_util.h"
#include <internal/domaininternal.h>

boolean TRP_V1_MarshalLocalASRequest( TRP_V1_Data data )
{
    boolean ret = TRUE;

    switch(data->opCode)
    {
    case TRP_V1_OP_KEK:
        {
            data->I_len = DOFKeyRequest_MarshallByteLength(data->RequestData.kekData->initKeyRequest, TRUE) +
                          DOFObjectID_GetBytesSize(DOFObjectIDAuthentication_GetObjectID(data->RequestData.kekData->groupID));
            data->stream = OALStream_CreateRW(data->I_len);

            if(!data->stream ||
               !DOFKeyRequest_Marshal(data->stream, data->RequestData.kekData->initKeyRequest, TRUE) ||
               !DOFObjectID_Marshal(DOFObjectIDAuthentication_GetObjectID(data->RequestData.kekData->groupID), data->stream))
            {
                goto FAIL;
            }
            data->flags.bits.retainedStream = 1;
            data->I = (uint8*)OALStream_GetReadPtr(data->stream);
        }
        break;
    case TRP_V1_OP_RANDOM:
        {
            data->I_len = DOFKeyRequest_MarshallByteLength(data->RequestData.randomData->initKeyRequest, TRUE);
            data->stream = OALStream_CreateRW(data->I_len);

            if(!data->stream ||
               !DOFKeyRequest_Marshal(data->stream, data->RequestData.randomData->initKeyRequest, TRUE))
            {
                goto FAIL;
            }
            data->flags.bits.retainedStream = 1;
            data->I = (uint8*)OALStream_GetReadPtr(data->stream);
        }
        break;
    case TRP_V1_OP_SESSION:
        {
            data->I_len = OALStream_GetDataSize(data->RequestData.sessionData->I);
            data->R_len = OALStream_GetDataSize(data->RequestData.sessionData->R);
            data->I = (uint8*)OALStream_GetReadPtr(data->RequestData.sessionData->I);
            data->R = (uint8*)OALStream_GetReadPtr(data->RequestData.sessionData->R);
        }
        break;
    case TRP_V1_OP_SECURITY_SCOPE:
    case TRP_V1_OP_SECURITY_SCOPE_EXT:
        {
            data->I_len = 1 +
                          DOFKeyRequest_MarshallByteLength(data->RequestData.securityScopeData->initKeyRequest, TRUE) +
                          DOFObjectID_GetBytesSize(DOFObjectIDAuthentication_GetObjectID(data->RequestData.securityScopeData->secNodeID))
                          + ((data->opCode == TRP_V1_OP_SECURITY_SCOPE_EXT)?(1+data->RequestData.securityScopeData->modeLen):0);
            data->stream = OALStream_CreateRW(data->I_len);

            if(!data->stream ||
               !OALStream_Write8(data->stream, data->RequestData.securityScopeData->grantedDuration) ||
               !DOFKeyRequest_Marshal(data->stream, data->RequestData.securityScopeData->initKeyRequest, TRUE) ||
               !DOFObjectID_Marshal(DOFObjectIDAuthentication_GetObjectID(data->RequestData.securityScopeData->secNodeID), data->stream))
            {
                goto FAIL;
            }

            if(data->opCode == TRP_V1_OP_SECURITY_SCOPE_EXT)
            {
                if(!OALStream_Write8(data->stream, data->RequestData.securityScopeData->secType) ||
                   !OALStream_Write(data->stream, data->RequestData.securityScopeData->modeLen, data->RequestData.securityScopeData->mode))
                {
                    goto FAIL;
                }
            }

            data->flags.bits.retainedStream = 1;
            data->I = (uint8*)OALStream_GetReadPtr(data->stream);
        }
        break;
    case TRP_V1_OP_CREDENTIAL:
        {
            data->stream = OALStream_CreateRW(DOFCredentials_MarshalSize(data->RequestData.credentialData->credentials,TRUE));
            data->RequestData.credentialData->localcredentials = data->RequestData.credentialData->credentials;
            data->RequestData.credentialData->credentials = NULL;
            if(!data->stream ||
               !DOFCredentials_Internal_Marshal(data->RequestData.credentialData->localcredentials, data->stream, TRUE) ||
               (DOFCredentials_Unmarshal(&data->RequestData.credentialData->credentials, data->secDomain, data->stream, TRUE) != CREDENTIALS_SUCCESSFUL_UNMARSHAL))
            {
                DOFCredentials_Destroy(data->RequestData.credentialData->localcredentials);
                goto FAIL;
            }
            OALStream_Release(data->stream, FALSE);
            data->stream = NULL;
        }
        break;
    case TRP_V1_OP_LOCAL_DOMAIN:
        {
            data->RequestData.domainData->responseA_len = DOFObjectID_GetBytesSize(DOFObjectIDAuthentication_GetObjectID(data->RequestData.domainData->remNodeID));
            data->stream = OALStream_CreateRW(data->RequestData.domainData->responseA_len);

            if( !data->RequestData.domainData->initRemoteKeyGrant )
            {
                OALStream_SetReadOffset(data->RequestData.domainData->rd, data->I_len);
                data->RequestData.domainData->initRemoteKeyGrant = OALKeyGrant_Unmarshal(data->RequestData.domainData->rd);
            }

            if(!data->stream ||
               !DOFObjectID_Marshal(DOFObjectIDAuthentication_GetObjectID(data->RequestData.domainData->remNodeID), data->stream))
            {
                goto FAIL;
            }
            data->flags.bits.retainedStream = 1;
            data->RequestData.domainData->responseA = (uint8*)OALStream_GetReadPtr(data->stream);
        }
        break;
    case TRP_V1_OP_REMOTE_DOMAIN:
        {
            data->I_len = DOFKeyRequest_MarshallByteLength(data->RequestData.domainData->initRemoteKeyRequest, TRUE);
            data->RequestData.domainData->rd = OALStream_CreateRW(data->I_len+64);

            data->I = (uint8*)OALStream_GetReadPtr(data->RequestData.domainData->rd);
            if(!data->RequestData.domainData->rd ||
               !DOFKeyRequest_Marshal(data->RequestData.domainData->rd, data->RequestData.domainData->initRemoteKeyRequest, TRUE))
            {
                goto FAIL;
            }
        }
        break;
    case TRP_V1_OP_VALIDATE_CREDS:
        {}
        break;
    default:
        {
            ret = FALSE;
        }
        break;
    }
    return ret;
FAIL:
    OALStream_Release(data->stream, FALSE);
    data->stream = NULL;
    return FALSE;
}

boolean TRP_V1_UnmarshalLocalASRequest( TRP_V1_Data data )
{
    boolean ret = FALSE;
    uint8 opCode = 0;

    OALStream_Read8(data->A, &opCode);

    switch(opCode)
    {
    case TRP_V1_OP_REJECT:
        {
            data->opCode = TRP_V1_OP_REJECT;
            if( OALStream_Read8(data->A, &data->errorCode))
            {
                if( data->errorCode == TRP_V1_BAD_MODE )
                {
                    uint32 len = OALStream_GetDataSize(data->A);
                    if( len > 0 )
                    {
                        const uint8* ptr = OALStream_GetReadPtr(data->A);
                        data->errorData = DOFMem_Alloc(len);
                        if( !data->errorData )
                        {
                            return FALSE;
                        }
                        PCRMem_Copy(data->errorData, ptr, len);
                    }
                }
                data->trp_status = DOFERROR_OK;
            }
            return TRUE;
        }
        break;
    case TRP_V1_OP_KEK:
        {
            TRP_V1_KEKData_t* kekData = data->RequestData.kekData;

            if(TRP_V1_Unmarshal_KeyGrant( data->A, &kekData->initKeyGrant ) )
            {
                data->A_pointer = (uint8*)OALStream_GetReadPtr(data->A);
                if( TRP_V1_Unmarshal_KEKInitBlock( data->A, kekData ) )
                {
                    data->A_len = (OALStream_GetReadPtr(data->A) - data->A_pointer);
                    ret = TRUE;
                }
            }
        }
        break;
    case TRP_V1_OP_RANDOM:
        {
            TRP_V1_RandomData_t* randData = data->RequestData.randomData;

            if(TRP_V1_Unmarshal_KeyGrant( data->A, &randData->initKeyGrant ) )
            {
                ret = TRUE;
            }
        }
        break;
    case TRP_V1_OP_SESSION:
        {
            TRP_V1_SessionData_t* sessData = data->RequestData.sessionData;
            OALStream old = data->A;

            if(TRP_V1_Unmarshal_KeyGrant( data->A, &sessData->respKeyGrant ) &&
               TRP_V1_Unmarshal_KeyGrant( data->A, &sessData->initKeyGrant ) )
            {
                data->A_len = OALStream_GetDataSize(old);
                data->A = OALStream_CreateRW(data->A_len);
                if( data->A )
                {
                    OALStream_Write(data->A,data->A_len,OALStream_GetReadPtr(old));
                    data->A_pointer = (uint8*)OALStream_GetReadPtr(data->A);
                    if( TRP_V1_Unmarshal_SessAuthBlock( old, sessData ) )
                    {
                        OALStream_Release(old,FALSE);
                        ret = TRUE;
                    }
                }
            }
        }
        break;
    case TRP_V1_OP_SECURITY_SCOPE:
    case TRP_V1_OP_SECURITY_SCOPE_EXT:
        {
            TRP_V1_SecurityScopeData_t* ssData = data->RequestData.securityScopeData;

            if(TRP_V1_Unmarshal_KeyGrant(data->A, &ssData->initKeyGrant))
            {
                data->A_pointer = (uint8*)OALStream_GetReadPtr(data->A);
                if(OALStream_Read8(data->A, &ssData->grantedDuration) &&
                   TRP_V1_Unmarshal_SecurityScopeInit(data->A, ssData))
                {
                    data->A_len = (OALStream_GetReadPtr(data->A) - data->A_pointer);
                    ret = TRUE;
                }
            }
        }
        break;
    case TRP_V1_OP_CREDENTIAL:
        {
            TRP_V1_CredentialData_t* credData = data->RequestData.credentialData;
            uint8 currentStage = 2;
            OALStream side1 = NULL;
            OALStream side2 = NULL;

            if(DOFCredentials_Unmarshal(&credData->localcredentials, data->secDomain, data->A, FALSE ) == CREDENTIALS_SUCCESSFUL_UNMARSHAL)
            {
                while(!DOFCredentials_IsResolved(credData->localcredentials) && currentStage > 0)
                {
                    side1 = OALStream_CreateRW( DOFCredentials_MarshalSize(credData->localcredentials, TRUE));
                    if( side1 )
                    {
                        DOFCredentials_Internal_Marshal( credData->localcredentials, side1, TRUE);
                        DOFCredentials_Unmarshal(&credData->credentials, data->secDomain, side1, TRUE);
                        side2 = OALStream_CreateRW( DOFCredentials_MarshalSize(credData->credentials, FALSE));
                        if( side2 )
                        {
                            DOFCredentials_Internal_Marshal(credData->credentials, side2, FALSE);
                            DOFCredentials_Unmarshal(&credData->localcredentials, data->secDomain, side2, FALSE);
                            OALStream_Release(side2, FALSE);
                            --currentStage;
                        }
                        else
                        {
                            currentStage = 0; /* memory error exit early */
                        }
                        OALStream_Release(side1, FALSE);
                    }
                    else
                    {
                        currentStage = 0; /* memory error exit early */
                    }
                }
            }

            if( DOFCredentials_IsResolved(credData->localcredentials) )
            {
                ret = TRUE;
            }

            DOFCredentials_Destroy(credData->credentials);
            credData->credentials = credData->localcredentials;
            credData->localcredentials = NULL;
        }
        break;
    case TRP_V1_OP_LOCAL_DOMAIN:
        {
            TRP_V1_DomainData_t* domData = data->RequestData.domainData;

            if(TRP_V1_Unmarshal_KeyGrant(data->A, &domData->initLocalKeyGrant))
            {
                data->A_pointer = (uint8*)OALStream_GetReadPtr(data->A);
                if( DOFERROR_OK == OALMarshal_StreamUncompress4( data->A, &domData->localDomainRespBlock) )
                {
                    data->A_len = (OALStream_GetReadPtr(data->A) - data->A_pointer);
                    ret = TRUE;
                }
            }
        }
        break;
    case TRP_V1_OP_REMOTE_DOMAIN:
        {
            TRP_V1_DomainData_t* domData = data->RequestData.domainData;

            if(TRP_V1_Unmarshal_KeyGrant_ToStream(data->A, domData->rd))
            {
                OALStream_AdjustReadOffset(data->A,-64);
                if(TRP_V1_Unmarshal_KeyGrant(data->A, &domData->initRemoteKeyGrant) &&
                   TRP_V1_Unmarshal_SecurityNodeID(data->A, &domData->remNodeID))
                {
                    ret = TRUE;
                }
            }
        }
        break;
    case TRP_V1_OP_VALIDATE_CREDS:
        {
            TRP_V1_ValidateCredsData_t* vcData = data->RequestData.validateData;

            vcData->bLen = OALStream_GetDataSize(data->A);
            vcData->validB = PCRMem_Alloc(vcData->bLen);
            if( vcData->validB )
            {
                if( OALStream_Read(data->A, vcData->bLen, vcData->validB) )
                {
                    ret = TRUE;
                }
            }
        }
        break;
    default:
        break;
    }
    return ret;
}

static boolean KeyGrant_Verify( DOFCredentials creds, OALKeyGrant kg, DOFObjectIDDomain domainID, uint8* I, uint32 I_len, uint8* R, uint32 R_len, uint8* A, uint32 A_len, uint8* Ks )
{
    OALSecureHMACDigest digest;
    OALSecureHMACContext ctx;
    DOFObjectID domain = DOFObjectIDDomain_GetObjectID(domainID);

    OALKeyGrant_ExtractK( kg, DOFCredentials_GetKey(creds), Ks );

    OALSecureHMAC_Start( &ctx, (uint8*)DOFCredentials_GetKey(creds));
    OALSecureHMAC_Digest( &ctx, (domain ? DOFObjectID_GetBytesSize( domain ) : 0), DOFObjectID_GetBytes( domain ));
    if(I_len > 0)
        OALSecureHMAC_Digest( &ctx, I_len, I );
    if(R_len > 0)
        OALSecureHMAC_Digest( &ctx, R_len, R );
    if(A_len > 0)
        OALSecureHMAC_Digest( &ctx, A_len, A );
    OALSecureHMAC_Digest( &ctx, 32, Ks );
    OALSecureHMAC_Finish( &ctx, digest );

    if(PCRMem_Compare(digest,kg->mac,32) != 0)
    {
        PCRMem_Set( Ks, 0, 32);
        return FALSE;
    }
    return TRUE;
}

boolean TRP_V1_ValidateLocalASRequest( TRP_V1_Data data, DOFObjectIDDomain domain )
{
    switch(data->opCode)
    {
    case TRP_V1_OP_REJECT:
        return TRUE;
        break;
    case TRP_V1_OP_KEK:
        return KeyGrant_Verify(data->RequestData.kekData->initKeyRequest->credentials, data->RequestData.kekData->initKeyGrant, domain, data->I, data->I_len, NULL, 0, data->A_pointer, data->A_len, data->k);
        break;
    case TRP_V1_OP_RANDOM:
        return KeyGrant_Verify(data->RequestData.randomData->initKeyRequest->credentials, data->RequestData.randomData->initKeyGrant, domain, data->I, data->I_len, NULL, 0, data->A_pointer, data->A_len, data->k);
        break;
    case TRP_V1_OP_SESSION:
        return KeyGrant_Verify(data->RequestData.sessionData->respBlock->credentials, data->RequestData.sessionData->respKeyGrant, domain, data->I, data->I_len, data->R, data->R_len, data->A_pointer, data->A_len, data->k);
        break;
    case TRP_V1_OP_SECURITY_SCOPE:
    case TRP_V1_OP_SECURITY_SCOPE_EXT:
        return KeyGrant_Verify(data->RequestData.securityScopeData->initKeyRequest->credentials, data->RequestData.securityScopeData->initKeyGrant, domain, data->I, data->I_len, NULL, 0, data->A_pointer, data->A_len, data->k);
        break;
    case TRP_V1_OP_CREDENTIAL:
        return TRUE;
        break;
    case TRP_V1_OP_LOCAL_DOMAIN:
        return KeyGrant_Verify(data->RequestData.domainData->initRemoteKeyRequest->credentials, data->RequestData.domainData->initLocalKeyGrant, domain, data->I, data->I_len, NULL, 0, data->A_pointer, data->A_len, data->k);
        break;
    case TRP_V1_OP_REMOTE_DOMAIN:
        return TRUE;
        break;
    case TRP_V1_OP_VALIDATE_CREDS:
        return TRUE;
        break;
    default:
        break;
    }
    return FALSE;
}

TRP_V1_Data TRP_V1_UnmarshalForwardedLocalASRequest( TRP_V1_Data data, DOFAuthenticator domain )
{
    TRP_V1_Data ret;
    TRP_V1_PassData_t* passData = data->RequestData.passData;

    switch( data->opCode )
    {
    case TRP_V1_OP_CREDENTIAL:
        {
            OALStream stream = NULL;
            DOFObjectIDDomain obj = NULL;
            uint8 opcode;
            ret = (TRP_V1_Data_t*)DOFMem_Alloc(sizeof(TRP_V1_Data_t) + sizeof(TRP_V1_CredentialData_t));
            if( !ret )
                return NULL;

            PCRMem_Set(ret,0,sizeof(TRP_V1_Data_t) + sizeof(TRP_V1_CredentialData_t));
            ret->RequestData.credentialData = (TRP_V1_CredentialData_t*)(((uint8*)ret) + sizeof(TRP_V1_Data_t));

            ret->domain = DOFAuthenticator_Copy(domain);
            if( !ret->domain )
            {
                DOFMem_Free( ret );
                return NULL;
            }

            ret->secDomain = DOFObjectIDDomain_Copy(data->secDomain);
            if( !ret->secDomain )
            {
                DOFAuthenticator_Destroy( ret->domain );
                DOFMem_Free(ret);
                return NULL;
            }

            stream = OALStream_CreateRO(passData->streamData,passData->length);
            if(stream == NULL || !OALStream_Read8(stream,&opcode) || ((obj = DOFObjectIDDomain_Unmarshal(stream)) == NULL) || DOFCredentials_Unmarshal(&ret->RequestData.credentialData->credentials, DOFAuthenticator_GetDomainID(domain), stream, TRUE ) != CREDENTIALS_SUCCESSFUL_UNMARSHAL)
            {
                DOFObjectIDDomain_Destroy( ret->secDomain );
                DOFAuthenticator_Destroy( ret->domain );
                DOFObjectIDDomain_Destroy(obj);
                OALStream_Release(stream, FALSE);
                DOFMem_Free(ret);
                return NULL;
            }
            else
            {
                DOFObjectIDDomain_Destroy(obj);
                OALStream_Release(stream, FALSE);
            }
            ret->flags.bits.retainedStream = 1;
            ret->opCode = data->opCode;
            ret->flags.flags = data->flags.flags;
        }
        break;
    case TRP_V1_OP_REMOTE_DOMAIN:
        {
            uint8 opcode;
            ret = (TRP_V1_Data_t*)DOFMem_Alloc(sizeof(TRP_V1_Data_t));
            if( ret )
            {
                PCRMem_Set(ret,0,sizeof(TRP_V1_Data_t));
                ret->RequestData.domainData = (TRP_V1_DomainData_t*)DOFMem_Alloc(sizeof(TRP_V1_DomainData_t));
                if( ret->RequestData.domainData )
                {
                    PCRMem_Set(ret->RequestData.domainData, 0, sizeof(TRP_V1_DomainData_t));

                    ret->stream = OALStream_CreateRO(passData->streamData,passData->length);
                    if( ret->stream && OALStream_Read8(ret->stream,&opcode) &&
                      ((ret->RequestData.domainData->remDomain = DOFObjectIDDomain_Unmarshal(ret->stream)) != NULL) &&
                      ((ret->RequestData.domainData->localDomain = DOFObjectIDDomain_Unmarshal(ret->stream)) != NULL))
                    {
                        ret->I = (uint8*)OALStream_GetReadPtr(ret->stream);
                        if( TRP_V1_Unmarshal_KeyRequest( ret->stream, &ret->RequestData.domainData->initRemoteKeyRequest, ret->RequestData.domainData->localDomain))
                        {
                            ret->domain = DOFAuthenticator_Copy(domain);
                            if( ret->domain )
                            {
                                ret->secDomain = DOFObjectIDDomain_Copy(data->secDomain);
                                if( ret->secDomain )
                                {
                                    uint8* temp = (uint8*)OALStream_GetReadPtr( ret->stream );
                                    ret->I_len = (temp - ret->I);
                                    ret->opCode = data->opCode;
                                    ret->flags.flags = data->flags.flags;

                                    ret->flags.bits.retainedStream = 1;
                                    return ret;
                                }
                                DOFObjectIDDomain_Destroy( ret->secDomain );
                            }
                            DOFAuthenticator_Destroy( ret->domain );
                        }
                    }
                    DOFMem_Free(ret->RequestData.domainData);
                }
                TRP_V1_FreeLocalASRequestDestroy(ret);
            }
            return NULL;
        }
        break;
    default:
        return NULL;
        break;
    }

    return ret;
}

TRP_V1_Data TRP_V1_LocalASDiscoveryRequest( TRP_V1_Data data, DOFAuthenticator domain )
{
    TRP_V1_Data ret;

    switch( data->opCode )
    {
    case TRP_V1_OP_CREDENTIAL:
        {
            ret = (TRP_V1_Data_t*)DOFMem_Alloc(sizeof(TRP_V1_Data_t) + sizeof(TRP_V1_CredentialData_t));
            if(ret)
            {
                PCRMem_Set(ret,0,sizeof(TRP_V1_Data_t) + sizeof(TRP_V1_CredentialData_t));
                ret->RequestData.credentialData = (TRP_V1_CredentialData_t*)(((uint8*)ret) + sizeof(TRP_V1_Data_t));

                if((ret->RequestData.credentialData->credentials = DOFCredentials_CreateCopyWithDomain(data->RequestData.credentialData->credentials, DOFAuthenticator_GetDomainID(domain))) != NULL)
                {
                    ret->domain = DOFAuthenticator_Copy(domain);
                    if(ret->domain)
                    {
                        ret->secDomain = DOFObjectIDDomain_Copy(data->secDomain);
                        if(ret->secDomain)
                        {
                            ret->opCode = data->opCode;
                            ret->flags.flags = data->flags.flags;
                            return ret;
                        }
                        DOFAuthenticator_Destroy(ret->domain);
                    }
                    DOFCredentials_Destroy(ret->RequestData.credentialData->credentials);
                }
                DOFMem_Free(ret);
            }
            return NULL;
        }
        break;
    case TRP_V1_OP_REMOTE_DOMAIN:
        {
            ret = (TRP_V1_Data_t*)DOFMem_Alloc(sizeof(TRP_V1_Data_t));
            if( !ret )
                return NULL;

            PCRMem_Set(ret,0,sizeof(TRP_V1_Data_t));
            ret->RequestData.domainData = (TRP_V1_DomainData_t*)DOFMem_Alloc( sizeof(TRP_V1_DomainData_t) );

            if(ret->RequestData.domainData )
            {
                PCRMem_Set(ret->RequestData.domainData,0,sizeof(TRP_V1_DomainData_t));

                ret->RequestData.domainData->localDomain = DOFObjectIDDomain_Copy(data->RequestData.domainData->localDomain);
                if( ret->RequestData.domainData->localDomain )
                {
                    ret->RequestData.domainData->remDomain = DOFObjectIDDomain_Copy( DOFAuthenticator_GetDomainID(domain));
                    if( ret->RequestData.domainData->remDomain )
                    {
                        ret->RequestData.domainData->initRemoteKeyRequest = DOFKeyRequest_Create(data->RequestData.domainData->initRemoteKeyRequest->credentials, data->RequestData.domainData->initRemoteKeyRequest->nonceLength, data->RequestData.domainData->initRemoteKeyRequest->newLogin, data->RequestData.domainData->initRemoteKeyRequest->fixedPermissions, data->RequestData.domainData->initRemoteKeyRequest->nonce, data->RequestData.domainData->initRemoteKeyRequest->permission_set);
                        if( ret->RequestData.domainData->initRemoteKeyRequest )
                        {
                            ret->I_len = DOFKeyRequest_MarshallByteLength(ret->RequestData.domainData->initRemoteKeyRequest,TRUE);
                            ret->stream = OALStream_CreateRW(ret->I_len + 64);

                            if( ret->stream )
                            {
                                if(DOFKeyRequest_Marshal(ret->stream, ret->RequestData.domainData->initRemoteKeyRequest,TRUE))
                                {
                                    ret->secDomain = DOFObjectIDDomain_Copy(data->secDomain);
                                    if( ret->secDomain )
                                    {
                                        ret->domain = DOFAuthenticator_Copy(domain);
                                        if( ret->domain )
                                        {
                                            ret->I = (uint8*)OALStream_GetReadPtr(ret->stream);
                                            ret->opCode = data->opCode;
                                            ret->flags.flags = data->flags.flags;
                                            ret->flags.bits.retainedStream = 1;
                                            return ret;
                                        }
                                        DOFAuthenticator_Destroy( ret->domain );
                                    }
                                    DOFObjectIDDomain_Destroy( ret->secDomain );
                                }
                                OALStream_Release(ret->stream,FALSE);
                            }
                            DOFKeyRequest_Destroy(ret->RequestData.domainData->initRemoteKeyRequest);
                        }
                        DOFObjectIDDomain_Destroy(ret->RequestData.domainData->remDomain);
                    }
                    DOFObjectIDDomain_Destroy(ret->RequestData.domainData->localDomain);
                }
                DOFMem_Free(ret->RequestData.domainData);
            }
            DOFMem_Free(ret);
            return NULL;
        }
        break;
    default:
        return NULL;
        break;
    }

    return ret;
}
