/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include <dof/core.h>
#include <dof/type.h>
#include <dof/value.h>
#include "valueprivate.h"


DOFValue DOFValueArrayString_Create( uint16 encoding, uint16 len, const char * const * val )
{
    DOFValue rval;
    uint16 i;
    DOFType type;

    if ( len && ( !val || ( len & 0x8000 ) ) )
        return( NULL );
    type = DOFValueStringType_Create( encoding, 0x3FFFFF );
    rval = DOFValue_Create( DOFValueArrayType_Create( type, len, len ), len, NULL, FALSE, 0 );
    DOFType_Destroy( type );
    if ( rval )
    {
        for ( i = 0; i < len; i++ )
        {
            rval->value[i] = DOFValueString_Create( encoding, 0, val[i] );
            if ( !rval->value[i] )
                goto bail;
        }
    }
    return( rval );
 bail:
    DOFValue_Destroy( rval );
    return( NULL );
}
