#!/usr/bin/python
#
#  Copyright (c) 2010-2015, Panasonic Corporation.
#
#  Permission to use, copy, modify, and/or distribute this software for any
#  purpose with or without fee is hereby granted, provided that the above
#  copyright notice and this permission notice appear in all copies.
#
#  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
#  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
#  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
#  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
#  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
#  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

import genLevel1

#initialize the genLevel1 module
genLevel1.init('../../embunit/tools', './generation/Level1')

#get a level1 test generator
DOFValueArray = genLevel1.level1_generator('../../tests/Level1/DOFValueArray_unit.c', 'DOFValueArray')
DOFValueArray.add_include_filename('string.h')

def create_create(num, type, assert_test):
    create = genLevel1.normal_generator('DOFValueArray_Create_' + str(num), 'DOFArray_Create.tmpl')
    create.add_replacement(genLevel1.tr_swap('@@@num@@@', str(num)))
    create.add_replacement(genLevel1.tr_swap('@@@type@@@', type))
    create.add_replacement(genLevel1.tr_swap('@@@assert@@@', assert_test))
    DOFValueArray.add_test_generator(create)

def _create(template, type, num):
    test_name = 'DOFValueArray_Create_' + type + '_' + str(num)
    ret = genLevel1.normal_generator(test_name, template)
    ret.add_replacement(genLevel1.tr_swap('@@@name@@@', test_name))
    ret.add_replacement(genLevel1.tr_swap('@@@type@@@', type))
    DOFValueArray.add_test_generator(ret)
    return ret

def create_simple(type, num, data_type, data, extra=''):
    simple = _create('DOFArray_Create_valid.tmpl', type, num)
    simple.add_replacement(genLevel1.tr_swap('@@@data_type@@@', data_type))
    simple.add_replacement(genLevel1.tr_swap('@@@len@@@', '(sizeof(data) / sizeof(*data))'))
    if(type == 'String'):
        simple.add_replacement(genLevel1.tr_swap('@@@extraparam@@@', '3, '))
    else:
        simple.add_replacement(genLevel1.tr_swap('@@@extraparam@@@', ''))
    simple.add_replacement(genLevel1.tr_swap('@@@data@@@', data))
    if(extra != ''):
        extra += '\n    '
    simple.add_replacement(genLevel1.tr_swap('@@@extra@@@', extra))

def create_null(type, num):
    null = _create('DOFArray_Create_valid_null.tmpl', type, num)
    if(type == 'String'):
        null.add_replacement(genLevel1.tr_swap('@@@extraparam@@@', '3, '))
    else:
        null.add_replacement(genLevel1.tr_swap('@@@extraparam@@@', ''))

def create_invalid(type, num):
    invalid = _create('DOFArray_Create_invalid.tmpl', type, num)
    if(type == 'String'):
        invalid.add_replacement(genLevel1.tr_swap('@@@extraparam@@@', '3, '))
    else:
        invalid.add_replacement(genLevel1.tr_swap('@@@extraparam@@@', ''))

def create_trio(type, data_type, data, extra=''):
    create_simple(type, 1, data_type, data, extra)
    create_null(type, 2)
    create_invalid(type, 3)

def create_with_num(base_name, num, template):
    test_name = base_name + '_' + str(num)
    created = genLevel1.normal_generator(test_name, template)
    created.add_replacement(genLevel1.tr_swap('@@@num@@@', str(num)))
    DOFValueArray.add_test_generator(created)
    return created

def create_get_element(num, element):
    get_element = create_with_num('DOFValueArray_GetElement', num, 'DOFArray_GetElement.tmpl')
    get_element.add_replacement(genLevel1.tr_swap('@@@element@@@', str(element)))

def _create_to(type, num, data_type, data_ret_type, data, offset, count, assert_valid, free, template):
    to = genLevel1.normal_generator('DOFValueArray' + type + '_Extract_' + str(num), template)
    to.add_replacement(genLevel1.tr_swap('@@@type@@@', type))
    to.add_replacement(genLevel1.tr_swap('@@@num@@@', str(num)))
    to.add_replacement(genLevel1.tr_swap('@@@data_type@@@', data_type))
    to.add_replacement(genLevel1.tr_swap('@@@ret_data_type@@@', data_ret_type))
    to.add_replacement(genLevel1.tr_swap('@@@data@@@', data))
    to.add_replacement(genLevel1.tr_swap('@@@offset@@@', str(offset)))
    to.add_replacement(genLevel1.tr_swap('@@@count@@@', str(count)))
    to.add_replacement(genLevel1.tr_swap('@@@assert_valid@@@', str(assert_valid)))
    to.add_replacement(genLevel1.tr_swap('@@@free@@@', str(free)))
    if(type == 'String'):
        to.add_replacement(genLevel1.tr_swap('@@@extraparam@@@', '3, '))
    else:
        to.add_replacement(genLevel1.tr_swap('@@@extraparam@@@', ''))
    DOFValueArray.add_test_generator(to)
    return to

def create_to(type, data_type, data_ret_type, data, template='DOFArray_To.tmpl'):
    params = (0, 1, 1, 1), (7, 1, 1, 0), (0, 8, 1, 1), (4, 4, 1, 1), (8, 1, 0, 1), (0, 9, 0, 1), (4, 5, 0, 1), (0, 0, 0, 1)
    num = 1
    for p in params:
        _create_to(type, num, data_type, data_ret_type, data, p[0], p[1], p[2], p[3], template)
        num += 1

#the DOFValueArray_Create* tests

create_create(1, 'Int8', 'TEST_ASSERT')
create_create(2, 'UInt8', 'TEST_ASSERT')

create_trio('Boolean', 'boolean', 'FALSE, TRUE, FALSE')
create_trio('DateTime', 'DateTime', '0, 0xffffffffUL, 0xffffffffffffffffULL')
create_trio('Float32', 'float', '0.0f, 0.5f, 1.0f, -1.0f, -0.5f')
create_trio('Float64', 'double', '0.0, 0.5, 1.0, -1.0, -0.5')
create_trio('GUID', 'GUID', '{{0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15}}, {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}}, {{0xde, 0xad, 0xfa, 0xce, 0xde, 0xad, 0xbe, 0xef}}')
create_trio('Int16', 'int16', '0, 0x7fff, 0x3fff')
create_trio('Int32', 'int32', '0, -1, 0x7fffffffL')
create_trio('Int64', 'int64', '0, -1, 0x7fffffffffffffffLL')
create_trio('Int8', 'int8', '0, -1, 0x7f')

DOFValueArray.add_test_generator(genLevel1.normal_generator('DOFValueArray_Create_InterfaceID_1', 'DOFArray_Create_interfaceid.tmpl'))
create_null('InterfaceID', 2)
create_invalid('InterfaceID', 3)

DOFValueArray.add_test_generator(genLevel1.normal_generator('DOFValueArray_Create_ObjectID_1', 'DOFArray_Create_objectid.tmpl'))
create_null('ObjectID', 2)
create_invalid('ObjectID', 3)

create_trio('String', 'const char*', '"test1", "test2", "test3"')
create_trio('UInt16', 'uint16', '0, 0xffff')
create_trio('UInt32', 'uint32', '0, 0xffffffffUL')
create_trio('UInt64', 'uint64', '0, 0xffffffffffffffffULL')
create_trio('UInt8', 'uint8', '0, 0xff')
#create_trio('UTF8', 'const uint8*', '(const uint8*)"test1", (const uint8*)"test2", (const uint8*)"test3"')

#other tests

create_get_element(1, 0)
create_get_element(2, 7)
DOFValueArray.add_test_generator(genLevel1.normal_generator('DOFValueArray_GetElement_3', 'DOFArray_GetElement_oob.tmpl'))

DOFValueArray.add_test_generator(genLevel1.normal_generator('DOFValueArray_GetElements_1', 'DOFArray_GetElements.tmpl'))

DOFValueArray.add_test_generator(genLevel1.normal_generator('DOFValueArray_GetElementsCount_1', 'DOFArray_GetLength_1.tmpl'))
DOFValueArray.add_test_generator(genLevel1.normal_generator('DOFValueArray_GetElementsCount_2', 'DOFArray_GetLength_2.tmpl'))

#DOFValueArray_To* tests

create_to('Boolean', 'boolean', 'boolean', 'FALSE, TRUE, FALSE, TRUE, FALSE, FALSE, TRUE, TRUE')
create_to('DateTime', 'DateTime', 'DateTime', '0ULL, 0xffffULL, 0xffffffffULL, 0xffffffffffffffffULL, 1, 2, 3, 4')
create_to('Float32', 'float', 'float', '0.0f, 0.5f, -0.5f, 1.0f, -1.0f, 0.0f, 0.0f, 5.0f')
create_to('Float64', 'double', 'double', '0.0, 0.5, -0.5, 1.0, -1.0, 0.0, 0.0, 5.0')
create_to('GUID', 'GUID', 'GUID', '{{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}}, {{0, 0, 0, 1, 0, 0, 0, 2, 0, 0, 0, 3, 0, 0, 0, 4}}, {{0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 1, 2, 3, 4, 5}}, {{1, 1, 1, 1, 2, 2, 2, 2, 4, 4, 4, 4, 3, 3, 3, 3}}, {{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}}, {{0, 0, 0, 1, 0, 0, 0, 2, 0, 0, 0, 3, 0, 0, 0, 4}}, {{0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 1, 2, 3, 4, 5}}, {{1, 1, 1, 1, 2, 2, 2, 2, 4, 4, 4, 4, 3, 3, 3, 3}}')
create_to('Int16', 'int16', 'int16', '1, 2, 3, 4, 5, 0x7fff, -1, 0')
create_to('Int32', 'int32', 'int32', '1, 2, 3, 4, 5, 0x7fffffff, -1, 0')
create_to('Int64', 'int64', 'int64', '1, 2, 3, 4, 5, 0x7fffffffffffffffLL, -1LL, 0')
create_to('Int8', 'int8', 'int8', '1, 2, 3, 4, 5, 0x7f, -1, 0')
create_to('InterfaceID', '', '', '', 'DOFArray_To_interfaceid.tmpl')
create_to('ObjectID', '', '', '', 'DOFArray_To_objectid.tmpl')
create_to('String', 'const char*', 'char*', '"hello", "good-bye", "hello world", "do you want to play a game?", "it\'s peanut butter jelly time", "this test passes", "i hope it does", "i love sushi!"')
create_to('UInt16', 'uint16', 'uint16', '1, 2, 3, 4, 5, 0x7fff, 0xffff, 0')
create_to('UInt32', 'uint32', 'uint32', '1, 2, 3, 4, 5, 0x7fffffffUL, 0xffffffffUL, 0')
create_to('UInt64', 'uint64', 'uint64', '1, 2, 3, 4, 5, 0x7fffffffffffffffULL, 0xffffffffffffffffULL, 0')
create_to('UInt8', 'uint8', 'uint8', '1, 2, 3, 4, 5, 0x7f, 0xff, 0')
#create_to('UTF8', 'const uint8*', 'uint8*', '(const uint8*)"hello", (const uint8*)"good-bye", (const uint8*)"hello world", (const uint8*)"do you want to play a game?", (const uint8*)"it\'s peanut butter jelly time", (const uint8*)"this test passes", (const uint8*)"i hope it does", (const uint8*)"i love sushi!"')

DOFValueArray.generate()
