#!/usr/bin/python
#
#  Copyright (c) 2010-2015, Panasonic Corporation.
#
#  Permission to use, copy, modify, and/or distribute this software for any
#  purpose with or without fee is hereby granted, provided that the above
#  copyright notice and this permission notice appear in all copies.
#
#  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
#  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
#  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
#  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
#  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
#  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.


import genLevel1

def _add_name_type_val(generator, type, val, data):
    generator.add_replacement(genLevel1.tr_swap('@@@name@@@', generator.test_name))
    if(data != ''):
        data += '\n    '
    generator.add_replacement(genLevel1.tr_swap('@@@data@@@', data))
    generator.add_replacement(genLevel1.tr_swap('@@@type@@@', type))
    generator.add_replacement(genLevel1.tr_swap('@@@val@@@', val))

def _add_other(generator, other):
    if(other != ''):
        other += '\n    '
    generator.add_replacement(genLevel1.tr_swap('@@@other@@@', other))

def _add_buffer(generator, buffer):
    if(buffer != ''):
        generator.add_replacement(genLevel1.tr_swap('@@@ubuffer@@@', '_' + buffer))
    else:
        generator.add_replacement(genLevel1.tr_swap('@@@ubuffer@@@', ''))
    generator.add_replacement(genLevel1.tr_swap('@@@buffer@@@', buffer))

def create_simple(test_name, type, val, other='', data='', buffer=''):
    simple = genLevel1.normal_generator(test_name, 'DOFPrimitive_Create_simple.tmpl')
    _add_name_type_val(simple, type, val, data)
    _add_other(simple, other)
    _add_buffer(simple, buffer)
    return simple

def create_invalid(test_name, type, val, data='', buffer=''):
    invalid = genLevel1.normal_generator(test_name, 'DOFPrimitive_Create_invalid.tmpl')
    _add_name_type_val(invalid, type, val, data)
    _add_buffer(invalid, buffer)
    return invalid

def create_iid(test_name, data, other=''):
    iid = genLevel1.normal_generator(test_name, 'DOFPrimitive_Create_InterfaceID.tmpl')
    iid.add_replacement(genLevel1.tr_swap('@@@name@@@', iid.test_name))
    iid.add_replacement(genLevel1.tr_swap('@@@data@@@', data))
    _add_other(iid, other)
    return iid

def _create_to_type(template_name, test_name, type, create_type, ret_type, create_val, val, destroy_flag, compare, buffer='', free_command=''):
    to_type = genLevel1.normal_generator(test_name, template_name)
    _add_name_type_val(to_type, type, val, '')
    to_type.add_replacement(genLevel1.tr_swap('@@@create_type@@@', create_type))
    to_type.add_replacement(genLevel1.tr_swap('@@@ret_type@@@', ret_type))
    to_type.add_replacement(genLevel1.tr_swap('@@@destroy_flag@@@', destroy_flag))
    to_type.add_replacement(genLevel1.tr_swap('@@@create_val@@@', create_val))
    to_type.add_replacement(genLevel1.tr_swap('@@@compare@@@', compare))
    to_type.add_replacement(genLevel1.tr_swap('@@@free@@@', free_command))
    _add_buffer(to_type, buffer)
    return to_type

def create_to_type(test_name, type, create_type, ret_type, create_val, val, destroy_flag, compare, free_flag, buffer='', free_command='PCRMem_Free(ret)'):
    to_type = _create_to_type('DOFPrimitive_To_Type_valid.tmpl', test_name, type, create_type, ret_type, create_val, val, destroy_flag, compare, buffer, free_command)
    to_type.add_replacement(genLevel1.tr_swap('@@@free_flag@@@', free_flag))
    return to_type

def create_to_type_simple(test_name, type, create_type, val, destroy_flag, compare):
    return _create_to_type('DOFPrimitive_To_Type_simple.tmpl', test_name, type, create_type, '', '', val, destroy_flag, compare)

def create_to_type_invalid(test_name, type, wrong_type, ret_type, val, compare):
    return _create_to_type('DOFPrimitive_To_Type_invalid.tmpl', test_name, type, wrong_type, ret_type, '', val, '', compare)

def create_to_type_null(test_name, type, access):
    to_type = _create_to_type('DOFPrimitive_To_Type_null.tmpl', test_name, type, '', '', '', '', '', '')
    to_type.add_replacement(genLevel1.tr_swap('@@@access@@@', access))
    return to_type

#---------------------------------------------------------------------------#

#initialize the genLevel1 module
genLevel1.init('../../embunit/tools', './generation/Level1')

#get a level1 test generator
DOFPrimitive = genLevel1.level1_generator('../../tests/Level1/DOFPrimitive_unit.c', 'DOFPrimitive')
DOFPrimitive.add_include_filename('string.h')

#ah, the meat of it all.
DOFPrimitive.add_test_generator(create_simple('DOFValueBLOB_Create_1', 'BLOB', '5, (uint8*)"abcde"'))
DOFPrimitive.add_test_generator(create_simple('DOFValueBLOB_Create_2', 'BLOB', '0, NULL'))

DOFPrimitive.add_test_generator(create_simple('DOFValueBoolean_Create_1', 'Boolean', 'TRUE'))
DOFPrimitive.add_test_generator(create_simple('DOFValueBoolean_Create_2', 'Boolean', 'FALSE'))

DOFPrimitive.add_test_generator(create_simple('DOFValueDateTime_Create_1', 'DateTime', '0'))
DOFPrimitive.add_test_generator(create_simple('DOFValueDateTime_Create_2', 'DateTime', '0xffffffff'))

DOFPrimitive.add_test_generator(create_simple('DOFValueFloat32_Create_1', 'Float32', '0.32f'))
DOFPrimitive.add_test_generator(create_simple('DOFValueFloat32_Create_2', 'Float32', '-0.32f'))

DOFPrimitive.add_test_generator(create_simple('DOFValueFloat64_Create_1', 'Float64', '0.64'))
DOFPrimitive.add_test_generator(create_simple('DOFValueFloat64_Create_2', 'Float64', '-0.64'))

DOFPrimitive.add_test_generator(create_simple('DOFValueString_Create_1', 'String', '3, 5, "abcde"'))
DOFPrimitive.add_test_generator(create_simple('DOFValueString_Create_2', 'String', '17, 3, "abcdef"'))
DOFPrimitive.add_test_generator(create_invalid('DOFValueString_Create_3', 'String', '3, 6, "abcde"'))
DOFPrimitive.add_test_generator(create_simple('DOFValueString_Create_4', 'String', '3, 0, NULL'))
DOFPrimitive.add_test_generator(create_simple('DOFValueString_Create_5', 'String', '3, 0, "abcde"'))

DOFPrimitive.add_test_generator(create_simple('DOFValueString_Create_6', 'String', '3, 5, 5, (const uint8*)"a\\0cde"', '', '', 'Buffer'))
DOFPrimitive.add_test_generator(create_simple('DOFValueString_Create_7', 'String', '17, 3, 6, (const uint8*)"a\\0cdef"', '', '', 'Buffer'))
DOFPrimitive.add_test_generator(create_invalid('DOFValueString_Create_8', 'String', '3, 6, 5, (const uint8*)"a\\0cde"', '', 'Buffer'))
DOFPrimitive.add_test_generator(create_simple('DOFValueString_Create_9', 'String', '3, 0, 0, NULL', '', '', 'Buffer'))
DOFPrimitive.add_test_generator(create_simple('DOFValueString_Create_10', 'String', '3, 0, 5, (const uint8*)"a\\0cde"', '', '', 'Buffer'))

DOFPrimitive.add_test_generator(create_simple('DOFValueGUID_Create_1', 'GUID', 'guid', '', 'const GUID guid = {{0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15}};'))

DOFPrimitive.add_test_generator(create_simple('DOFValueInt16_Create_1', 'Int16', '32767'))
DOFPrimitive.add_test_generator(create_simple('DOFValueInt16_Create_2', 'Int16', '-32767'))

DOFPrimitive.add_test_generator(create_simple('DOFValueInt32_Create_1', 'Int32', '2147483647L'))
DOFPrimitive.add_test_generator(create_simple('DOFValueInt32_Create_2', 'Int32', '-2147483647L'))

DOFPrimitive.add_test_generator(create_simple('DOFValueInt64_Create_1', 'Int64', '4294967295LL'))
DOFPrimitive.add_test_generator(create_simple('DOFValueInt64_Create_2', 'Int64', '-4294967295LL'))

DOFPrimitive.add_test_generator(create_simple('DOFValueInt8_Create_1', 'Int8', '127'))
DOFPrimitive.add_test_generator(create_simple('DOFValueInt8_Create_2', 'Int8', '-127'))

DOFPrimitive.add_test_generator(create_iid('DOFValueInterfaceID_Create_1', '0x01'))
DOFPrimitive.add_test_generator(create_iid('DOFValueInterfaceID_Create_2', '0x0102'))
DOFPrimitive.add_test_generator(create_iid('DOFValueInterfaceID_Create_3', '0x01020304'))

DOFPrimitive.add_test_generator(create_simple('DOFValueObjectID_Create_1', 'ObjectID', 'DOFOBJECTID_BROADCAST'))
DOFPrimitive.add_test_generator(create_simple('DOFValueObjectID_Create_2', 'ObjectID', 'oid', 'DOFObjectID_Destroy(oid);', 'DOFObjectID oid = DOFObjectID_Create(DOFOBJECTIDCLASS_EMAIL, 7, (const uint8*)"a@b.com");'))

DOFPrimitive.add_test_generator(create_simple('DOFValueUInt16_Create_1', 'UInt16', '0'))
DOFPrimitive.add_test_generator(create_simple('DOFValueUInt16_Create_2', 'UInt16', '0xffff'))

DOFPrimitive.add_test_generator(create_simple('DOFValueUInt32_Create_1', 'UInt32', '0'))
DOFPrimitive.add_test_generator(create_simple('DOFValueUInt32_Create_2', 'UInt32', '0xffffffff'))

DOFPrimitive.add_test_generator(create_simple('DOFValueUInt64_Create_1', 'UInt64', '0'))
DOFPrimitive.add_test_generator(create_simple('DOFValueUInt64_Create_2', 'UInt64', '0x100000000ULL'))

DOFPrimitive.add_test_generator(create_simple('DOFValueUInt8_Create_1', 'UInt8', '0'))
DOFPrimitive.add_test_generator(create_simple('DOFValueUInt8_Create_2', 'UInt8', '0xff'))

DOFPrimitive.add_test_generator(create_simple('DOFValueBLOB_Get_1', 'BLOB', '5, (uint8*)"abcde"', 'TEST_ASSERT(!memcmp(DOFValueBLOB_Get(p), (uint8*)"abcde", 5));'))
DOFPrimitive.add_test_generator(create_simple('DOFValueBLOB_GetSize_1', 'BLOB', '5, (uint8*)"abcde"', 'TEST_ASSERT(DOFValueBLOB_GetSize(p)==5);'))

DOFPrimitive.add_test_generator(create_simple('DOFValueString_Get_1', 'String', '3, 5, "abcde"', 'TEST_ASSERT(!strcmp(DOFValueString_Get(p), "abcde"));'))
DOFPrimitive.add_test_generator(create_simple('DOFValueString_Get_2', 'String', '3, 0, ""', 'TEST_ASSERT(DOFValueString_Get(p)[0]==0);'))
DOFPrimitive.add_test_generator(create_simple('DOFValueString_Get_3', 'String', '3, 5, 5, (const uint8*)"a\\0cde"', 'TEST_ASSERT(!memcmp(DOFValueString_Get(p), "a\\0cde", 5));', '', 'Buffer'))
DOFPrimitive.add_test_generator(create_to_type_invalid('DOFValueString_Get_4', 'String', 'UInt8', 'const char*', '100', 'ret==0'))
DOFPrimitive.add_test_generator(create_to_type_null('DOFValueString_Get_5', 'String', 'Get'))
DOFPrimitive.add_test_generator(create_simple('DOFValueString_GetBuffer_1', 'String', '3, 5, 5, (const uint8*)"a\\0cde"', 'TEST_ASSERT(!memcmp(DOFValueString_GetBuffer(p), (const uint8*)"a\\0cde", 5));', '', 'Buffer'))
DOFPrimitive.add_test_generator(create_simple('DOFValueString_GetBufferSize_1', 'String', '3, 5, 5, (const uint8*)"a\\0cde"', 'TEST_ASSERT(DOFValueString_GetBufferSize(p)==5);', '', 'Buffer'))

DOFPrimitive.add_test_generator(create_to_type('DOFValueString_Extract_1', 'String', 'const char *', 'char *', '"abcde"', '3, 5, val', '1', '!memcmp(ret, val, 5)', '1'))
DOFPrimitive.add_test_generator(create_to_type('DOFValueString_Extract_2', 'String', 'const char *', 'char *', '"abcde"', '3, 5, val', '1', '!memcmp(ret, val, 5)', '1'))
DOFPrimitive.add_test_generator(create_to_type('DOFValueString_Extract_3', 'String', 'const char *', 'char *', '"abcde"', '3, 5, val', '1', '!memcmp(ret, val, 5)', '1'))
#TODO: Extract w/ invalid?
DOFPrimitive.add_test_generator(create_to_type_null('DOFValueString_Extract_5', 'String', 'Extract'))
DOFPrimitive.add_test_generator(create_to_type('DOFValueString_ExtractBuffer_1', 'String', 'const uint8*', 'uint8*', '(const uint8*)"a\\0cde"', '3, 5, 5, val', '1', '!memcmp(ret, val, 5)', '1', 'Buffer'))

DOFPrimitive.add_test_generator(create_iid('DOFValueInterfaceID_Get_1', '0x01', 'TEST_ASSERT(!DOFInterfaceID_Compare(iid, DOFValueInterfaceID_Get(p)));'))
DOFPrimitive.add_test_generator(create_iid('DOFValueInterfaceID_Get_2', '0x0102', 'TEST_ASSERT(!DOFInterfaceID_Compare(iid, DOFValueInterfaceID_Get(p)));'))
DOFPrimitive.add_test_generator(create_iid('DOFValueInterfaceID_Get_3', '0x01020304', 'TEST_ASSERT(!DOFInterfaceID_Compare(iid, DOFValueInterfaceID_Get(p)));'))

DOFPrimitive.add_test_generator(create_simple('DOFValueObjectID_Get_1', 'ObjectID', 'DOFOBJECTID_BROADCAST', 'TEST_ASSERT(DOFValueObjectID_Get(p)==DOFOBJECTID_BROADCAST);'))
DOFPrimitive.add_test_generator(create_simple('DOFValueObjectID_Get_2', 'ObjectID', 'oid', 'TEST_ASSERT(DOFValueObjectID_Get(p)==oid);\nDOFObjectID_Destroy(oid);', 'DOFObjectID oid = DOFObjectID_Create(DOFOBJECTIDCLASS_EMAIL, 7, (const uint8*)"a@b.com");'))

DOFPrimitive.add_test_generator(create_to_type('DOFValueBLOB_Extract_1', 'BLOB', 'const uint8*', 'uint8*', '(const uint8*)"abcde"', '5, val', '1', 'ret&&(!memcmp(ret, val, 5))', '1'))
DOFPrimitive.add_test_generator(create_to_type('DOFValueBLOB_Extract_2', 'BLOB', 'const uint8*', 'uint8*', 'NULL', '0, val', '1', 'ret', '1'))
DOFPrimitive.add_test_generator(create_to_type_invalid('DOFValueBLOB_Extract_3', 'BLOB', 'UInt8', 'const void *', '100', 'ret==NULL'))
DOFPrimitive.add_test_generator(create_to_type_null('DOFValueBLOB_Extract_4', 'BLOB', 'Get'))

DOFPrimitive.add_test_generator(create_to_type_simple('DOFValueBoolean_Get_1', 'Boolean', 'boolean', 'TRUE', '1', 'ret==TRUE'))
DOFPrimitive.add_test_generator(create_to_type_simple('DOFValueBoolean_Get_2', 'Boolean', 'boolean', 'FALSE', '0', 'ret==FALSE'))
DOFPrimitive.add_test_generator(create_to_type_invalid('DOFValueBoolean_Get_3', 'Boolean', 'UInt8', 'boolean', '100', 'ret==FALSE'))

DOFPrimitive.add_test_generator(create_to_type_simple('DOFValueDateTime_Get_1', 'DateTime', 'DateTime', '0', '1', 'ret==0'))
DOFPrimitive.add_test_generator(create_to_type_simple('DOFValueDateTime_Get_2', 'DateTime', 'DateTime', '0xffffffff', '0', 'ret==0xffffffff'))
DOFPrimitive.add_test_generator(create_to_type_invalid('DOFValueDateTime_Get_3', 'DateTime', 'UInt8', 'DateTime', '100', 'ret==0'))

DOFPrimitive.add_test_generator(create_to_type_simple('DOFValueFloat32_Get_1', 'Float32', 'float', '0.32f', '1', 'ret==0.32f'))
DOFPrimitive.add_test_generator(create_to_type_simple('DOFValueFloat32_Get_2', 'Float32', 'float', '-0.32f', '0', 'ret==-0.32f'))
DOFPrimitive.add_test_generator(create_to_type_invalid('DOFValueFloat32_Get_3', 'Float32', 'UInt8', 'float', '100', 'ret==0'))

DOFPrimitive.add_test_generator(create_to_type_simple('DOFValueFloat64_Get_1', 'Float64', 'double', '0.64f', '1', 'ret==0.64f'))
DOFPrimitive.add_test_generator(create_to_type_simple('DOFValueFloat64_Get_2', 'Float64', 'double', '-0.64f', '0', 'ret==-0.64f'))
DOFPrimitive.add_test_generator(create_to_type_invalid('DOFValueFloat64_Get_3', 'Float64', 'UInt8', 'double', '100', 'ret==0'))

DOFPrimitive.add_test_generator(create_to_type('DOFValueGUID_Get_1', 'GUID', 'GUID', 'GUID', '{{1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 1, 2, 3, 4, 5, 6}}', 'val', '1', '!memcmp(&ret, &val, sizeof(val))', '0'))
DOFPrimitive.add_test_generator(create_to_type('DOFValueGUID_Get_2', 'GUID', 'GUID', 'GUID', '{{1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 1, 2, 3, 4, 5, 6}}', 'val', '0', '!memcmp(&ret, &val, sizeof(val))', '0'))
DOFPrimitive.add_test_generator(create_to_type_invalid('DOFValueGUID_Get_3', 'GUID', 'UInt8', 'GUID', '100', '(((char*)&ret)[0]==0)&&(((char*)&ret)[1]==0)&&(((char*)&ret)[2]==0)&&(((char*)&ret)[3]==0)&&(((char*)&ret)[4]==0)&&(((char*)&ret)[5]==0)&&(((char*)&ret)[6]==0)&&(((char*)&ret)[7]==0)&&(((char*)&ret)[8]==0)&&(((char*)&ret)[9]==0)&&(((char*)&ret)[10]==0)&&(((char*)&ret)[11]==0)&&(((char*)&ret)[12]==0)&&(((char*)&ret)[13]==0)&&(((char*)&ret)[14]==0)&&(((char*)&ret)[15]==0)'))

DOFPrimitive.add_test_generator(create_to_type_simple('DOFValueInt16_Get_1', 'Int16', 'int16', '32767', '1', 'ret==32767'))
DOFPrimitive.add_test_generator(create_to_type_simple('DOFValueInt16_Get_2', 'Int16', 'int16', '-32768', '1', 'ret==-32768'))
DOFPrimitive.add_test_generator(create_to_type_invalid('DOFValueInt16_Get_3', 'Int16', 'UInt8', 'int16', '100', 'ret==0'))

DOFPrimitive.add_test_generator(create_to_type_simple('DOFValueInt32_Get_1', 'Int32', 'int32', '2147483647', '1', 'ret==2147483647L'))
DOFPrimitive.add_test_generator(create_to_type_simple('DOFValueInt32_Get_2', 'Int32', 'int32', '-2147483647', '1', 'ret==-2147483647L'))
DOFPrimitive.add_test_generator(create_to_type_invalid('DOFValueInt32_Get_3', 'Int32', 'UInt8', 'int32', '100', 'ret==0'))

DOFPrimitive.add_test_generator(create_to_type_simple('DOFValueInt64_Get_1', 'Int64', 'int64', '4294967295LL', '1', 'ret==4294967295LL'))
DOFPrimitive.add_test_generator(create_to_type_simple('DOFValueInt64_Get_2', 'Int64', 'int64', '-4294967295LL', '1', 'ret==-4294967295LL'))
DOFPrimitive.add_test_generator(create_to_type_invalid('DOFValueInt64_Get_3', 'Int64', 'UInt8', 'int64', '100', 'ret==0'))

DOFPrimitive.add_test_generator(create_to_type_simple('DOFValueInt8_Get_1', 'Int8', 'int8', '127', '1', 'ret==127'))
DOFPrimitive.add_test_generator(create_to_type_simple('DOFValueInt8_Get_2', 'Int8', 'int8', '-128', '1', 'ret==-128'))
DOFPrimitive.add_test_generator(create_to_type_invalid('DOFValueInt8_Get_3', 'Int8', 'UInt8', 'int8', '100', 'ret==0'))

DOFPrimitive.add_test_generator(create_to_type('DOFValueInterfaceID_Extract_1', 'InterfaceID', 'DOFInterfaceID', 'DOFInterfaceID', 'DOFInterfaceID_Create(5, 0x01)', 'val', '1', '!DOFInterfaceID_Compare(val, ret)', '1'))
DOFPrimitive.add_test_generator(create_to_type('DOFValueInterfaceID_Extract_2', 'InterfaceID', 'DOFInterfaceID', 'DOFInterfaceID', 'DOFInterfaceID_Create(5, 0x0102)', 'val', '1', '!DOFInterfaceID_Compare(val, ret)', '1'))
DOFPrimitive.add_test_generator(create_to_type('DOFValueInterfaceID_Extract_3', 'InterfaceID', 'DOFInterfaceID', 'DOFInterfaceID', 'DOFInterfaceID_Create(5, 0x01020304)', 'val', '1', '!DOFInterfaceID_Compare(val, ret)', '1'))
DOFPrimitive.add_test_generator(create_to_type_invalid('DOFValueInterfaceID_Extract_4', 'InterfaceID', 'UInt8', 'DOFInterfaceID', '100', 'ret==NULL'))

DOFPrimitive.add_test_generator(create_to_type('DOFValueObjectID_Extract_1', 'ObjectID', 'DOFObjectID', 'DOFObjectID', 'DOFOBJECTID_BROADCAST', 'val', '1', '!DOFObjectID_Compare(DOFOBJECTID_BROADCAST, ret)', '1', '', 'DOFObjectID_Destroy(ret)'))
DOFPrimitive.add_test_generator(create_to_type('DOFValueObjectID_Extract_2', 'ObjectID', 'DOFObjectID', 'DOFObjectID', 'DOFObjectID_Create(DOFOBJECTIDCLASS_EMAIL, 7, (const uint8*)"a@b.com")', 'val', '1', '!DOFObjectID_Compare(val, ret)', '1', '', 'DOFObjectID_Destroy(val); DOFObjectID_Destroy(ret)'))

DOFPrimitive.add_test_generator(create_to_type_simple('DOFValueUInt16_Get_1', 'UInt16', 'uint16', '0', '1', 'ret==0'))
DOFPrimitive.add_test_generator(create_to_type_simple('DOFValueUInt16_Get_2', 'UInt16', 'uint16', '65535', '1', 'ret==65535'))
DOFPrimitive.add_test_generator(create_to_type_invalid('DOFValueUInt16_Get_3', 'UInt16', 'UInt8', 'int16', '100', 'ret==0'))

DOFPrimitive.add_test_generator(create_to_type_simple('DOFValueUInt32_Get_1', 'Int32', 'uint32', '0', '1', 'ret==0'))
DOFPrimitive.add_test_generator(create_to_type_simple('DOFValueUInt32_Get_2', 'Int32', 'uint32', '0xffffffff', '1', 'ret==0xffffffff'))
DOFPrimitive.add_test_generator(create_to_type_invalid('DOFValueUInt32_Get_3', 'UInt32', 'UInt8', 'int32', '100', 'ret==0'))

DOFPrimitive.add_test_generator(create_to_type_simple('DOFValueUInt64_Get_1', 'UInt64', 'uint64', '0', '1', 'ret==0'))
DOFPrimitive.add_test_generator(create_to_type_simple('DOFValueUInt64_Get_2', 'UInt64', 'uint64', '0x100000000ULL', '1', 'ret==0x100000000ULL'))
DOFPrimitive.add_test_generator(create_to_type_invalid('DOFValueUInt64_Get_3', 'UInt64', 'UInt8', 'int64', '100', 'ret==0'))

DOFPrimitive.add_test_generator(create_to_type_simple('DOFValueUInt8_Get_1', 'UInt8', 'uint8', '0', '1', 'ret==0'))
DOFPrimitive.add_test_generator(create_to_type_simple('DOFValueUInt8_Get_2', 'UInt8', 'uint8', '255', '1', 'ret==255'))
DOFPrimitive.add_test_generator(create_to_type_invalid('DOFValueUInt8_Get_3', 'UInt8', 'Int8', 'int8', '100', 'ret==0'))


#----------------------------------------------------------------------------#

#now make the tests
DOFPrimitive.generate()
