#!/usr/bin/python
#
#  Copyright (c) 2010-2015, Panasonic Corporation.
#
#  Permission to use, copy, modify, and/or distribute this software for any
#  purpose with or without fee is hereby granted, provided that the above
#  copyright notice and this permission notice appear in all copies.
#
#  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
#  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
#  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
#  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
#  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
#  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.


import genLevel1

#initialize the genLevel1 module
genLevel1.init('../../embunit/tools', './generation/Level1')

#get a level1 test generator
DOFType = genLevel1.level1_generator('../../tests/Level1/DOFType_unit.c', 'DOFType')
DOFType.add_include_filename('string.h')

def _add_simple(simple, type, num, params, assert_valid):
    simple.add_replacement(genLevel1.tr_swap('@@@type@@@', type))
    simple.add_replacement(genLevel1.tr_swap('@@@num@@@', str(num)))
    simple.add_replacement(genLevel1.tr_swap('@@@params@@@', params))
    simple.add_replacement(genLevel1.tr_swap('@@@assert_valid@@@', str(assert_valid)))
    DOFType.add_test_generator(simple)
        
def create_simple(type, num, template, assert_valid, params=''):
    prefix='DOFValue'
    created = genLevel1.normal_generator(prefix + type + 'Type_Create_' + str(num), template)
    _add_simple(created, type, num, params, assert_valid)
    return created

def create_simple_default(type, num, assert_valid, params=''):
    return create_simple(type, num, 'DOFType_Create_Prim.tmpl', assert_valid, params)
    
def create_get_element_type(template, num, create_type, params, array_array):
    element = genLevel1.normal_generator('DOFType_GetElementType_' + str(num), template)
    element.add_replacement(genLevel1.tr_swap('@@@num@@@', str(num)))
    element.add_replacement(genLevel1.tr_swap('@@@create_type@@@', create_type))
    element.add_replacement(genLevel1.tr_swap('@@@create_params@@@', params))
    element.add_replacement(genLevel1.tr_swap('@@@array_array@@@', str(array_array)))
    DOFType.add_test_generator(element)
    
#-------------------------------------------------------------------#

for type in ('Boolean', 'DateTime', 'Float32', 'Float64', 'GUID', 'Int16', 'Int32', 'Int64', 'Int8', 'InterfaceID', 'ObjectID', 'UInt16', 'UInt32', 'UInt64', 'UInt8'):
    create_simple_default(type, 1, 1)
del type

#array data is type, num, params (for type creation), min array len, max array len, and then 1 if it should be valid and 0 if it should be null
for array_data in (('Int8', 1, '', 0, 5, 1), ('Int8', 2, '', 5, 5, 1), ('String', 3, '3, 10', 5, 5, 1), ('String', 4, '106, 10', 5, 5, 1), ('BLOB', 5, '10, 20', 5, 5, 1), ('Int8', 8, '', 0, 0, 1), ('Int8', 9, '', 5, 3, 0)):
    created = create_simple('Array', str(array_data[1]), 'DOFType_Create_Array.tmpl', array_data[5], array_data[2])
    created.add_replacement(genLevel1.tr_swap('@@@create_type@@@', array_data[0]))
    created.add_replacement(genLevel1.tr_swap('@@@min@@@', str(array_data[3])))
    created.add_replacement(genLevel1.tr_swap('@@@max@@@', str(array_data[4])))
del array_data

DOFType.add_test_generator(genLevel1.normal_generator('DOFType_Create_Array_6', 'DOFType_Create_Array_Structure.tmpl'))
DOFType.add_test_generator(genLevel1.normal_generator('DOFType_Create_Array_7', 'DOFType_Create_Array_Array.tmpl'))

#blob data is num, assert valid, params
for blob_data in ((1, 1, '3, 5'), (2, 1, '5, 5'), (3, 0, '5, 3'), (4, 1, '0, 0')):
    create_simple_default('BLOB', blob_data[0], blob_data[1], blob_data[2])
del blob_data

#string data is num, assert valid, params
for string_data in ((1, 1, '3, 5'), (2, 1, '3, 0'), (3, 0, '2, 5'), (4, 0, '1, 5'), (5, 0, '0, 5')):
    create_simple_default('String', string_data[0], string_data[1], string_data[2])
del string_data
    
DOFType.add_test_generator(genLevel1.normal_generator('DOFType_Destroy_1', 'DOFType_Destroy_1.tmpl'))
DOFType.add_test_generator(genLevel1.normal_generator('DOFType_Destroy_2', 'DOFType_Destroy_2.tmpl'))

#for element_valid_data in ((1, 'PrimitiveInt8', '', 0), (2, 'Structure', '0, NULL', 0), (3, 'PrimitiveInt8', '', 1)):
#    create_get_element_type('DOFType_GetElementType.tmpl', element_valid_data[0], element_valid_data[1], element_valid_data[2], element_valid_data[3])
#del element_valid_data

#for invalid_element_data in ((4, 'PrimitiveInt8', ''), (5, 'Structure', '0, NULL')):
#    create_get_element_type('DOFType_GetElementType_invalid.tmpl', invalid_element_data[0], invalid_element_data[1], invalid_element_data[2], 0)
#del invalid_element_data

for encoding_data in ((1, 'String', '3, 5', 1), (2, 'Int8', '3, 5', 1)):
    created = genLevel1.normal_generator('DOFType_GetEncoding_' + str(encoding_data[0]), 'DOFType_GetEncoding.tmpl')
    _add_simple(created, encoding_data[1], encoding_data[0], encoding_data[2], encoding_data[3])
del encoding_data

DOFType.add_test_generator(genLevel1.normal_generator('DOFType_GetSubtypesCount_1', 'DOFType_GetFieldCount.tmpl'))

for field_type_data in range(1, 5):
    created = genLevel1.normal_generator('DOFType_GetSubtype_' + str(field_type_data), 'DOFType_GetFieldType.tmpl')
    created.add_replacement(genLevel1.tr_swap('@@@num@@@', str(field_type_data)))
    DOFType.add_test_generator(created)
del field_type_data

#data is Min/Max, test number, type, params, expected length, structure, array
for minmax_data in (('Max', 1, 'String', '3, 5', 5, 0, 0), ('Max', 2, 'BLOB', '16, 32', 32, 0, 0), ('Max', 3, 'Int8', '', 5, 0, 1), ('Max', 4, 'Int8', '', 0, 1, 0), ('Min', 1, 'Int8', '', 0, 1, 0), ('Min', 2, 'String', '3, 5', 0, 0, 0), ('Min', 3, 'BLOB', '16, 32', 16, 0, 0), ('Min', 4, 'Int8', '', 1, 0, 1)):
    test_name = 'DOFType_Get' + minmax_data[0] + 'Length_' + str(minmax_data[1])
    created = genLevel1.normal_generator(test_name, 'DOFType_GetMinMaxLength.tmpl')
    created.add_replacement(genLevel1.tr_swap('@@@name@@@', test_name))
    created.add_replacement(genLevel1.tr_swap('@@@type@@@', minmax_data[2]))
    created.add_replacement(genLevel1.tr_swap('@@@params@@@', minmax_data[3]))
    created.add_replacement(genLevel1.tr_swap('@@@expected@@@', str(minmax_data[4])))
    created.add_replacement(genLevel1.tr_swap('@@@struct@@@', str(minmax_data[5])))
    created.add_replacement(genLevel1.tr_swap('@@@array@@@', str(minmax_data[6])))
    created.add_replacement(genLevel1.tr_swap('@@@minmax@@@', minmax_data[0]))
    DOFType.add_test_generator(created)
del minmax_data

for type_data in ((1, 'Int8', '', 'DOFTYPE_INT8'), (2, 'Int16', '', 'DOFTYPE_INT16'), (3, 'Int32', '', 'DOFTYPE_INT32'), (4, 'Int64', '', 'DOFTYPE_INT64'), (5, 'UInt8', '', 'DOFTYPE_UINT8'), (6, 'UInt16', '', 'DOFTYPE_UINT16'), (7, 'UInt32', '', 'DOFTYPE_UINT32'), (8, 'UInt64', '', 'DOFTYPE_UINT64'), (9, 'Float32', '', 'DOFTYPE_FLOAT32'), (10, 'Float64', '', 'DOFTYPE_FLOAT64'), (11, 'DateTime', '', 'DOFTYPE_DATETIME'), (12, 'Boolean', '', 'DOFTYPE_BOOLEAN'), (13, 'GUID', '', 'DOFTYPE_GUID'), (14, 'InterfaceID', '', 'DOFTYPE_INTERFACEID'), (15, 'ObjectID', '', 'DOFTYPE_OBJECTID'), (16, 'String', '3, 32', 'DOFTYPE_STRING'), (20, 'BLOB', '32, 64', 'DOFTYPE_BLOB'), (21, 'BLOB', '32, 32', 'DOFTYPE_BLOB')):
    created = genLevel1.normal_generator('DOFType_GetTypeID_' + str(type_data[0]), 'DOFType_GetType.tmpl')
    _add_simple(created, type_data[1], type_data[0], type_data[2], 0)
    created.add_replacement(genLevel1.tr_swap('@@@expected@@@', type_data[3]))
del type_data

DOFType.add_test_generator(genLevel1.normal_generator('DOFType_GetTypeID_22', 'DOFType_GetType_Structure.tmpl'))
DOFType.add_test_generator(genLevel1.normal_generator('DOFType_GetTypeID_23', 'DOFType_GetType_Array.tmpl'))

#for encoding_data in ((1, 'String', '3, 5', 'TRUE'), (2, 'Int8', '', 'FALSE')):
#    created = genLevel1.normal_generator('DOFType_HasEncoding_' + str(encoding_data[0]), 'DOFType_HasEncoding.tmpl')
#    _add_simple(created, encoding_data[1], encoding_data[0], encoding_data[2], 0)
#    created.add_replacement(genLevel1.tr_swap('@@@expected@@@', encoding_data[3]))
#del encoding_data

#is_data is num, create call, compare_type, structure, farray, varray, assert_valid
for is_data in ((1, 'DOFTypeTypedef_Create(DOFVALUEINT8_TYPE)', 'Defined', 0, 0, 0, 1), (2, 'DOFTypeTypedef_Create(NULL)', 'Defined', 0, 0, 0, 0)):
    created = genLevel1.normal_generator('DOFType_Is' + is_data[2] + '_' + str(is_data[0]), 'DOFType_IsType.tmpl')
    _add_simple(created, '', is_data[0], '', is_data[6])
    created.add_replacement(genLevel1.tr_swap('@@@create_call@@@', is_data[1]))
    created.add_replacement(genLevel1.tr_swap('@@@structure@@@', str(is_data[3])))
    created.add_replacement(genLevel1.tr_swap('@@@farray@@@', str(is_data[4])))
    created.add_replacement(genLevel1.tr_swap('@@@varray@@@', str(is_data[5])))
    created.add_replacement(genLevel1.tr_swap('@@@compare_type@@@', is_data[2]))
del is_data

#is_data is num, compare_type, ((type0, params0)...(type5, params5)), index_a, index_b, assert_valid
#for is_data in \
#    ((1, 'Compatible', (('Int8', ''), ('Int16', '')), 0, 1, 0),
#    (2, 'Compatible', (('Int8', ''), ('UInt8', '')), 0, 1, 0),
#    (3, 'Compatible', (('Int8', ''), ('Structure', '1, &types[0]'), ('Structure', '1, &types[0]')), 1, 2, 1), \
#    (4, 'Compatible', (('Int8', ''), ('UInt8', ''), ('Structure', '1, &types[0]'), ('Structure', '1, &types[1]')), 2, 3, 0), \
#    (5, 'Compatible', (('String', '32'), ('String', '64'), ('Structure', '1, &types[0]'), ('Structure', '1, &types[1]')), 2, 3, 1), \
#    (6, 'Compatible', (('String', '64'), ('String', '32'), ('Structure', '1, &types[0]'), ('Structure', '1, &types[1]')), 2, 3, 0), \
#    (7, 'Compatible', (('BLOB', '32, 32'), ('Structure', '1, &types[0]'), ('Structure', '1, &types[0]')), 1, 2, 1), \
#    (8, 'Compatible', (('BLOB', '32, 64'), ('BLOB', '32, 32'), ('Structure', '1, &types[0]'), ('Structure', '1, &types[1]')), 2, 3, 0), \
#    (9, 'Compatible', (('BLOB', '16, 32'), ('BLOB', '32, 32'), ('Structure', '1, &types[0]'), ('Structure', '1, &types[1]')), 2, 3, 0), \
#    (10, 'Compatible', (('FString', '32, "hello"'), ('Structure', '1, &types[0]'), ('Structure', '1, &types[1]'), ('Structure', '1, &types[1]')), 2, 3, 1), \
#    (11, 'Compatible', (('FString', '32, "hello"'), ('FString', '24, "hello"'), ('Array', 'types[0], 2, 2'), ('Array', 'types[1], 2, 2'), ('Structure', '1, &types[2]'), ('Structure', '1, &types[3]')), 4, 5, 0), \
#    (12, 'Compatible', (('Int8', ''), ('Array', 'types[0], 2, 2'), ('Array', 'types[0], 2, 2')), 1, 2, 1), \
#    (13, 'Compatible', (('Int8', ''), ('Array', 'types[0], 2, 4'), ('Array', 'types[0], 2, 2')), 1, 2, 0), \
#    (14, 'Compatible', (('Int8', ''), ('Int8', ''), ('Array', 'types[0], 1, 2'), ('Array', 'types[1], 2, 2')), 1, 2, 0), \
#    (15, 'Compatible', (('Int8', ''), ('UInt8', ''), ('Array', 'types[0], 1, 2'), ('Array', 'types[1], 1, 2')), 1, 2, 0), \
#    (16, 'Compatible', (('FString', '1, "hello"'), ('Structure', '1, &types[0]'), ('Array', 'types[1], 2, 2'), ('Array', 'types[1], 2, 2')), 2, 3, 1), \
#    (17, 'Compatible', (('FString', '32, "hello"'), ('FString', '24, "hello"'), ('Structure', '1, &types[0]'), ('Structure', '1, &types[1]'), ('Array', 'types[2], 2, 2'), ('Array', 'types[3], 2, 2')), 4, 5, 0),
#    (18, 'Compatible', (('Int8', ''),), 0, 1, 0),
#    (19, 'Compatible', (('Int8', ''),), 1, 0, 0),
#    (20, 'Compatible', (), 0, 0, 1),    
#    created = genLevel1.normal_generator('DOFType_Is' + is_data[1] + '_' + str(is_data[0]), 'DOFType_IsCompatibleEqual.tmpl')
#    created.add_replacement(genLevel1.tr_swap('@@@compare_type@@@', is_data[1]))
#    created.add_replacement(genLevel1.tr_swap('@@@assert_valid@@@', str(is_data[5])))
#    created.add_replacement(genLevel1.tr_swap('@@@index_a@@@', str(is_data[3])))
#    created.add_replacement(genLevel1.tr_swap('@@@index_b@@@', str(is_data[4])))
#    created.add_replacement(genLevel1.tr_swap('@@@num@@@', str(is_data[0])))
#    counter = 0
#    for sub_data in is_data[2]:
#        created.add_replacement(genLevel1.tr_swap('@@@create' + str(counter) + '@@@', '1'))
#        created.add_replacement(genLevel1.tr_swap('@@@type' + str(counter) + '@@@', sub_data[0]))
#        created.add_replacement(genLevel1.tr_swap('@@@params' + str(counter) + '@@@', sub_data[1]))
#        counter += 1
#    for i in range(counter, 6):
#        created.add_replacement(genLevel1.tr_swap('@@@create' + str(i) + '@@@', '0'))
#        created.add_replacement(genLevel1.tr_swap('@@@type' + str(counter) + '@@@', ''))
#        created.add_replacement(genLevel1.tr_swap('@@@params' + str(counter) + '@@@', ''))
#    DOFType.add_test_generator(created)
#del is_data

DOFType.generate()
