#!/usr/bin/python
#
#  Copyright (c) 2010-2015, Panasonic Corporation.
#
#  Permission to use, copy, modify, and/or distribute this software for any
#  purpose with or without fee is hereby granted, provided that the above
#  copyright notice and this permission notice appear in all copies.
#
#  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
#  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
#  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
#  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
#  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
#  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

from __future__ import with_statement

import sys
import os

#do not invoke directly! NOT THREAD SAFE!

embunit_path = ""
input_path = ""

#---------------------------------------------------------------------#

def init(_embunit_path, _input_path):
    global embunit_path, input_path
    embunit_path = _embunit_path
    input_path = _input_path

#---------------------------------------------------------------------#

class token_replacement:
    "Parent class of a token replacement operation."
    def __init__(self, match_token):
        self.match_token = match_token
        
    def do_replace(self):
        print "This method is intended to be pure virutal, so child classes must override it."
        assert 0
        
class tr_swap(token_replacement):
    "Simple token swap. Replaces token with given string or value."
    def __init__(self, match_token, replacement):
        token_replacement.__init__(self, match_token)
        self.replacement = replacement

    def do_replace(self, data):
        "Replaces the matcing token with the given replacement"
        backup = ""
        while(data != backup):
            backup = data
            data = data.replace(self.match_token, self.replacement)
        return data

#------------------------------------------------------------------------#

class test_generator:
    "Parent class of all individual tests."
        
    def __init__(self, test_name, template_filename):
        self.token_replacements = []
        self.test_name = test_name
        self.template_filename = input_path + '/' + template_filename
        with open(self.template_filename) as f:
            self.template_data = f.read()
        
    def add_replacement(self, replacement):
        self.token_replacements.append(replacement)
    
    def expand_template(self, data):
        "Expands the template into the output file."
        for replacement in self.token_replacements:
            data = replacement.do_replace(data)
        return data

class normal_generator(test_generator):
    "Handle the usual test case."
    def __init__(self, test_name, template_filename):
        test_generator.__init__(self, test_name, template_filename)
        self.add_replacement(tr_swap('static void ' + test_name + '(void)\n{\n\tTEST_FAIL("no implementation");\n}\n', self.template_data))

class setUp_generator(test_generator):
    "Handle the setUp template."
    def __init__(self, template_filename):
        test_generator.__init__(self, 'setUp', template_filename)
        self.add_replacement(tr_swap('static void setUp(void)\n{\n\t/* initialize */\n}\n', self.template_data))
        
class tearDown_generator(test_generator):
    "Handle the tearDown template."
    def __init__(self, template_filename):
        test_generator.__init__(self, 'tearDown', template_filename)
        self.add_replacement(tr_swap('static void tearDown(void)\n{\n\t/* terminate */\n}\n', self.template_data))
    
#-------------------------------------------------------------------------#

class level1_generator:
    "Parent class of the whole level1 test. A level1_generator holds many test_generators, which hold many token_replacements."
    def __init__(self, output_filename, module_name):
        self.output_filename = output_filename
        self.module_name = module_name
        self.t = None
        self.output_data = ""
        self.include_filenames = []
        self.test_generators = []
        
        self.add_include_filename('dof/oal.h')
    
    def read_output_data(self):
        with open(self.module_name+'_unit.c') as f:
            self.output_data = f.read()
    
    def write_output_data(self):
        with open(self.module_name+'_unit.c', 'w') as f:
            f.write(self.output_data)
    
    def add_test_generator(self, generator):
        self.test_generators.append(generator)
    
    def add_include_filename(self, include_filename):
        self.include_filenames.append(include_filename)

    def tcuppa(self, generator):
        command = embunit_path + '/tcuppa ' + self.module_name + ' '
    
        for include in self.include_filenames:
            command = command + include + ' '
    
        command = command + generator.test_name
        ret = os.system(command)
        assert not ret
        
        #read output data in
        self.read_output_data()
        
        self.output_data = "/* @@@ Generated File! Do not modify! @@@ */\n\n" + self.output_data
        
        #tcuppa has made a file, so fill in the setUp/tearDown part of them
        self.output_data = setUp_generator(self.module_name+'_setUp.tmpl').expand_template(self.output_data)
        self.output_data = tearDown_generator(self.module_name+'_tearDown.tmpl').expand_template(self.output_data)
        
        #finally set t to tuma for additional tests
        self.t = self.tuma
        
    def tuma(self, generator):
        command = embunit_path + '/tuma ' + self.module_name + ' ' + generator.test_name
        ret = os.system(command)
        assert not ret
        self.read_output_data()
        
    def generate(self):
        "Generates the output file."
        if(len(self.test_generators)==0):
            print "Must have some test_generator instances added."
            assert 0

        #start with calling tcuppa, and then switch to tuma after the first one
        self.t = self.tcuppa

        #iterate over all generators to create the file
        for generator in self.test_generators:
            #use tcuppa or tuma to add tests to file
            self.t(generator)
            #have the generator expand its template
            self.output_data = generator.expand_template(self.output_data)
            self.write_output_data()
        
        #now copy the file to the output directory
        os.system("mv ./" + self.module_name + "_unit.c " + self.output_filename)
        return
