/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include <dof/core.h>
#include <dof/type.h>
#include <dof/value.h>
#include <dof/interfaceid.h>
#include "valueprivate.h"


/** @addtogroup DOFValue
@{
**/


/*
**  "Private" Type and Method Declarations
*/


/*
**  "Public" Methods
*/

/** @internal
    Create a new malloced buffer from the contents of the specified @ref
    DOFValue.

    This is a helper function that can be used to extract string/blob types
    from a DOFValue.

    @note This allocates resources that must later be deallocated by
    calling PCRMem_Free().

    @param  value                The DOFValue to be extracted.
    @param  expectedType    The expected element type of the array (for type checking).
    @param  destroyDOFValue  Destroy the @a value before return (if TRUE).

    @return The new buffer from @a value, or NULL if it could not be created.
**/
uint8 * DOFValue_ToSingleMalloc( const DOFValue value, DOFTypeID expectedType, boolean destroyDOFValue )
{
    uint8 * rval = NULL;
    DOFTypeID typeID = DOFType_GetTypeID( DOFValue_GetDOFType( value ) );

    if( value && ( expectedType == DOFTYPE_VOID || typeID == expectedType ) )
    {
        /* Strings are stored with a NULL terminator, just in case.  It is *not* stored in length. */
        boolean isString = (expectedType == DOFTYPE_STRING);
        /* Allocate the block to hold the data. */
        rval = (uint8 *) PCRMem_Alloc( value->length + isString );
        if ( rval )
        {
            PCRMem_Copy( rval, &value->data[0], value->length + isString );
        }
    }

    if ( destroyDOFValue )
        DOFValue_Destroy( value );

    return( rval );
}


/**
@}
**/
