/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include <dof/core.h>
#include <dof/value.h>
#include <dof/type.h>
#include <internal/type.h>
#include "valueprivate.h"


/** @addtogroup DOFValueStructure
@{
**/


/*
**  "Private" Type and Method Declarations
*/

/*
**  "Public" Methods
*/


DOFValue DOFValueStructure_Create( uint16 fieldCount, const DOFValue fields[] )
{
    DOFType type;
    DOFValue value;

    if( fieldCount && ( !fields || ( fieldCount & 0x8000 ) ) )
        return NULL;

    type = DOFValueStructureType_Create_Values( fieldCount, fields );
    value = DOFValueStructure_Create_Type(type, fields);
    DOFType_Destroy(type);

    return value;
}


DOFValue DOFValueStructure_Create_Type( const DOFType type, const DOFValue fields[] )
{
    uint32 fieldCount, i;

    if ( !type || DOFType_GetTypeID( type ) != DOFTYPE_STRUCTURE )
        return( NULL );
    fieldCount = DOFType_GetSubtypesCount( type );
    for ( i = 0; i < fieldCount; i++ )
    {
        DOFType fieldTypeSpec;

        fieldTypeSpec = DOFType_GetSubtype( type, i );
        if ( !fieldTypeSpec )
            return NULL;
        if ( !DOFValue_CheckCompatible( fields[i], fieldTypeSpec ) )
            return NULL;
    }
    return( DOFValue_Create( DOFType_Copy( type ), fieldCount, (const uint8 *)fields, FALSE, 0 ) );
}


uint16 DOFValueStructure_GetFieldsCount( const DOFValue self )
{
    if( !self || DOFType_GetTypeID(self->type) != DOFTYPE_STRUCTURE )
        return 0;

    return( self->length );
}


DOFValue DOFValueStructure_GetField( const DOFValue self, uint16 field )
{
    if( !self || DOFType_GetTypeID(self->type) != DOFTYPE_STRUCTURE )
        return NULL;

    if ( field < self->length )
        return( self->value[field] );
    return( NULL );
}


const DOFValue * DOFValueStructure_GetFields( const DOFValue self )
{
    if( !self || DOFType_GetTypeID(self->type) != DOFTYPE_STRUCTURE )
        return NULL;

    return( self->value );
}


/**
@}
**/
