/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include <dof/core.h>
#include <dof/type.h>
#include <dof/value.h>
#include "valueprivate.h"


float64 * DOFValueArrayFloat64_Extract( const DOFValue value, uint16 elementOffset, uint16 elementCount )
{
    float64 * rval = NULL;
    DOFType valueType = DOFValue_GetDOFType( value );
    DOFTypeID valueTypeID = DOFType_GetTypeID( valueType );

    /* The request is invalid if there is no value, it is not the correct type, or the elementOffset and/or elementCount specify results that are outside the array bounds. */
    if( value &&
        valueTypeID == DOFTYPE_ARRAY &&
        DOFType_GetTypeID( DOFType_GetSubtype( valueType, 0 ) ) == DOFTYPE_FLOAT64 &&
        elementOffset < value->length &&
        elementCount > 0 &&
        ( (uint32) elementOffset + elementCount ) <= value->length )
    {
        rval = (float64 *) PCRMem_Alloc( sizeof(float64) * elementCount );
        if ( rval )
        {
            while ( elementCount-- )
                rval[elementCount] = DOFValueFloat64_Get( value->value[elementCount + elementOffset] );
        }
    }

    return( rval );
}
