/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include <dof/core.h>
#include <dof/type.h>
#include <dof/value.h>
#include "valueprivate.h"
#include <dof/interfaceid.h>


DOFInterfaceIDList DOFValueArrayInterfaceID_Extract( const DOFValue value, uint16 elementOffset, uint16 elementCount )
{
    DOFInterfaceIDList rval = NULL;
    DOFType valueType = DOFValue_GetDOFType( value );
    DOFTypeID valueTypeID = DOFType_GetTypeID( valueType );

    /* The request is invalid if there is no value, it is not the correct type, or the elementOffset and/or elementCount specify results that are outside the array bounds. */
    if( value &&
        valueTypeID == DOFTYPE_ARRAY &&
        DOFType_GetTypeID( DOFType_GetSubtype( valueType, 0 ) ) == DOFTYPE_INTERFACEID &&
        elementOffset < value->length &&
        elementCount > 0 &&
        ( (uint32) elementOffset + elementCount ) <= value->length )
    {
        /* Allocate the List object. */
        rval = (DOFInterfaceIDList) DOFMem_Alloc( sizeof( struct DOFInterfaceIDList_t ) );
        if ( rval )
        {
            rval->count = elementCount;
            /* Allocate the List itself. */
            rval->list  = (DOFInterfaceID*) DOFMem_Alloc( elementCount * sizeof(DOFInterfaceID) );
            if ( rval->list )
            {
                PCRMem_Set( rval->list, 0, rval->count * sizeof(DOFInterfaceID) );
                while ( elementCount-- )
                {
                    rval->list[elementCount] = DOFValueInterfaceID_Extract( value->value[elementCount + elementOffset] );
                    if ( !rval->list[elementCount] )
                    {
                        goto fail;
                    }
                }
            }
            else
            {
              fail:
                DOFInterfaceIDList_Destroy( rval );
                rval = NULL;
            }
        }
    }

    return( rval );
}
