/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include <dof/core.h>
#include <dof/type.h>
#include <dof/value.h>
#include <dof/error.h>
#include <internal/value.h>


/** @addtogroup DOFValue
@{
**/


/*
**  "Private" Type and Method Declarations
*/

extern PCRMutex ref_count_mutex;

/*
**  "Public" Methods
*/

void DOFValueList_Destroy( uint16 valueCount, DOFValue valueList[] )
{
    uint16 i;

    for ( i = 0; i < valueCount; i++ )
    {
        DOFValue_Destroy( valueList[i] );
        valueList[i] = NULL;
    }
}


DOFValue * DOFValueListAlloc_Create( uint16 valueCount )
{
    DOFValue * rval = DOFMem_Alloc( sizeof(DOFValue) * ( (valueCount)? valueCount : 1 ) );

    if ( rval )
    {
        PCRMem_Set( rval, 0, sizeof(DOFValue) * valueCount );
    }
    return rval;
}


void DOFValueListAlloc_Destroy( uint16 valueCount, DOFValue * valueList )
{
    if ( valueList )
    {
        DOFValueList_Destroy( valueCount, valueList );
        DOFMem_Free( valueList );
    }
}


DOFValue * DOFValueListAlloc_Copy( uint16 valueCount, const DOFValue valueList[] )
{
    uint16 i;
    DOFValue * rval = DOFValueListAlloc_Create( valueCount );

    if ( rval )
    {
        for ( i = 0; i < valueCount; i++ )
        {
            if ( valueList[i] )
            {
                rval[i] = DOFValue_Copy( valueList[i] );
                if ( !rval[i] )
                {
                    DOFValueList_Destroy( valueCount, rval );
                    DOFMem_Free( rval );
                    return NULL;
                }
            }
        }
    }
    return rval;
}


/**
@}
**/
