/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef _DOF_VALUEPRIVATE_H
#define _DOF_VALUEPRIVATE_H

#include <dof/core.h>
#include <internal/value.h>


#ifdef  __cplusplus
extern "C" {
#endif

/** @addtogroup DOFValue
@{
**/


/** @internal
    Internal DOFValue representation, suitable for all types of DOFValues.
**/
struct DOFValue_t
{
    uint32   refCount;       /**< Reference Count. */
    uint32   length;         /**< Element Count (for Array types), Field Count (for Structure types), or data size (for all other types).  **/
    DOFType  type;           /**< Type Specification. **/
    uint16   encoding;       /**< MIBenum representing coded character set of String (only for String type). **/
    uint8    data[1];        /**< Actual data.  Space is allocated appropriately starting at this location for the data that needs to be contained.  **/
    DOFValue value[1];       /**< Contained values (for Array and Structure types).  Space is allocated appropriately starting at this location for the values that need to be contained.  **/
};

DOFValue DOFValue_Create( DOFType type, uint32 size, const uint8 *data, boolean isMarshalled, uint16 encoding );
uint8 ** DOFValueArray_ToComboMalloc( const DOFValue value, DOFTypeID expectedType, uint16 elementOffset, uint16 elementCount, boolean destroyDOFValueArray );
uint8 * DOFValue_ToSingleMalloc( const DOFValue value, DOFTypeID expectedType, boolean destroyDOFValue );


/**
@}
**/


#ifdef  __cplusplus
}
#endif

#endif /* _DOF_VALUEPRIVATE_H */
