/*
**  Copyright (c) 2010-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#ifndef DOF_PCRTYPES_H
#define DOF_PCRTYPES_H

/*
** Includes
*/

#include <dof/arch_types.h>


/*
** Definitions
*/

#ifndef DOF_PCR_DECLSPEC
/**
 * Platform-specific public function prototype declaration specification.
 * Some platforms make use of this identifier to specially mark pulic functions (e.g., for library import/export, etc.).
 * By default, this is empty.
 */
# define DOF_PCR_DECLSPEC
#endif

#ifndef NULL
/** Standard definition of NULL. **/
# define NULL   ((void *)0)
#endif

#ifndef FALSE
/** Boolean FALSE value. **/
# ifdef  __cplusplus
#  define FALSE false
# else
#  define FALSE 0
# endif
#endif

#ifndef TRUE
/** Boolean TRUE value. **/
# ifdef  __cplusplus
#  define TRUE  true
# else
#  define TRUE  1
# endif
#endif

#ifndef lengthof
/** Determine the length of a constant array. **/
# define lengthof(x)    ( sizeof( (x) ) / sizeof( (x)[0] ) )
#endif

#ifndef offsetof
/** Determine the offset of a member within a structure. **/
# define offsetof(type, member) ((uint32)((uint8*)&((type*)0)->member - (uint8*)0))
#endif


#ifdef  __cplusplus
extern "C" {
#endif


/*
** Typedefs
*/

/** @defgroup PCRTypes Standard Types and Definitions
    Common core basic types and definitions upon which the DOF C-Language Object Access Library is built.

    @ingroup PCR
@{
**/

typedef uint8_t     uint8;           /**< Unsigned 8-bit integer. **/
typedef uint16_t    uint16;          /**< Unsigned 16-bit integer. **/
typedef uint32_t    uint32;          /**< Unsigned 32-bit integer. **/
typedef uint64_t    uint64;          /**< Unsigned 64-bit integer. **/
typedef int8_t      int8;            /**< Signed 8-bit integer. **/
typedef int16_t     int16;           /**< Signed 16-bit integer. **/
typedef int32_t     int32;           /**< Signed 32-bit integer. **/
typedef int64_t     int64;           /**< Signed 64-bit integer. **/

typedef float       float32;         /**< 32-bit floating point value. **/
typedef double      float64;         /**< 64-bit floating point value. **/

#ifdef __cplusplus
typedef bool        boolean;         /**< Boolean value.  Zero is false, nonzero is true.  **/
#else
typedef uint8       boolean;         /**< Boolean value.  Zero is false, nonzero is true.  **/
#endif
typedef int64       DateTime;        /**< Date & Time value (64-bit).  Represented as milliseconds since epoch of 1/1/1970. **/
typedef struct { uint8 bytes[16]; }  GUID; /**< Globally Unique Identifier. **/

/**
@}
**/


#ifdef  __cplusplus
}
#endif

#endif /* DOF_PCRTYPES_H */
