/*
**  Copyright (c) 2014-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include "config.h"
#include <dof/pcr.h>

/* Standard headers */
#include <string.h>
#include <stdlib.h>

void PCRMem_Free(void * mem)
{
    free(mem);  /* For NULL Memory, must do nothing and safely return. */
}


void * PCRMem_Alloc(uint32 size)
{
#ifdef PCRMEM_ALLOC_ASSERT
    void * ret = malloc(size);
    PCR_Assert(ret);
    return(ret);
#else
    return(malloc(size));
#endif
}


void * PCRMem_Realloc(void * mem, uint32 size)
{
    PCR_Assert(mem);

#ifdef PCRMEM_ALLOC_ASSERT
    {
        void * ret = realloc(mem, size);
        PCR_Assert(ret);
        return(ret);
    }
#else
    return(realloc(mem, size));
#endif
}


int PCRMem_Compare(const void * buf1, const void * buf2, uint32 n)
{
    if(!n) return 0;

    PCR_Assert(buf1&&buf2);
    return memcmp((buf1),(buf2),(n));
}


void PCRMem_Copy( void *dest, const void *src, uint32 n )
{
    if(!n) return;

    PCR_Assert(dest&&src);

    memcpy((dest),(src),(n));
}


void PCRMem_Move( void *dest, const void *src, uint32 n )
{
    if(!n) return;

    PCR_Assert(dest&&src);

    memmove((dest),(src),(n));
}


void PCRMem_Set( void *ptr, uint8 c, uint32 n )
{
    if(!n) return;

    PCR_Assert(ptr);

    memset((ptr),(c),(n));

}

