/*
**  Copyright (c) 2014-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include "config.h"
#include <dof/pcr.h>
#include <internal/mutex.h>
#include <internal/pcrinternal.h>

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>

void PCR_Init( void )
{
    if(PCRSecureRandom_Init())      /* Report failure more appropriately */
    {
        PCRRandom_Seed( PCRTime_GetMS() ^ PCRSecureRandom_Get() ); /* Could just use secure random seed. */
        PCRMutex_Init();
    }
}


void PCR_Shutdown( void )
{
    PCRSecureRandom_Shutdown();
}


void PCR_AssertFail( const char *exprString, const char *fileString, uint32 line )
{
    fprintf( stderr, "Assertion `%s' failed at %s:%lu.\n", exprString, fileString, (unsigned long) line );
    fflush( stderr );
    PCR_Die( 0 );
}

void PCR_ValidateFail( const char *exprString, const char *fileString, uint32 line )
{
    fprintf( stderr, "Unrecoverable error: validation `%s' failed at %s:%lu.\n", exprString, fileString, (unsigned long) line );
    fflush( stderr );
    PCR_Die( 1 );
}

void PCR_Die( uint32 code )
{
    abort();
}

