/*
**  Copyright (c) 2014-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include "config.h"
#include <dof/pcr.h>
#include <internal/pcrinternal.h>

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>

static FILE * randomSource = NULL;

void PCRRandom_Generate(uint32 len, uint8 *buf)
{
    while ( len-- > 0 )
        buf[len] = random() >> 23; /* Use upper most bits of randomness (MSB is always zero from random call). */
}


void PCRRandom_Seed( uint32 seed )
{
    srandom(seed);
}


uint32 PCRRandom_Get( void )
{
    /* Obtain full 32-bits of randomness (a single random call only gives 31 bits). */
    return ( random() << 16 ) | ( random() & 0xffff );
}

boolean PCRSecureRandom_Init(void)
{
    randomSource = fopen("/dev/urandom", "r");
    return (randomSource != NULL);
}

void PCRSecureRandom_Shutdown(void)
{
    fclose(randomSource);
}

void PCRSecureRandom_Generate(uint32   len,
                              uint8  * buf)
{
    fread(buf, sizeof(uint8), len, randomSource);
}

uint32 PCRSecureRandom_Get(void)
{
    uint32 ret;

    fread(&ret, sizeof(uint32), 1, randomSource);
    return ret;
}

