/*
**  Copyright (c) 2014-2015, Panasonic Corporation.
**
**  Permission to use, copy, modify, and/or distribute this software for any
**  purpose with or without fee is hereby granted, provided that the above
**  copyright notice and this permission notice appear in all copies.
**
**  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
**  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
**  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
**  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
**  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
**  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
**  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include "config.h"
#include <dof/pcr.h>

/* Standard headers */
#include <pthread.h> 
#include <errno.h>
#include <sys/select.h>
#include <sys/time.h>

/* Internal thread representation. */
struct PCRThread_t
{
    pthread_t   thread;
};


PCRThread PCRThread_Create(PCRThreadCall threadFunc, void * threadParam)
{
    PCRThread thread;

    PCR_Assert(threadFunc);

    thread = (PCRThread)PCRMem_Alloc(sizeof(struct PCRThread_t));
    PCRMem_Set( thread, 0, sizeof(struct PCRThread_t) );
    if ( thread )
    {
        if ( pthread_create( &thread->thread, NULL, (void*(*)(void *))threadFunc, threadParam ) != 0 )
        {
            PCRMem_Free(thread);
            thread = NULL;
        }
    }
    return(thread);
}


PCRStatus PCRThread_Join(PCRThread thread)
{
    if ( thread == NULL || pthread_join(thread->thread, NULL) )
        return(PCRSTATUS_ERROR);
    return(PCRSTATUS_OK);
}


void PCRThread_Sleep(uint32 mseconds)
{
    struct timeval timeout;
    uint32 start = PCRTime_GetMS(), end;

    for (;;)
    {
        timeout.tv_sec  = mseconds / 1000;
        timeout.tv_usec = ( mseconds % 1000 ) * 1000;

        /* Select is used to perform a sleep */
        if (select(0, 0, 0, 0, &timeout) != 0 && errno == EINTR && ((end = PCRTime_GetMS()) - start < mseconds))
        {
            mseconds -= end - start;
            continue;
        }
        break;
    }
}


PCRStatus PCRThread_Yield(void)
{
    int retval = sched_yield();     /* Returns 0 for success, -1 for not success. */

    if (retval)
        return(PCRSTATUS_ERROR);
    return(PCRSTATUS_OK);
}


void PCRThread_Destroy(PCRThread thread)
{
    PCRMem_Free(thread);
}

